/*****************************************************************************
 *
 * txinfomsg.h - TX INFO API Message Definitions
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                         PROPRIETARY NOTICE
 *
 *  (c)Copyright 1988-2009 Dialogic Corporation.  All Rights Reserved
 *
 *  The following is licensed program material and is considered
 *  proprietary and confidential.  No part of this program may be
 *  reproduced or used in any form or by any means without the express
 *  written permission of Dialogic Corporation.
 *
 *****************************************************************************/

#ifndef _TXINFOMSG_H
#define _TXINFOMSG_H

/* ----- dependent includes ----- */
#include "txnuser.h"
#include "cosinfo.h"

/*---------------------------------------------------------------------------*
 * System Identification - information identifying the specific TX board type
 *---------------------------------------------------------------------------*/
#define TXINFO_SYSID_AVAIL          8   /* number of U32s available for future use */
typedef struct
{
    U32                 cpNum;          /* CP (Communications Processor) number */
    U32                 boardType;      /* board type identifier [see cosinfo.h - TXxxx_SYSID] */
    U32                 serialNum;      /* TX board serial number */
    U32                 kernelVersion;  /* kernel version */
    U32                 kernelRevision; /* kernel revision */

    U32                 avail[TXINFO_SYSID_AVAIL];      /* room for future expansion */
} TXINFO_SYSID;

/* ----- System Utilization definitions ----- */
#define TXINFO_SYSUTIL_BUCKETS      16  /* number of utilization buckets maintained */
#define TXINFO_SYSUTIL_BUCKET_WRAP  0xF /* mask to apply when handling CPU bucket wrap */
#define TXINFO_SYSUTIL_AVAIL        5   /* number of U32s available for future use */

/*---------------------------------------------------------------------------*
 * System Utilization block - area used to compute CPU utilization
 *---------------------------------------------------------------------------*/
typedef struct
{
    U32         idleCount;      /* counter incremented whenever CPU is idle [cleared once a second] */
    U32         idlePeak;       /* peak value that idleCount has ever reached */
    U32         lastBucket;     /* last cpuBucket that was updated (index into cpuBucket array) */
    U32         cpuBucket[TXINFO_SYSUTIL_BUCKETS];      /* per second history of idleCount */

    U32         avail[TXINFO_SYSUTIL_AVAIL];            /* room for future expansion */
} TXINFO_SYSUTIL;

/*---------------------------------------------------------------------------*
 * System Timestamp - time of day
 *---------------------------------------------------------------------------*/
typedef struct
{
    U32         year;
    U32         month;
    U32         day;
    U32         hour;
    U32         min;
    U32         sec;
    U32         zone;
    U32         msec;
} TXINFO_TIMESTAMP;

/*---------------------------------------------------------------------------*
 * System Time block - time passage handling area
 *---------------------------------------------------------------------------*/
#define TXINFO_SYSTIME_AVAIL        4   /* number of U32s available for future use */
typedef struct
{
    U32                 bootMsecs;      /* number of milliseconds since boot */
    U32                 epochSecs;      /* number of seconds since epoch */
    U32                 bmsecEpoch;     /* number of msecs since boot when epoch secs were recorded */
    TXINFO_TIMESTAMP    sysTime;        /* system time in timestamp format */
    U32                 bmsecWraps;     /* number of times that milliseconds since boot has wrapped */
    U32                 avail[TXINFO_SYSTIME_AVAIL];    /* room for future expansion */
} TXINFO_SYSTIME;

/*---------------------------------------------------------------------------*
 * System Status - information about current system status
 *---------------------------------------------------------------------------*/
#define TXINFO_SYSSTATUS_AVAIL      32  /* number of U32s available for future use */
typedef struct
{
    U32                 cpuTemp;        /* current CPU temperature (in tenths of degrees Celcius) */
    U32                 boardTemp;      /* current board temperature (in tenths of degrees Celcius) */
    U32                 fanState;       /* current state of the on-board fan */
#define TXINFO_FAN_UNKNOWN      0       /* fan state in unknown (state has not been read yet) */
#define TXINFO_FAN_RUNNING      1       /* fan is running normally (no stop detected) */
#define TXINFO_FAN_STOPPED      2       /* fan has stopped */
#define TXINFO_FAN_RESTARTED    3       /* fan had stopped but has now restarted */
#define TXINFO_FAN_SLOW         4       /* fan is running slow */
#define TXINFO_FAN_NORMAL       1       /* fan is running noarmally */
#define TX5_FAN_THRESH_NORMAL   4500    /* fan is running noarmally if equal to or above this value */

    U32                 avail[TXINFO_SYSSTATUS_AVAIL];  /* room for future expansion */
} TXINFO_SYSSTATUS;

/*---------------------------------------------------------------------------*
 * TXINFO_SYSTEM_INFO - System-Level Information
 *---------------------------------------------------------------------------*/
#define TXINFO_SYSTEM_INFO_AVAIL    32          /* number of U32's available for future use */
typedef struct
{
    TXINFO_SYSID        id;                     /* system identification */
    TXINFO_SYSUTIL      util;                   /* system utilization information */
    TXINFO_SYSTIME      time;                   /* system time handling */
    TXINFO_SYSSTATUS    status;                 /* current system-level status */
    U32                 avail[TXINFO_SYSTEM_INFO_AVAIL];
} TXINFO_SYSTEM_INFO;

/*###########################################################################*
 *
 * Message Protocol Definitions
 *
 * NOTE: All fields passed between the TX board and the host system
 *       must be in network byte order.
 *
 *###########################################################################*/

/*---------------------------------------------------------------------------*
 * TXINFO_HDR - common header used for all TX INFO API messages
 *---------------------------------------------------------------------------*/
#define TXINFO_HDR_AVAIL            4           /* number of U32's available for future use */
typedef struct
{
    U32                     msgfunc;            /* message function */
#define TXINFO_MSG_SYSTEM_INFO  0x01            /* get overall system information */

    U32                     msgtype;            /* type of message */
#define TXINFO_REQUEST          0x01
#define TXINFO_RESPONSE         0x02
#define TXINFO_NOTIFICATION     0x03

    U32                     msgkey;             /* key (provided by user in REQUEST; duplicated in RESPONSE; special in NOTIF) */
    U32                     errorcode;          /* any error code associated with message (see coserror.h) */
    U32                     errorparam;         /* parameter associated with error code (error-code-specific) */
    U32                     avail[TXINFO_HDR_AVAIL];
} TXINFO_HDR;

/*---------------------------------------------------------------------------*
 * TXINFO_SYSTEM_INFO_REQ - System-level information request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXINFO_HDR                hdr;
} TXINFO_SYSTEM_INFO_REQ;

/*---------------------------------------------------------------------------*
 * TXINFO_SYSTEM_INFO_RSP - System-level information response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXINFO_HDR                hdr;
    TXINFO_SYSTEM_INFO_REQ    req;                /* copy of original request */
    TXINFO_SYSTEM_INFO        system;             /* system-level information */
} TXINFO_SYSTEM_INFO_RSP;

/*###########################################################################*
 * FULL MESSAGE UNION - single structure overlay for all messages
 *###########################################################################*/

/*---------------------------------------------------------------------------*
 * TXINFO_MSG - single structure representing all TXINFO messages
 *---------------------------------------------------------------------------*/
typedef union __txinfo_msg
{
    TXINFO_HDR            common;

    union
    {
        TXINFO_SYSTEM_INFO_REQ      info;
    } request;

    union
    {
        TXINFO_SYSTEM_INFO_RSP      info;
    } response;

    union
    {
        TXINFO_HDR                  reserved;                   /* TXINFO NOTIFICATIONS: reserved for future use */
    } notif;

} TXINFO_MSG;

#endif  /* _TXINFOMSG_H */
