/*****************************************************************************
 *
 * txipmsg.h - TX IP (Internet Protocol) Control API Message Definitions
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                         PROPRIETARY NOTICE
 *
 *  (c)Copyright 1988-2009 Dialogic Corporation.  All Rights Reserved
 *
 *  The following is licensed program material and is considered
 *  proprietary and confidential.  No part of this program may be
 *  reproduced or used in any form or by any means without the express
 *  written permission of Dialogic Corporation.
 *
 *****************************************************************************/

#ifndef _TXIPMSG_H
#define _TXIPMSG_H

/* ----- dependent includes ----- */
#include "txnuser.h"

/*###########################################################################*
 *
 * GENERAL USE IP STRUCTURES AND DEFINITIONS
 *
 *###########################################################################*/

#define TXIP_NAME_SZ            16          /* characters reserved for general ASCII name */
#define TXIP_OWNER_SZ           8           /* characters reserved for name of owner */

#define TXIP_STATE_CREATED      1           /* entity has been created, but has not been attached */
#define TXIP_STATE_ATTACHING    2           /* entity is attaching to a given IP layer */
#define TXIP_STATE_DISABLED     3           /* entity is attached to IP layer, but is currently disabled */
#define TXIP_STATE_ENABLING     4           /* entity is currently being enabled */
#define TXIP_STATE_ENABLED      5           /* entity is enabled and ready for data passing */
#define TXIP_STATE_DOWN         6           /* entity is enabled but communication link is currently down */
#define TXIP_STATE_BINDING      7           /* entity has not yet been successfully bound to an address */
#define TXIP_STATE_UP           8           /* entity is enabled and communication link is currently up */
#define TXIP_STATE_DISABLING    9           /* entity is being disabled */
#define TXIP_STATE_DETACHING    10          /* entity is detaching from a given IP layer */
#define TXIP_STATE_DEAD         0xFF        /* entity no longer exists */

#define TXIP_BROADCAST_IPV4     0xFFFFFFFF  /* IPv4 broadcast address */
#define TXIP_ADDR_IPV6_CNT      4           /* number of U32 entries in IPv6 IP address */

#define TXIP_SIZE_ANY           0xFFFFFFFF  /* special indicator for "any size value" */
#define TXIP_STATS_ZERO         1           /* standard value for zeroFlag to indicate stats should be zero'd after reported */
#define TXIP_STATS_MASK         0xAAAAAAAA  /* special value used for zeroflag to request mask of stats supported */
#define TXIP_STAT_SUPPORTED     0xFFFFFFFF  /* stats value returned for each supported stats field (based on specific entity type) */

#define TXIP_DISABLED           0           /* option disabled */
#define TXIP_ENABLED            1           /* option enabled */

/*---------------------------------------------------------------------------*
 * TXIP_ADDR_IPV4 - IPv4 IP Address
 *---------------------------------------------------------------------------*/
typedef struct
{
    U32                 addr32;
} TXIP_ADDR_IPV4;

/*---------------------------------------------------------------------------*
 * TXIP_ADDR_IPV6 - IPv6 IP Address
 *---------------------------------------------------------------------------*/
typedef struct
{
    U32                 addr32[TXIP_ADDR_IPV6_CNT];
} TXIP_ADDR_IPV6;

/*---------------------------------------------------------------------------*
 * TXIP_ADDR - IP Address
 *---------------------------------------------------------------------------*/
typedef struct
{
    U32                 addrType;           /* type of IP address respresentation (all definitions shuold be identical regardless of endianness) */
#define TXIP_NONE   0x00000000              /* IP address has not been defined */
#define TXIP_IPV4   0x44444444              /* IPv4 address */
#define TXIP_IPV6   0x66666666              /* IPv6 address */
#define TXIP_RSRV   0xEEEEEEEE              /* reserved IP address entry: used to mark slot as reserved before address, etc. is known */

    /* ----- type-specific IP address ----- */
    union
    {
        TXIP_ADDR_IPV4  ipv4;
        TXIP_ADDR_IPV6  ipv6;
        U8              reserved[28];
    } u;
} TXIP_ADDR;

/*---------------------------------------------------------------------------*
 * TXIP_DATA_STATS - Basic IP Data Communication Statistics (Uni-directional)
 *---------------------------------------------------------------------------*/
#define TXIP_DATA_STATS_AVAIL       4       /* number of U32's available for future use */
typedef struct
{
    U32                 frames;             /* total number of frames processed */
    U32                 bytes;              /* total number of bytes processed */
    U32                 fails;              /* number of frames that did not complete successfully */
    U32                 lastReason;         /* reason for last failure (see coserror.h) */
    U32                 lastInfo;           /* optional additional information associated with last failure reason */
    U32                 avail[TXIP_DATA_STATS_AVAIL];
} TXIP_DATA_STATS;

/*---------------------------------------------------------------------------*
 * TXIP_COMM_STATS - Basic IP Communication Statistics (Bi-directional)
 *---------------------------------------------------------------------------*/
#define TXIP_COMM_STATS_AVAIL       4       /* number of U32's available for future use */
typedef struct
{
    TXIP_DATA_STATS     rx;                 /* receive stats */
    TXIP_DATA_STATS     tx;                 /* transmit stats */
    U32                 avail[TXIP_COMM_STATS_AVAIL];
} TXIP_COMM_STATS;

/*---------------------------------------------------------------------------*
 * TXIP_FRAG_D_STATS - Ip Fragmentation Statistics (Uni-directional)
 *---------------------------------------------------------------------------*/
#define TXIP_FRAG_D_STATS_AVAIL     4       /* number of U32's available for future use */
typedef struct
{
    U32                 moreFrags;          /* number of fragments with the "more fragments" bit set */
    U32                 endFrags;           /* number of fragments without "more fragments" bit set (end-of-datagram fragments) */
    U32                 pending;            /* number of fragments currently pending (awaiting remaining fragments of datagram) */
    U32                 errors;             /* number of IP fragmentatio-related errors encountered */
    U32                 lastReason;         /* reason for last error (see coserror.h) */
    U32                 lastInfo;           /* optional additional information associated with last error reason */
    U32                 avail[TXIP_FRAG_D_STATS_AVAIL];
} TXIP_FRAG_D_STATS;

/*---------------------------------------------------------------------------*
 * TXIP_FRAG_STATS - FRAG Statistics (Bi-directional)
 *---------------------------------------------------------------------------*/
#define TXIP_FRAG_STATS_AVAIL       4       /* number of U32's available for future use */
typedef struct
{
    TXIP_FRAG_D_STATS   rx;                 /* receive stats */
    TXIP_FRAG_D_STATS   tx;                 /* transmit stats */
    U32                 avail[TXIP_FRAG_STATS_AVAIL];
} TXIP_FRAG_STATS;

/*---------------------------------------------------------------------------*
 * TXIP_ICMP_D_STATS - ICMP Statistics (Uni-directional)
 *---------------------------------------------------------------------------*/
#define TXIP_ICMP_D_STATS_AVAIL     4       /* number of U32's available for future use */
typedef struct
{
    U32                 echoRequest;        /* number of ICMP Echo Request messages */
    U32                 echoReply;          /* number of ICMP Echo Reply messages */
    U32                 timestampRequest;   /* number of ICMP Timestamp Request messages */
    U32                 timestampReply;     /* number of ICMP Timestamp Reply messages */
    U32                 infoRequest;        /* number of ICMP Information Request messages */
    U32                 infoReply;          /* number of ICMP Information Reply messages */

    U32                 destUnreachable;    /* number of ICMP Destination Unreachable messages */
    U32                 sourceQuench;       /* number of ICMP Source Quench messages */
    U32                 redirect;           /* number of ICMP Redirect messages */
    U32                 timeExceeded;       /* number of ICMP Time Exceeded messages */
    U32                 paramProblem;       /* number of ICMP Parameter Problem messages */
    U32                 checksumMismatch;   /* number of ICMP messages with checksum mismatch */
    U32                 replyUnexpected;    /* number of ICMP reply messages when not expected */
    U32                 unknown;            /* number of ICMP messages of unknown type */
    U32                 errors;             /* number of ICMP-level errors */
    U32                 lastReason;         /* reason for last error (see coserror.h) */
    U32                 lastInfo;           /* optional additional information associated with last error reason */

    U32                 avail[TXIP_ICMP_D_STATS_AVAIL];
} TXIP_ICMP_D_STATS;

/*---------------------------------------------------------------------------*
 * TXIP_ICMP_STATS - ICMP Statistics (Bi-directional)
 *---------------------------------------------------------------------------*/
#define TXIP_ICMP_STATS_AVAIL       4       /* number of U32's available for future use */
typedef struct
{
    TXIP_ICMP_D_STATS   rx;                 /* receive stats */
    TXIP_ICMP_D_STATS   tx;                 /* transmit stats */
    U32                 avail[TXIP_ICMP_STATS_AVAIL];
} TXIP_ICMP_STATS;

/*###########################################################################*
 *
 * OVERALL NETWORK MANAGEMENT
 *
 *###########################################################################*/

/*---------------------------------------------------------------------------*
 * TXIP_NETWORK_CONFIG - Overall Network-Level Configuration
 *---------------------------------------------------------------------------*/
#define TXIP_NETWORK_CONFIG_AVAIL   8           /* number of U32's available for future use */
typedef struct
{
    U32                 configMask;             /* mask indicating valid config fields in remainder of structure */
#define TXIP_NETWORK_CFG_MATEIPADDR     (1<<0)  /* set = mateIpAddr valid */

    TXIP_ADDR           mateIpAddr;             /* IP address of mate entity (redundancy mate) */
    U32                 avail[TXIP_NETWORK_CONFIG_AVAIL];
} TXIP_NETWORK_CONFIG;

/*---------------------------------------------------------------------------*
 * TXIP_NETWORK_STATUS - Current status of Overall Network
 *---------------------------------------------------------------------------*/
#define TXIP_NETWORK_STATUS_AVAIL   8           /* number of U32's available for future use */
typedef struct
{
    U32                 udpCount;               /* number of different UDP ports that UDP protocol layer is demultiplexing to */
    U32                 protoCount;             /* number of different protocols that IP layer is demultiplexing to */
    U32                 routeCount;             /* total number of entries in the routing table */
    U32                 ifaceCount;             /* total number of interfaces */
    U32                 ethCount;               /* total number of Ethernet interfaces */
    U32                 pingCount;              /* total number of PINGs being tracked */
    U32                 avail[TXIP_NETWORK_STATUS_AVAIL];
} TXIP_NETWORK_STATUS;

/*---------------------------------------------------------------------------*
 * TXIP_NETWORK_INFO - Information describing the overall network
 *---------------------------------------------------------------------------*/
#define TXIP_NETWORK_INFO_AVAIL     8           /* number of U32's available for future use */
typedef struct
{
    TXIP_NETWORK_CONFIG config;
    TXIP_NETWORK_STATUS status;
    U32                 avail[TXIP_NETWORK_INFO_AVAIL];
} TXIP_NETWORK_INFO;

/*###########################################################################*
 *
 * UDP PROTOCOL LAYER MANAGEMENT
 *
 *###########################################################################*/

#define TXIP_UDP_COUNT              8           /* maximum number of UDP port entries (in summary record) */

/*---------------------------------------------------------------------------*
 * TXIP_UDP_CONFIG - UDP Port-specific configuration
 *---------------------------------------------------------------------------*/
#define TXIP_UDP_CONFIG_AVAIL       4           /* number of U32's available for future use */
typedef struct
{
    U32                 configMask;             /* mask indicating valid config fields in remainder of structure */
#define TXIP_UDP_CFG_NAME               (1<<0)  /* set = name valid */
#define TXIP_UDP_CFG_OWNER              (1<<1)  /* set = owner valid */

    S8                  name[TXIP_NAME_SZ];     /* ASCII name identifying usage of UDP port */
    S8                  owner[TXIP_OWNER_SZ];   /* ASCII name of UDP port owner */
    U32                 avail[TXIP_UDP_CONFIG_AVAIL];
} TXIP_UDP_CONFIG;

/*---------------------------------------------------------------------------*
 * TXIP_UDP_STATUS - Current status of specific UDP port
 *---------------------------------------------------------------------------*/
#define TXIP_UDP_STATUS_AVAIL       4           /* number of U32's available for future use */
typedef struct
{
    U32                 state;                  /* current state of UDP port (TXIP_STATE_xxx) */
    U32                 avail[TXIP_UDP_STATUS_AVAIL];
} TXIP_UDP_STATUS;

/*---------------------------------------------------------------------------*
 * TXIP_UDP_INFO - Information describing a UDP port
 *---------------------------------------------------------------------------*/
#define TXIP_UDP_INFO_AVAIL         4           /* number of U32's available for future use */
typedef struct
{
    TXIP_UDP_CONFIG     config;
    TXIP_UDP_STATUS     status;
    U32                 avail[TXIP_UDP_INFO_AVAIL];
} TXIP_UDP_INFO;

/*---------------------------------------------------------------------------*
 * TXIP_UDP_STATS - Statistics maintained per UDP port
 *---------------------------------------------------------------------------*/
#define TXIP_UDP_STATS_AVAIL        4           /* number of U32's available for future use */
typedef struct
{
    TXIP_COMM_STATS     comm;                   /* basic communication statistics */
    U32                 avail[TXIP_UDP_STATS_AVAIL];
} TXIP_UDP_STATS;

/*---------------------------------------------------------------------------*
 * TXIP_UDP_SUMM - Summary entry for a specific UDP port
 *---------------------------------------------------------------------------*/
#define TXIP_UDP_SUMM_AVAIL         4           /* number of U32's available for future use */
typedef struct
{
    U32                 udpPort;                /* unique UDP port number used as demultiplexing value */
    TXIP_UDP_STATUS     status;                 /* current status of the UDP port */
    TXIP_UDP_STATS      stats;                  /* UDP port-specific statisitcs */
    U32                 avail[TXIP_UDP_SUMM_AVAIL];
} TXIP_UDP_SUMM;

/*---------------------------------------------------------------------------*
 * TXIP_UDP_SUMMARY - Summary information for UDP ports
 *---------------------------------------------------------------------------*/
#define TXIP_UDP_SUMMARY_AVAIL      4           /* number of U32's available for future use */
typedef struct
{
    U32                 udpCount;               /* number of different UDP ports that UDP protocol layer is demultiplexing to */
    U32                 baseIndex;              /* index into total UDP port set IDing slot represented by udp[0] */
    U32                 avail[TXIP_UDP_SUMMARY_AVAIL];
    TXIP_UDP_SUMM       udp[TXIP_UDP_COUNT];    /* maximum number of UDP ports provided in a summary record */
} TXIP_UDP_SUMMARY;

/*###########################################################################*
 *
 * IP PROTOCOL LAYER MANAGEMENT
 *
 *###########################################################################*/

#define TXIP_PROTO_COUNT     8                  /* maximum number of protocol entries (in summary record) */

/*---------------------------------------------------------------------------*
 * TXIP_PROTO_CONFIG - IP Protocol-specific configuration
 *---------------------------------------------------------------------------*/
#define TXIP_PROTO_CONFIG_AVAIL     4           /* number of U32's available for future use */
typedef struct
{
    U32                 configMask;             /* mask indicating valid config fields in remainder of structure */
#define TXIP_PROTO_CFG_NAME             (1<<0)  /* set = name valid */
#define TXIP_PROTO_CFG_OWNER            (1<<1)  /* set = owner valid */

    S8                  name[TXIP_NAME_SZ];     /* ASCII name identifying usage of IP protocol */
    S8                  owner[TXIP_OWNER_SZ];   /* ASCII name of IP protocol owner */
    U32                 avail[TXIP_PROTO_CONFIG_AVAIL];
} TXIP_PROTO_CONFIG;

/*---------------------------------------------------------------------------*
 * TXIP_PROTO_STATUS - Current status of specific IP protocol layer
 *---------------------------------------------------------------------------*/
#define TXIP_PROTO_STATUS_AVAIL     4           /* number of U32's available for future use */
typedef struct
{
    U32                 state;                  /* current state of IP protocol layer (TXIP_STATE_xxx) */
    U32                 avail[TXIP_PROTO_STATUS_AVAIL];
} TXIP_PROTO_STATUS;

/*---------------------------------------------------------------------------*
 * TXIP_PROTO_INFO - Information describing an IP protocol layer
 *---------------------------------------------------------------------------*/
#define TXIP_PROTO_INFO_AVAIL       4           /* number of U32's available for future use */
typedef struct
{
    TXIP_PROTO_CONFIG   config;
    TXIP_PROTO_STATUS   status;
    U32                 avail[TXIP_PROTO_INFO_AVAIL];
} TXIP_PROTO_INFO;

/*---------------------------------------------------------------------------*
 * TXIP_PROTO_STATS - Statistics maintained per IP protocol layer
 *---------------------------------------------------------------------------*/
#define TXIP_PROTO_STATS_AVAIL      4           /* number of U32's available for future use */
typedef struct
{
    TXIP_COMM_STATS     comm;                   /* basic communication statistics */
    U32                 avail[TXIP_PROTO_STATS_AVAIL];
} TXIP_PROTO_STATS;

/*---------------------------------------------------------------------------*
 * TXIP_PROTO_SUMM - Summary entry for a specific IP protocol
 *---------------------------------------------------------------------------*/
#define TXIP_PROTO_SUMM_AVAIL       4           /* number of U32's available for future use */
typedef struct
{
    U32                 protocol;               /* unique protocol number used as demultiplexing value */
    TXIP_PROTO_STATUS   status;                 /* current status of the protocol layer */
    TXIP_PROTO_STATS    stats;                  /* protocol layer-specific statisitcs */
    U32                 avail[TXIP_PROTO_SUMM_AVAIL];
} TXIP_PROTO_SUMM;

/*---------------------------------------------------------------------------*
 * TXIP_PROTO_SUMMARY - Summary information for IP protocols
 *---------------------------------------------------------------------------*/
#define TXIP_PROTO_SUMMARY_AVAIL    4           /* number of U32's available for future use */
typedef struct
{
    U32                 protoCount;                 /* number of different protocols that IP layer is demultiplexing to */
    U32                 baseIndex;                  /* index into total protocol set IDing slot represented by proto[0] */
    U32                 avail[TXIP_PROTO_SUMMARY_AVAIL];
    TXIP_PROTO_SUMM     proto[TXIP_PROTO_COUNT];    /* maximum number of protocols provided in a summary record */
} TXIP_PROTO_SUMMARY;

/*###########################################################################*
 *
 * IP ROUTE TABLE MANAGEMENT
 *
 *###########################################################################*/

#define TXIP_ROUTE_ENTRY_CNT        6           /* maximum number of entries in a route table report */

/*---------------------------------------------------------------------------*
 * TXIP_ROUTE_IFACE - IP Route to Interface
 *---------------------------------------------------------------------------*/
#define TXIP_ROUTE_IFACE_AVAIL      4           /* number of U32's available for future use */
typedef struct
{
    U32                 ifNum;
    U32                 avail[TXIP_ROUTE_IFACE_AVAIL];
} TXIP_ROUTE_IFACE;

/*---------------------------------------------------------------------------*
 * TXIP_ROUTE_GW - IP Route to Gateway
 *---------------------------------------------------------------------------*/
#define TXIP_ROUTE_GW_AVAIL         4           /* number of U32's available for future use */
typedef struct
{
    TXIP_ADDR           addr;
    U32                 avail[TXIP_ROUTE_GW_AVAIL];
} TXIP_ROUTE_GW;

/*---------------------------------------------------------------------------*
 * TXIP_ROUTE - IP Route
 *---------------------------------------------------------------------------*/
#define TXIP_ROUTE_AVAIL            4           /* number of U32's available for future use */
typedef struct
{
    U32                 routeType;              /* type of route (all definitions should be identical regardless of endianness) */
#define TXIP_ROUTE_NOT_USED     0               /* unused route table entry */
#define TXIP_ROUTE_INTERFACE    0x11111111      /* route that is associated with a specific interface */
#define TXIP_ROUTE_GATEWAY      0x22222222      /* route to a remote gateway address */

    U32                 routeIndex;             /* index into route table array */
    U32                 routeStatus;            /* flags indicating current route status */
#define TXIP_ROUTE_ACTIVE       (1<<0)          /* set = route is currently active; clr = route inactive */
#define TXIP_ROUTE_DELETED      (1<<1)          /* set = route has been deleted */

    TXIP_ADDR           ipAddr;
    TXIP_ADDR           mask;
    union
    {
        TXIP_ROUTE_IFACE    iface;
        TXIP_ROUTE_GW       gw;
        U8                  reserved[48];
    } u;
    TXIP_DATA_STATS     usage;                  /* route usage statistics */
    U32                 avail[TXIP_ROUTE_AVAIL];
} TXIP_ROUTE;

/*---------------------------------------------------------------------------*
 * TXIP_ROUTE_TABLE - IP Routing Table
 *---------------------------------------------------------------------------*/
#define TXIP_ROUTE_TABLE_AVAIL      4           /* number of U32's available for future use */
typedef struct
{
    U32                 routeCount;                         /* total number of entries in the routing table */
    U32                 baseIndex;                          /* index into total route set IDing slot represented by route[0] */
    TXIP_ROUTE          route[TXIP_ROUTE_ENTRY_CNT];        /* route table entries */
    U32                 avail[TXIP_ROUTE_TABLE_AVAIL];
} TXIP_ROUTE_TABLE;

/*###########################################################################*
 *
 * IP INTERFACE MANAGEMENT
 *
 *###########################################################################*/

#define TXIP_IFACE_COUNT            4           /* maximum number of interfaces in summary message */

/*---------------------------------------------------------------------------*
 * TXIP_IFACE_CONFIG - Interface configuration
 *---------------------------------------------------------------------------*/
#define TXIP_IFACE_CONFIG_AVAIL     4           /* number of U32's available for future use */
typedef struct
{
    U32                 configMask;             /* mask indicating valid config fields in remainder of structure */
#define TXIP_IFACE_CFG_PRIMARY              (1<<0)  /* set = valid primary */
#define TXIP_IFACE_CFG_GATEWAY              (1<<1)  /* set = valid gateway */
#define TXIP_IFACE_CFG_DHCP_USE             (1<<2)  /* set = valid dhcpUse */
#define TXIP_IFACE_CFG_PING_BLOCK           (1<<3)  /* set = valid pingBLock */
#define TXIP_IFACE_CFG_LINK_MTU             (1<<4)  /* set = valid linkMTU */
#define TXIP_IFACE_CFG_FRAG_DROP            (1<<5)  /* set = vaild fragDropTimeout */
#define TXIP_IFACE_CFG_HOP_LIMIT            (1<<6)  /* set = valid hopLimit */
#define TXIP_IFACE_CFG_ICMP_RATE_LIMIT      (1<<7)  /* set = valid icmpRateLimit */
#define TXIP_IFACE_CFG_TXAPI_ACCESS         (1<<8)  /* set = valid txapiAccess */
#define TXIP_IFACE_CFG_IPV6_SUPPORT         (1<<9)  /* set = valid ipv6Support */
#define TXIP_IFACE_CFG_IPSEC_USE            (1<<10) /* set = valid ipSecUse */
#define TXIP_IFACE_CFG_ND_ATTEMPTS          (1<<11) /* set = valid ndAttempts */
#define TXIP_IFACE_CFG_ND_RETRAN_TIMEOUT    (1<<12) /* set = valid ndRetranTimeout */
#define TXIP_IFACE_CFG_ND_REACH_TIMEOUT     (1<<13) /* set = valid ndReachTimeout */
#define TXIP_IFACE_CFG_ARP_RETRY_INTERVAL   (1<<14) /* set = valid arpRetryInterval */
#define TXIP_IFACE_CFG_ARP_MAX_RETRIES      (1<<15) /* set = valid arpMaxRetries */
#define TXIP_IFACE_CFG_ARP_REACHABLE_TIME   (1<<16) /* set = valid arpReachableTime */
#define TXIP_IFACE_CFG_ARP_UNUSED_TIME      (1<<17) /* set = valid arpUnusedTime */

    TXIP_ROUTE          primary;                /* primary inteface route (widest mask interface route) */
    TXIP_ROUTE          gateway;                /* primary gateway route (widest mask gateway route) */
    U32                 dhcpUse;                /* whether DHCP (IP address discovery) use is enabled */
    U32                 pingBlock;              /* whether automatic response to PING messages is blocked */
    U32                 linkMTU;                /* link MTU (Maximum Transmission Unit) */
#define TXIP_IFACE_CFG_MTU_MIN  60
#define TXIP_IFACE_CFG_MTU_MAX  1500

    U32                 fragDropTimeout;        /* Maximum time to hold onto fragments before dropping incomplete datagram (in milliseconds) */
    U32                 hopLimit;               /* hop limit for this link */
    U32                 icmpRateLimit;          /* rate limiter (number of messages per second for ICMPv6) */
    U32                 txapiAccess;            /* whether TX API requests should be accepted over this interface */
    U32                 ipv6Support;            /* whether IPv6 is supported for this interface */
    U32                 ipSecUse;               /* whether IPSec (security) in use on this interface */
    U32                 ndAttempts;             /* number of Neighbor Discovery Retransmission attempts */
    U32                 ndRetranTimeout;        /* Neighbor Discovery Retransmission timeout (in milliseconds) */
    U32                 ndReachTimeout;         /* Neighbor Discovery Reachable timeout (in milliseconds) */
    U32                 arpRetryInterval;       /* interval (in msecs) at which unanswered ARP requests are retried */
    U32                 arpMaxRetries;          /* maximum number of ARP request retries before aging ARP entry */
    U32                 arpReachableTime;       /* maximum amount of time (in msecs) without receiving from a remote before probing to verify remote is still reachable */
    U32                 arpUnusedTime;          /* maximum amount of time (in msecs) that an unused ARP entry remains in the ARP table */
    U32                 avail[TXIP_IFACE_CONFIG_AVAIL];
} TXIP_IFACE_CONFIG;

/* ----- IP interface configuration defaults ----- */
#define TXIP_DEFAULT_DHCP_USE           TXIP_DISABLED
#define TXIP_DEFAULT_PING_BLOCK         TXIP_DISABLED
#define TXIP_DEFAULT_LINK_MTU           1500
#define TXIP_DEFAULT_FRAG_DROP_TIMEOUT  10000
#define TXIP_DEFAULT_HOP_LIMIT          128
#define TXIP_DEFAULT_ICMP_RATE_LIMIT    100
#define TXIP_DEFAULT_TXAPI_ACCESS       TXIP_DISABLED
#define TXIP_DEFAULT_IPV6_SUPPORT       TXIP_DISABLED
#define TXIP_DEFAULT_IPSEC_USE          TXIP_DISABLED
#define TXIP_DEFAULT_ND_ATTEMPTS        4
#define TXIP_DEFAULT_ND_RETRAN_TIMEOUT  4000
#define TXIP_DEFAULT_ND_REACH_TIMEOUT   30000
#define TXIP_DEFAULT_ARP_RETRY_INTERVAL 1000
#define TXIP_DEFAULT_ARP_MAX_RETRIES    15
#define TXIP_DEFAULT_ARP_REACHABLE_TIME (60*1000)
#define TXIP_DEFAULT_ARP_UNUSED_TIME    (3*60*60*1000)

/*---------------------------------------------------------------------------*
 * TXIP_IFACE_STATUS - Current status of specific interface
 *---------------------------------------------------------------------------*/
#define TXIP_IFACE_STATUS_AVAIL     4           /* number of U32's available for future use */
typedef struct
{
    U32                 state;                  /* current state of IP interface (TXIP_STATE_xxx) */
    U32                 avail[TXIP_IFACE_STATUS_AVAIL];
} TXIP_IFACE_STATUS;

/*---------------------------------------------------------------------------*
 * TXIP_IFACE_INFO - Information describing an interface
 *---------------------------------------------------------------------------*/
#define TXIP_IFACE_INFO_AVAIL       4           /* number of U32's available for future use */
typedef struct
{
    TXIP_IFACE_CONFIG   config;
    TXIP_IFACE_STATUS   status;
    U32                 avail[TXIP_IFACE_INFO_AVAIL];
} TXIP_IFACE_INFO;

/*---------------------------------------------------------------------------*
 * TXIP_IFACE_STATS - Statistics maintained per interface
 *---------------------------------------------------------------------------*/
#define TXIP_IFACE_STATS_AVAIL      4           /* number of U32's available for future use */
typedef struct
{
    TXIP_COMM_STATS     comm;                   /* basic communication statistics */
    TXIP_FRAG_STATS     frag;                   /* IP fragmentation statistics */
    TXIP_ICMP_STATS     icmp;                   /* ICMP statistics */
    U32                 avail[TXIP_IFACE_STATS_AVAIL];
} TXIP_IFACE_STATS;

/*---------------------------------------------------------------------------*
 * TXIP_IFACE_SUMM - Summary entry for a specific IP interface
 *---------------------------------------------------------------------------*/
#define TXIP_IFACE_SUMM_AVAIL       4           /* number of U32's available for future use */
typedef struct
{
    U32                 ifNum;                  /* unique number used to identify the interface */
    TXIP_IFACE_STATUS   status;                 /* current status of the interface */
    TXIP_IFACE_STATS    stats;                  /* interface level statisitcs */
    U32                 avail[TXIP_IFACE_SUMM_AVAIL];
} TXIP_IFACE_SUMM;

/*---------------------------------------------------------------------------*
 * TXIP_IFACE_SUMMARY - Summary information for IP interfaces
 *---------------------------------------------------------------------------*/
#define TXIP_IFACE_SUMMARY_AVAIL    4           /* number of U32's available for future use */
typedef struct
{
    U32                 ifaceCount;                 /* total number of interfaces */
    U32                 baseIndex;                  /* index into total interface set IDing slot represented by iface[0] */
    U32                 avail[TXIP_IFACE_SUMMARY_AVAIL];
    TXIP_IFACE_SUMM     iface[TXIP_IFACE_COUNT];    /* maximum number of interfaces provided in a summary record */
} TXIP_IFACE_SUMMARY;

/*###########################################################################*
 *
 * IP ETHERNET LINK MANAGEMENT
 *
 *###########################################################################*/

#define TXIP_MAC_ADDR_LEN           6           /* number of bytes representing an Ethernet MAC address */

#define TXIP_ETH_COUNT              4           /* maximum number of Ethernet links provided in summary message */

/*---------------------------------------------------------------------------*
 * TXIP_ETH_CONFIG - Ethernet link configuration
 *---------------------------------------------------------------------------*/
#define TXIP_ETH_CONFIG_AVAIL       3           /* number of U32's available for future use */
typedef struct
{
    U32                 configMask;             /* mask indicating valid config fields in remainder of structure */
#define TXIP_ETH_CFG_MACADDR            (1<<0)  /* set = valid macAddr */
#define TXIP_ETH_CFG_MONCHAN            (1<<1)  /* set = valid monChan */
#define TXIP_ETH_CFG_PHYCFG             (1<<2)  /* set = valid phyCfg */

    U8                  macAddr[TXIP_MAC_ADDR_LEN]; /* Ethernet MAC address */
    U8                  avail1[2];
    U32                 monChan;                /* host channel receiving Ethernet link monitor messages (0 = monitor INACTIVE) */
    U32                 phyCfg;                 /* PHY configuration */
#define ETH_SPEED_AUTO          (0x00 << 0)     /* speed has not been determined */
#define ETH_SPEED_10Mb          (0x01 << 0)     /* 10 Megabits per second */
#define ETH_SPEED_100Mb         (0x02 << 0)     /* 100 Megabits per second */
#define ETH_SPEED_MASK          (0x0f << 0)

#define ETH_DUPLEX_AUTO         (0x00 << 4)     /* mode has not been determined */
#define ETH_DUPLEX_FULL         (0x01 << 4)
#define ETH_DUPLEX_HALF         (0x02 << 4)
#define ETH_DUPLEX_MASK         (0x0f << 4)

#define ETH_MDIXCTL_AUTO        (0x00 << 8)
#define ETH_MDIXCTL_MDIX        (0x01 << 8)
#define ETH_MDIXCTL_NOMDIX      (0x02 << 8)
#define ETH_MDIXCTL_MASK        (0x0f << 8)

#define ETH_CFG_AUTO    (ETH_SPEED_AUTO | ETH_DUPLEX_AUTO | ETH_MDIXCTL_AUTO)

    U32                 avail2[TXIP_ETH_CONFIG_AVAIL];
} TXIP_ETH_CONFIG;

/*---------------------------------------------------------------------------*
 * TXIP_ETH_STATUS - Current status of specific Ethernet link
 *---------------------------------------------------------------------------*/
#define TXIP_ETH_STATUS_AVAIL       4           /* number of U32's available for future use */
typedef struct
{
    U32                 state;                  /* current state of Ethernet link (TXIP_STATE_xxx) */
    U32                 speed;                  /* operating speed */
#define TXIP_SPEED_UNKNOWN  0                   /* speed has not been determined */
#define TXIP_SPEED_10MB     1                   /* 10 Megabits per second */
#define TXIP_SPEED_100MB    2                   /* 100 Megabits per second */
#define TXIP_SPEED_1GB      3                   /* 1 Gigabit per second */

    U32                 mode;                   /* mode that Ethernet is operating at */
#define TXIP_MODE_UNKNOWN   0                   /* mode has not been determined */
#define TXIP_HALF_DUPLEX    1
#define TXIP_FULL_DUPLEX    2

    U32                 avail[TXIP_ETH_STATUS_AVAIL];
} TXIP_ETH_STATUS;

/*---------------------------------------------------------------------------*
 * TXIP_ETH_INFO - Information describing an Ethernet link
 *---------------------------------------------------------------------------*/
#define TXIP_ETH_INFO_AVAIL         4           /* number of U32's available for future use */
typedef struct
{
    TXIP_ETH_CONFIG     config;
    TXIP_ETH_STATUS     status;
    U32                 avail[TXIP_ETH_INFO_AVAIL];
} TXIP_ETH_INFO;

/*---------------------------------------------------------------------------*
 * TXIP_ETH_STATS - Statistics maintained per Ethernet link
 *---------------------------------------------------------------------------*/
#define TXIP_ETH_STATS_AVAIL        4           /* number of U32's available for future use */
#define TXIP_ETH_STATS_TX_OK_AVAIL  2           /* number of U32's available for future use */
#define TXIP_ETH_STATS_TX_ERR_AVAIL 2           /* number of U32's available for future use */
#define TXIP_ETH_STATS_RX_OK_AVAIL  2           /* number of U32's available for future use */
#define TXIP_ETH_STATS_RX_ERR_AVAIL 2           /* number of U32's available for future use */
typedef struct
{
    TXIP_COMM_STATS     comm;                   /* basic communication statistics */

    /* ----- additional transmit statistics ----- */
    U32                 txFragments;            /* number of fragments transmitted */
    U32                 txMulticast;            /* number of multicast transmits */
    U32                 txBroadcast;            /* number of broadcast messages transmitted */
    U32                 txPause;                /* number of flow control pause messages transmitted */
    U32                 txControl;              /* number of control frames transmitted */
    U32                 txJabber;               /* number of jabber frames transmitted */
    U32                 availTxSuccess[TXIP_ETH_STATS_TX_OK_AVAIL];

    U32                 txQueueOverruns;        /* number of transmits attempted when transmit queue already full */
    U32                 txUnderruns;            /* number of frames not transmitted due to internal underrun (DMA failure, etc.) */
    U32                 txUndersized;           /* number of undersized frames transmitted */
    U32                 txOversized;            /* number of oversized frames transmitted */
    U32                 txCarrierLost;          /* number of frames transmitted when deassertion of carrier sense was detected */
    U32                 txFscErrors;            /* number of FCS errors transmitted */
    U32                 txUnknown;              /* number of transmit frames with unknown OP code */
    U32                 txDeferred;             /* number of frames that were deferred before transmission due to link activity */
    U32                 txExcessDeferral;       /* number of frames transmitted with excessive edeferral */
    U32                 txCollisions;           /* number of frame transmits with single collisions */
    U32                 txMultiCollisions;      /* number of frame transmits with multiple collisons */
    U32                 txLateCollisions;       /* number of frames not transmitted due to collision outside normal collision window */
    U32                 txExcessCollisions;     /* number of frame transmits with excessive collisions (aborted transmits) */
    U32                 txTotalCollisions;      /* total number of collisions detected while transmitting */
    U32                 txHostEthMonFails;      /* number of Ethernet data tracing (Ethernet monitoring) messages that could not be transmitted to host */
    U32                 availTxError[TXIP_ETH_STATS_TX_ERR_AVAIL];

    /* ----- additional receive statistics ----- */
    U32                 rxFragments;            /* number of fragments received */
    U32                 rxMulticast;            /* number of multicast packets received */
    U32                 rxBroadcast;            /* number of broadcast packets received */
    U32                 rxPause;                /* number of flow control pause messages received */
    U32                 rxControl;              /* number of control frames received */
    U32                 rxJabber;               /* number of Jabber receives */
    U32                 availRxSuccess[TXIP_ETH_STATS_RX_OK_AVAIL];

    U32                 rxNoMemory;             /* number of failed memory allocation attempts */
    U32                 rxOverruns;             /* number of frames not received due to internal resource limitation */
    U32                 rxUndersized;           /* number of undersized frames received */
    U32                 rxOversized;            /* number of oversized frames received */
    U32                 rxCarrierLost;          /* number of receive frame errors due to carrier sense lost */
    U32                 rxFscErrors;            /* number of FCS errors received */
    U32                 rxUnknown;              /* number of received frames with unknown OP code */
    U32                 rxCollisions;           /* number of times collision detected while receiving a frame */
    U32                 rxCRCErrors;            /* number of frames received with CRC errors */
    U32                 rxAlignErrors;          /* number of frames received with alignment errors */
    U32                 rxNoResources;          /* number of frames received but discarded due to lack of resources */
    U32                 rxFrameLength;          /* number of received frames of invalid length */
    U32                 rxCodeErrors;           /* number of receive code errors detected */
    U32                 rxOtherErrors;          /* number of frames received due to any other error */
    U32                 availRxError[TXIP_ETH_STATS_RX_ERR_AVAIL];

    U32                 avail[TXIP_ETH_STATS_AVAIL];
} TXIP_ETH_STATS;

/*---------------------------------------------------------------------------*
 * TXIP_ETH_MONITOR - Monitor Ethernet link
 *---------------------------------------------------------------------------*/
#define TXIP_ETH_MON_AVAIL      4               /* number of U32's available for future use */
typedef struct
{
    U32                 linkNum;                /* unique number used to identify the Ethernet link (0 = all Ethernet links) */
    U32                 monChan;                /* host channel to receive Ethernet link monitor messages (0 = monitor INACTIVE) */
    U32                 avail1[TXIP_ETH_MON_AVAIL];
} TXIP_ETH_MON;

/*---------------------------------------------------------------------------*
 * TXIP_ETH_SUMM - Summary entry for a specific Ethernet link
 *---------------------------------------------------------------------------*/
#define TXIP_ETH_SUMM_AVAIL         4           /* number of U32's available for future use */
typedef struct
{
    U32                 linkNum;                /* unique number used to identify the Ethernet link */
    TXIP_ETH_STATUS     status;                 /* current status of the Ethernet link */
    TXIP_ETH_STATS      stats;                  /* Ethernet type-specific statisitcs */
    U32                 avail[TXIP_ETH_SUMM_AVAIL];
} TXIP_ETH_SUMM;

/*---------------------------------------------------------------------------*
 * TXIP_ETH_SUMMARY - Summary information for Ethernet links
 *---------------------------------------------------------------------------*/
#define TXIP_ETH_SUMMARY_AVAIL      4           /* number of U32's available for future use */
typedef struct
{
    U32                 ethCount;               /* total number of Ethernet links */
    U32                 baseIndex;              /* index into total Ethernet set IDing slot represented by eth[0] */
    U32                 avail[TXIP_ETH_SUMMARY_AVAIL];
    TXIP_ETH_SUMM       eth[TXIP_ETH_COUNT];    /* maximum number of Ethernet links provided in a summary record */
} TXIP_ETH_SUMMARY;

/*###########################################################################*
 *
 * PING MANAGEMENT
 *
 *###########################################################################*/

#define TXIP_PING_COUNT                 2       /* maximum number of PINGs in a summary record */

/*---------------------------------------------------------------------------*
 * TXIP_PING_PATTERN - data pattern placed in echo request (and expected in echo reply)
 *---------------------------------------------------------------------------*/
#define TXIP_PING_DEFAULT_MSG "TX PING:@ABCDEFG"

typedef struct
{
    U8                  fillMethod;             /* pattern fill method */
#define TXIP_PING_FILL_MSG      0               /* fill all data with the repeating ASCII test message (TXIP_PING_DEFAULT_MSG) */
#define TXIP_PING_FILL_SPREAD   1               /* fill with pattern spread (repeating from fillMin to fillMax [by fillStep]) */

    U8                  fillMin;                /* minimum value used to pattern fill data buffer */
    U8                  fillMax;                /* maximum value used to pattern fill data buffer */
    U8                  fillStep;               /* amount that pattern fill value is stepped up [incremented] by NOTE: 0 treated as 1 */
} TXIP_PING_PATTERN;

/*---------------------------------------------------------------------------*
 * TXIP_PING_IPHDR - options that control fields in the IP header
 *---------------------------------------------------------------------------*/
typedef struct
{
    U8                  ttl;                    /* time-to-live used in each transmitted request */
    U8                  tos;                    /* type-of-service used in each transmitted request NOTE: RFC 792 lists tos as 0-only */
    U8                  ipOpt;                  /* IP options to set for each transmitted request */
#define TXIP_PING_DONT_FRAG     (1<<0)          /* set = specify DONT FRAGMENT in each transmitted ping request */
#define TXIP_PING_RECORD_ROUTE  (1<<1)          /* set = specify RECORD ROUTE IP option in each transmitted ping request */
#define TXIP_PING_RECORD_TIME   (1<<2)          /* set = specify RECORD TIMESTAMPS IP option in each transmitted ping request */

    U8                  avail;
} TXIP_PING_IPHDR;

/*---------------------------------------------------------------------------*
 * TXIP_PING_OPTIONS - PING options
 *
 * NOTE: Any option set to 0 causes default behavior.
 *---------------------------------------------------------------------------*/
#define TXIP_PING_OPTIONS_AVAIL     4           /* number of U32's available for future use */

typedef struct
{
    U32                 requestType;            /* type of request to transmit */
#define TXIP_PING_ECHO          0               /* transmit echo requests and expect echo replies */
#define TXIP_PING_TIMESTAMP     0x11111111      /* transmit timestamp requests and expect timestamp replies */
#define TXIP_PING_INFORMATION   0x22222222      /* transmit information requests and expect information replies */

    U32                 stopCount;              /* stop after transmitting stopCount ping requests (and receiving stopCount replies [or timing out]) */
    U32                 requestTime;            /* number of milliseconds between ping requests to remote endpoint */
    U32                 replyTime;              /* number of milliseconds to allow for ping reply before counting remote endpoint as unreachable */
    U32                 echoSize;               /* set number of data bytes included in echo request (and expected in echo response) */
    TXIP_PING_PATTERN   echoPattern;            /* control over data pattern that is placed in each echo request (and expected in each echo reply) */
    TXIP_PING_IPHDR     ipHdr;                  /* options that control fields in the IP header */
    U32                 avail[TXIP_PING_OPTIONS_AVAIL];
} TXIP_PING_OPTIONS;

/* ----- defaults for PING options ----- */
#define TXIP_PING_DEFAULT_FILL_METHOD   TXIP_PING_FILL_MSG  /* fill all data with the repeating ASCII test message (TXIP_PING_DEFAULT_MSG) */
#define TXIP_PING_DEFAULT_TTL           60      /* time-to-live used in each transmitted request */
#define TXIP_PING_DEFAULT_TOS           0       /* type-of-service used in each transmitted request */
#define TXIP_PING_DEFAULT_IPOPT         0       /* transmit ping requests without any IP options set */
#define TXIP_PING_DEFAULT_REQ_TYPE      TXIP_PING_ECHO      /* transmit echo requests and expect echo replies */
#define TXIP_PING_DEFAULT_STOP_COUNT    5       /* transmit 5 ping requests and expect 5 ping replies */
#define TXIP_PING_DEFAULT_REQ_TIME      1000    /* transmit a ping request once every second */
#define TXIP_PING_DEFAULT_REPLY_TIME    8000    /* allow 8 seconds for reply before remote endpoint counted as unreachable */
#define TXIP_PING_DEFAULT_ECHO_SIZE     56      /* 56 data bytes (+ 8-byte ICMP header yields 64 payload bytes) */

/*---------------------------------------------------------------------------*
 * TXIP_PING_ROUTE - Information about gateways/routers, etc. along route PING has taken
 *---------------------------------------------------------------------------*/
#define TXIP_PING_ROUTE_CNT         10          /* max number of recorded routes (reported in ping reply) */

#define TXIP_PING_ROUTE_AVAIL       4           /* number of U32's available for future use */
typedef struct
{
    U32                 timestamp;              /* timestamp that was optionally reported with IP address of remote (0 used when no timestamp info available) */
    TXIP_ADDR           remote;                 /* IP address of the remote entity (gateway/router, etc.) that the PING was routed through */
    U32                 avail[TXIP_PING_ROUTE_AVAIL];
} TXIP_PING_ROUTE;

/*---------------------------------------------------------------------------*
 * TXIP_PING_STATUS - Current status of specific PING
 *---------------------------------------------------------------------------*/
#define TXIP_PING_STATUS_AVAIL      5           /* number of U32's available for future use */
typedef struct
{
    U32                 pingState;              /* current state of PING */
#define TXIP_PING_S_AVAIL       1                   /* PING descriptor is available for use (no ping currently represented) */
#define TXIP_PING_S_NO_RX       2                   /* PING started but no PING response received yet; no RX timeout yet */
#define TXIP_PING_S_UNREACH     3                   /* PING request(s) out; no PING response ever received; RX timeout(s) have occurred */
#define TXIP_PING_S_LIVE        4                   /* endpoint considered "live" [have received from endpoint and no timeouts within interval] */
#define TXIP_PING_S_PARTIAL     5                   /* endpoint considered partially reachable since some replies have been received while some have timed out */
#define TXIP_PING_S_LOST        6                   /* endpoint considered "lost" [have not received from endpoint within entire monitoring period] */
#define TXIP_PING_S_STOPPED     7                   /* PING stopped by operator (no further requests sent, but replies still handled) */

    U32                 pingIndex;              /* index used to uniquely ID each PING known to the system */
    U32                 pingRouteCount;         /* number of route entries in the last received reply that contained route entries */
    TXIP_PING_ROUTE     pingRoute[TXIP_PING_ROUTE_CNT];     /* last received set of recorded routes */
    U32                 pingRouteOverflow;                  /* number of IP modules that could not register their route entries */
    U32                 avail[TXIP_PING_STATUS_AVAIL];
} TXIP_PING_STATUS;

/*---------------------------------------------------------------------------*
 * TXIP_PING_INFO - Information describing an active PING
 *---------------------------------------------------------------------------*/
#define TXIP_PING_INFO_AVAIL        4           /* number of U32's available for future use */
typedef struct
{
    TXIP_ADDR           local;
    TXIP_ADDR           remote;
    TXIP_PING_OPTIONS   options;
    TXIP_PING_STATUS    status;
    U32                 avail[TXIP_PING_INFO_AVAIL];
} TXIP_PING_INFO;

/*---------------------------------------------------------------------------*
 * TXIP_PING_STATS - Statistics maintained per active PING
 *---------------------------------------------------------------------------*/
#define TXIP_PING_STATS_AVAIL       4           /* number of U32's available for future use */
typedef struct
{
    TXIP_COMM_STATS     comm;                   /* basic communication statistics */
    U32                 rttMin;                 /* minimum round trip time */
    U32                 rttAverage;             /* average round trip time */
    U32                 rttMax;                 /* maximum round trip time */
    U32                 replyLost;              /* number of expected replies considered lost (timed out) */
    U32                 replyOutOfOrder;        /* number of times reply received out of order */
    U32                 replyDuplicate;         /* number of duplicate reply packets received */
    U32                 replyDamaged;           /* number of reply packets that were received with unexpected values (considered damaged) */
    U32                 replyExpected;          /* number of times reply received as expected */
    U32                 avail[TXIP_PING_STATS_AVAIL];
} TXIP_PING_STATS;

/*---------------------------------------------------------------------------*
 * TXIP_PING_SUMM - Summary entry for a specific PING
 *---------------------------------------------------------------------------*/
#define TXIP_PING_SUMM_AVAIL        4           /* number of U32's available for future use */
typedef struct
{
    TXIP_ADDR           local;
    TXIP_ADDR           remote;
    TXIP_PING_STATUS    status;                 /* current status of the PING */
    TXIP_PING_STATS     stats;                  /* Ethernet type-specific statisitcs */
    U32                 avail[TXIP_PING_SUMM_AVAIL];
} TXIP_PING_SUMM;

/*---------------------------------------------------------------------------*
 * TXIP_PING_SUMMARY - Summary information for current PINGs
 *---------------------------------------------------------------------------*/
#define TXIP_PING_SUMMARY_AVAIL     4           /* number of U32's available for future use */
typedef struct
{
    U32                 pingCount;              /* total number of PINGs being tracked */
    U32                 baseIndex;              /* index into total PING set IDing slot represented by ping[0] */
    U32                 avail[TXIP_PING_SUMMARY_AVAIL];
    TXIP_PING_SUMM      ping[TXIP_PING_COUNT];  /* maximum number of PINGs provided in a summary record */
} TXIP_PING_SUMMARY;

/*###########################################################################*
 *
 * Message Protocol Definitions
 *
 * NOTE: All fields passed between the TX board and the host system
 *       must be in network byte order.
 *
 *###########################################################################*/

/*---------------------------------------------------------------------------*
 * TXIP_HDR - common header used for all TX IP API messages
 *---------------------------------------------------------------------------*/
#define TXIP_HDR_AVAIL              4           /* number of U32's available for future use */
typedef struct
{
    U32                     msgfunc;            /* message function */
#define TXIP_MSG_NETWORK_CONFIG 0x01            /* set overall network-level configuration                        */
#define TXIP_MSG_NETWORK_INFO   0x02            /* get overall network-level information                          */
#define TXIP_MSG_UDP_SUMMARY    0x11            /* get summary report showing breakdown per UDP port              */
#define TXIP_MSG_UDP_CONFIG     0x12            /* set UDP port-specific configuration                            */
#define TXIP_MSG_UDP_INFO       0x13            /* get information about a particular UDP port                    */
#define TXIP_MSG_UDP_STATS      0x14            /* get (and optionally zero) statistics for a given UDP port      */
#define TXIP_MSG_PROTO_SUMMARY  0x21            /* get a summary report showing breakdown per IP protocol         */
#define TXIP_MSG_PROTO_CONFIG   0x22            /* set IP protocol-specific configuration                         */
#define TXIP_MSG_PROTO_INFO     0x23            /* get information about a particular IP protocol                 */
#define TXIP_MSG_PROTO_STATS    0x24            /* get (and optionally zero) statistics for a given IP protocol   */
#define TXIP_MSG_ROUTE_TABLE    0x31            /* add a route through a specified interface                      */
#define TXIP_MSG_ROUTE_ADD      0x32            /* add a gateway route                                            */
#define TXIP_MSG_ROUTE_ADD_GW   0x33            /* delete an interface route                                      */
#define TXIP_MSG_ROUTE_DEL      0x34            /* delete a gateway route                                         */
#define TXIP_MSG_ROUTE_DEL_GW   0x35            /* get route that would be used for given local and remote addr's */
#define TXIP_MSG_ROUTE_GET      0x36            /* get report of current IP routing table                         */
#define TXIP_MSG_IFACE_SUMMARY  0x41            /* get a summary report showing breakdown per IP interface        */
#define TXIP_MSG_IFACE_CONFIG   0x42            /* set IP interface-specific configuration                        */
#define TXIP_MSG_IFACE_INFO     0x43            /* get information about a particular IP interface                */
#define TXIP_MSG_IFACE_STATS    0x44            /* get (and optionally zero) statistics for a given IP interface  */
#define TXIP_MSG_ETH_SUMMARY    0x51            /* get a summary report showing breakdown per Ethernet link       */
#define TXIP_MSG_ETH_CONFIG     0x52            /* set Ethernet link-specific configuration                       */
#define TXIP_MSG_ETH_INFO       0x53            /* get information about a particular Ethernet link               */
#define TXIP_MSG_ETH_STATS      0x54            /* get (and optionally zero) statistics for a given Ethernet link */
#define TXIP_MSG_ETH_MONITOR    0x55            /* set host channel to receive Ethernet link monitor messages     */
#define TXIP_MSG_PING_SUMMARY   0x61            /* get a summary report showing breakdown per active PING         */
#define TXIP_MSG_PING_START     0x62            /* start PINGing the given remote IP address                      */
#define TXIP_MSG_PING_STOP      0x63            /* stop the specified PING                                        */
#define TXIP_MSG_PING_INFO      0x64            /* get information about a particular PING                        */
#define TXIP_MSG_PING_STATS     0x65            /* get (and optionally zero) statistics for a given PING          */

    U32                     msgtype;            /* type of message */
#define TXIP_REQUEST        0x01
#define TXIP_RESPONSE       0x02
#define TXIP_NOTIFICATION   0x03

    U32                     msgkey;             /* key (provided by user in REQUEST; duplicated in RESPONSE; special in NOTIF) */
    U32                     errorcode;          /* any error code associated with message (see coserror.h) */
    U32                     errorparam;         /* parameter associated with error code (error-code-specific) */
    U32                     avail[TXIP_HDR_AVAIL];
} TXIP_HDR;

/*###########################################################################*
 * Overall Network Management
 *###########################################################################*/

/*---------------------------------------------------------------------------*
 * TXIP_NETWORK_CONFIG_REQ - Overall Network-level configuration request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_NETWORK_CONFIG     config;             /* overall network-level configuration */
} TXIP_NETWORK_CONFIG_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_NETWORK_CONFIG_RSP - Overall Network-level configuration response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_NETWORK_CONFIG_REQ req;                /* copy of original request */
} TXIP_NETWORK_CONFIG_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_NETWORK_INFO_REQ - Network-level information request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
} TXIP_NETWORK_INFO_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_NETWORK_INFO_RSP - Network-level information response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_NETWORK_INFO_REQ   req;                /* copy of original request */
    TXIP_NETWORK_INFO       info;               /* network-level information */
} TXIP_NETWORK_INFO_RSP;

/*###########################################################################*
 * UDP Port Management
 *###########################################################################*/

/*---------------------------------------------------------------------------*
 * TXIP_UDP_SUMMARY_REQ - UDP layer summary request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
} TXIP_UDP_SUMMARY_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_UDP_SUMMARY_RSP - UDP layer summary response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_UDP_SUMMARY_REQ    req;                /* copy of original request */
    TXIP_UDP_SUMMARY        summary;            /* summary of UDP port usage */
} TXIP_UDP_SUMMARY_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_UDP_CONFIG_REQ - UDP port-level configuration request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    U32                     udpPort;            /* UDP port number */
    TXIP_UDP_CONFIG         config;             /* configuration for the given UDP port */
} TXIP_UDP_CONFIG_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_UDP_CONFIG_RSP - UDP port-level configuration response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_UDP_CONFIG_REQ     req;                /* copy of original request */
} TXIP_UDP_CONFIG_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_UDP_INFO_REQ - information about a given UDP port request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    U32                     udpPort;            /* UDP port number */
} TXIP_UDP_INFO_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_UDP_INFO_RSP - information about a given UDP port response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_UDP_INFO_REQ       req;                /* copy of original request */
    TXIP_UDP_INFO           info;               /* current information for the given UDP port */
} TXIP_UDP_INFO_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_UDP_STATS_REQ - UDP port statistics request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    U32                     udpPort;            /* UDP port number */
    U32                     zeroFlag;           /* whether to zero statistics after current values are reported */
} TXIP_UDP_STATS_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_UDP_STATS_RSP -  UDP port statistics response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_UDP_STATS_REQ      req;                /* copy of original request */
    TXIP_UDP_STATS          stats;              /* current UDP port statistics */
} TXIP_UDP_STATS_RSP;

/*###########################################################################*
 * IP Protocol-Layer Management
 *###########################################################################*/

/*---------------------------------------------------------------------------*
 * TXIP_PROTO_SUMMARY_REQ - Protocol-Layer summary request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
} TXIP_PROTO_SUMMARY_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_PROTO_SUMMARY_RSP - Protocol-Layer summary response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_PROTO_SUMMARY_REQ  req;                /* copy of original request */
    TXIP_PROTO_SUMMARY      summary;            /* summary of Protocol usage */
} TXIP_PROTO_SUMMARY_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_PROTO_CONFIG_REQ - Protocol-Layer configuration request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    U32                     protocol;           /* IP protocol number */
    TXIP_PROTO_CONFIG       config;             /* configuration for the given Protocol-Layer */
} TXIP_PROTO_CONFIG_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_PROTO_CONFIG_RSP - Protocol-Layer configuration response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_PROTO_CONFIG_REQ   req;                /* copy of original request */
} TXIP_PROTO_CONFIG_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_PROTO_INFO_REQ - information about a given Protocol-Layer request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    U32                     protocol;           /* IP protocol number */
} TXIP_PROTO_INFO_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_PROTO_INFO_RSP - information about a given Protocol-Layer response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_PROTO_INFO_REQ     req;                /* copy of original request */
    TXIP_PROTO_INFO         info;               /* current information for the given Protocol-Layer */
} TXIP_PROTO_INFO_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_PROTO_STATS_REQ - Protocol-Layer statistics request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    U32                     protocol;           /* IP protocol number */
    U32                     zeroFlag;           /* whether to zero statistics after current values are reported */
} TXIP_PROTO_STATS_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_PROTO_STATS_RSP -  Protocol-Layer statistics response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_PROTO_STATS_REQ    req;                /* copy of original request */
    TXIP_PROTO_STATS        stats;              /* current Protocol-Layer statistics */
} TXIP_PROTO_STATS_RSP;

/*###########################################################################*
 * IP Route Table Management
 *###########################################################################*/

/*---------------------------------------------------------------------------*
 * TXIP_ROUTE_TABLE_REQ - Route table request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
} TXIP_ROUTE_TABLE_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_ROUTE_TABLE_RSP - Route table response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ROUTE_TABLE_REQ    req;                /* copy of original request */
    TXIP_ROUTE_TABLE        table;              /* table of IP Routes */
} TXIP_ROUTE_TABLE_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_ROUTE_ADD_REQ - Route add request (interfrace route)
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ADDR               ipAddr;             /* IP address */
    TXIP_ADDR               mask;               /* mask showing significant bits of IP address */
    U32                     ifNum;              /* interface number that route is associated with */
} TXIP_ROUTE_ADD_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_ROUTE_ADD_RSP - Route add response (interface route)
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ROUTE_ADD_REQ      req;                /* copy of original request */
} TXIP_ROUTE_ADD_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_ROUTE_ADD_GW_REQ - Route add request (gateway route)
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ADDR               ipAddr;             /* IP address */
    TXIP_ADDR               mask;               /* mask showing significant bits of IP address */
    TXIP_ADDR               gateway;            /* gateway to route packets to */
} TXIP_ROUTE_ADD_GW_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_ROUTE_ADD_GW_RSP - Route add response (gateway route)
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ROUTE_ADD_GW_REQ   req;                /* copy of original request */
} TXIP_ROUTE_ADD_GW_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_ROUTE_DEL_REQ - Route delete request (interfrace route)
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ADDR               ipAddr;             /* IP address */
    TXIP_ADDR               mask;               /* mask showing significant bits of IP address */
    U32                     ifNum;              /* interface number that route is associated with */
} TXIP_ROUTE_DEL_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_ROUTE_DEL_RSP - Route delete response (interface route)
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ROUTE_DEL_REQ      req;                /* copy of original request */
} TXIP_ROUTE_DEL_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_ROUTE_DEL_GW_REQ - Route delete request (gateway route)
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ADDR               ipAddr;             /* IP address */
    TXIP_ADDR               mask;               /* mask showing significant bits of IP address */
    TXIP_ADDR               gateway;            /* gateway to route packets to */
} TXIP_ROUTE_DEL_GW_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_ROUTE_DEL_GW_RSP - Route delete response (gateway route)
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ROUTE_DEL_GW_REQ   req;                /* copy of original request */
} TXIP_ROUTE_DEL_GW_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_ROUTE_GET_REQ - get Route request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ADDR               local;              /* local IP address used during route lookup */
    TXIP_ADDR               remote;             /* remote IP address used during route lookup */
} TXIP_ROUTE_GET_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_ROUTE_GET_RSP - get Route response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ROUTE_GET_REQ      req;                /* copy of original request */
    TXIP_ROUTE              route;              /* Route associated with given IP address */
} TXIP_ROUTE_GET_RSP;

/*###########################################################################*
 * IP Interface Management
 *###########################################################################*/

/*---------------------------------------------------------------------------*
 * TXIP_IFACE_SUMMARY_REQ - Interface summary request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
} TXIP_IFACE_SUMMARY_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_IFACE_SUMMARY_RSP - Interface summary response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_IFACE_SUMMARY_REQ  req;                /* copy of original request */
    TXIP_IFACE_SUMMARY      summary;            /* summary of IP Interfaces */
} TXIP_IFACE_SUMMARY_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_IFACE_CONFIG_REQ - Interface configuration request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    U32                     ifNum;              /* interface number */
    TXIP_IFACE_CONFIG       config;             /* configuration for the given Interface */
} TXIP_IFACE_CONFIG_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_IFACE_CONFIG_RSP - Interface configuration response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_IFACE_CONFIG_REQ   req;                /* copy of original request */
} TXIP_IFACE_CONFIG_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_IFACE_INFO_REQ - information about a given Interface request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    U32                     ifNum;              /* interface number */
} TXIP_IFACE_INFO_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_IFACE_INFO_RSP - information about a given Interface response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_IFACE_INFO_REQ     req;                /* copy of original request */
    TXIP_IFACE_INFO         info;               /* current information for the given Interface */
} TXIP_IFACE_INFO_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_IFACE_STATS_REQ - Interface statistics request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    U32                     ifNum;              /* interface number */
    U32                     zeroFlag;           /* whether to zero statistics after current values are reported */
} TXIP_IFACE_STATS_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_IFACE_STATS_RSP -  Interface statistics response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_IFACE_STATS_REQ    req;                /* copy of original request */
    TXIP_IFACE_STATS        stats;              /* current Interface statistics */
} TXIP_IFACE_STATS_RSP;

/*###########################################################################*
 * IP Ethernet Link Management
 *###########################################################################*/

/*---------------------------------------------------------------------------*
 * TXIP_ETH_SUMMARY_REQ - UDP layer summary request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
} TXIP_ETH_SUMMARY_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_ETH_SUMMARY_RSP - UDP layer summary response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ETH_SUMMARY_REQ    req;                /* copy of original request */
    TXIP_ETH_SUMMARY        summary;            /* summary of Ethernet Link usage */
} TXIP_ETH_SUMMARY_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_ETH_CONFIG_REQ - Ethernet Link-level configuration request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    U32                     linkNum;            /* Ethernet link number */
    TXIP_ETH_CONFIG         config;             /* configuration for the given Ethernet Link */
} TXIP_ETH_CONFIG_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_ETH_CONFIG_RSP - Ethernet Link-level configuration response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ETH_CONFIG_REQ     req;                /* copy of original request */
} TXIP_ETH_CONFIG_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_ETH_INFO_REQ - information about a given Ethernet Link request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    U32                     linkNum;            /* Ethernet link number */
} TXIP_ETH_INFO_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_ETH_INFO_RSP - information about a given Ethernet Link response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ETH_INFO_REQ       req;                /* copy of original request */
    TXIP_ETH_INFO           info;               /* current information for the given Ethernet Link */
} TXIP_ETH_INFO_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_ETH_STATS_REQ - Ethernet Link statistics request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    U32                     linkNum;            /* Ethernet link number */
    U32                     zeroFlag;           /* whether to zero statistics after current values are reported */
} TXIP_ETH_STATS_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_ETH_STATS_RSP -  Ethernet Link statistics response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ETH_STATS_REQ      req;                /* copy of original request */
    TXIP_ETH_STATS          stats;              /* current Ethernet Link statistics */
} TXIP_ETH_STATS_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_ETH_MON_REQ - Ethernet Link monitor request
 *
 * Activate Ethernet traffic monitoring to the host channel provided
 * (monChan = 0 deactivate any traffic monitoring).
 *
 * Each time a packet is received or successfully transmitted over the given
 * Ethernet link a copy is sent to the registered host application in the form
 * of a TXLOG_TRACE message [trace_id = TXLOG_ETH_xxx] (see txalarm.h).
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    U32                     linkNum;            /* Ethernet link number */
    U32                     monChan;            /* host channel that is to receive Ethernet link monitor messages */
#define TXIP_MSG_ETH_MON_DEFAULT_CH 0x2F        /* default host channel number (ss7trace) */

} TXIP_ETH_MON_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_ETH_MON_RSP -  Ethernet Link monitor response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ETH_MON_REQ        req;                /* copy of original request */
} TXIP_ETH_MON_RSP;

/*###########################################################################*
 * PING Management
 *###########################################################################*/

/*---------------------------------------------------------------------------*
 * TXIP_PING_SUMMARY_REQ - PING summary request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
} TXIP_PING_SUMMARY_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_PING_SUMMARY_RSP - PING summary response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_PING_SUMMARY_REQ   req;                /* copy of original request */
    TXIP_PING_SUMMARY       summary;            /* summary of current PINGs */
} TXIP_PING_SUMMARY_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_PING_START_REQ - PING start request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ADDR               local;              /* local IP address */
    TXIP_ADDR               remote;             /* remote IP address */
    TXIP_PING_OPTIONS       options;            /* PING options */
} TXIP_PING_START_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_PING_START_RSP - PING start response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_PING_START_REQ     req;                /* copy of original request */
} TXIP_PING_START_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_PING_STOP_REQ - PING stop request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ADDR               local;              /* local IP address */
    TXIP_ADDR               remote;             /* remote IP address */
} TXIP_PING_STOP_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_PING_STOP_RSP - PING stop response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_PING_STOP_REQ      req;                /* copy of original request */
} TXIP_PING_STOP_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_PING_INFO_REQ - information about a given PING request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ADDR               local;              /* local IP address */
    TXIP_ADDR               remote;             /* remote IP address */
} TXIP_PING_INFO_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_PING_INFO_RSP - information about a given PING response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_PING_INFO_REQ      req;                /* copy of original request */
    TXIP_PING_INFO          info;               /* current information for the given PING */
} TXIP_PING_INFO_RSP;

/*---------------------------------------------------------------------------*
 * TXIP_PING_STATS_REQ - PING statistics request
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_ADDR               local;              /* local IP address */
    TXIP_ADDR               remote;             /* remote IP address */
    U32                     zeroFlag;           /* whether to zero statistics after current values are reported */
} TXIP_PING_STATS_REQ;

/*---------------------------------------------------------------------------*
 * TXIP_PING_STATS_RSP -  PING statistics response
 *---------------------------------------------------------------------------*/
typedef struct
{
    TXIP_HDR                hdr;
    TXIP_PING_STATS_REQ     req;                /* copy of original request */
    TXIP_PING_STATS         stats;              /* current PING statistics */
} TXIP_PING_STATS_RSP;

/*###########################################################################*
 * FULL MESSAGE UNION - single structure overlay for all messages
 *
 * NOTE: These are not really FULL message size definitions since many
 *       messages end with a variable length array.
 *       This message union fully describes the minimal structures for each
 *       message type.  Use the TXIP_HDR field: msgcount to determine the
 *       actual number of array elements.
 *###########################################################################*/

/*---------------------------------------------------------------------------*
 * TXIP_MSG - single structure representing all TXIP messages
 *---------------------------------------------------------------------------*/
typedef union __txip_msg
{
    TXIP_HDR            common;

    union
    {
        union
        {
            TXIP_NETWORK_CONFIG_REQ config;
            TXIP_NETWORK_INFO_REQ   info;
        } network;

        union
        {
            TXIP_UDP_SUMMARY_REQ    summary;
            TXIP_UDP_CONFIG_REQ     config;
            TXIP_UDP_INFO_REQ       info;
            TXIP_UDP_STATS_REQ      stats;
        } udp;

        union
        {
            TXIP_PROTO_SUMMARY_REQ  summary;
            TXIP_PROTO_CONFIG_REQ   config;
            TXIP_PROTO_INFO_REQ     info;
            TXIP_PROTO_STATS_REQ    stats;
        } proto;

        union
        {
            TXIP_ROUTE_TABLE_REQ    table;
            TXIP_ROUTE_ADD_REQ      add;
            TXIP_ROUTE_ADD_GW_REQ   addGw;
            TXIP_ROUTE_DEL_REQ      del;
            TXIP_ROUTE_DEL_GW_REQ   delGw;
            TXIP_ROUTE_GET_REQ      get;
        } route;

        union
        {
            TXIP_IFACE_SUMMARY_REQ  summary;
            TXIP_IFACE_CONFIG_REQ   config;
            TXIP_IFACE_INFO_REQ     info;
            TXIP_IFACE_STATS_REQ    stats;
        } iface;

        union
        {
            TXIP_ETH_SUMMARY_REQ    summary;
            TXIP_ETH_CONFIG_REQ     config;
            TXIP_ETH_INFO_REQ       info;
            TXIP_ETH_STATS_REQ      stats;
            TXIP_ETH_MON_REQ        mon;
        } eth;

        union
        {
            TXIP_PING_SUMMARY_REQ   summary;
            TXIP_PING_START_REQ     start;
            TXIP_PING_STOP_REQ      stop;
            TXIP_PING_INFO_REQ      info;
            TXIP_PING_STATS_REQ     stats;
        } ping;
    } request;

    union
    {
        union
        {
            TXIP_NETWORK_CONFIG_RSP config;
            TXIP_NETWORK_INFO_RSP   info;
        } network;

        union
        {
            TXIP_UDP_SUMMARY_RSP    summary;
            TXIP_UDP_CONFIG_RSP     config;
            TXIP_UDP_INFO_RSP       info;
            TXIP_UDP_STATS_RSP      stats;
        } udp;

        union
        {
            TXIP_PROTO_SUMMARY_RSP  summary;
            TXIP_PROTO_CONFIG_RSP   config;
            TXIP_PROTO_INFO_RSP     info;
            TXIP_PROTO_STATS_RSP    stats;
        } proto;

        union
        {
            TXIP_ROUTE_TABLE_RSP    table;
            TXIP_ROUTE_ADD_RSP      add;
            TXIP_ROUTE_ADD_GW_RSP   add_gw;
            TXIP_ROUTE_DEL_RSP      del;
            TXIP_ROUTE_DEL_GW_RSP   del_gw;
            TXIP_ROUTE_GET_RSP      get;
        } route;

        union
        {
            TXIP_IFACE_SUMMARY_RSP  summary;
            TXIP_IFACE_CONFIG_RSP   config;
            TXIP_IFACE_INFO_RSP     info;
            TXIP_IFACE_STATS_RSP    stats;
        } iface;

        union
        {
            TXIP_ETH_SUMMARY_RSP    summary;
            TXIP_ETH_CONFIG_RSP     config;
            TXIP_ETH_INFO_RSP       info;
            TXIP_ETH_STATS_RSP      stats;
            TXIP_ETH_MON_RSP        mon;
        } eth;

        union
        {
            TXIP_PING_SUMMARY_RSP   summary;
            TXIP_PING_START_RSP     start;
            TXIP_PING_STOP_RSP      stop;
            TXIP_PING_INFO_RSP      info;
            TXIP_PING_STATS_RSP     stats;
        } ping;
    } response;

    union
    {
        TXIP_HDR        reserved;                   /* TXIP NOTIFICATIONS: reserved for future use */
    } notif;

} TXIP_MSG;

#endif  /* _TXIPMSG_H */
