/*****************************************************************************
 * txdynamic.c - TX Dynamic Switching of SS7 Link Demonstration Utility
 *
 * This utility shows how to dynamically switch an SS7 link across various
 * TDM channels without rebooting the TX boards involved.
 * The TXDYNAMIC program communicates with 2 TX boards that are connected via
 * either the H.100 or H.110 bus.  The TXDYNAMIC program can also switch
 * the SS7 link across T1/E1 trunks (if indicated on the command line).
 *
 * The 2 TX boards are provided as 'slave' and 'master' boards.
 * The 'master' board's link connection is switched first with the 'slave'
 * board being switched after both boards have reported a link failure.
 *
 * Before running this test, perform an ss7load of the master and slave
 * TX boards using the configuration files provided with this demonstration
 * application.  After the boards have been loaded, run TXDYNAMIC.
 *
 * NOTE: To test switching over T1/E1/J1 (in addition to switching over
 *       H.100/H.110), uncomment the appropriate trunk configuration entries
 *       from the txcfg files before loading the TX boards; then specify
 *       the trunk type (-t|-e|-j) to cause TXDYNAMIC to also test switching
 *       over the given trunk type.  The test assumes that each trunk of the
 *       master board is connected to same trunk number of the slave board.
 *
 * Test phases:
 *
 *       <<startup>>
 *          ||
 *       [PHASE_CAPS - reading switching capabilities of both boards]
 *          ||
 *       <<register to receive alarms>>
 *          ||
 *       [PHASE_WAIT_CONNECT - waiting for alarms indicating link up (initially connected)]
 *     ____ ||
 *   //---->\/
 *  ||   <<move master board's TDM connection>>
 *  ||      ||
 *  ||      \/
 *  ||   [PHASE_WAIT_DOWN - waiting for both boards to alarm link down]
 *  || ____ ||
 *  | /---->\/
 *  ||   <<move slave board's TDM connection>>
 *  ||      ||
 *  ||      \/
 *  ||   [PHASE_WAIT_UP - waiting for both boards to alarm link up]
 *  ||      ||
 *   \\____//
 *     ---- (advance stream or trunk under test)
 *
 * NOTE: For the H.100/H.110 bus, the test uses timeslot 1 on each
 *       H.100/H.110 stream (advancing the stream number by 2 each pass to
 *       maintain the standard even/odd convention).
 *       For T1/E1 trunks, the test traverses all available
 *       trunks using timeslot 1.
 *
 *****************************************************************************/

/*****************************************************************************
 **
 **                         PROPRIETARY NOTICE
 **
 **  (c)Copyright 2000-2009 Dialogic Corporation. All Rights Reserved.
 **
 **
 **  The following is licensed program material and is considered
 **  proprietary and confidential.  No part of this program may be
 **  reproduced or used in any form or by any means without the
 **  express written permission of Dialogic Corporation.
 **
 *****************************************************************************/

/* ----- standard C includes ----- */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

/* ----- common include files ----- */
#include "txnuser.h"
#include "txcpi.h"
#include "tx_core.h"
#include "t1e1type.h"
#include "swidef.h"
#include "txswi.h"
#include "txswiapi.h"
#include "tx4_swi.h"
#include "txalarm.h"

/* ----- Modification History: --------------------------------------------- */
#define MH_VERSION      2           /* identifier of major release version */
#define MH_REVISION     0           /* identifier of release revision */

/* ----- default TX board numbers ----- */
#define DEFAULT_MASTER      1       /* board acting as initial clock master */
#define DEFAULT_SLAVE       2       /* board acting as initial clock slave */

/* ----- test range definitions ----- */
#define HSTREAM_COUNT       32      /* number of H.100/H.110 streams */
#define TRUNK_COUNT         16      /* maximum number of trunks on a single TX board */
#define TEST_PORT           1       /* port number being switched */
#define TEST_TIMESLOT       1       /* timeslot used for all test connections */

/* ----- alarms of interest ----- */
#define TXALARM_LINK_UP     18179
#define TXALARM_LINK_DOWN   18180

/* externs */

extern char            *optarg;
extern int              optind, opterr, optopt;
extern CPI_WAIT_TYPE    stdInput;

/*###########################################################################*
 *
 * LOCAL FUNCTION PROTOTYPES
 *
 *###########################################################################*/
S16  HandleNotif        ( TX_HANDLE fd, S16 events );
void CpiaAlarmRxNotif   ( TX_HANDLE handle, void *chkey );
void CpiaAlarmTxNotif   ( TX_HANDLE handle, void *chkey, CPIPKT *msg,
                          void *tx_key, CPI_ERR_TYPE ccode );
void CpiaSwiRxNotif     ( TX_HANDLE handle, void *chkey );
void CpiaSwiTxNotif     ( TX_HANDLE handle, void *chkey, CPIPKT *msg,
                          void *tx_key, CPI_ERR_TYPE ccode );

/*###########################################################################*
 *
 * LOCAL STRUCTURE DEFINITONS
 *
 *###########################################################################*/

/*****************************************************************************
 * PER-TX-BOARD INFORMATION
 *****************************************************************************/
typedef struct
{
    U16         boardnum;       /* TX board number (also called CP number) */
    U8          index;          /* either MASTER_BOARD_INDEX or SLAVE_BOARD_INDEX */
    U8          link_state;     /* state of SS7 link being monitored */
#define LINK_UNKNOWN    0       /* no alarms received for link being monitored */
#define LINK_DOWN       1       /* link currently reported as down */
#define LINK_UP         2       /* link currently reported as up */

    TX_HANDLE   swi_handle;     /* CPI handle used for SWI API requests/responses */
    TX_HANDLE   alarm_handle;   /* CPI handle used to receive alarms */
    U16         trunk_count;    /* # of T1/E1 trunks on board */
    U16         avail1;         /* return to natural alignment */
    U8          curr_bus;       /* current bus being tested */
    U8          avail2[3];      /* return to natural alignment */
    U32         curr_stream;    /* current stream number (or trunk number) under test */

} TX_INFO;

/*###########################################################################*
 *
 * GLOBALS
 *
 *###########################################################################*/

/* ----- globals kept per board ----- */
TX_INFO     board_info[2];
#define MASTER_BOARD_INDEX  0
#define SLAVE_BOARD_INDEX   1

/* ----- globals not related to a given TX board ----- */
U16         host_channel = PT_SWI;  /* CPI channel used locally for SWI API messages */
U32         test_passes = 0;        /* number of test passes to perform (0 = 'infinite') */
U16         trunk_count = 0;        /* number of trunk connections to test */
U16         trunk_test = 0;         /* test to perform via trunks */
#define TRUNK_TEST_T1       1
#define TRUNK_TEST_E1       2
#define TRUNK_TEST_J1       3

U8          show_all_alarms = 0;    /* whether to show all received alarms (0 = only show link up|down alarms) */

U32         display_options = 0;    /* alarm output display options [default: NONE] */
#define TXALM_DISPLAY_SEVERITY  (1<<0)  /* set = display alarm severity as part of alarm message output */

/* ----- current test phase tracking ----- */
U8          curr_phase = 0;                     /* current phase of test */
#define PHASE_CAPS          1       /* getting switching capabilities */
#define PHASE_WAIT_CONNECT  2       /* waiting for alarms indicating link up (initially connected) */
#define PHASE_WAIT_DOWN     3       /* waiting for both boards to alarm link down */
#define PHASE_WAIT_UP       4       /* waiting for both boards to alarm link up */

U16         curr_expect = 0;                /* current count of expected responses */
U32         curr_passes = 0;                /* number of test passes that have occurred */

/* ----- names used for current status display ----- */
S8         *phase_name[] =
{
    "NOT USED",
    "getting switching capabilities",
    "waiting for initial connect alarms",
    "waiting for link down alarms",
    "waiting for link up alarms"
};

S8         *trunk_name[] =
{
    "NOT USED",
    "T1",
    "E1",
    "J1"
};

/*###########################################################################*
 *
 * ALARM ASSIST FUNCTIONS
 *
 *###########################################################################*/

/****************************************************************************
 * ProcessAlarm - process received alarm message (format output line)
 *
 * Inputs: pkt     - CPI packet holding received alarm
 *                   or NULL = write banner header to output line
 *         outline - address where output line (or error message) stored
 *         options - set of display options
 *
 * Output: alarm message [outline holds formatted alarm string]
 *         or NULL if error detected [outline holds error message]
 ****************************************************************************/
TXLOG_ALARM_MSG *ProcessAlarm( CPIPKT *pkt, S8 *outline, U32 options )
{
    TXLOG_ALARM_MSG    *msg;
    int                 slen;
    S8                  severity;
    S8                 *legacy_taskname;
    S8                 *alarm_text;
    U16                 source_board;
    struct tm           timestamp;
    time_t              secs_epoch;

    /* ----- write banner header to output line if no packet provided */
    if (pkt == NULL)
    {
        if (options & TXALM_DISPLAY_SEVERITY)
            strcpy( outline, "<MM/DD/YY HH:MM:SS> TaskName Bd Sev Number Message\n" );
        else
            strcpy( outline, "<MM/DD/YY HH:MM:SS> TaskName Bd Number Message\n" );
        return( NULL );
    }

    /* ----- verify size of packet is big enough to hold alarm message ----- */
    if (pkt->hdr.length < (sizeof(CPIHDR) + TXLOG_ALARM_MIN_SZ))
    {
        sprintf( outline, "Message of %d bytes too small to be alarm\n", pkt->hdr.length );
        return( NULL );
    }

    /* ----- determine TX board that was source of alarm ----- */
    source_board = BOARD(pkt->hdr.src);
    if (source_board > MAXBOARD)
    {
        sprintf( outline, "Message from invalid board number: %d\n", source_board );
        return( NULL );
    }

    /* ----- perform all required field swabbing ----- */
    msg = (TXLOG_ALARM_MSG *)pkt->data;
    msg->hdr.secs_epoch = cpi_cptoh_l( msg->hdr.secs_epoch );   /* seconds since epoch - log record timestamp */
    msg->hdr.logs_lost = cpi_cptoh_l( msg->hdr.logs_lost );     /* count of log records lost [not delivered to receiver] */
    msg->alarm.number = cpi_cptoh_s( msg->alarm.number );       /* unique number identifying alarm */
    msg->alarm.severity = cpi_cptoh_s( msg->alarm.severity );   /* severity of alarm */

    /* ----- determine severity code ----- */
    if (options & TXALM_DISPLAY_SEVERITY)
    {
        switch( msg->alarm.severity )
        {
            case TXALARM_SPECIFIC:  /* severity determined by specific alarm number */
                severity = 'S';
                break;

            case TXALARM_INFO:      /* Informational */
                severity = 'I';
                break;

            case TXALARM_WARNING:   /* Warning */
                severity = 'W';
                break;

            case TXALARM_ERROR:     /* Error */
                severity = 'E';
                break;

            case TXALARM_CRITICAL:  /* Critical Error */
                severity = 'C';
                break;

            default:
                severity = '?';     /* show that severity code is unknown */
        }
    }

    /* ----- locate text of alarm message ----- */
    legacy_taskname = (S8 *)msg->alarm.message;
    slen = strlen( legacy_taskname );
    alarm_text = (S8 *)&msg->alarm.message[slen + 1];

    /* ----- compute timestamp of alarm ----- */
    /* NOTE: The TX board sends time by providing seconds since epoch.
     *       These seconds are used by the "localtime" function to calculate the local time.
     */
    secs_epoch = (time_t)msg->hdr.secs_epoch;   /* get local copy so represented as valid native time_t */
    timestamp = *localtime( &secs_epoch );      /* call the library function to calculate local time */

    /* ----- format the alarm output string ----- */
    if (options & TXALM_DISPLAY_SEVERITY)
        sprintf( outline, "<%02d/%02d/%02d %02d:%02d:%02d> %s %2d [%c] %6d %s\n",
                 timestamp.tm_mon+1,
                 timestamp.tm_mday,
                 (timestamp.tm_year+1900) % 100,
                 timestamp.tm_hour,
                 timestamp.tm_min,
                 timestamp.tm_sec,
                 msg->hdr.task_name, /* space-padded task name */
                 source_board,
                 severity,
                 msg->alarm.number,
                 alarm_text );
    else
        sprintf( outline, "<%02d/%02d/%02d %02d:%02d:%02d> %s %2d %6d %s\n",
                 timestamp.tm_mon+1,
                 timestamp.tm_mday,
                 (timestamp.tm_year+1900) % 100,
                 timestamp.tm_hour,
                 timestamp.tm_min,
                 timestamp.tm_sec,
                 msg->hdr.task_name, /* space-padded task name */
                 source_board,
                 msg->alarm.number,
                 alarm_text );

    return( msg );
}

/*###########################################################################*
 *
 * TX SWI API ACCESS FUNCTIONS
 *
 *###########################################################################*/

/****************************************************************************
 * GetSwitchCaps - Query H.100/H.110 switch capabilities of a TX board
 *
 * Inputs: txinfo - information describing TX board to query
 ****************************************************************************/
void GetSwitchCaps( TX_INFO *txinfo )
{
    U32                 req_size;
    CPIPKT             *pkt;
    U32                 status;
    short               cpistatus;

    /* ----- determine the size of the request packet */
    req_size = txswiGetSwitchCaps( NULL );
    if (req_size == 0)
    {
        fprintf( stderr, "ERROR: Unable to determine packet size for swiGetSwitchCaps.\n" );
        exit( 1 );
    }

    /* ----- create the request msg */
    pkt = malloc( req_size );
    txswiAddress( pkt, req_size, txinfo->boardnum, host_channel );
    status = txswiGetSwitchCaps( pkt );
    if (status != 0)
    {
        fprintf( stderr, "ERROR: %d (building swiGetSwitchCaps packet)\n",
                 status );
        free( pkt );
        exit( 1 );
    }

    /* ----- send the request msg */
    cpistatus = cpia_send( txinfo->swi_handle, pkt, NULL );
    if (cpistatus != CPI_SUCCESS)
    {
        fprintf( stderr, "ERROR: Failed send of swiGetSwitchCaps to board %d:",
                 txinfo->boardnum );
        fprintf( stderr, "       CPI error: %s", cpi_get_error_str( cpi_get_last_error( ) ) );
        free( pkt );
        exit( 1 );
    }
}

/****************************************************************************
 * DisableOutputs - Break port connections currently defined on board
 *
 * Inputs: txinfo - information describing TX board to break connections for
 ****************************************************************************/
void DisableOutputs( TX_INFO *txinfo )
{
    SWI_TERMINUS    outterm[2];
    U32             req_size;
    CPIPKT         *pkt;
    U32             status;
    short           cpistatus;

    /* ----- determine the size of the request packet */
    req_size = txswiDisableOutput( NULL, outterm, 2 );
    if (req_size == 0)
    {
        fprintf( stderr, "\nERROR: Unable to determine packet size for swiDisableOutput.\n" );
        exit( 1 );
    }

    /* ----- break output to communications processor ----- */
    outterm[0].bus = MVIP95_LOCAL_BUS;
    outterm[0].stream = TX_LS_COMM_PRIMARY;
    outterm[0].timeslot = TEST_PORT - 1;    /* port n uses timeslot n-1 */

    /* ----- determine external connection to break ----- */
    if (txinfo->curr_bus == MVIP95_MVIP_BUS)
    {   /* break connection on H.100/H.110 bus */
        outterm[1].bus = MVIP95_MVIP_BUS;
        outterm[1].timeslot = TEST_TIMESLOT;    /* always using this external timeslot */
        if (txinfo->index == MASTER_BOARD_INDEX)
        {   /* inputting from odd stream number and outputting to even stream number */
            outterm[1].stream = txinfo->curr_stream;
        }
        else
        {   /* inputting from even stream number and outputting to odd stream number */
            outterm[1].stream = txinfo->curr_stream + 1;
        }
    }
    else
    {   /* break connection on local bus (trunk) */
        outterm[1].bus = MVIP95_LOCAL_BUS;
        outterm[1].stream = TX_LS_TRUNK( txinfo->curr_stream ); /* curr_stream holds trunk number */
        outterm[1].timeslot = TEST_TIMESLOT;
    }

    /* ----- create the request msg */
    pkt = malloc( req_size );
    txswiAddress( pkt, req_size, txinfo->boardnum, host_channel );
    status = txswiDisableOutput( pkt, outterm, 2 );
    if (status != 0)
    {
        fprintf( stderr, "ERROR: %d (building swiDisableOutput packet)\n",
                 status );
        free( pkt );
        exit( 1 );
    }

    /* ----- send the request msg */
    cpistatus = cpia_send( txinfo->swi_handle, pkt, NULL );
    if (cpistatus != CPI_SUCCESS)
    {
        fprintf( stderr, "ERROR: Failed send of swiDisableOutput to board %d:",
                 txinfo->boardnum );
        fprintf( stderr, "       CPI error: %s", cpi_get_error_str( cpi_get_last_error( ) ) );
        free( pkt );
        exit( 1 );
    }
}

/****************************************************************************
 * MakeConnections - Make new port connections
 *
 * Inputs: txinfo - information describing TX board to connect
 ****************************************************************************/
void MakeConnections( TX_INFO *txinfo )
{
    SWI_TERMINUS    interm[2],outterm[2];
    U32             req_size;
    CPIPKT         *pkt;
    U32             status;
    short           cpistatus;

    /* ----- determine the size of the request packet */
    req_size = txswiMakeConnection( NULL, interm, outterm, 2 );
    if (req_size == 0)
    {
        fprintf( stderr, "\nERROR: Unable to determine packet size for swiMakeConnection.\n" );
        exit( 1 );
    }

    /* ----- setup terminus information for communications processor connections ----- */
    interm[0].bus = MVIP95_LOCAL_BUS;
    interm[0].stream = TX_LS_COMM_PRIMARY;;
    interm[0].timeslot = TEST_PORT - 1;     /* port n uses timeslot n-1 */

    outterm[1].bus = MVIP95_LOCAL_BUS;
    outterm[1].stream = TX_LS_COMM_PRIMARY;;
    outterm[1].timeslot = TEST_PORT - 1;    /* port n uses timeslot n-1 */

    /* ----- determine external connections to make ----- */
    if (txinfo->curr_bus == MVIP95_MVIP_BUS)
    {   /* make connections on H.100/H.110 bus */
        outterm[0].bus = MVIP95_MVIP_BUS;
        interm[1].bus = MVIP95_MVIP_BUS;
        if (txinfo->index == MASTER_BOARD_INDEX)
        {   /* inputting from odd stream number and outputting to even stream number */
            outterm[0].stream = txinfo->curr_stream;
            interm[1].stream = txinfo->curr_stream + 1;
        }
        else
        {   /* inputting from even stream number and outputting to odd stream number */
            outterm[0].stream = txinfo->curr_stream + 1;
            interm[1].stream = txinfo->curr_stream;
        }
        outterm[0].timeslot = TEST_TIMESLOT;    /* always using this external timeslot */
        interm[1].timeslot = TEST_TIMESLOT;     /* always using this external timeslot */
    }
    else
    {   /* make connections on local bus (trunk) */
        outterm[0].bus = MVIP95_LOCAL_BUS;
        outterm[0].stream = TX_LS_TRUNK( txinfo->curr_stream ); /* curr_stream holds trunk number */
        outterm[0].timeslot = TEST_TIMESLOT;
        interm[1].bus = MVIP95_LOCAL_BUS;
        interm[1].stream = TX_LS_TRUNK( txinfo->curr_stream );  /* curr_stream holds trunk number */
        interm[1].timeslot = TEST_TIMESLOT;
    }

    /* ----- create the request msg */
    pkt = malloc( req_size );
    txswiAddress( pkt, req_size, txinfo->boardnum, host_channel );
    status = txswiMakeConnection( pkt, interm, outterm, 2 );
    if (status != 0)
    {
        fprintf( stderr, "ERROR: %d (building swiMakeConnection packet)\n",
                 status );
        free( pkt );
        exit( 1 );
    }

    /* ----- send the request msg */
    cpistatus = cpia_send( txinfo->swi_handle, pkt, NULL );
    if (cpistatus != CPI_SUCCESS)
    {
        fprintf( stderr, "ERROR: Failed send of swiMakeConnection to board %d:",
                 txinfo->boardnum );
        fprintf( stderr, "       CPI error: %s", cpi_get_error_str( cpi_get_last_error( ) ) );
        free( pkt );
        exit( 1 );
    }
}

/*###########################################################################*
 *
 * TEST ENGINE
 *
 *###########################################################################*/

/****************************************************************************
 * AdvanceTest - Advance to next stream or trunk to be tested
 ****************************************************************************/
void AdvanceTest( TX_INFO *txinfo )
{
    /* advance stream (or trunk) under test */
    if (txinfo->curr_bus == MVIP95_MVIP_BUS)
    {   /* currently testing SS7 link over H.100/H.110 bus */
        txinfo->curr_stream += 2;       /* advance to next stream pair */
        if (txinfo->curr_stream >= 31)
        {   /* done checking H.100 streams */
            if (trunk_test == 0)
            {   /* no trunk test so back to H.100/H.110 stream 0 */
                if (txinfo->index == MASTER_BOARD_INDEX)
                    curr_passes++;
                txinfo->curr_stream = 0;
            }
            else
            {   /* advance to start of trunk test */
                txinfo->curr_bus = MVIP95_LOCAL_BUS;
                txinfo->curr_stream = 1;        /* start with trunk number 1 */
            }
        }
    }
    else
    {   /* currently testing SS7 link over trunk (via local bus) */
        txinfo->curr_stream++;        /* advance to next trunk */
        if (txinfo->curr_stream > trunk_count)
        {   /* done checking trunks... back to H.100/H.110 stream */
            if (txinfo->index == MASTER_BOARD_INDEX)
                curr_passes++;
            txinfo->curr_bus = MVIP95_MVIP_BUS;
            txinfo->curr_stream = 0;
        }
    }

    if ( (test_passes != 0) && (curr_passes >= test_passes) )
    {
        printf( "\n\n%d TEST PASSES COMPLETED SUCCESSFULLY\n\n", curr_passes );
        exit(0);
    }

    if (txinfo->curr_bus == MVIP95_MVIP_BUS)
        printf( "\nSWITCHING board %d\'s port 1 to H.100/H.110 streams %d and %d...\n",
                txinfo->boardnum, txinfo->curr_stream, txinfo->curr_stream+1 );
    else
        printf( "\nSWITCHING board %d\'s port 1 to %s trunk %d...\n",
                txinfo->boardnum, trunk_name[trunk_test], txinfo->curr_stream );
}

/****************************************************************************
 * MovePort - Move port definition of master or slave board
 *
 * This function moves the port definition of the board indicated.
 * Moving a port involves breaking the current port definition and
 * making a new port definition.
 *
 * INPUTS: index - index of board to move port for
 ****************************************************************************/
void MovePort( U8 index )
{
    /* advance test phase based on which board port is being moved */
    if (index == MASTER_BOARD_INDEX)
        curr_phase = PHASE_WAIT_DOWN;   /* waiting for alarms indicating link down */
    else
        curr_phase = PHASE_WAIT_UP;     /* waiting for alarms indicating link up */

    /* move board's port 1 definition [DYNAMIC SWITCHING] */
    curr_expect += 4;                   /* 4 notifications expected:
                                         *     DisableOutputs response (from this board)
                                         *     MakeConnections response (from this board)
                                         *     link down alarm (from both boards)
                                         */

    /* NOTE: SWI messages are processed in order so OK to break current outputs and then
     *       immediately define new connections.
     */
    DisableOutputs( &board_info[index] );
    AdvanceTest( &board_info[index] );  /* advance stream (or trunk) under test */
    MakeConnections( &board_info[index] );
}

/****************************************************************************
 * CurrPhaseAlarm - Perform functions for current test phase [ALARM RX]
 *
 * This function handles all received alarms.
 * The received alarm has already been verified as a valid alarm.
 * This function continues the test based on the current test phase and the
 * type of alarm just received.
 *
 * INPUTS: txinfo - information describing the TX board that alarm rcvd from
 *         alarm  - verified alarm message (received from TX board)
 ****************************************************************************/
void CurrPhaseAlarm( TX_INFO *txinfo, TXLOG_ALARM_MSG *msg )
{
    /* ----- handle information in the received alarm ----- */
    switch (curr_phase)
    {
        case PHASE_CAPS:            /* getting switching capabilities */
            fprintf( stderr, "\nERROR: Received ALARM before registered for alarms!\n" );
            exit( 1 );
            break;

        case PHASE_WAIT_CONNECT:    /* waiting for alarms indicating link up (initially connected) */
            if ( (msg->alarm.number == TXALARM_LINK_UP) &&
                 (txinfo->link_state != LINK_UP) )
            {
                txinfo->link_state = LINK_UP;
                curr_expect--;
            }
            break;

        case PHASE_WAIT_DOWN:       /* waiting for both boards to alarm link down */
            if ( (msg->alarm.number == TXALARM_LINK_DOWN) &&
                 (txinfo->link_state != LINK_DOWN) )
            {
                txinfo->link_state = LINK_DOWN;
                curr_expect--;
            }
            break;

        case PHASE_WAIT_UP:         /* waiting for both boards to alarm link up */
            if ( (msg->alarm.number == TXALARM_LINK_UP) &&
                 (txinfo->link_state != LINK_UP) )
            {
                txinfo->link_state = LINK_UP;
                curr_expect--;
            }
            break;

        default:
            fprintf( stderr, "\nERROR: Unknown test phase [$%X]\n",
                     curr_phase );
            exit( 1 );
    }

    /* ----- determine next test phase ----- */
    if (curr_expect == 0)
    {   /* last expected alarm received for current test phase so determine next phase */
        switch (curr_phase)
        {
            case PHASE_WAIT_CONNECT:    /* DONE waiting for alarms indicating link up (initially connected case) */
            case PHASE_WAIT_UP:         /* DONE waiting for alarms indicating link up */
                /* move master board's port 1 definition */
                MovePort( MASTER_BOARD_INDEX );     /* perform DYNAMIC SWITCHING on master board */
                break;

            case PHASE_WAIT_DOWN:       /* DONE waiting for alarms indicating link down */
                /* move slave board's port 1 definition */
                MovePort( SLAVE_BOARD_INDEX );      /* perform DYNAMIC SWITCHING on slave board */
                break;

            default:
                fprintf( stderr, "\nERROR: Unknown test phase [$%X]\n",
                         curr_phase );
                exit( 1 );
        }
    }
}

/****************************************************************************
 * CurrPhaseSwi - Perform functions for current test phase [SWI response RX]
 *
 * This function handles all received SWI messages.
 * The received message has already been verified as a valid response.
 * This function stores any required information from the response message
 * and then continues the test based on the current test phase.
 *
 * INPUTS: txinfo - information describing the TX board that msg rcvd from
 *         msg    - verified response message (received from TX board)
 ****************************************************************************/
void CurrPhaseSwi( TX_INFO *txinfo, TXSWI_MSG *msg )
{
    SWI_LOCALDEV_DESC  *localdev;
    U16                 i;
    CPI_ERR_TYPE        cpi_error;

    /* ----- handle information in the received message ----- */
    switch (curr_phase)
    {
        case PHASE_CAPS:            /* DONE getting switching capabilities for one TX board */
            /* ----- save info from GetSwitchCaps response ----- */
            txinfo->trunk_count = 0;
            for (i = 0; i < msg->response.get_switch_caps.args.numlocalstreams; i++)
            {   /* for each local stream */
                localdev = &msg->response.get_switch_caps.localdev[i];
                if ( (localdev->deviceid == MVIP95_X_TRUNK_DEVICE) &&   /* stream connected to trunk ...AND... */
                     (i < TX_LS_MV_TRUNK_BASE) )                        /* NOT in range used for MVIPAPI compatibility */
                {   /* found a trunk */
                    txinfo->trunk_count++;                              /* NOTE: will count each trunk twice since provided as stream pair [see /= 2 below] */
                }
            }
            txinfo->trunk_count /= 2;   /* each trunk is reported twice (as pair of local streams) so cut count in half */
            if (trunk_count > txinfo->trunk_count)
                trunk_count = txinfo->trunk_count;  /* reduce to number of trunks available on each board */
            break;

        case PHASE_WAIT_CONNECT:    /* waiting for alarms indicating link up (initially connected) */
            fprintf( stderr, "\nERROR: Received SWI response when waiting connect!\n" );
            exit( 1 );
            break;

        case PHASE_WAIT_DOWN:       /* waiting for both boards to alarm link down */
            /* Here, responses to DisableOutputs and MakeConnections are received */
            break;

        case PHASE_WAIT_UP:         /* waiting for both boards to alarm link up */
            /* Here, responses to DisableOutputs and MakeConnections are received */
            break;

        default:
            fprintf( stderr, "\nERROR: Unknown test phase [$%X]\n",
                     curr_phase );
            exit( 1 );
    }

    /* ----- determine next test phase ----- */
    curr_expect--;           /* 1 less response outstanding */
    if (curr_expect == 0)
    {   /* last expected response received for current test phase so determine next phase */
        switch (curr_phase)
        {
            case PHASE_CAPS:            /* DONE getting switching capabilities for both TX boards */
                /* register to receive alarms */
                printf( "\nWAITING for initial connection alarms...\n" );
                curr_phase = PHASE_WAIT_CONNECT;    /* advance test phase */
                curr_expect += 2;   /* 2 link up alarms expected (one from each board) */

                for (i = MASTER_BOARD_INDEX; i <= SLAVE_BOARD_INDEX; i++)
                {
                    txinfo = &board_info[i];
                    txinfo->alarm_handle = cpia_open( txinfo, txinfo->boardnum, PT_INF,
                                                      CpiaAlarmRxNotif, CpiaAlarmTxNotif );
                    if (txinfo->alarm_handle < 0)
                    {
                        cpi_error = cpi_get_last_error( );
                        fprintf( stderr, "ERROR: could not register for ALARM channel (%d) for board number %d.\n",
                                 PT_INF, txinfo->boardnum );
                        fprintf( stderr, "       CPI error: %s\n",
                                 cpi_get_error_str( cpi_error ) );
                        exit( 1 );
                    }

                    tx_core_install_handler( cpi_wait_obj( txinfo->alarm_handle ), txinfo->alarm_handle, HandleNotif );
                    tx_core_set_poll( cpi_wait_obj( txinfo->alarm_handle ), TX_CORE_INPUT );
                }
                break;

            case PHASE_WAIT_CONNECT:    /* DONE waiting for alarms indicating link up (initially connected case) */
            case PHASE_WAIT_UP:         /* DONE waiting for alarms indicating link up */
                /* move master board's port 1 definition */
                MovePort( MASTER_BOARD_INDEX );     /* perform DYNAMIC SWITCHING on master board */
                break;

            case PHASE_WAIT_DOWN:       /* DONE waiting for alarms indicating link down */
                /* move slave board's port 1 definition */
                MovePort( SLAVE_BOARD_INDEX );      /* perform DYNAMIC SWITCHING on slave board */
                break;

            default:
                fprintf( stderr, "\nERROR: Unknown test phase [$%X]\n",
                         curr_phase );
                exit( 1 );
        }
    }
}

/*###########################################################################*
 *
 * NOTIFICATION ROUTINES
 *
 *###########################################################################*/

/****************************************************************************
 * GetCommand - get command line from stdInput
 *
 * This routine handles command lines from stdInput.
 *
 * INPUT:   handle - handle provided by TX CORE API
 *          events - set of events that have occurred
 *
 * OUTPUT:  value provided to TX CORE (whether to continue or terminate)
 ****************************************************************************/
short GetCommand( TX_HANDLE handle, short events )
{
    S8          input[80];
    S16         txcore_result = TX_CORE_SUCCESS;
    TX_INFO    *txinfo;

    /* ----- check if keyboard input event did occur ----- */
    if (tx_core_kbhit( ) == FALSE)
        return( TX_CORE_SUCCESS );      /* no keyboard activity after all */

    if (tx_core_process_events( TX_CORE_INPUT, events ) == FALSE)
        return( TX_CORE_SUCCESS );      /* no keyboard activity after all */

    /* ----- get the user command (from stdin [keyboard]) ----- */
    tx_core_gets( input, 80 );
    if (strlen( input ) == (TX_CORE_CR_POS - 1))
        strcpy( input, "?" );   /* convert CR to request for supported commands */
    else if (strlen( input ) > 1)
    {
        printf( "Invalid command [%s].\n", input );
        strcpy( input, "?" );   /* convert to request for supported commands */
    }

    printf( "\n" );             /* start from new line (beyond progress bar) */
    printf( "\n" );             /* blank line to separate user-requested output */
    switch (tolower( (int)input[0] ))
    {
        case 'c':               /* show current test control settings */
            txinfo = &board_info[MASTER_BOARD_INDEX];
            printf( "Current Test Control Settings:\n" );
            printf( "==============================================================\n" );
            printf( "       Current Phase: %s\n", phase_name[curr_phase] );
            printf( "   Expected Messages: %d\n", curr_expect );
            if (txinfo->curr_bus == MVIP95_MVIP_BUS)
            {
                printf( "             Testing: H.100/H.110 Streams %d and %d (timeslot 1)\n",
                        txinfo->curr_stream, txinfo->curr_stream + 1 );
            }
            else
            {
                printf( "             Testing: %s Trunk %d (channel 1)\n",
                        trunk_name[trunk_test], txinfo->curr_stream );
            }
            printf( "         Test Passes: %d\n", curr_passes );
            break;

        case '?':               /* display list of supported commands */
            printf( "Supported Commands:\n" );
            printf( "=============================================\n" );
            printf( "  C - show current test control settings\n" );
            printf( "  ? - show list of supported commands\n" );
            printf( "  Q - quit\n" );
            break;

        case 'q':               /* quit */
            txcore_result = TX_CORE_EXIT_POLL_LOOP; /* tell TX CORE to quit */
            break;
    }

    return( txcore_result );
}

/****************************************************************************
 * HandleNotif - handle notification of asynchronous CPI completion
 *
 * This function is upcalled whenever the TX CORE layer has detected an
 * asynchronous event occurring for one of the CPI channels.
 * These async events include receive notifications and transmit complete
 * notifications.  The event itself is not handled by this function.
 * This is a trigger to call the cpia_intr function which will either handle
 * the event internally or upcall the CpiaRxNotif or CpiaTxNotif function.
 *
 * INPUT:   fd     - file descriptor that event has occurred for
 *          events - set of events that have occurred
 *
 * OUTPUT:  value provided to TX CORE (whether to continue or terminate)
 *          [always returning TX_CORE_SUCCESS since never done at this point]
 ****************************************************************************/
S16 HandleNotif( TX_HANDLE fd, S16 events )
{
    cpia_intr( fd );

    return( TX_CORE_SUCCESS );
}

/****************************************************************************
 * CpiaAlarmRxNotif - notification of incoming alarm msg (from TX board)
 *
 * INPUT:   handle - handle to CPI channel that alarm was received over
 *          txinfo - TXDYNAMIC's control key associated with given TX board
 ****************************************************************************/
void CpiaAlarmRxNotif( TX_HANDLE handle, void *chkey )
{
    TX_INFO            *txinfo = (TX_INFO *)chkey;
    CPIPKT              rcvBuf;
    S16                 len;
    CPI_ERR_TYPE        cpi_error;
    TXLOG_ALARM_MSG    *msg;
    char                outLine[256];

    /* ----- read the received message ----- */
    len = sizeof( CPIPKT );
    if (cpia_get_data( handle, &rcvBuf, &len ) == CPI_ERROR)
    {
        cpi_error = cpi_get_last_error( );
        fprintf( stderr, "\nERROR: ALARM Receive error (%s)\n",
                 cpi_get_error_str( cpi_error ) );
        exit( 1 );
    }

    if (len < sizeof(TXLOG_ALARM_MSG))
    {
        fprintf( stderr, "\nERROR: Message received over ALARM channel too small to be alarm:\n" );
        fprintf( stderr, "      %d byte message received when %d bytes expected.\n",
                 len, sizeof(TXLOG_ALARM_MSG) );
        exit( 1 );
    }

    /* ----- process received alarm (and format alarm output string) ----- */
    strcpy( outLine, "ALARM: " );       /* start with header used for all alarm messages */
    msg = ProcessAlarm( &rcvBuf,
                        &outLine[strlen(outLine)],
                        display_options );
    if (msg == NULL)
    {
        printf( outLine );              /* display error message */
        exit(1);                        /* error while processing alarm so abort */
    }

    if ( (show_all_alarms) ||
         ( (msg->alarm.number == TXALARM_LINK_UP) && (txinfo->link_state != LINK_UP) ) ||
         ( (msg->alarm.number == TXALARM_LINK_DOWN) && (txinfo->link_state != LINK_DOWN) ) )
        printf( outLine );              /* display received alarm */

    /* ----- handle received alarm by current test phase ----- */
    CurrPhaseAlarm( txinfo, msg );      /* continue test based on current test phase */
}

/****************************************************************************
 * CpiaAlarmTxNotif - notification of msg transmit complete over alarm chan
 *
 * INPUT:   handle - handle to CPI channel that message was transmitted over
 *          txinfo - TXDYNAMIC's control key associated with given TX board
 *          msg    - address of message that has just been transmitted
 *          tx_key - TXDYNAMIC's control key associated with this transmit
 *          ccode  - completion code of transmitted packet
 ****************************************************************************/
void CpiaAlarmTxNotif( TX_HANDLE handle, void *chkey, CPIPKT *msg,
                       void *tx_key, CPI_ERR_TYPE ccode )
{
    TX_INFO    *txinfo = (TX_INFO *)chkey;

    /* handle all transmit notifications as errors since never send on alarm channel */
    fprintf( stderr, "\nERROR: transmit complete to board %d on alarm channel!\n",
             txinfo->boardnum );
    exit( 1 );
}

/****************************************************************************
 * CpiaSwiRxNotif - notification of incoming SWI msg receive (from TX board)
 *
 * INPUT:   handle - handle to CPI channel that message was received over
 *          txinfo - TXDYNAMIC's control key associated with given TX board
 ****************************************************************************/
void CpiaSwiRxNotif( TX_HANDLE handle, void *chkey )
{
    TX_INFO        *txinfo = (TX_INFO *)chkey;
    CPIPKT          rcvBuf;
    S16             len;
    CPI_ERR_TYPE    cpi_error;
    U32             ccode;
    TXSWI_MSG      *msg;

    /* ----- read the received message ----- */
    len = sizeof( CPIPKT );
    if (cpia_get_data( handle, &rcvBuf, &len ) == CPI_ERROR)
    {
        cpi_error = cpi_get_last_error( );
        fprintf( stderr, "\nERROR: SWI Receive error (%s)\n",
                 cpi_get_error_str( cpi_error ) );
        exit( 1 );
    }

    /* ----- process (and verify) received message ----- */
    msg = txswiReply( &rcvBuf, len, &ccode );
    if ( (msg == NULL) || (ccode != 0) )
    {   /* invalid message received */

        fprintf( stderr, "\nERROR: invalid SWI packet received from board %d\n",
                 txinfo->boardnum );
        if (ccode != 0)
            fprintf( stderr, "       MVIP95 error %d while processing reply\n",
                     ccode );
        exit( 1 );
    }

    /* ----- make sure message is a response ----- */
    if (msg->common.msgtype != TXSWI_RESPONSE)
    {   /* NON-response received... error since haven't registered for notifications */

        fprintf( stderr, "\nERROR: received SWI message from board %d is not a response [$%X]\n",
                 txinfo->boardnum, msg->common.msgtype );
        exit( 1 );
    }

    /* ----- handle validated response message by current test phase ----- */
    CurrPhaseSwi( txinfo, msg );    /* continue test based on current test phase */
}

/****************************************************************************
 * CpiaSwiTxNotif - notification of SWI msg transmit (to TX board) completed
 *
 * For the TXDYNAMIC program, an SWI transmit completes whenever a REQUEST
 * sent by TXDYNAMIC has reached the target task on the board.
 *
 * INPUT:   handle - handle to CPI channel that message was transmitted over
 *          txinfo - TXDYNAMIC's control key associated with given TX board
 *          msg    - address of message that has just been transmitted
 *          tx_key - TXDYNAMIC's control key associated with this transmit
 *          ccode  - completion code of transmitted packet
 ****************************************************************************/
void CpiaSwiTxNotif( TX_HANDLE handle, void *chkey, CPIPKT *msg,
                     void *tx_key, CPI_ERR_TYPE ccode )
{
    TX_INFO    *txinfo = (TX_INFO *)chkey;

    free( msg );                /* all done with request message */
}

/*###########################################################################*
 *
 * MAIN FUNCTIONS
 *
 *###########################################################################*/

/****************************************************************************
 * ShowUsage - show command-line usage
 ****************************************************************************/
void ShowUsage( char *myname )
{
    printf( "Usage:\n" );
    printf( "  %s [-m master] [-s slave] [-p passes] [-c channel] [-a] [-v] [-t|e|j]\n", myname );
    printf( "where                                               [DEFAULT]\n" );
    printf( "  -m : sets master TX board (1st to switch)         [board %d]\n",
            DEFAULT_MASTER );
    printf( "  -s : sets slave TX board (2nd to switch)          [board %d]\n",
            DEFAULT_SLAVE );
    printf( "  -p : sets number of test passes                   [INFINITE]\n" );
    printf( "  -c : sets host-side channel number                [channel %d]\n",
            PT_SWI );
    printf( "  -a : show all received alarms                     [ONLY LINK UP|DOWN]\n" );
    printf( "  -v : show alarm severity                          [NOT SHOWN]\n" );
    printf( "  -t : perform T1 test between TX boards            [NO T1 TEST]\n" );
    printf( "  -e : perform E1 test between TX boards            [NO E1 TEST]\n" );
    printf( "  -j : perform J1 test between TX boards            [NO J1 TEST]\n" );
    printf( "\n" );
}

/****************************************************************************
 * main
 ****************************************************************************/
int main( int argc, char *argv[] )
{
    S8             *myname;
    char           *str;
    U32             i;
    TX_INFO        *txinfo;
    CPI_ERR_TYPE    cpi_error;
    S32             cps[MAXBOARD + 1];

    myname = argv[0];

    printf( "\n" );
    printf( "********************************************\n" );
    printf( "* DYNAMIC SS7 SWITCHING DEMONSTRATION V%d.%d *\n", MH_VERSION, MH_REVISION );
    printf( "********************************************\n" );
    printf( "\n" );

    /*------------------------------------------------------------------------*
     * Set defaults
     *------------------------------------------------------------------------*/
    for (i = MASTER_BOARD_INDEX; i <= SLAVE_BOARD_INDEX; i++)
    {
        txinfo = &board_info[i];
        memset( txinfo, 0, sizeof(TX_INFO) );
        if (i == MASTER_BOARD_INDEX)
            txinfo->boardnum = DEFAULT_MASTER;
        else
            txinfo->boardnum = DEFAULT_SLAVE;
        txinfo->index = (U8)i;
        txinfo->swi_handle = CPI_INVALID_HANDLE;
        txinfo->alarm_handle = CPI_INVALID_HANDLE;
        txinfo->curr_bus = MVIP95_MVIP_BUS;
        txinfo->curr_stream = 0;
    }

    /*------------------------------------------------------------------------*
     * Parse all command-line options
     *------------------------------------------------------------------------*/
    i = 1;
    while (i < (U32)argc)
    {
        str = argv[i];
        if (*str == '-')
        {
            str++;
            switch (*str)
            {
                case 'm':
                case 'M':
                    str++;
                    if (isdigit(*str))
                        board_info[MASTER_BOARD_INDEX].boardnum = (U16)atoi(str);
                    else
                    {
                        i++;
                        if ( (i < (U32)argc) && (isdigit(*argv[i])) )
                            board_info[MASTER_BOARD_INDEX].boardnum = (U16)atoi(argv[i]);
                        else
                        {
                            printf("\nError: expecting board number after -m option.\n");
                            exit( 1 );
                        }
                    }
                    break;

                case 's':
                case 'S':
                    str++;
                    if (isdigit(*str))
                        board_info[SLAVE_BOARD_INDEX].boardnum = (U16)atoi(str);
                    else
                    {
                        i++;
                        if ( (i < (U32)argc) && (isdigit(*argv[i])) )
                            board_info[SLAVE_BOARD_INDEX].boardnum = (U16)atoi(argv[i]);
                        else
                        {
                            printf("\nError: expecting board number after -s option.\n");
                            exit( 1 );
                        }
                    }
                    break;

                case 'p':
                case 'P':
                    str++;
                    if (isdigit(*str))
                        test_passes = (U32)atoi(str);
                    else
                    {
                        i++;
                        if ( (i < (U32)argc) && (isdigit(*argv[i])) )
                            test_passes = (U32)atoi(argv[i]);
                        else
                        {
                            printf("\nError: expecting number of test passes after -p option.\n");
                            exit( 1 );
                        }
                    }
                    break;

                case 'c':
                case 'C':
                    str++;
                    if (isdigit(*str))
                        host_channel = (U16)atoi(str);
                    else
                    {
                        i++;
                        if ( (i < (U32)argc) && (isdigit(*argv[i])) )
                            host_channel = (U16)atoi(argv[i]);
                        else
                        {
                            printf("\nError: expecting host channel after -c option.\n");
                            exit( 1 );
                        }
                    }
                    break;

                case 'a':
                    show_all_alarms = 1;
                    break;

                case 'v':                           /* display alarm severity as part of alarm message output */
                    display_options |= TXALM_DISPLAY_SEVERITY;
                    break;

                case 't':
                    trunk_test = TRUNK_TEST_T1;
                    trunk_count = TRUNK_COUNT;      /* set to max trunk count (trimmed based on board capabilities) */
                    break;

                case 'e':
                    trunk_test = TRUNK_TEST_E1;
                    trunk_count = TRUNK_COUNT;      /* set to max trunk count (trimmed based on board capabilities) */
                    break;

                case 'j':
                    trunk_test = TRUNK_TEST_J1;
                    trunk_count = TRUNK_COUNT;      /* set to max trunk count (trimmed based on board capabilities) */
                    break;

                default:
                    ShowUsage( myname );
                    exit( 1 );
            }                       /* switch (*++str) */
        }                           /* if (*str == '-') */
        else
        {
            ShowUsage( myname );
            exit( 1 );
        }
        ++i;
    }                               /* while i < argc */

    /*------------------------------------------------------------------------*
     * Check that specified parameters make sense
     *------------------------------------------------------------------------*/
    if (board_info[MASTER_BOARD_INDEX].boardnum ==
        board_info[SLAVE_BOARD_INDEX].boardnum)
    {
        fprintf( stderr, "Master and slave TX boards cannot both be board number %d\n",
                 board_info[MASTER_BOARD_INDEX].boardnum );
        exit( 1 );
    }
    if (board_info[MASTER_BOARD_INDEX].boardnum > MAXBOARD)
    {
        fprintf( stderr, "ERROR: master board number must be between 1 and %d.\n", MAXBOARD );
        exit( 1 );
    }
    if (board_info[SLAVE_BOARD_INDEX].boardnum > MAXBOARD)
    {
        fprintf( stderr, "ERROR: slave board number must be between 1 and %d.\n", MAXBOARD );
        exit( 1 );
    }

    /*------------------------------------------------------------------------*
     * Initialize Core Library
     *------------------------------------------------------------------------*/
    if (tx_core_init( ) != TX_CORE_SUCCESS)
    {
        printf( "!!! Could not initialize core routines\n" );
        exit( 1 );
    }

    /*------------------------------------------------------------------------*
     * Initialize CPI library
     *------------------------------------------------------------------------*/
    cpi_init( 0, NULL );

    /*------------------------------------------------------------------------*
     * Get board model number (since not supported by legacy boards)
     *------------------------------------------------------------------------*/
    if (cpi_get_resources(MAXBOARD, cps) == CPI_ERROR)
    {
        cpi_error = cpi_get_last_error( );
        fprintf( stderr, "ERROR: could not determine board model numbers.\n" );
        fprintf( stderr, "       CPI error: %s\n",
                 cpi_get_error_str( cpi_error ) );
        exit( 1 );
    }

    /*------------------------------------------------------------------------*
     * Open a CPI channel to the TX boards indicated (for SWI messages)
     *------------------------------------------------------------------------*/
    for (i = MASTER_BOARD_INDEX; i <= SLAVE_BOARD_INDEX; i++)
    {
        txinfo = &board_info[i];
        txinfo->swi_handle = cpia_open( txinfo, txinfo->boardnum, host_channel,
                                        CpiaSwiRxNotif, CpiaSwiTxNotif );
        if (txinfo->swi_handle < 0)
        {
            cpi_error = cpi_get_last_error( );
            fprintf( stderr, "ERROR: could not register for SWI channel (%d) for board number %d.\n",
                     host_channel, txinfo->boardnum );
            fprintf( stderr, "       CPI error: %s\n",
                     cpi_get_error_str( cpi_error ) );
            exit( 1 );
        }

        tx_core_install_handler( cpi_wait_obj( txinfo->swi_handle ), txinfo->swi_handle, HandleNotif );
        tx_core_set_poll( cpi_wait_obj( txinfo->swi_handle ), TX_CORE_INPUT );
    }

    /*------------------------------------------------------------------------*
     * Install Operator Command Handler (attaching to keyboard events)
     *------------------------------------------------------------------------*/
    tx_core_install_handler( stdInput, CPI_INVALID_HANDLE, GetCommand );
    tx_core_set_poll( stdInput, TX_CORE_INPUT );

    /*------------------------------------------------------------------------*
     * Get switching capabilities of both boards
     *------------------------------------------------------------------------*/
    printf( "\nGETTING TX board switching capabilities...\n" );
    curr_phase = PHASE_CAPS;    /* remember current test phase */
    curr_expect += 2;            /* 2 responses expected (one from each board) */
    GetSwitchCaps( &board_info[MASTER_BOARD_INDEX] );
    GetSwitchCaps( &board_info[SLAVE_BOARD_INDEX] );

    /*------------------------------------------------------------------------*
     * Enter polling loop (remaining in loop until user quits program)
     *------------------------------------------------------------------------*/
    tx_core_poll_loop( TX_CORE_INFINITE );

    /*------------------------------------------------------------------------*
     * Close CPI channels
     *------------------------------------------------------------------------*/
    for (i = MASTER_BOARD_INDEX; i <= SLAVE_BOARD_INDEX; i++)
    {
        txinfo = &board_info[i];
        if (txinfo->swi_handle != CPI_INVALID_HANDLE)
            cpi_close( txinfo->swi_handle );
        if (txinfo->alarm_handle != CPI_INVALID_HANDLE)
            cpi_close( txinfo->alarm_handle );
    }

    return( 0 );
}
