/*****************************************************************************
 * txinfo.c - TX Information Display Utility
 *
 * This utility shows how to obtain system information from a TX board.
 *
 *****************************************************************************/

/*****************************************************************************
 **
 **                         PROPRIETARY NOTICE
 **
 **  (c)Copyright 1995-2009 Dialogic Corporation. All Rights Reserved.
 **
 **
 **  The following is licensed program material and is considered
 **  proprietary and confidential.  No part of this program may be
 **  reproduced or used in any form or by any means without the
 **  express written permission of Dialogic Corporation.
 **
 *****************************************************************************/

/* TX board information display tool */
#include <ctype.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "txcpi.h"
#include "tx_core.h"
#include "txinfoapi.h"

/* ----- Modification History: --------------------------------------------- */
#define MH_VERSION      2           /* identifier of major release version */
#define MH_REVISION     0           /* identifier of release revision */

/*###########################################################################*
 *
 * GLOBALS
 *
 *###########################################################################*/

/* ----- CPI interface */
TX_HANDLE   handle;
CPIPKT      info_reqmsg;            /* information request message */

/* ----- command-line information */
U16         board = 1;              /* default: request information from TX board 1 */

/* ----- names of each known TX board system ID */
char       *board_type_tx4000       = "TX 4000";
char       *board_type_tx4000c      = "TX 4000C";
char       *board_type_tx4000_20    = "TX 4000/20";
char       *board_type_tx4000_20c   = "TX 4000/20C";
char       *board_type_tx5000e      = "TX 5000E";
char       *board_type_tx5500e      = "TX 5500E";

char       *fan_state_unknown       = "UNKNOWN";    /* fan state in unknown (state has not been read yet) */
char       *fan_state_normal        = "NORMAL";     /* fan is running normally (no stop detected) */
char       *fan_state_stopped       = "STOPPED";    /* fan has stopped */
char       *fan_state_restarted     = "RESTARTED";  /* fan had stopped but has now restarted */
char       *fan_state_slow          = "SLOW";       /* fan is running slow */

char       *name_unknown            = "UNKNOWN";

/* ----- local prototypes */
void DisplaySysInfo( TXINFO_SYSTEM_INFO *sysInfo );
void DisplayCpuStatus( TXINFO_SYSTEM_INFO *sysInfo );

/*###########################################################################*
 *
 * NOTIFICATION ROUTINES
 *
 *###########################################################################*/

/****************************************************************************
 * HandleNotif - handle notification of asynchronous CPI completion
 *
 * This function is upcalled whenever the TX CORE layer has detected an
 * asynchronous event occurring for the CPI channel.
 * These async events include receive notifications and transmit complete
 * notifications.  The event itself is not handled by this function.
 * This is a trigger to call the cpia_intr function which will either handle
 * the event internally or upcall the CpiaRxNotif or CpiaTxNotif function.
 *
 * INPUT:   fd     - file descriptor that event has occurred for
 *          events - set of events that have occurred
 *
 * OUTPUT:  value provided to TX CORE (whether to continue or terminate)
 *          [always returning TX_CORE_SUCCESS since never done at this point]
 ****************************************************************************/
S16 HandleNotif( TX_HANDLE fd, S16 events )
{
    cpia_intr( fd );

    return( TX_CORE_SUCCESS );
}

/****************************************************************************
 * CpiaRxNotif - handle notification of incoming receive (from the TX board)
 *
 * INPUT:   handle - handle to CPI channel that message was received over
 *          ckhey - any control key associated with given CPI channel
 ****************************************************************************/
void CpiaRxNotif( TX_HANDLE handle, void *chkey )
{
    CPIPKT              rcvBuf;
    S16                 len;
    CPI_ERR_TYPE        cpi_error;
    TXINFO_MSG         *msg;
    U32                 ccode;

    /* ----- read the received message ----- */
    len = sizeof( CPIPKT );
    if (cpia_get_data( handle, &rcvBuf, &len ) == CPI_ERROR)
    {
        cpi_error = cpi_get_last_error( );
        fprintf( stderr, "\nERROR: Receive error (%s)\n",
                 cpi_get_error_str( cpi_error ) );
        exit( 1 );
    }

    /* ----- process (and verify) received system information message ----- */
    msg = txinfoReply( &rcvBuf, len, &ccode );
    if ( (msg == NULL) || (ccode != 0) )
    {   /* invalid message received */

        if ( (msg != NULL) && (msg->common.msgtype == TXINFO_NOTIFICATION) )
        {
            printf( "Invalid INFO notification message received.\n" );
        }
        else if (msg == NULL)
        {
            printf( "Invalid INFO message received.\n" );
        }
        else if (msg->common.msgtype != TXINFO_RESPONSE)
        {
            printf( "Invalid message [type 0x%X] received.\n", msg->common.msgtype );
        }

        if (ccode != 0)
        {
            printf( "  Error code from txinfoReply: 0x%08X\n", ccode );
        }

        return;
    }

    /* validated message received */

    /* display the TX board system information */
    printf( "\n" );
    DisplaySysInfo( &msg->response.info.system );

    /* display CPU status instead of system information summary */
    printf( "\n" );
    DisplayCpuStatus( &msg->response.info.system );

    exit( 0 );
}

/****************************************************************************
 * CpiaTxNotif - handle notification of transmit (to TX board) completed
 *
 * For the TXINFO program, a transmit completes whenever a REQUEST
 * sent by TXINFO has reached the target task on the board.
 *
 * INPUT:   handle - handle to CPI channel that message was transmitted over
 *          txinfo - TXINFO's control key associated with given TX board
 *          msg    - address of message that has just been transmitted
 *          tx_key - TXINFO's control key associated with this transmit
 *          ccode  - completion code of transmitted packet
 ****************************************************************************/
void CpiaTxNotif( TX_HANDLE handle, void *chkey, CPIPKT *msg,
                  void *tx_key, CPI_ERR_TYPE ccode )
{
    /* nothing to do here since request is a global (would free otherwise) */
}

/*###########################################################################*
 *
 * TX SYSTEM INFORMATION RECORD DISPLAY
 *
 *###########################################################################*/

/****************************************************************************
 * TXTempString - Convert from celcius temp in tenths to whole and tenths (Celcius and Ferenheit)
 ****************************************************************************/
void TXTempString( U32 ctemp, S8 *celcius, S8 *ferenheit )
{
    U32     curr_c, whole_c, tenths_c;
    U32     curr_f, whole_f, tenths_f;

    curr_c = ctemp;             /* get local copy of current temp so value cannot change during compuations */
    if (ctemp == 0)
    {
        whole_c = 0;
        tenths_c = 0;
        whole_f = 0;
        tenths_f = 0;
    }
    else
    {
        whole_c = curr_c / 10;        /* convert from tenths of degrees C to degrees C */
        tenths_c = curr_c - (whole_c * 10);
        /* now compute current Ferenheit temperature: 1.8C + 32 = F
        *                                            18C + 320 = 10F
        * ... but curr_c is in "tenths of degrees C"   curr_c/10 = C
        *     so multiply by 10 again to be able to use curr_c ...
        *                                            18curr_c + 3200 = 100F
        */
        curr_f = ((18 * curr_c) + 3200) / 10;       /* current degrees fereneheit [in tenths of a degree] */
        whole_f = curr_f / 10;        /* convert from tenths of degrees F to degrees F */
        tenths_f = curr_f - (whole_f * 10);
    }

    sprintf( celcius, "%u.%u", whole_c, tenths_c );
    sprintf( ferenheit, "%u.%u", whole_f, tenths_f );
}

/****************************************************************************
 * TXBoardType - return ASCII name of TX board system ID type provided
 ****************************************************************************/
char *TXBoardType( U32 sysId )
{
    char *name;

    switch( sysId )
    {
        case TX4000_SYSID:
            name = board_type_tx4000;
            break;

        case TX4000C_SYSID:
            name = board_type_tx4000c;
            break;

        case TX4000_20_SYSID:
            name = board_type_tx4000_20;
            break;

        case TX4000_20C_SYSID:
            name = board_type_tx4000_20c;
            break;

        case TX5000E_SYSID:
            name = board_type_tx5000e;
            break;

        case TX5500E_SYSID:
            name = board_type_tx5500e;
            break;

        default:
            name = name_unknown;
    }

    return( name );
}

/****************************************************************************
 * TXFanState - return ASCII name of TX fan state
 ****************************************************************************/
char *TXFanState( U32 fanState )
{
    char   *name;

    switch( fanState )
    {
        case TXINFO_FAN_UNKNOWN:            /* fan state in unknown (state has not been read yet) */
            name = fan_state_unknown;
            break;

        case TXINFO_FAN_NORMAL:            /* fan is running normally (no stop detected) */
            name = fan_state_normal;
            break;

        case TXINFO_FAN_STOPPED:            /* fan has stopped */
            name = fan_state_stopped;
            break;

        case TXINFO_FAN_RESTARTED:          /* fan had stopped but has now restarted */
            name = fan_state_restarted;
            break;

        case TXINFO_FAN_SLOW:               /* fan is running slow */
            name = fan_state_slow;
            break;

        default:
            name = name_unknown;
    }

    return( name );
}

/****************************************************************************
 * DisplaySysInfo - Display system information provided by TX board
 ****************************************************************************/
void DisplaySysInfo( TXINFO_SYSTEM_INFO *sysInfo )
{
    TXINFO_TIMESTAMP   *sysTime = &sysInfo->time.sysTime;   /* system time in timestamp format */

    printf( "---------------------------------\n" );
    printf( "TX board %d System Information:\n", board );
    printf( "---------------------------------\n" );
    printf( "\n" );

    /* system identification */
    printf( "    ID: cpNum: %d, BoardType: %s, serialNum: %u, kernel:%u.%u\n",
            sysInfo->id.cpNum,
            TXBoardType( sysInfo->id.boardType ),
            sysInfo->id.serialNum,
            sysInfo->id.kernelVersion, sysInfo->id.kernelRevision );
    printf( "\n" );

    /* system utilization */
    printf( "  UTIL: idleCount: %d, idlePeak: %d\n",
            sysInfo->util.idleCount,
            sysInfo->util.idlePeak );
    printf( "\n" );

    /* time handling */
    printf( "  TIME: bootMsecs: %d (bmsecWraps: %u), epochSecs: %d\n",
            sysInfo->time.bootMsecs,
            sysInfo->time.bmsecWraps,
            sysInfo->time.epochSecs );
    printf( "        %d/%d/%d %02d:%02d:%02d (+%d msecs)\n",
            sysTime->month, sysTime->day, sysTime->year,
            sysTime->hour, sysTime->min, sysTime->sec,
            sysTime->msec );
    printf( "\n" );
}

/****************************************************************************
 * DisplayCpuStatus - Display CPU status information provided by TX board
 ****************************************************************************/
void DisplayCpuStatus( TXINFO_SYSTEM_INFO *sysInfo )
{
    U32             i;
    U32             total,average,peak,curr,x;
    S8              curr_bar[52],average_bar[52];
    U32             temp;
    S8              celcius[16], ferenheit[16];

    printf( "--------------------------\n" );
    printf( "TX board %d CPU Status:\n", board );
    printf( "--------------------------\n" );
    printf( "\n" );

    /* sum up all the buckets (to get average utilization) */
    total = 0;
    peak = sysInfo->util.idlePeak;
    for (i = 0; i < TXINFO_SYSUTIL_BUCKETS; i++)
    {
        total += sysInfo->util.cpuBucket[i];
        if (peak < sysInfo->util.cpuBucket[i])
            peak = sysInfo->util.cpuBucket[i];   /* handle case where peak not accurate */
    }
    if (peak == 0)
    {   /* all buckets are ZERO'd so treat as 100% CPU */
        average = 100;
        curr = 100;
    }
    else
    {   /* non-zero peak */
        average = total / TXINFO_SYSUTIL_BUCKETS;   /* average idle count */
        average = 100 - (average / (peak/100));     /* convert to percentage */
        curr = sysInfo->util.cpuBucket[sysInfo->util.lastBucket];
        curr = 100 - (curr / (peak/100));           /* convert to percentage */
    }

    memset( curr_bar, '=', 50);                     /* assume 100% bar graph */
    x = curr / 2;                                   /* each '=' represents 2% */
    curr_bar[x] = '\0';
    memset( average_bar, '=', 50);
    x = average / 2;
    average_bar[x] = '\0';

    printf( "CPU Util_____ 0...10...20...30...40...50...60...70...80...90...100\n" );
    printf( " Current: %3u %s\n", curr, curr_bar );
    printf( " Average: %3u %s\n", average, average_bar );
    printf( "\n" );

    /* CPU status */
    temp = sysInfo->status.cpuTemp;
    TXTempString( temp, celcius, ferenheit );
    printf( "CPU Temp: %sC [%sF] degrees\n", celcius, ferenheit );
    temp = sysInfo->status.boardTemp;
    TXTempString( temp, celcius, ferenheit );
    printf( "Brd Temp: %sC [%sF] degrees\n", celcius, ferenheit );
    printf( "CPU  Fan: %s\n", TXFanState( sysInfo->status.fanState ) );
    printf( "\n" );
}

/*###########################################################################*
 *
 * MAIN FUNCTIONS
 *
 *###########################################################################*/

/****************************************************************************
 * ShowUsage - show command-line usage
 ****************************************************************************/
void ShowUsage( char *myname )
{
    printf("\n" );
    printf("TX Information Utility V%d.%d\n", MH_VERSION, MH_REVISION);
    printf("(c)Copyright 1995-2009 Dialogic Corporation.  All Rights Reserved.\n");
    printf("\n" );
    printf( "Usage: %s [-b board]\n", myname );
    printf( "\n" );
    printf( "where                                               [DEFAULT]\n" );
    printf( "  -b : sets TX board to request information from    [board 1]\n" );
    printf( "\n" );
}

/****************************************************************************
 * main
 ****************************************************************************/
int main( int argc, char *argv[] )
{
    S8             *myname;
    char           *str;
    U32             i;
    CPI_ERR_TYPE    cpi_error;
    U32             req_size;
    CPIPKT         *pkt;
    U32             status;
    U32             cpistatus;

    myname = argv[0];

    /*------------------------------------------------------------------------*
     * Parse all command-line options
     *------------------------------------------------------------------------*/
    i = 1;
    while (i < (U32)argc)
    {
        str = argv[i];
        if (*str == '-')
        {
            str++;
            switch (*str)
            {
                case 'b':
                case 'B':
                    str++;
                    if (isdigit(*str))
                        board = (U16)atoi(str);
                    else
                    {
                        i++;
                        if ( (i < (U32)argc) && (isdigit(*argv[i])) )
                            board = (U16)atoi(argv[i]);
                        else
                        {
                            printf("\nError: expecting board number after -b option.\n");
                            exit( 1 );
                        }
                    }
                    break;

                default:
                    ShowUsage( myname );
                    exit( 1 );
            }
        }
        else
        {
            ShowUsage( myname );
            exit( 1 );
        }
        ++i;
    }

    /*------------------------------------------------------------------------*
     * Initialize Core Library
     *------------------------------------------------------------------------*/
    if (tx_core_init( ) != TX_CORE_SUCCESS)
    {
        printf( "!!! Could not initialize core routines\n" );
        exit( 1 );
    }

    /*------------------------------------------------------------------------*
     * Initialize CPI library
     *------------------------------------------------------------------------*/
    cpi_init( 0, NULL );

    /*------------------------------------------------------------------------*
     * Open a CPI channel to the TX board indicated on the command line
     *------------------------------------------------------------------------*/
    handle = cpia_open( NULL, board, PT_MGR,
                        CpiaRxNotif, CpiaTxNotif );
    if (handle < 0)
    {
        cpi_error = cpi_get_last_error( );
        fprintf( stderr, "ERROR: could not open driver for board number %d.\n",
                 board );
        fprintf( stderr, "       CPI error: %s\n",
                    cpi_get_error_str( cpi_error ) );
        exit( 1 );
    }

    tx_core_install_handler( cpi_wait_obj( handle ), handle, HandleNotif );
    tx_core_set_poll( cpi_wait_obj( handle ), TX_CORE_INPUT );

    /*------------------------------------------------------------------------*
     * Issue system information request
     *------------------------------------------------------------------------*/

    /* ----- determine the size of the request packet */
    req_size = txinfoSystemInfo( NULL, 0 );
    if (req_size == 0)
    {
        printf( "Error sizing SYSTEM INFO.\n" );

        exit( 1 );
    }

    /* ----- create the request msg */
    pkt = (CPIPKT *)malloc( req_size );
    txinfoAddress( pkt, req_size, board, PT_MGR );
    status = txinfoSystemInfo( pkt, 0 );
    if (status != 0)
    {
        printf( "Error [0x%08X] building SYSTEM INFO.\n", status );
        free( pkt );

        exit( 1 );
    }

    /* ----- send the request msg */
    cpistatus = cpia_send( handle, pkt, NULL );
    if (cpistatus != CPI_SUCCESS)
    {
        printf( "Error while sending SYSTEM INFO:\n" );
        printf( "CPI error: %s\n", cpi_get_error_str(cpi_get_last_error()) );
        free( pkt );

        exit( 1 );
    }

    /*------------------------------------------------------------------------*
     * Enter polling loop (remaining in loop until exit is called from RX notif)
     *------------------------------------------------------------------------*/
    tx_core_poll_loop( TX_CORE_INFINITE );

    /*------------------------------------------------------------------------*
     * Close CPI channel
     *------------------------------------------------------------------------*/
    cpi_close( handle );

    return( 0 );
}

