/*****************************************************************************
 * txsdemo.c - TX SWI API Demonstration Utility
 *
 * This utility shows an example usage of the TX SWI API.
 * The TXSDEMO program communicates with 2 TX boards that are connected via
 * either the H.100 or H.110 bus.  The TXSDEMO program can also test
 * communication between 2 TX boards connected via T1/E1 cables.
 *
 * The 2 TX boards are provided as 'slave' and 'master' boards.
 * The 'master' board begins the test as the board mastering the clock
 * [H.100/H.110 A clock OR loop master (for T1/E1/J1 test)].
 * Once a test pass has completed with the initial master/slave clocking,
 * the master becomes the clock slave for the next pass.
 *
 * Test phases:
 *
 *       [PHASE_CAPS - getting switching capabilities]
 *     ____ ||
 *   //---->\/
 *  ||   [PHASE_CLOCK - configuring clocking]
 *  ||      ||
 *  ||      \/
 *  ||   [PHASE_CONFIG - configuring trunks] <<<< SKIP if no trunk test <<<<
 *  || ____ ||
 *  | /---->\/
 *  ||   [PHASE_CONNECT - making connections on receiver board]
 *  ||      ||
 *  ||      \/
 *  ||   [PHASE_PATTERN - configuring patterns on transmitter board]
 *  ||      ||
 *  ||      \/
 *  ||   [PHASE_SAMPLE - collecting samples on receiver board]
 *  ||      ||
 *  ||      \/
 *  ||   [PHASE_BREAK - breaking connections]
 *  ||      ||
 *   \\____//
 *     ---- (advance stream or trunk under test)
 *
 * NOTE: For the H.100/H.110 bus, the test begins with the 'master' board
 *       outputting patterns to the odd stream numbers.  The 'slave' board
 *       connects odd stream inputs to even stream outputs.
 *
 *       Once all connections have been verified, the 'master' is changed
 *       to output patterns to the even stream numbers (with the 'slave'
 *       board connecting even stream inputs to odd stream outputs).
 *
 *****************************************************************************/

/*****************************************************************************
 **
 **                         PROPRIETARY NOTICE
 **
 **  (c)Copyright 2000-2009 Dialogic Corporation.  All Rights Reserved.
 **
 **
 **  The following is licensed program material and is considered
 **  proprietary and confidential.  No part of this program may be
 **  reproduced or used in any form or by any means without the
 **  express written permission of Dialogic Corporation.
 **
 *****************************************************************************/

/* ----- standard C includes ----- */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

#ifdef UNIX
#include <unistd.h>
#endif	                                /* UNIX */

/* ----- common include files ----- */
#include "txnuser.h"
#include "txcpi.h"
#include "tx_core.h"
#include "t1e1type.h"
#include "swidef.h"
#include "txswi.h"
#include "txswiapi.h"

/* ----- Modification History: --------------------------------------------- */
#define MH_VERSION      2           /* identifier of major release version */
#define MH_REVISION     0           /* identifier of release revision */

/* ----- default TX board numbers ----- */
#define DEFAULT_MASTER      1       /* board acting as initial clock master */
#define DEFAULT_SLAVE       2       /* board acting as initial clock slave */

#define HSTREAM_COUNT       32      /* number of H.100/H.110 streams */
#define TRUNK_COUNT         16      /* maximum number of trunks on a single TX board */

/* externs */

extern char            *optarg;
extern int              optind, opterr, optopt;
extern CPI_WAIT_TYPE    stdInput;

/* SLEEP macro = sleep for # of milliseconds specified */
#ifdef UNIX
#define SLEEP(x) usleep(x * 1000);  /* convert from millisecs to microsecs */
#else
#define SLEEP(x) Sleep(x)
#endif

/*###########################################################################*
 *
 * LOCAL STRUCTURE DEFINITONS
 *
 *###########################################################################*/

/*****************************************************************************
 * PER-STREAM/TRUNK INFORMATION
 *****************************************************************************/
typedef struct
{
    U32         match;      /* number of patterns sampled as expected */
    U32         mismatch;   /* number of pattern mismatches */
} TEST_STATS;

/*****************************************************************************
 * PER-TX-BOARD INFORMATION
 *****************************************************************************/
typedef struct
{
    U16         boardnum;       /* TX board number (also called CP number) */
    U8          index;          /* either MASTER_BOARD_INDEX or SLAVE_BOARD_INDEX */
    U8          avail;
    TX_HANDLE   txhandle;       /* CPI handle used to access the given board */
    U16         trunk_count;    /* # of T1/E1 trunks on board */
    U8          pad[2];

    TEST_STATS  hstream[HSTREAM_COUNT]; /* per-stream (H.100/H.110) stats */
    TEST_STATS  trunk[TRUNK_COUNT];     /* per-trunk stats */
} TX_INFO;

/*###########################################################################*
 *
 * GLOBALS
 *
 *###########################################################################*/

/* ----- globals kept per board ----- */
TX_INFO     board_info[2];
#define MASTER_BOARD_INDEX  0
#define SLAVE_BOARD_INDEX   1

/* ----- globals not related to a given TX board ----- */
U16         host_channel = PT_SWI;  /* CPI channel used locally */
U32         test_passes = 0;        /* number of test passes to perform (0 = 'infinite') */
U16         trunk_count = 0;        /* number of trunk connections to test */
U16         trunk_test = 0;         /* test to perform via trunks */
#define TRUNK_TEST_T1       1
#define TRUNK_TEST_E1       2
#define TRUNK_TEST_J1       3

U8          exit_on_error = 0;      /* whether to exit test on 1st error detected */

#define EVEN_STREAMS        0
#define ODD_STREAMS         1

/* ----- current test phase tracking ----- */
U8          curr_master = MASTER_BOARD_INDEX;   /* which board acting as clock master */
U8          curr_sender = MASTER_BOARD_INDEX;   /* which board is sending patterns */
U8          curr_pat_out = ODD_STREAMS;         /* whether outputting patterns to even or odd streams */
U8          curr_bus = MVIP95_MVIP_BUS;         /* current bus being tested */
U8          curr_phase = 0;                     /* current phase of test */
#define PHASE_CAPS          1       /* getting switching capabilities */
#define PHASE_CLOCK         2       /* configuring clocking */
#define PHASE_CONFIG        3       /* configuring trunks */
#define PHASE_CONNECT       4       /* making connections on receiver board */
#define PHASE_PATTERN       5       /* configuring patterns on transmitter board */
#define PHASE_SAMPLE        6       /* collecting samples on receiver board */
#define PHASE_BREAK         7       /* breaking connections */

U16         curr_stream = 0;                /* current stream number (or trunk) being tested */
U16         curr_expect = 0;                /* current count of expected responses */
U32         curr_passes = 0;                /* number of test passes that have occurred */
BYTE        curr_skew = 0;                  /* pattern modified so consecutive tests use different patterns */
#define MAX_SKEW 20                         /* value when skew resets */

/* ----- progress bar (showing that test is progressing) ----- */
S8          progress_bar[] = "________________________________ | ________"
                             "\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b"
                             "\b\b\b"
                             "\b\b\b\b\b\b\b\b";
#define PROG_BAR_H100 0         /* index into progress bar for H.100 stream 0 */
#define PROG_BAR_T1E1 35        /* index into progress bar for T1E1 trunk 1 */

#define STATE_BYSTATS   0       /* state determined based on match/mismatch stats */
#define STATE_MAKING    '+'     /* making connections over stream/trunk */
#define STATE_SAMPLING  '*'     /* sampling patterns over stream/trunk */
#define STATE_BREAKING  'x'     /* breaking connections over stream/trunk */

#define NOT_TESTED      '_'     /* progress bar indicator for untested stream/trunk */
#define TEST_PASS       '.'     /* progress bar indicator for test that has passed */
#define TEST_FAIL       'F'     /* progress bar indicator for test that has failed */

/* ----- trunk configurations ----- */
TxT1E1ConfigEx  t1cfg_opts =
{
    /* standard configuration (TxT1E1Config) */
    {   FRAMING_D4, ENCODING_B8ZS, 2, 0, 0, 0, 0, 0 },

    /* extended configuration */
    0, 0, 0,                    /* alarmDetect, autoEnable, autoDisable */
    0, 0, 0,                    /* spareBits, spareNational, serviceWord */
    T1_XPM_0, T1_XPM_1, T1_XPM_2, T1_XPM_3, /* txPulseMask */
    0, T1E1_X_RX_IN_THRESH, T1E1_X_SLICER_55,     /* rxControl, rxInputThresh, rxSlicerThresh */
    T1E1_X_PULSE_DETECT,        /* pulseDetect */
    T1E1_X_PULSE_RECOVERY,      /* pulseRecovery */
    0, T1_X_SYNC_2OF4,          /* syncProc, selectSync */
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,           /* reserved */
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0
};

TxT1E1ConfigEx  e1cfg_opts =
{
    /* standard configuration (TxT1E1Config) */
    {   FRAMING_CCS, ENCODING_HDB3, 0, 0, 0, 0, 0, 0 },

    /* extended configuration */
    0, 0, 0,                    /* alarmDetect, autoEnable, autoDisable */
    0, E1_X_SPARE_NATIONAL, 0,  /* spareBits, spareNational, serviceWord */
    E1_XPM_0, E1_XPM_1, E1_XPM_2, E1_XPM_3, /* txPulseMask */
    0, T1E1_X_RX_IN_THRESH, T1E1_X_SLICER_55, /* rxControl, rxInputThresh, rxSlicerThresh */
    T1E1_X_PULSE_DETECT,                    /* pulseDetect */
    T1E1_X_PULSE_RECOVERY,                  /* pulseRecovery */
    0, 0,                                   /* syncProc, selectSync */
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,           /* reserved */
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0
};

/*###########################################################################*
 *
 * PROGRESS DISPLAY FUNCTIONS
 *
 *###########################################################################*/

/****************************************************************************
 * ProgressBar - Display the progress bar (optionally showing title lines)
 *
 * This routine shows the progress of the TXSDEMO program.
 *
 * INPUT:   show_title - flag indicating whether to show title line
 *                       0 = DO NOT SHOW (assume cursor at progress bar line)
 *                       else show title lines before showing progress bar
 ****************************************************************************/
void ProgressBar( U8 show_title )
{
    if (show_title)
    {
        printf( "----- H.100/H.110 Streams ------ | T1/E1/J1\n" );
        printf( "          1         2         3  | Trunks\n" );
        printf( "01234567890123456789012345678901 | 12345678\n" );
        printf( "================================ | ========\n" );
    }

    printf( progress_bar );
}

/****************************************************************************
 * ProgressState - Change state of progress bar
 *
 * This routine manages all changes to the test progress bar.
 *
 * INPUT: base_index - base index into progress bar (PROG_BAR_xxx)
 *        stream     - <stream number> (0..31) or <trunk number> (0..15)
 *        state      - current state of stream (or trunk) [STATE_xxx]
 ****************************************************************************/
void ProgressState( U16 base_index, U16 stream, S8 state )
{
    TEST_STATS *stats_a,*stats_b;

    switch( state )
    {
        case STATE_BYSTATS:     /* state determined based on match/mismatch stats */
            switch( base_index )
            {
                case PROG_BAR_H100:
                    stats_a = &board_info[MASTER_BOARD_INDEX].hstream[stream];
                    stats_b = &board_info[SLAVE_BOARD_INDEX].hstream[stream];
                    break;

                case PROG_BAR_T1E1:
                    stats_a = &board_info[MASTER_BOARD_INDEX].trunk[stream];
                    stats_b = &board_info[SLAVE_BOARD_INDEX].trunk[stream];
                    break;

                default:
                    fprintf( stderr, "\nERROR: unknown base index [$%X] provided to ProgressState\n",
                             base_index );
                    exit( 1 );
            }

            if ( (stats_a->mismatch == 0) && (stats_b->mismatch == 0) )
                progress_bar[base_index + stream] = TEST_PASS;
            else
                progress_bar[base_index + stream] = TEST_FAIL;
            break;

        case NOT_TESTED:        /* stream (or trunk) has not been tested yet */
        case STATE_MAKING:      /* making connections over stream/trunk */
        case STATE_SAMPLING:    /* sampling patterns over stream/trunk */
        case STATE_BREAKING:    /* breaking connections over stream/trunk */
            progress_bar[base_index + stream] = state;
            break;

        default:
            fprintf( stderr, "\nERROR: unknown state [$%X] provided to ProgressState\n",
                     state );
            exit( 1 );
    }
}

/*###########################################################################*
 *
 * T1/E1/J1 ASSIST FUNCTIONS
 *
 *###########################################################################*/

/****************************************************************************
 * TrunkTimeslotRange - set valid timeslot range based on trunk type
 *
 * Inputs: ts_low  - pointer where low timeslot to be stored
 *         ts_high - pointer where high timeslot to be stored
 *         (global trunk_test determines trunk type)
 *
 * Output: count of timeslots
 ****************************************************************************/
void TrunkTimeslotRange( unsigned *ts_low, unsigned *ts_high )
{
    switch (trunk_test)
    {
        case TRUNK_TEST_T1:
        case TRUNK_TEST_J1:
            *ts_low = 0;
            *ts_high = TX_LS_TIMESLOT_MAX_T1;       /* T1|J1 timeslots */
            break;

        case TRUNK_TEST_E1:
            *ts_low = 1;                            /* skip timeslot 0 for E1 */
            *ts_high = TX_LS_TIMESLOT_MAX_E1;       /* E1 timeslots */
            break;

        default:
            fprintf( stderr, "ERROR: Unknown trunk test (%d)\n",
                     trunk_test );
            exit( 1 );
    }
}

/*###########################################################################*
 *
 * TX SWI API ACCESS FUNCTIONS
 *
 *###########################################################################*/

/****************************************************************************
 * GetSwitchCaps - Query H.100/H.110 switch capabilities of a TX board
 *
 * Inputs: txinfo - information describing TX board to query
 ****************************************************************************/
void GetSwitchCaps( TX_INFO *txinfo )
{
    U32                 req_size;
    CPIPKT             *pkt;
    U32                 status;
    short               cpistatus;

    /* ----- determine the size of the request packet */
    req_size = txswiGetSwitchCaps( NULL );
    if (req_size == 0)
    {
        fprintf( stderr, "ERROR: Unable to determine packet size for swiGetSwitchCaps.\n" );
        exit( 1 );
    }

    /* ----- create the request msg */
    pkt = malloc( req_size );
    txswiAddress( pkt, req_size, txinfo->boardnum, host_channel );
    status = txswiGetSwitchCaps( pkt );
    if (status != 0)
    {
        fprintf( stderr, "ERROR: %d (building swiGetSwitchCaps packet)\n",
                 status );
        free( pkt );
        exit( 1 );
    }

    /* ----- send the request msg */
    cpistatus = cpia_send( txinfo->txhandle, pkt, NULL );
    if (cpistatus != CPI_SUCCESS)
    {
        fprintf( stderr, "ERROR: Failed send of swiGetSwitchCaps to board %d:",
                 txinfo->boardnum );
        fprintf( stderr, "       CPI error: %s", cpi_get_error_str( cpi_get_last_error( ) ) );
        free( pkt );
        exit( 1 );
    }
}

/****************************************************************************
 * ConfigTrunk - Configure trunk mode
 *
 * Inputs: txinfo    - information describing TX board
 *         trunk_num - trunk number to configure
 ****************************************************************************/
void ConfigTrunk( TX_INFO *txinfo, U16 trunk_num )
{
    SWI_LOCALSTREAM_ARGS    args;
    TxT1E1ConfigEx          config;
    U32                     req_size;
    CPIPKT                 *pkt;
    U32                     status;
    short                   cpistatus;

    /* ----- common initialization ----- */
    memset( &args, 0, sizeof(SWI_LOCALSTREAM_ARGS) );
    args.parameterid = TX_X_CONFIG_CARRIER;
    args.localstream = TX_LS_TRUNK( trunk_num );

    /* ----- trunk type-specific initialization ----- */
    switch (trunk_test)
    {
        case TRUNK_TEST_T1:    /* test trunks in T1 mode */
            args.deviceid = MVIP95_T1_TRUNK_DEVICE;

            /* ----- start from copy of global extended config area */
            memcpy( &config, &t1cfg_opts, sizeof(TxT1E1ConfigEx) );
            break;

        case TRUNK_TEST_E1:    /* test trunks in E1 mode */
            args.deviceid = MVIP95_E1_TRUNK_DEVICE;

            /* ----- start from copy of global extended config area */
            memcpy( &config, &e1cfg_opts, sizeof(TxT1E1ConfigEx) );
            break;

        case TRUNK_TEST_J1:    /* test trunks in J1 mode */
            args.deviceid = MVIP95_X_J1_TRUNK_DEVICE;

            /* ----- start from copy of global extended config area */
            memcpy( &config, &t1cfg_opts, sizeof(TxT1E1ConfigEx) );
            break;

        default:
            fprintf( stderr, "ERROR: unknown trunk test [$%X]\n",
                     trunk_test );
            exit( 1 );
    }

    /* ----- clocking configuration ----- */
    if (txinfo->index == curr_master)
        config.std.loopMaster = LOOPTIMING_MASTER;  /* this board is acting as clock master */
    else
        config.std.loopMaster = LOOPTIMING_SLAVE;   /* this board is acting as clock slave */

    /* ----- determine the size of the request packet */
    req_size = txswiConfigLocalStream( NULL, &args, &config, sizeof(TxT1E1ConfigEx) );
    if (req_size == 0)
    {
        fprintf( stderr, "\nERROR: Unable to determine packet size for swiConfigLocalStream.\n" );
        exit( 1 );
    }

    /* ----- create the request msg */
    pkt = malloc( req_size );
    txswiAddress( pkt, req_size, txinfo->boardnum, host_channel );
    status = txswiConfigLocalStream( pkt, &args, &config, sizeof(TxT1E1ConfigEx) );
    if (status != 0)
    {
        fprintf( stderr, "ERROR: %d (building swiConfigLocalStream packet)\n",
                 status );
        free( pkt );
        exit( 1 );
    }

    /* ----- send the request msg */
    cpistatus = cpia_send( txinfo->txhandle, pkt, NULL );
    if (cpistatus != CPI_SUCCESS)
    {
        fprintf( stderr, "ERROR: Failed send of swiConfigLocalStream to board %d:",
                 txinfo->boardnum );
        fprintf( stderr, "       CPI error: %s", cpi_get_error_str( cpi_get_last_error( ) ) );
        free( pkt );
        exit( 1 );
    }
}

/****************************************************************************
 * MakeConnections - Make all desired connections on receiver board
 *
 * Inputs: txinfo - information describing TX board to connect
 *
 * NOTE: For H.100/H.110 we want to define 128 connections,
 *       but 128 connection definitions cannot fit within a single
 *       CPI packet.  The connection definitions are therefore made
 *       using 2 different requests (of 64 connections each).
 ****************************************************************************/
void MakeConnections( TX_INFO *txinfo )
{
    SWI_TERMINUS    interm[64],outterm[64];
    SWI_TERMINUS    interm_b[64],outterm_b[64];
    unsigned        ts_low,ts_high,ts_count,i;
    U32             req_size;
    CPIPKT         *pkt;
    U32             status;
    short           cpistatus;
    DWORD           in_stream,out_stream;

    /* ----- determine the size of the request packet */
    if (curr_bus == MVIP95_MVIP_BUS)
    {
        ts_low = 0;
        ts_high = 63;                       /* 64 timeslots (in 2 diff requests) per H.100 stream */
    }
    else
    {
        TrunkTimeslotRange( &ts_low, &ts_high );   /* timeslot count deteremined by trunk type */
    }

    ts_count = (ts_high - ts_low + 1);
    req_size = txswiMakeConnection( NULL, interm, outterm, ts_count );
    if (req_size == 0)
    {
        fprintf( stderr, "\nERROR: Unable to determine packet size for swiMakeConnection.\n" );
        exit( 1 );
    }

    /* ----- determine input and output stream numbers ----- */
    if (curr_bus == MVIP95_MVIP_BUS)
    {
        if (curr_pat_out == ODD_STREAMS)
        {   /* inputting from odd stream numbers and outputting to even stream numbers */
            in_stream = (DWORD)curr_stream + 1;
            out_stream = (DWORD)curr_stream;
        }
        else
        {   /* inputting from even stream numbers and outputting to odd stream numbers */
            in_stream = (DWORD)curr_stream;
            out_stream = (DWORD)curr_stream + 1;
        }
    }

    /* ----- fill in all connection entries ----- */
    for (i = 0; i < ts_count; i++)
    {
        if (curr_bus == MVIP95_MVIP_BUS)
        {
            interm[i].bus = MVIP95_MVIP_BUS;
            interm[i].stream = in_stream;
            interm[i].timeslot = i;
            outterm[i].bus = MVIP95_MVIP_BUS;
            outterm[i].stream = out_stream;
            outterm[i].timeslot = i;

            interm_b[i].bus = MVIP95_MVIP_BUS;
            interm_b[i].stream = in_stream;
            interm_b[i].timeslot = i + 64;
            outterm_b[i].bus = MVIP95_MVIP_BUS;
            outterm_b[i].stream = out_stream;
            outterm_b[i].timeslot = i + 64;
        }
        else
        {   /* T1|E1|J1 trunk being tested */
            interm[i].bus = MVIP95_LOCAL_BUS;
            interm[i].stream = TX_LS_TRUNK( curr_stream );      /* curr_stream holds trunk number */
            interm[i].timeslot = i + ts_low;
            outterm[i].bus = MVIP95_LOCAL_BUS;
            outterm[i].stream = TX_LS_TRUNK( curr_stream );     /* curr_stream holds trunk number */
            outterm[i].timeslot = i + ts_low;
        }
    }

    /* ----- create the request msg */
    pkt = malloc( req_size );
    txswiAddress( pkt, req_size, txinfo->boardnum, host_channel );
    status = txswiMakeConnection( pkt, interm, outterm, ts_count );
    if (status != 0)
    {
        fprintf( stderr, "ERROR: %d (building swiMakeConnection packet)\n",
                 status );
        free( pkt );
        exit( 1 );
    }

    /* ----- send the request msg */
    cpistatus = cpia_send( txinfo->txhandle, pkt, NULL );
    if (cpistatus != CPI_SUCCESS)
    {
        fprintf( stderr, "ERROR: Failed send of swiMakeConnection to board %d:",
                 txinfo->boardnum );
        fprintf( stderr, "       CPI error: %s", cpi_get_error_str( cpi_get_last_error( ) ) );
        free( pkt );
        exit( 1 );
    }

    if (curr_bus == MVIP95_MVIP_BUS)
    {   /* had to split into 2 requests so issue second request */
        /* ----- create the 2nd request msg */
        pkt = malloc( req_size );
        txswiAddress( pkt, req_size, txinfo->boardnum, host_channel );
        status = txswiMakeConnection( pkt, interm_b, outterm_b, ts_count );
        if (status != 0)
        {
            fprintf( stderr, "ERROR: %d (building 2nd swiMakeConnection packet)\n",
                     status );
            free( pkt );
            exit( 1 );
        }

        /* ----- send the 2nd request msg */
        cpistatus = cpia_send( txinfo->txhandle, pkt, NULL );
        if (cpistatus != CPI_SUCCESS)
        {
            fprintf( stderr, "ERROR: Failed send of 2nd swiMakeConnection to board %d:",
                     txinfo->boardnum );
            fprintf( stderr, "       CPI error: %s", cpi_get_error_str( cpi_get_last_error( ) ) );
            free( pkt );
            exit( 1 );
        }
    }

    if (curr_bus == MVIP95_MVIP_BUS)
        ProgressState( PROG_BAR_H100, (U16)in_stream, STATE_MAKING );
    else
        ProgressState( PROG_BAR_T1E1, (U16)(curr_stream - 1), STATE_MAKING );
    ProgressBar( 0 );           /* refresh progress bar */
}

/****************************************************************************
 * ConfigBoardClock - Configure H.100/H.110 clocking on given board
 *
 * Inputs: txinfo - information describing TX board to configure
 ****************************************************************************/
void ConfigBoardClock( TX_INFO *txinfo )
{
    SWI_CLOCK_ARGS  args;
    U32             req_size;
    CPIPKT         *pkt;
    U32             status;
    short           cpistatus;

    memset( &args, 0, sizeof(SWI_CLOCK_ARGS) );
    args.size = sizeof(SWI_CLOCK_ARGS);

    args.clocktype = MVIP95_H100_CLOCKING;
    if (txinfo->index == curr_master)
    {
        args.clocksource = MVIP95_SOURCE_INTERNAL;
        args.ext.h100.clockmode = MVIP95_H100_MASTER_A;
    }
    else
        args.clocksource = MVIP95_SOURCE_H100_A;

    args.ext.h100.autofallback = MVIP95_H100_DISABLE_AUTO_FB;

    /* ----- determine the size of the request packet */
    req_size = txswiConfigBoardClock( NULL, &args );
    if (req_size == 0)
    {
        fprintf( stderr, "\nERROR: Unable to determine packet size for swiConfigBoardClock.\n" );
        exit( 1 );
    }

    /* ----- create the request msg */
    pkt = malloc( req_size );
    txswiAddress( pkt, req_size, txinfo->boardnum, host_channel );
    status = txswiConfigBoardClock( pkt, &args );
    if (status != 0)
    {
        fprintf( stderr, "ERROR: %d (building swiConfigBoardClock packet)\n",
                 status );
        free( pkt );
        exit( 1 );
    }

    /* ----- send the request msg */
    cpistatus = cpia_send( txinfo->txhandle, pkt, NULL );
    if (cpistatus != CPI_SUCCESS)
    {
        fprintf( stderr, "ERROR: Failed send of swiConfigBoardClock to board %d:",
                 txinfo->boardnum );
        fprintf( stderr, "       CPI error: %s", cpi_get_error_str( cpi_get_last_error( ) ) );
        free( pkt );
        exit( 1 );
    }
}

/****************************************************************************
 * SendPatterns - Cause board to send test patterns
 *
 * Inputs: txinfo - information describing TX board to act as transmitter
 *
 * NOTE: For H.100/H.110 we want to define 128 patterns,
 *       but 128 pattern definitions cannot fit within a single
 *       CPI packet.  The pattern definitions are therefore made
 *       using 2 different requests (of 64 patterns each).
 ****************************************************************************/
void SendPatterns( TX_INFO *txinfo )
{
    SWI_TERMINUS    outterm[64],outterm_b[64];
    BYTE            pattern[64],pattern_b[64];
    unsigned        ts_low,ts_high,ts_count,i;
    U32             req_size;
    CPIPKT         *pkt;
    U32             status;
    short           cpistatus;
    DWORD           out_stream;

    /* ----- determine the size of the request packet */
    if (curr_bus == MVIP95_MVIP_BUS)
    {
        ts_low = 0;
        ts_high = 63;                       /* 64 timeslots (in 2 diff requests) per H.100 stream */
    }
    else
    {
        TrunkTimeslotRange( &ts_low, &ts_high );    /* timeslot count deteremined by trunk type */
    }

    ts_count = (ts_high - ts_low + 1);
    req_size = txswiSendPattern( NULL, pattern, outterm, ts_count );
    if (req_size == 0)
    {
        fprintf( stderr, "\nERROR: Unable to determine packet size for swiSendPattern.\n" );
        exit( 1 );
    }

    /* ----- determine output stream number ----- */
    if (curr_bus == MVIP95_MVIP_BUS)
    {
        if (curr_pat_out == ODD_STREAMS)
        {   /* outputting patterns to odd stream numbers */
            out_stream = (DWORD)curr_stream + 1;
        }
        else
        {   /* outputting patterns to even stream numbers */
            out_stream = (DWORD)curr_stream;
        }
    }

    /* ----- fill in all connection entries ----- */
    for (i = 0; i < ts_count; i++)
    {
        if (curr_bus == MVIP95_MVIP_BUS)
        {
            outterm[i].bus = MVIP95_MVIP_BUS;
            outterm[i].stream = out_stream;
            outterm[i].timeslot = i;
            pattern[i] = i+1+curr_skew;         /* skip pattern $00 */

            outterm_b[i].bus = MVIP95_MVIP_BUS;
            outterm_b[i].stream = out_stream;
            outterm_b[i].timeslot = i + 64;
            pattern_b[i] = i+64+1+curr_skew;    /* skip pattern $00 */
        }
        else
        {
            outterm[i].bus = MVIP95_LOCAL_BUS;
            outterm[i].stream = TX_LS_TRUNK( curr_stream );     /* curr_stream holds trunk number */
            outterm[i].timeslot = i + ts_low;
            pattern[i] = i+1+curr_skew;         /* skip pattern $00 */
        }
    }

    /* ----- create the request msg */
    pkt = malloc( req_size );
    txswiAddress( pkt, req_size, txinfo->boardnum, host_channel );
    status = txswiSendPattern( pkt, pattern, outterm, ts_count );
    if (status != 0)
    {
        fprintf( stderr, "ERROR: %d (building swiSendPattern packet)\n",
                 status );
        free( pkt );
        exit( 1 );
    }

    /* ----- send the request msg */
    cpistatus = cpia_send( txinfo->txhandle, pkt, NULL );
    if (cpistatus != CPI_SUCCESS)
    {
        fprintf( stderr, "ERROR: Failed send of swiSendPattern to board %d:",
                 txinfo->boardnum );
        fprintf( stderr, "       CPI error: %s", cpi_get_error_str( cpi_get_last_error( ) ) );
        free( pkt );
        exit( 1 );
    }

    if (curr_bus == MVIP95_MVIP_BUS)
    {   /* had to split into 2 requests so issue second request */
        /* ----- create the 2nd request msg */
        pkt = malloc( req_size );
        txswiAddress( pkt, req_size, txinfo->boardnum, host_channel );
        status = txswiSendPattern( pkt, pattern_b, outterm_b, ts_count );
        if (status != 0)
        {
            fprintf( stderr, "ERROR: %d (building 2nd swiSendPattern packet)\n",
                     status );
            free( pkt );
            exit( 1 );
        }

        /* ----- send the 2nd request msg */
        cpistatus = cpia_send( txinfo->txhandle, pkt, NULL );
        if (cpistatus != CPI_SUCCESS)
        {
            fprintf( stderr, "ERROR: Failed send of 2nd swiSendPattern to board %d:",
                     txinfo->boardnum );
            fprintf( stderr, "       CPI error: %s", cpi_get_error_str( cpi_get_last_error( ) ) );
            free( pkt );
            exit( 1 );
        }
    }

    if (curr_bus == MVIP95_MVIP_BUS)
        ProgressState( PROG_BAR_H100, (U16)out_stream, STATE_MAKING );
    else
        ProgressState( PROG_BAR_T1E1, (U16)(curr_stream - 1), STATE_MAKING );
    ProgressBar( 0 );           /* refresh progress bar */
}

/****************************************************************************
 * SampleInputs - Sample test patterns being received by board
 *
 * Inputs: txinfo - information describing TX board to act as receiver
 ****************************************************************************/
void SampleInputs( TX_INFO *txinfo )
{
    SWI_TERMINUS    interm[128];
    unsigned        ts_low,ts_high,ts_count,i;
    U32             req_size;
    CPIPKT         *pkt;
    U32             status;
    short           cpistatus;
    DWORD           in_stream;

    /* ----- determine the size of the request packet */
    if (curr_bus == MVIP95_MVIP_BUS)
    {
        ts_low = 0;
        ts_high = 127;                       /* 128 timeslots per H.100 stream */
    }
    else
    {
        TrunkTimeslotRange( &ts_low, &ts_high );    /* timeslot count deteremined by trunk type */
    }

    ts_count = (ts_high - ts_low + 1);
    req_size = txswiSampleInput( NULL, interm, ts_count );
    if (req_size == 0)
    {
        fprintf( stderr, "\nERROR: Unable to determine packet size for swiSampleInput.\n" );
        exit( 1 );
    }

    /* ----- determine input stream number ----- */
    if (curr_bus == MVIP95_MVIP_BUS)
    {
        if (curr_pat_out == ODD_STREAMS)
        {   /* inputting from odd stream numbers and outputting to even stream numbers */
            in_stream = (DWORD)curr_stream + 1;
        }
        else
        {   /* inputting from even stream numbers and outputting to odd stream numbers */
            in_stream = (DWORD)curr_stream;
        }
    }

    /* ----- fill in all connection entries ----- */
    for (i = 0; i < ts_count; i++)
    {
        if (curr_bus == MVIP95_MVIP_BUS)
        {
            interm[i].bus = MVIP95_MVIP_BUS;
            interm[i].stream = in_stream;
            interm[i].timeslot = i;
        }
        else
        {
            interm[i].bus = MVIP95_LOCAL_BUS;
            interm[i].stream = TX_LS_TRUNK( curr_stream );  /* curr_stream holds trunk number */
            interm[i].timeslot = i + ts_low;
        }
    }

    /* ----- create the request msg */
    pkt = malloc( req_size );
    txswiAddress( pkt, req_size, txinfo->boardnum, host_channel );
    status = txswiSampleInput( pkt, interm, ts_count );
    if (status != 0)
    {
        fprintf( stderr, "ERROR: %d (building swiSampleInput packet)\n",
                 status );
        free( pkt );
        exit( 1 );
    }

    /* ----- send the request msg */
    cpistatus = cpia_send( txinfo->txhandle, pkt, NULL );
    if (cpistatus != CPI_SUCCESS)
    {
        fprintf( stderr, "ERROR: Failed send of swiSampleInput to board %d:",
                 txinfo->boardnum );
        fprintf( stderr, "       CPI error: %s", cpi_get_error_str( cpi_get_last_error( ) ) );
        free( pkt );
        exit( 1 );
    }

    if (curr_bus == MVIP95_MVIP_BUS)
        ProgressState( PROG_BAR_H100, (U16)in_stream, STATE_SAMPLING );
    else
        ProgressState( PROG_BAR_T1E1, (U16)(curr_stream - 1), STATE_SAMPLING );
    ProgressBar( 0 );           /* refresh progress bar */
}

/****************************************************************************
 * DisableOutputs - Break connections/patterns currently defined on board
 *
 * Inputs: txinfo - information describing TX board to break connections for
 ****************************************************************************/
void DisableOutputs( TX_INFO *txinfo )
{
    SWI_TERMINUS    outterm[128];
    unsigned        ts_low,ts_high,ts_count,i;
    U32             req_size;
    CPIPKT         *pkt;
    U32             status;
    short           cpistatus;
    DWORD           out_stream;

    /* ----- determine the size of the request packet */
    if (curr_bus == MVIP95_MVIP_BUS)
    {
        ts_low = 0;
        ts_high = 127;                       /* 128 timeslots per H.100 stream */
    }
    else
    {
        TrunkTimeslotRange( &ts_low, &ts_high );    /* timeslot count deteremined by trunk type */
    }

    ts_count = (ts_high - ts_low + 1);
    req_size = txswiDisableOutput( NULL, outterm, ts_count );
    if (req_size == 0)
    {
        fprintf( stderr, "\nERROR: Unable to determine packet size for swiDisableOutput.\n" );
        exit( 1 );
    }

    /* ----- determine output stream number ----- */
    if (curr_bus == MVIP95_MVIP_BUS)
    {
        if (curr_pat_out == ODD_STREAMS)
        {   /* inputting from odd stream numbers and outputting to even stream numbers */
            if (curr_sender == txinfo->index)
            {   /* disabling patterns */
                out_stream = (DWORD)curr_stream + 1;    /* sender board outputting to odd streams */
            }
            else
            {   /* disabling connections */
                out_stream = (DWORD)curr_stream;        /* receiver board connecting from odd to even streams */
            }
        }
        else
        {   /* inputting from even stream numbers and outputting to odd stream numbers */
            if (curr_sender == txinfo->index)
                out_stream = (DWORD)curr_stream;        /* sender board outputting to even streams */
            else
                out_stream = (DWORD)curr_stream + 1;    /* receiver board connecting from even to odd streams */
        }
    }

    /* ----- fill in all connection entries ----- */
    for (i = 0; i < ts_count; i++)
    {
        if (curr_bus == MVIP95_MVIP_BUS)
        {
            outterm[i].bus = MVIP95_MVIP_BUS;
            outterm[i].stream = out_stream;
            outterm[i].timeslot = i;
        }
        else
        {
            outterm[i].bus = MVIP95_LOCAL_BUS;
            outterm[i].stream = TX_LS_TRUNK( curr_stream );     /* curr_stream holds trunk number */
            outterm[i].timeslot = i + ts_low;
        }
    }

    /* ----- create the request msg */
    pkt = malloc( req_size );
    txswiAddress( pkt, req_size, txinfo->boardnum, host_channel );
    status = txswiDisableOutput( pkt, outterm, ts_count );
    if (status != 0)
    {
        fprintf( stderr, "ERROR: %d (building swiDisableOutput packet)\n",
                 status );
        free( pkt );
        exit( 1 );
    }

    /* ----- send the request msg */
    cpistatus = cpia_send( txinfo->txhandle, pkt, NULL );
    if (cpistatus != CPI_SUCCESS)
    {
        fprintf( stderr, "ERROR: Failed send of swiDisableOutput to board %d:",
                 txinfo->boardnum );
        fprintf( stderr, "       CPI error: %s", cpi_get_error_str( cpi_get_last_error( ) ) );
        free( pkt );
        exit( 1 );
    }

    if (curr_bus == MVIP95_MVIP_BUS)
    {
        if (curr_pat_out == ODD_STREAMS)
            ProgressState( PROG_BAR_H100, (U16)(curr_stream + 1), STATE_BREAKING );
        else
            ProgressState( PROG_BAR_H100, (U16)curr_stream, STATE_BREAKING );
    }
    else
        ProgressState( PROG_BAR_T1E1, (U16)(curr_stream - 1), STATE_BREAKING );
    ProgressBar( 0 );           /* refresh progress bar */
}

/*###########################################################################*
 *
 * TEST RESULT CHECKING
 *
 *###########################################################################*/

/****************************************************************************
 * CheckSamples - check set of samples against expected patterns
 *
 * This function determines whether a test phase has completed successfully.
 * The function maintains statistics indicating the number of times
 * that patterns matched and did not match for a given stream (or trunk).
 *
 * INPUTS: txinfo - information describing the TX board that msg rcvd from
 *         msg    - verified response message (response to SampleInput)
 ****************************************************************************/
void CheckSamples( TX_INFO *txinfo, TXSWI_MSG *msg )
{
    unsigned        ts_low,ts_high,ts_count,i;
    U8             *data = &msg->response.sample_input.data[0];
    TEST_STATS     *stats;
    DWORD           in_stream;

    /* ----- determine input stream number ----- */
    if (curr_bus == MVIP95_MVIP_BUS)
    {
        ts_low = 0;
        ts_high = 127;                       /* 128 timeslots per H.100 stream */

        if (curr_pat_out == ODD_STREAMS)
        {   /* inputting from odd stream numbers and outputting to even stream numbers */
            in_stream = (DWORD)curr_stream + 1;
        }
        else
        {   /* inputting from even stream numbers and outputting to odd stream numbers */
            in_stream = (DWORD)curr_stream;
        }
        stats = &txinfo->hstream[in_stream];
    }
    else
    {
        TrunkTimeslotRange( &ts_low, &ts_high );    /* timeslot count deteremined by trunk type */
        in_stream = TX_LS_TRUNK( curr_stream );     /* curr_stream holds trunk number */

        stats = &txinfo->trunk[curr_stream - 1];
    }

    ts_count = (ts_high - ts_low + 1);

    /* ----- compare all sampled patterns ----- */
    for (i = 0; i < ts_count; i++)
    {
        if (data[i] == i+1+curr_skew)
            stats->match++;
        else
        {
            stats->mismatch++;
            printf( "\nPATTERN MISMATCH (pass %d): terminus %d:%d sampled as [0x%02X] when expecting [0x%02X]\n",
                    curr_passes + 1,
                    in_stream, i + ts_low, data[i], i+1+curr_skew );
            if (exit_on_error)
                exit(1);
        }
    }

    /* ----- advance the pattern skew value (so next pass using different patterns) */
    curr_skew += 2;
    if (curr_skew > MAX_SKEW)
        curr_skew = 0;
}

/****************************************************************************
 * CheckPasses - check all test statistics to determine result of test passes
 *
 * This routine determines whether all test passes were successful.
 * If all passes were successful, the function exits with a success message.
 * If any failures are detected, failure info is displayed and the function
 * exists with an error indicator.
 ****************************************************************************/
void CheckPasses( )
{
    U16             i,j;
    TX_INFO        *txinfo;
    TEST_STATS     *stats;
    TEST_STATS      h_total,f_total;
    U8              result = 0;         /* default: test completely successful */

    /* ----- init overall test result records ----- */
    h_total.match = 0;
    h_total.mismatch = 0;
    f_total.match = 0;
    f_total.mismatch = 0;

    for( i = MASTER_BOARD_INDEX; i <= SLAVE_BOARD_INDEX; i++)
    {
        txinfo = &board_info[i];
        for (j = 0; j < HSTREAM_COUNT; j++)
        {
            stats = &txinfo->hstream[j];
            h_total.match += stats->match;
            h_total.mismatch += stats->mismatch;
        }
        if (trunk_test != 0)
        {
            for (j = 0; j < TRUNK_COUNT; j++)
            {
                stats = &txinfo->trunk[j];
                f_total.match += stats->match;
                f_total.mismatch += stats->mismatch;
            }
        }
    }

    if ( (h_total.mismatch == 0) && (f_total.mismatch == 0) )
        printf( "\nSUCCESS!\n" );
    else
    {
        result = 1;
        printf( "\nERRORS!\n" );
        printf( "     H.100/H.110 bus totals: %d matches\n", h_total.match );
        printf( "                             %d mismatches\n", h_total.mismatch );
        if (trunk_test != 0)
        {
            printf( "   Trunk (LOCAL bus) totals: %d matches\n", f_total.match );
            printf( "                             %d mismatches\n", f_total.mismatch );
        }
    }

    exit( result );
}

/*###########################################################################*
 *
 * TEST ENGINE
 *
 *###########################################################################*/

/****************************************************************************
 * AdvanceTestMvip - Advance to next test to be performed over MVIP bus
 *
 * INPUTS: NONE
 *
 * OUTPUT: new test phase
 ****************************************************************************/
U8 AdvanceTestMvip( )
{
    U8      new_phase = PHASE_CONNECT;  /* default: next phase = make new connections */

    if (curr_pat_out == ODD_STREAMS)
        curr_pat_out = EVEN_STREAMS;    /* switch to outputting patterns on even streams */
    else
    {   /* done checking patterns out to even streams */
        curr_pat_out = ODD_STREAMS;
        if (curr_sender == MASTER_BOARD_INDEX)
            curr_sender = SLAVE_BOARD_INDEX;    /* flip sender and receiver */
        else
        {   /* done checking flipped sender & receiver */
            curr_sender = MASTER_BOARD_INDEX;   /* flip sender and receiver */

            /* flip clock master */
            if (curr_master == MASTER_BOARD_INDEX)
                curr_master = SLAVE_BOARD_INDEX;
            else
            {   /* all test phases complete... back to initial test */
                curr_master = MASTER_BOARD_INDEX;
                curr_passes++;
                if ( (test_passes != 0) && (curr_passes >= test_passes) )
                {   /* just completed number of test passes requested */
                    CheckPasses( );     /* check on results of all test passes */
                    /* NOTE: CheckPasses calls exit so never execute beyond this call */
                }
            }

            new_phase = PHASE_CLOCK;            /* changed clocking */
        }
    }

    curr_stream = 0;        /* back to testing streams 0 and 1 */

    return( new_phase );
}

/****************************************************************************
 * AdvanceTest - Advance to next stream/trunk to be tested
 *
 * This function handles all changes to the particular stream, trunk number,
 * clocking source, etc. to be tested in the next test pass.
 *
 * Test advancement occurs as follows:
 * (with LOCAL phases only occurring if indicated on command line)
 *
 *      Bus     H.100 Clock Master  Pattern Sender  H.100 Pattern Out
 *      ======= ==================  ==============  =================
 *      MVIP    <master board>      <master board>  odd streams
 *      <<<advance stream number pair until each stream pair tested>>>
 *      LOCAL   <master board>      <master board>  -----------------
 *      <<<advance trunk number until each trunk tested>>>
 *      MVIP    <master board>      <master board>  even streams
 *      <<<advance stream number pair until each stream pair tested>>>
 *      LOCAL   <master board>      <master board>  -----------------
 *      <<<advance trunk number until each trunk tested>>>
 *      MVIP    <master board>      <slave board>   odd streams
 *      <<<advance stream number pair until each stream pair tested>>>
 *      LOCAL   <master board>      <slave board>   -----------------
 *      <<<advance trunk number until each trunk tested>>>
 *      MVIP    <master board>      <slave board>   even streams
 *      <<<advance stream number pair until each stream pair tested>>>
 *      LOCAL   <master board>      <slave board>   -----------------
 *      <<<advance trunk number until each trunk tested>>>
 *      MVIP    <slave board>       <master board>  odd streams
 *      <<<advance stream number pair until each stream pair tested>>>
 *      LOCAL   <slave board>       <master board>  -----------------
 *      <<<advance trunk number until each trunk tested>>>
 *      MVIP    <slave board>       <master board>  even streams
 *      <<<advance stream number pair until each stream pair tested>>>
 *      LOCAL   <slave board>       <master board>  -----------------
 *      <<<advance trunk number until each trunk tested>>>
 *      MVIP    <slave board>       <slave board>   odd streams
 *      <<<advance stream number pair until each stream pair tested>>>
 *      LOCAL   <slave board>       <slave board>   -----------------
 *      <<<advance trunk number until each trunk tested>>>
 *      MVIP    <slave board>       <slave board>   even streams
 *      <<<advance stream number pair until each stream pair tested>>>
 *      LOCAL   <slave board>       <slave board>   -----------------
 *      <<<advance trunk number until each trunk tested>>>
 *
 ****************************************************************************/
void AdvanceTest( )
{
    U8      new_phase = PHASE_CONNECT;  /* default: next phase = make new connections */

    /* advance stream (or trunk) under test */
    if (curr_bus == MVIP95_MVIP_BUS)
    {   /* currently testing streams of H.100/H.110 bus */
        curr_stream += 2;       /* advance to next stream pair */
        if (curr_stream >= 31)
        {   /* done checking H.100 streams */
            if (trunk_test == 0)
            {   /* no trunk test so advance to next H.100/H.110 test */
                new_phase = AdvanceTestMvip( );
            }
            else
            {   /* advance to start of trunk test */
                curr_bus = MVIP95_LOCAL_BUS;
                curr_stream = 1;        /* start with trunk number 1 */
            }
        }
    }
    else
    {   /* currently testing streams of local bus */
        curr_stream++;        /* advance to next trunk */
        if (curr_stream > trunk_count)
        {   /* done checking trunks... advance to next H.100 test */
            curr_bus = MVIP95_MVIP_BUS;
            new_phase = AdvanceTestMvip( );
        }
    }

    /* perform actions leading to new test phase */
    curr_phase = new_phase;     /* remember the current test phase */
    switch( curr_phase )
    {
        case PHASE_CLOCK:       /* time to change clocking configuration */
            /* configure clocking on each board */
            curr_expect += 2;   /* 2 responses expected (one from each board) */
            ConfigBoardClock( &board_info[MASTER_BOARD_INDEX] );
            ConfigBoardClock( &board_info[SLAVE_BOARD_INDEX] );
            break;

        case PHASE_CONNECT:     /* time to make new connections on receiver board */
            if (curr_bus == MVIP95_MVIP_BUS)
                curr_expect += 2;        /* expecting responses to 2 requests [had to split] */
            else
                curr_expect++;           /* expecting response to 1 request */
            if (curr_sender == MASTER_BOARD_INDEX)
                MakeConnections( &board_info[SLAVE_BOARD_INDEX] );
            else
                MakeConnections( &board_info[MASTER_BOARD_INDEX] );
            break;

        default:
            fprintf( stderr, "\nERROR: unknown new test phase [$%X] provided to AdvanceTest\n",
                     new_phase );
            exit( 1 );
    }

}

/****************************************************************************
 * CurrPhase - Perform all functions for the current test phase
 *
 * This function handles all received messages.
 * The received message has already been verified as a valid response.
 * This function stores any required information from the response message
 * and then continues the test based on the current test phase.
 *
 * INPUTS: txinfo - information describing the TX board that msg rcvd from
 *         msg    - verified response message (received from TX board)
 ****************************************************************************/
void CurrPhase( TX_INFO *txinfo, TXSWI_MSG *msg )
{
    SWI_LOCALDEV_DESC  *localdev;
    U16                 i,j;

    /* ----- handle information in the received message ----- */
    switch (curr_phase)
    {
        case PHASE_CAPS:        /* DONE getting switching capabilities for one TX board */
            /* ----- save info from GetSwitchCaps response ----- */
            txinfo->trunk_count = 0;
            for (i = 0; i < msg->response.get_switch_caps.args.numlocalstreams; i++)
            {   /* for each local stream */
                localdev = &msg->response.get_switch_caps.localdev[i];
                if ( (localdev->deviceid == MVIP95_X_TRUNK_DEVICE) &&   /* stream connected to trunk ...AND... */
                     (i < TX_LS_MV_TRUNK_BASE) )                        /* NOT in range used for MVIPAPI compatibility */
                {   /* found a trunk */
                    txinfo->trunk_count++;                              /* NOTE: will count each trunk twice since provided as stream pair [see /= 2 below] */
                }
            }
            txinfo->trunk_count /= 2;   /* each trunk is reported twice (as pair of local streams) so cut count in half */
            break;

        case PHASE_SAMPLE:      /* DONE collecting samples on receiver board */
            /* ----- check samples collected against expected patterns ----- */
            CheckSamples( txinfo, msg );
            break;

        case PHASE_CLOCK:       /* DONE configuring clocking */
        case PHASE_CONFIG:      /* DONE configuring trunk */
        case PHASE_CONNECT:     /* DONE making connections on receiver board */
        case PHASE_PATTERN:     /* DONE configuring patterns on transmitter board */
        case PHASE_BREAK:       /* DONE breaking connections */
            break;          /* no special response handling */

        default:
            fprintf( stderr, "\nERROR: Unknown test phase [$%X]\n",
                     curr_phase );
            exit( 1 );
    }

    /* ----- determine next test phase ----- */
    curr_expect--;                          /* 1 less response outstanding */
    if (curr_expect == 0)
    {   /* last expected response received for current test phase so determine next phase */
        switch (curr_phase)
        {
            case PHASE_CAPS:
                /* configure clocking on each board */
                curr_phase = PHASE_CLOCK;
                curr_expect += 2;           /* 2 responses expected (one from each board) */
                ConfigBoardClock( &board_info[MASTER_BOARD_INDEX] );
                ConfigBoardClock( &board_info[SLAVE_BOARD_INDEX] );
                break;

            case PHASE_CLOCK:
                SLEEP(500);                 /* delay for 1/2 second to allow clocking to settle */

                if (trunk_test != 0)
                {   /* testing trunks (as well as H.100/H.110) */
                    trunk_count = board_info[MASTER_BOARD_INDEX].trunk_count;
                    if (trunk_count > board_info[SLAVE_BOARD_INDEX].trunk_count)
                        trunk_count = board_info[SLAVE_BOARD_INDEX].trunk_count;
                    if (trunk_count == 0)
                    {
                        fprintf( stderr, "\nERROR: Trunk test only valid if trunks on both boards.\n" );
                        exit( 1 );
                    }

                    curr_phase = PHASE_CONFIG;
                    curr_expect += (2 * trunk_count);    /* number of responses expected (one from each trunk on both boards) */
                    for (i = MASTER_BOARD_INDEX; i <= SLAVE_BOARD_INDEX; i++)
                        for (j = 1; j <= trunk_count; j++)
                            ConfigTrunk( &board_info[i], j );
                }
                else
                {   /* not testing any trunks so start H.100/H.110 test */

                    /* make receiver board connections */
                    curr_phase = PHASE_CONNECT;
                    curr_expect += 2;       /* expecting responses to 2 requests [had to split] */
                    if (curr_sender == MASTER_BOARD_INDEX)
                        MakeConnections( &board_info[SLAVE_BOARD_INDEX] );
                    else
                        MakeConnections( &board_info[MASTER_BOARD_INDEX] );
                }
                break;

            case PHASE_CONFIG:
                SLEEP(10000);               /* delay for 5 seconds to allow trunks to sync up */

                /* make receiver board connections */
                curr_phase = PHASE_CONNECT;
                if (curr_bus == MVIP95_MVIP_BUS)
                    curr_expect += 2;       /* expecting responses to 2 requests [had to split] */
                else
                    curr_expect++;          /* expecting response to 1 request */
                if (curr_sender == MASTER_BOARD_INDEX)
                    MakeConnections( &board_info[SLAVE_BOARD_INDEX] );
                else
                    MakeConnections( &board_info[MASTER_BOARD_INDEX] );
                break;

            case PHASE_CONNECT:
                /* configure patterns on transmitter board */
                curr_phase = PHASE_PATTERN;
                if (curr_bus == MVIP95_MVIP_BUS)
                    curr_expect += 2;       /* expecting responses to 2 requests [had to split] */
                else
                    curr_expect++;          /* expecting response to 1 request */
                if (curr_sender == MASTER_BOARD_INDEX)
                    SendPatterns( &board_info[MASTER_BOARD_INDEX] );
                else
                    SendPatterns( &board_info[SLAVE_BOARD_INDEX] );
                break;

            case PHASE_PATTERN:
                SLEEP(250);                 /* delay for 1/4 second to allow patterns (& connects) to settle */

                /* collect samples on receiver board */
                curr_phase = PHASE_SAMPLE;
                curr_expect++;              /* expecting response to 1 request */
                if (curr_sender == MASTER_BOARD_INDEX)
                    SampleInputs( &board_info[SLAVE_BOARD_INDEX] );
                else
                    SampleInputs( &board_info[MASTER_BOARD_INDEX] );
                break;

            case PHASE_SAMPLE:
                /* break connections */
                curr_phase = PHASE_BREAK;
                curr_expect += 2;           /* expecting responses to 2 requests (one from each board) */
                DisableOutputs( &board_info[MASTER_BOARD_INDEX] );
                DisableOutputs( &board_info[SLAVE_BOARD_INDEX] );
                break;

            case PHASE_BREAK:
                /* update progrss bar with PASS/FAIL for test just completed */
                if (curr_bus == MVIP95_MVIP_BUS)
                {
                    if (curr_pat_out == ODD_STREAMS)
                        ProgressState( PROG_BAR_H100, (U16)(curr_stream + 1), STATE_BYSTATS );
                    else
                        ProgressState( PROG_BAR_H100, curr_stream, STATE_BYSTATS );
                }
                else
                    ProgressState( PROG_BAR_T1E1, (U16)(curr_stream - 1), STATE_BYSTATS );
                ProgressBar( 0 );           /* refresh progress bar */

                AdvanceTest( );             /* advance stream (or trunk) under test */
                break;

            default:
                fprintf( stderr, "\nERROR: Unknown test phase [$%X]\n",
                         curr_phase );
                exit( 1 );
        }
    }
}

/*###########################################################################*
 *
 * NOTIFICATION ROUTINES
 *
 *###########################################################################*/

/****************************************************************************
 * GetCommand - get command line from stdInput
 *
 * This routine handles command lines from stdInput.
 *
 * INPUT:   handle - handle provided by TX CORE API
 *          events - set of events that have occurred
 *
 * OUTPUT:  value provided to TX CORE (whether to continue or terminate)
 ****************************************************************************/
short GetCommand( TX_HANDLE handle, short events )
{
    S8          input[80];
    S16         txcore_result = TX_CORE_SUCCESS;
    TX_INFO    *txinfo;
    U16         i,stream;

    /* ----- check if keyboard input event did occur ----- */
    if (tx_core_kbhit( ) == FALSE)
        return( TX_CORE_SUCCESS );      /* no keyboard activity after all */

    if (tx_core_process_events( TX_CORE_INPUT, events ) == FALSE)
        return( TX_CORE_SUCCESS );      /* no keyboard activity after all */

    /* ----- get the user command (from stdin [keyboard]) ----- */
    tx_core_gets( input, 80 );
    if (strlen( input ) == (TX_CORE_CR_POS - 1))
        strcpy( input, "?" );   /* convert CR to request for supported commands */
    else if (strlen( input ) > 1)
    {
        printf( "Invalid command [%s].\n", input );
        strcpy( input, "?" );   /* convert to request for supported commands */
    }

    printf( "\n" );             /* start from new line (beyond progress bar) */
    printf( "\n" );             /* blank line to separate user-requested output */
    switch (tolower( (int)input[0] ))
    {
        case 'k':               /* show key */
            printf( "Progress Bar KEY:\n" );
            printf( "==============================================================\n" );
            printf( "  %c - stream (or trunk) has not been tested\n",                   NOT_TESTED );
            printf( "  %c - making connections across stream (or trunk)\n",             STATE_MAKING );
            printf( "  %c - sampling for expected patterns across stream (or trunk)\n", STATE_SAMPLING );
            printf( "  %c - breaking connections across stream (or trunk)\n",           STATE_BREAKING );
            printf( "  %c - pattern test PASSED for stream (or trunk)\n",               TEST_PASS );
            printf( "  %c - pattern test FAILED for stream (or trunk)\n",               TEST_FAIL );
            break;

        case 'c':               /* show current test control settings */
            printf( "Current Test Control Settings:\n" );
            printf( "==============================================================\n" );
            printf( "       Current Phase: %d\n", curr_phase );
            printf( "        Clock Master: Board %d\n", board_info[curr_master].boardnum );
            printf( "      Pattern Sender: Board %d\n", board_info[curr_sender].boardnum );
            printf( "      Pattern Output: %d\n", curr_pat_out );
            printf( "        Pattern Skew: %d\n", curr_skew );
            printf( "            Test Bus: %d\n", curr_bus );
            printf( "         Test Stream: %d\n", curr_stream );
            printf( "  Expected Responses: %d\n", curr_expect );
            printf( "         Test Passes: %d\n", curr_passes );
            break;

        case 's':               /* show statistics */
            printf( "Pattern Checking Statistics:\n" );
            printf( "\n" );
            printf( "              |      Board %d      |      Board %d\n",
                    board_info[MASTER_BOARD_INDEX].boardnum,
                    board_info[SLAVE_BOARD_INDEX].boardnum );
            printf( "Type   Stream | Match    Mismatch | Match    Mismatch\n" );
            printf( "====== ====== | ======== ======== | ======== ========\n" );
            for (i = 0; i < HSTREAM_COUNT; i ++)
            {   /* for each H.100/H.110 stream */
                printf( "H.100  %3d    | %8d %8d | %8d %8d\n",
                        i,
                        board_info[MASTER_BOARD_INDEX].hstream[i].match,
                        board_info[MASTER_BOARD_INDEX].hstream[i].mismatch,
                        board_info[SLAVE_BOARD_INDEX].hstream[i].match,
                        board_info[SLAVE_BOARD_INDEX].hstream[i].mismatch );
            }
            for (i = 0; i < TRUNK_COUNT; i++)
            {   /* for each trunk */
                printf( "TRUNK  %3d    | %8d %8d | %8d %8d\n",
                        i+1,
                        board_info[MASTER_BOARD_INDEX].trunk[i].match,
                        board_info[MASTER_BOARD_INDEX].trunk[i].mismatch,
                        board_info[SLAVE_BOARD_INDEX].trunk[i].match,
                        board_info[SLAVE_BOARD_INDEX].trunk[i].mismatch );
            }
            printf( "\n" );
            printf( "%d TEST PASSES COMPLETED\n", curr_passes );
            break;

        case 'r':               /* reset statistics */
            /* ----- clear all the stats ----- */
            for (i = MASTER_BOARD_INDEX; i <= SLAVE_BOARD_INDEX; i++)
            {
                txinfo = &board_info[i];

                for (stream = 0; stream < HSTREAM_COUNT; stream++)
                {
                    txinfo->hstream[stream].match = 0;
                    txinfo->hstream[stream].mismatch = 0;
                }
                for (stream = 0; stream < TRUNK_COUNT; stream++)
                {
                    txinfo->trunk[stream].match = 0;
                    txinfo->trunk[stream].mismatch = 0;
                }
            }

            /* ----- convert all TEST_FAIL indicators to NOT_TESTED (yet) ----- */
            for (i = 0; i < HSTREAM_COUNT; i += 2)
            {   /* for each H.100/H.110 stream pair */
                if (!((curr_bus == MVIP95_MVIP_BUS) && (curr_stream == i)))
                {   /* not stream pair currently under test so reset state */
                    ProgressState( PROG_BAR_H100, i, NOT_TESTED );
                    ProgressState( PROG_BAR_H100, (U16)(i+1), NOT_TESTED );
                }
            }
            for (i = 0; i < TRUNK_COUNT; i++)
            {   /* for each trunk */
                if (!((curr_bus == MVIP95_LOCAL_BUS) && (curr_stream == i)))
                {   /* not stream currently under test so reset state */
                    ProgressState( PROG_BAR_T1E1, i, NOT_TESTED );
                }
            }
            break;

        case '?':               /* display list of supported commands */
            printf( "Supported Commands:\n" );
            printf( "=============================================\n" );
            printf( "  K - show key (symbols used in progress bar)\n" );
            printf( "  C - show current test control settings\n" );
            printf( "  S - show statistics\n" );
            printf( "  R - reset statistics\n" );
            printf( "  ? - show list of supported commands\n" );
            printf( "  Q - quit\n" );
            break;

        case 'q':               /* quit */
            txcore_result = TX_CORE_EXIT_POLL_LOOP; /* tell TX CORE to quit */
            break;
    }

    if (txcore_result != TX_CORE_EXIT_POLL_LOOP)
        ProgressBar( 1 );       /* not quitting so re-display progress bar */

    return( txcore_result );
}

/****************************************************************************
 * HandleNotif - handle notification of asynchronous CPI completion
 *
 * This function is upcalled whenever the TX CORE layer has detected an
 * asynchronous event occurring for one of the CPI channels.
 * These async events include receive notifications and transmit complete
 * notifications.  The event itself is not handled by this function.
 * This is a trigger to call the cpia_intr function which will either handle
 * the event internally or upcall the CpiaRxNotif or CpiaTxNotif function.
 *
 * INPUT:   fd     - file descriptor that event has occurred for
 *          events - set of events that have occurred
 *
 * OUTPUT:  value provided to TX CORE (whether to continue or terminate)
 *          [always returning TX_CORE_SUCCESS since never done at this point]
 ****************************************************************************/
S16 HandleNotif( TX_HANDLE fd, S16 events )
{
    cpia_intr( fd );

    return( TX_CORE_SUCCESS );
}

/****************************************************************************
 * CpiaRxNotif - handle notification of incoming receive (from a TX board)
 *
 * INPUT:   handle - handle to CPI channel that message was received over
 *          txinfo - TXSDEMO's control key associated with given TX board
 ****************************************************************************/
void CpiaRxNotif( TX_HANDLE handle, void *chkey )
{
    TX_INFO        *txinfo = (TX_INFO *)chkey;
    CPIPKT          rcvBuf;
    S16             len;
    CPI_ERR_TYPE    cpi_error;
    U32             ccode;
    TXSWI_MSG      *msg;

    /* ----- read the received message ----- */
    len = sizeof( CPIPKT );
    if (cpia_get_data( handle, &rcvBuf, &len ) == CPI_ERROR)
    {
        cpi_error = cpi_get_last_error( );
        fprintf( stderr, "\nERROR: Receive error (%s)\n",
                 cpi_get_error_str( cpi_error ) );
        exit( 1 );
    }

    /* ----- process (and verify) received message ----- */
    msg = txswiReply( &rcvBuf, len, &ccode );
    if ( (msg == NULL) || (ccode != 0) )
    {   /* invalid message received */

        fprintf( stderr, "\nERROR: invalid packet received from board %d\n",
                 txinfo->boardnum );
        if (ccode != 0)
            fprintf( stderr, "       MVIP95 error %d while processing reply\n",
                     ccode );
        exit( 1 );
    }

    /* ----- make sure message is a response ----- */
    if (msg->common.msgtype != TXSWI_RESPONSE)
    {   /* NON-response received... error since haven't registered for notifications */

        fprintf( stderr, "\nERROR: received message from board %d is not a response [$%X]\n",
                 txinfo->boardnum, msg->common.msgtype );
        exit( 1 );
    }

    /* ----- handle validated response message by current test phase ----- */
    CurrPhase( txinfo, msg );   /* continue test based on current test phase */
}

/****************************************************************************
 * CpiaTxNotif - handle notification of transmit (to TX board) completed
 *
 * For the TXSDEMO program, a transmit completes whenever a REQUEST
 * sent by TXSDEMO has reached the target task on the board.
 *
 * INPUT:   handle - handle to CPI channel that message was transmitted over
 *          txinfo - TXSDEMO's control key associated with given TX board
 *          msg    - address of message that has just been transmitted
 *          tx_key - TXSDEMO's control key associated with this transmit
 *          ccode  - completion code of transmitted packet
 ****************************************************************************/
void CpiaTxNotif( TX_HANDLE handle, void *chkey, CPIPKT *msg,
                  void *tx_key, CPI_ERR_TYPE ccode )
{
    TX_INFO    *txinfo = (TX_INFO *)chkey;

    free( msg );                /* all done with request message */
}

/*###########################################################################*
 *
 * MAIN FUNCTIONS
 *
 *###########################################################################*/

/****************************************************************************
 * ShowUsage - show command-line usage
 ****************************************************************************/
void ShowUsage( char *myname )
{
    printf("\n" );
    printf("TX SWI Demonstration Application V%d.%d\n", MH_VERSION, MH_REVISION);
    printf("(c)Copyright 2000-2009 Dialogic Corporatio.  All Rights Reserved.n\n");
    printf("\n" );
    printf( "Usage:\n" );
    printf( "  %s [-m master] [-s slave] [-p passes] [-c channel] [-t|e|j] [-x]\n", myname );
    printf( "where                                               [DEFAULT]\n" );
    printf( "  -m : sets TX board that starts as clock master    [board %d]\n",
            DEFAULT_MASTER );
    printf( "  -s : sets TX board that starts slaved to clock    [board %d]\n",
            DEFAULT_SLAVE );
    printf( "  -p : sets number of test passes                   [INFINITE]\n" );
    printf( "  -c : sets host-side channel number                [channel %d]\n",
            PT_SWI );
    printf( "  -t : Perform T1 test between TX boards            [NO T1 TEST]\n" );
    printf( "  -e : Perform E1 test between TX boards            [NO E1 TEST]\n" );
    printf( "  -j : Perform J1 test between TX boards            [NO J1 TEST]\n" );
    printf( "  -x : Exit on any error                            [continue]\n" );
    printf( "\n" );
}

/****************************************************************************
 * main
 ****************************************************************************/
int main( int argc, char *argv[] )
{
    S8             *myname;
    char           *str;
    U32             i;
    TX_INFO        *txinfo;
    CPI_ERR_TYPE    cpi_error;
    S32             cps[MAXBOARD + 1];

    myname = argv[0];

    /*------------------------------------------------------------------------*
     * Set to defaults
     *------------------------------------------------------------------------*/
    txinfo = &board_info[MASTER_BOARD_INDEX];
    txinfo->boardnum = DEFAULT_MASTER;      /* board that starts as clock master */
    txinfo->index = MASTER_BOARD_INDEX;
    txinfo = &board_info[SLAVE_BOARD_INDEX];
    txinfo->boardnum = DEFAULT_SLAVE;       /* board that starts as clock slave */
    txinfo->index = SLAVE_BOARD_INDEX;

    /*------------------------------------------------------------------------*
     * Parse all command-line options
     *------------------------------------------------------------------------*/
    i = 1;
    while (i < (U32)argc)
    {
        str = argv[i];
        if (*str == '-')
        {
            str++;
            switch (*str)
            {
                case 'm':
                case 'M':
                    str++;
                    if (isdigit(*str))
                        board_info[MASTER_BOARD_INDEX].boardnum = (U16)atoi(str);
                    else
                    {
                        i++;
                        if ( (i < (U32)argc) && (isdigit(*argv[i])) )
                            board_info[MASTER_BOARD_INDEX].boardnum = (U16)atoi(argv[i]);
                        else
                        {
                            printf("\nError: expecting board number after -m option.\n");
                            exit( 1 );
                        }
                    }
                    break;

                case 's':
                case 'S':
                    str++;
                    if (isdigit(*str))
                        board_info[SLAVE_BOARD_INDEX].boardnum = (U16)atoi(str);
                    else
                    {
                        i++;
                        if ( (i < (U32)argc) && (isdigit(*argv[i])) )
                            board_info[SLAVE_BOARD_INDEX].boardnum = (U16)atoi(argv[i]);
                        else
                        {
                            printf("\nError: expecting board number after -s option.\n");
                            exit( 1 );
                        }
                    }
                    break;

                case 'p':
                    str++;
                    if (isdigit(*str))
                        test_passes = (U32)atoi(str);
                    else
                    {
                        i++;
                        if ( (i < (U32)argc) && (isdigit(*argv[i])) )
                            test_passes = (U32)atoi(argv[i]);
                        else
                        {
                            printf("\nError: expecting number of test passes after -p option.\n");
                            exit( 1 );
                        }
                    }
                    break;

                case 'c':
                case 'C':
                    str++;
                    if (isdigit(*str))
                        host_channel = (U16)atoi(str);
                    else
                    {
                        i++;
                        if ( (i < (U32)argc) && (isdigit(*argv[i])) )
                            host_channel = (U16)atoi(argv[i]);
                        else
                        {
                            printf("\nError: expecting host channel after -c option.\n");
                            exit( 1 );
                        }
                    }
                    break;

                case 't':
                    trunk_test = TRUNK_TEST_T1;
                    break;

                case 'e':
                    trunk_test = TRUNK_TEST_E1;
                    break;

                case 'j':
                    trunk_test = TRUNK_TEST_J1;
                    break;

                case 'x':
                    exit_on_error = 1;
                    break;

                default:
                    ShowUsage( myname );
                    exit( 1 );
            }                       /* switch (*++str) */
        }                           /* if (*str == '-') */
        else
        {
            ShowUsage( myname );
            exit( 1 );
        }
        ++i;
    }                               /* while i < argc */

    /*------------------------------------------------------------------------*
     * Check that specified parameters make sense
     *------------------------------------------------------------------------*/
    if (board_info[MASTER_BOARD_INDEX].boardnum ==
        board_info[SLAVE_BOARD_INDEX].boardnum)
    {
        fprintf( stderr, "Master and slave TX boards cannot both be board number %d\n",
                 board_info[MASTER_BOARD_INDEX].boardnum );
        exit( 1 );
    }
    if (board_info[MASTER_BOARD_INDEX].boardnum > MAXBOARD)
    {
        fprintf( stderr, "ERROR: master board number must be between 1 and %d.\n", MAXBOARD );
        exit( 1 );
    }
    if (board_info[SLAVE_BOARD_INDEX].boardnum > MAXBOARD)
    {
        fprintf( stderr, "ERROR: slave board number must be between 1 and %d.\n", MAXBOARD );
        exit( 1 );
    }

    /*------------------------------------------------------------------------*
     * Initialize Core Library
     *------------------------------------------------------------------------*/
    if (tx_core_init( ) != TX_CORE_SUCCESS)
    {
        printf( "!!! Could not initialize core routines\n" );
        exit( 1 );
    }

    /*------------------------------------------------------------------------*
     * Initialize CPI library
     *------------------------------------------------------------------------*/
    cpi_init( 0, NULL );

    /*------------------------------------------------------------------------*
     * Get board model number (since not supported by legacy boards)
     *------------------------------------------------------------------------*/
    if (cpi_get_resources(MAXBOARD, cps) == CPI_ERROR)
    {
        cpi_error = cpi_get_last_error( );
        fprintf( stderr, "ERROR: could not determine board model numbers.\n" );
        fprintf( stderr, "       CPI error: %s\n",
                 cpi_get_error_str( cpi_error ) );
        exit( 1 );
    }

    /*------------------------------------------------------------------------*
     * Open a CPI channel to the TX boards indicated on the command line
     *------------------------------------------------------------------------*/
    for (i = MASTER_BOARD_INDEX; i <= SLAVE_BOARD_INDEX; i++)
    {
        txinfo = &board_info[i];
        txinfo->txhandle = cpia_open( txinfo, txinfo->boardnum, host_channel,
                                      CpiaRxNotif, CpiaTxNotif );
        if (txinfo->txhandle < 0)
        {
            cpi_error = cpi_get_last_error( );
            fprintf( stderr, "ERROR: could not open driver for board number %d.\n",
                     txinfo->boardnum );
            fprintf( stderr, "       CPI error: %s\n",
                     cpi_get_error_str( cpi_error ) );
            exit( 1 );
        }

        tx_core_install_handler( cpi_wait_obj( txinfo->txhandle ), txinfo->txhandle, HandleNotif );
        tx_core_set_poll( cpi_wait_obj( txinfo->txhandle ), TX_CORE_INPUT );
    }

    /*------------------------------------------------------------------------*
     * Install Operator Command Handler (attaching to keyboard events)
     *------------------------------------------------------------------------*/
    tx_core_install_handler( stdInput, CPI_INVALID_HANDLE, GetCommand );
    tx_core_set_poll( stdInput, TX_CORE_INPUT );

    /*------------------------------------------------------------------------*
     * Get switching capabilities of both boards
     *------------------------------------------------------------------------*/
    curr_phase = PHASE_CAPS;    /* remember current test phase */
    curr_expect += 2;            /* 2 responses expected (one from each board) */
    GetSwitchCaps( &board_info[MASTER_BOARD_INDEX] );
    GetSwitchCaps( &board_info[SLAVE_BOARD_INDEX] );

    /*------------------------------------------------------------------------*
     * Display Progress Bar
     *------------------------------------------------------------------------*/
    ProgressBar( 1 );

    /*------------------------------------------------------------------------*
     * Enter polling loop (remaining in loop until user quits program)
     *------------------------------------------------------------------------*/
    tx_core_poll_loop( TX_CORE_INFINITE );

    /*------------------------------------------------------------------------*
     * Close CPI channels
     *------------------------------------------------------------------------*/
    for (i = MASTER_BOARD_INDEX; i <= SLAVE_BOARD_INDEX; i++)
    {
        txinfo = &board_info[i];
        cpi_close( txinfo->txhandle );
    }

    return( 0 );
}
