/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**                          Copyright 2006 - 2009                      **
**                Dialogic Corporation.  All Rights Reserved            **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation.                         **
**                                                                      **
*************************************************************************/

/*
 * moncfg.c
 *
 * Scans ascii text configuration file and downloads configuration to monitor
 * task on specified board.
 */

/* header include files -- defines (.h) */
#ifdef UNIX
#include <strings.h>
#include <stdlib.h>
#endif

#include <sys/types.h>
#include <stdio.h>
#include <errno.h>

#include "ss7type.h"
#include "monmapi.h"
#include "cfgscan.h"
#include "monver.h"

/* global variables */
U8      myInst;
U8      myEntId;
short   bDebug = 0;                  /* prints each packet sent/received */
int     maxports = 0;

char configFile[256] = "MON.cfg";
static char *usage = "Usage: moncfg [ -b <board number> ] [-f <filename>]\n";
char buf[80];


/* Handy macro to get & range check a long, print a warning if its out
 * of range, and copy the value (possibly cast to a smaller type)
 * to "dest" if its in range; assumes "dest" is previously set to
 * default value. "FROM" version scans from a known position in file
 * (rather than start of file) and returns current position.
 */
#define GET_CHECK_INT( name, min, max, dest, cast )         \
{                                                           \
    if (getcfgval(name, CFGT_INT, &temp, 0) > 0) {          \
        if ( temp < min || temp > max ) {                   \
        printf( "Warning: %s [%ld] out of range, setting to default [%d]\n",\
                name, temp, dest );                         \
        } else {                                            \
            dest = (cast) temp;                             \
        }                                                   \
    }                                                       \
}

#define GET_CHECK_INT_FROM( name, min, max, dest, cast, from, next )    \
{                                                                       \
    if( (next = getcfgval(name, CFGT_INT, &temp, from)) > 0) {          \
        if ( temp < min || temp > max ) {                               \
            printf( "Warning: %s [%ld] out of range, setting to default [%d]\n",\
                name, temp, dest );                                     \
        } else {                                                        \
            dest = (cast) temp;                                         \
        }                                                               \
    }                                                                   \
}

/* Handy macro to scan the config file for a timer value and set the
 * enabled/disabled status based on the timer value (0 == disabled).
 * Assumes timer value is set to default before calling.
 */
#define GET_TIMER( name, timer )                            \
{                                                           \
  if (getcfgval(name, CFGT_INT, &temp, 0) > 0)              \
    timer.value = (U16) temp;                               \
  timer.enable = (timer.value != 0) ? 1 : 0;                \
}

#define GET_TIMER_FROM( name, timer, from, next )               \
{                                                               \
  if( (next = getcfgval(name, CFGT_INT, &temp, from)) > 0)      \
    timer.value = (U16) temp;                                   \
  timer.enable = (timer.value != 0) ? 1 : 0;                    \
}
/*
* doMonGenCfg
*
* Configures monitor general parameters
*/
void doMonGenCfg( U8 board )
{
    MonGenCfg          gencfg;
    MON_STATUS         status;
    U32                temp, last, next;

    last = 0;                   /* Last line we read */

    MonInitGenCfg(board, &gencfg);

    if (getcfgval("RECV_FISU", CFGT_BOOL, (void *) &temp, last) > 0 )
        gencfg.recvFisu = (Bool) temp;

    if (getcfgval("RECV_LSSU", CFGT_BOOL, (void *) &temp, last) > 0 )
        gencfg.recvLssu = (Bool) temp;

    if (getcfgval("MTP2HDR", CFGT_BOOL, (void *) &temp, last) > 0 )
        gencfg.mtp2Hdr = (Bool) temp;

    GET_CHECK_INT_FROM( "NUM_BUFS",      50,   10000, gencfg.numBuffers,     U16, last, next );
    GET_CHECK_INT_FROM( "SIZE_BUFS",    300,   10000, gencfg.sizeBuffers,    U16, last, next );

    GET_TIMER_FROM( "TIMER_TIND", gencfg.tInd, last, next );

    /* generate the config request */
    status = MonSetGenCfg( board, &gencfg );
    switch( status )
    {
        case MON_SUCCESS:
            break;

        case MON_TIMEOUT:
            printf( "Monitor General Cfg: No response from board %d\n", board );
            break;

        case MON_BOARD:
            printf( "Monitor General Cfg: Invalid board %d\n", board );
            break;

        case MON_HANDLE:
            printf( "Monitor General Cfg: Board %d not initialized\n", board );
            break;

        case MON_RESPONSE:
            printf( "Monitor General Cfg: Board %d returned unexpected response\n", board );
            break;

        default:
            printf( "Monitor General Cfg Error: [%d]\n", status);
            break;
    }


} /* doMonGenCfg */

/*
* doMonLinkCfg
*
* Configures monitor links
*/
void doMonLinkCfg( U8 board )
{
    MonLinkCfg          linkcfg;
    MON_STATUS          status;
    int                 i;
    U32                 temp, last, next;
    U16                 linkNum;
    U8                  linkType;

    last = 0;                   /* Last line we read */
    for( i = 0; ; i++ )
    {
        /* Get link number */
        if ((next = getcfgval("LINK", CFGT_INT, (void *) &temp, last)) <= 0)
        {
            break;                      /* no more links */
        }
        linkNum = (short) temp;

        if (getcfgval("LINK_TYPE", CFGT_STRING, (void *) buf, last) > 0 )
        {
            if ((strcmp(buf, "CCITT") == 0) || (strcmp(buf, "ITU") == 0))
                linkType = MON_LINK_ITU;
            else if ((strcmp(buf, "CCITT_EXT") == 0) || (strcmp(buf, "ITU_EXT") == 0))
                linkType = MON_LINK_ITU_EXT;
            else if (strcmp(buf, "ANSI") == 0)
                linkType = MON_LINK_ANSI;
            else if (strcmp(buf, "ANSI_EXT") == 0)
                linkType = MON_LINK_ANSI_EXT;
            else
            {
                linkType = MON_LINK_ANSI;
                printf( "MTP2 Link Config: Link %d LINK_TYPE [%s] unknown - defaulted to ANSI\n",
                    linkNum, buf );
            }
        }

        status = MonInitLinkCfg(board, &linkcfg, linkNum, linkType);
        if( status != MON_SUCCESS )
        {
            /* should never happen, we've already validated link type */
            printf( "Monitor Link Config: link %d config initialization failed\n",
                linkNum, buf );
            continue;
        }

        GET_CHECK_INT_FROM( "PORT_NUM",          1,     (U32)maxports, linkcfg.portNum,     U16, last, next );
        GET_CHECK_INT_FROM( "MAX_FRAME",        64,    1024, linkcfg.maxFrameLen, U16, last, next );
        GET_CHECK_INT_FROM( "NUM_BUFFERS",      50,   10000, linkcfg.numBuffers,  U16, last, next );
        GET_CHECK_INT_FROM( "SIZE_BUFFERS",    250,    500, linkcfg.sizeBuffers, U16, last, next );

        if (getcfgval("PORT_TYPE", CFGT_STRING, (void *) buf, last) > 0 )
            {
            if (strcmp(buf, "TDM") == 0)
                linkcfg.portType = MON_PTYPE_TDM;
            else if (strcmp(buf, "SERIAL") == 0)
                linkcfg.portType = MON_PTYPE_SERIAL;
            else
                {
                printf( "Monitor Link Port Type [%s] unknown - defaulted to TDM\n",
                    buf );
                linkcfg.portType = MON_PTYPE_TDM;
                }
            }


        linkcfg.dataEnc = DAT_NRZ;
        if (getcfgval("DATA_ENC", CFGT_STRING, (void *) &buf, last) > 0 )
            {
            if (strcmp(buf, "NRZ") == 0)
                linkcfg.dataEnc = DAT_NRZ;      /* NRZ */
            else if (strcmp(buf, "NRZI") == 0)
                linkcfg.dataEnc = DAT_NRZI;     /* NRZI */
            else
                {
                printf( "Monitor Link Config: DATA_ENC [%s] unknown - defaulted to NRZ\n",
                    buf );
                }
            }

        /* generate the config request */
        status = MonSetLinkCfg( board, &linkcfg );
        switch( status )
        {
            case MON_SUCCESS:
                break;

            case MON_TIMEOUT:
                printf( "Monitor Link Cfg: No response from board %d\n", board );
                break;

            case MON_BOARD:
                printf( "Monitor Link Cfg: Invalid board %d\n", board );
                break;

            case MON_HANDLE:
                printf( "Monitor Link Cfg: Board %d not initialized\n", board );
                break;

            case MON_RESPONSE:
                printf( "Monitor Link Cfg: Board %d returned unexpected response\n", board );
                break;

            default:
                printf( "Monitor Link Cfg: Link %d caused error [%d]\n",
                    linkcfg.linkNum, status);
                break;
        }

        /* skip up to end of section */
        next = getcfgval("END", CFGT_INT, &temp, last);
        last = next;

    }   /* end for loop */

} /* doMonLinkCfg */

/*
 * main
 *
 * parses parameters (board number, config file name) from command line,
 * calls appropriate config routines.
 */
int main( int argc, char *argv[] )
{

    MON_STATUS         status;
    unsigned short      board = 1;                  /* default board is 1 */
    int                 i;
    char                *p;
    char                sbuf[CPSTRSIZE], *vstr = sbuf; /* dummy for old cpi_init */
    S32                 cps[MAXBOARD+1];                   /* type of CP for each possible CP # */

    printf("\n\n%s: sample monitor configuration application version %s %s\n\n",
            argv[0], MONCFG_VERSION, __DATE__);

    /* parse command line for config file name & board # */
    for(i = 1; i < argc; i++)
    {
		p = argv[i];
        if (*p == '-')
        {
            switch (*++p)
            {
            case 'f' :     /* configuration file name */
                i++;
                if (argv[i] == NULL)
                {
                	printf( usage );
                	exit ( 1 );
                }

                strncpy( configFile, argv[i], sizeof( configFile ) - 1 );
                configFile[sizeof( configFile ) - 1] = '\0';  /* ensure null term */
				break;

			case 'b' :    /* Set Board number */
				i++;
                if((argv[i] == NULL) || (sscanf(argv[i], "%hi", &board) != 1) || (board < 1) || (board > 8) )
                {
                    printf( usage );
                    exit( 1 );
                }
				break;

            case 'd' :
                bDebug = 1;              /* i/o packet dumps */
                CDebug = 1;             /* config scanner debugging */
                break;

			default:
                printf( usage );
                exit( 1 );
                break;
			}       /* end switch */
        }
        else
        {
            printf( usage );
            exit( 1 );
		}

	}       /* end for */

    /* Initialize the monitor management library */
    status = MonMgmtInit( (U8)board, MON_ENT, 0 );
    if( status != MON_SUCCESS )
    {
        printf( "Unable to open management library, error %d\n", status );
		exit(1);
	}

    /* Get the board type */
    cpi_init( 0, &vstr );

    /* determine how many boards are present in the system */
    if (cpi_get_resources( MAXBOARD, cps ) != CPI_SUCCESS)
    {
        printf( "Could not get resources from driver\n" );

        exit( 1 );
    }
    switch(cps[board])
    {
        case TX_4000 :
            maxports = 32;
            break;
        default:
            maxports = 128;             /* for TX5000 family */
            break;
    }

    /* initialize config file scanner library */
    if( initcfg( configFile ) != 0 )
    {
        /* initcfg() prints error message if it fails */
        exit( 1 );
    }

    doMonGenCfg( (U8)board );
    doMonLinkCfg( (U8)board );

    /* allow libraries to clean up and terminate */
    endcfg();
    (void) MonMgmtTerm( (U8)board );               /* ignore return code */

    exit (0);           /* Exit code success */
}
