/* Monitor management test application
 *
 *  Copyright 2006 - 2009 Dialogic Corporation.  All Rights Reserved
 *
 * Sample monitor management application; sends commands to monitor task on Tx series
 * processor and waits for response (status and optional data, such as entity
 * status or statistics).
 *
 * To run:
 *
 *  monmgr  [-b <board>]
 *
 *      <board>     Optional Board number to open (defaults to 1)
 *
 * User Commands:
 *
 *      LINK    <link number>  START | STOP
 *      TRACE   trace_level
 *      GET     GEN
 *      GET     LINK    <link number> | *
 *      GET     FILTERS <link number> | *
 *      STATS   GEN
 *      STATS   LINK    <link number> [ RESET ]
 *      FILTER  <link nmber> DPC|OPC|SIO|CLEAR <filter value>
 *      ?       [<command name>]        ! help all commands or specific command
 *      BOARD   <board number>          ! attach to new board
 *      Q                               ! quit
 */


#include <stdio.h>
#include <ctype.h>
#ifdef UNIX
#define getch	getchar
#else
#include <conio.h>          /* For getch() call */
#endif
#include <string.h>
#include <stdlib.h>
#include "ss7type.h"
#include "monmapi.h"
#include "cfgscan.h"                    /* for point code display and scanning utilities */
#include "monver.h"


void linkCtrl( U32 *parms );
void cfgReq( U32 *parms );
void statsReq( U32 *parms );
void filterCtrl( U32 *parms );
void traceCtrl( U32 *parms );
void HelpCommand( U32 *parms );
void setCP( U32 *parms );
void quit( U32 *parms );

char * filterNames[] =
{
    "???",
    "MON_FILTER_SIO",
    "MON_FILTER_DPC",
    "MON_FILTER_OPC"
};



/* lookup tables for translating names into values and vice versa */
struct lookupEnt {
    char            *name;
    U32   value;
};

/* parameter name tables */
#define MON_START_LINK  1
#define MON_STOP_LINK   2
struct lookupEnt LinkCtrlLookupTbl[] = {	/* link control action values */
    { "START", MON_START_LINK },
    { "STOP",  MON_STOP_LINK },
	{ 0,  0 }/* end of table */
};

#define MON_GEN_CFG     1
#define MON_LINK_CFG    2
#define MON_FILTER      3
struct lookupEnt CfgReqLookupTbl[] = {   /* MTP3 alarms control action values  */
    { "GEN", MON_GEN_CFG },
    { "LINK", MON_LINK_CFG },
    { "FILTERS", MON_FILTER },
	{ 0,	0 }/* end of table */
};

#define MON_GEN_STATS   1
#define MON_LINK_STATS  2
struct lookupEnt StatsReqLookupTbl[] = {    /* MTP3 flow control action values  */
    { "GEN", MON_GEN_STATS },
    { "LINK", MON_LINK_STATS },
	{ 0,	0 }/* end of table */
};

#define ALL_LINKS   69

#define MON_FILTER_CLEAR_VAL 255
struct lookupEnt FilterCtrlLookupTbl[] = {  /* status command values */
    { "SIO", MON_FILTER_SIO },
    { "DPC", MON_FILTER_DPC },
    { "OPC", MON_FILTER_OPC },
    { "CLEAR", MON_FILTER_CLEAR_VAL },
	{ 0,	0 }/* end of table */
};

struct lookupEnt ResetLookupTbl[] = {     /* stats reset flag values */
    { "RESET", 255 },
	{ 0,  0 }/* end of table */
};

struct lookupEnt CmdLookupTbl[] =
{
      { "LINK",       1 },
      { "GET",        2 },
      { "STATS",      3 },
      { "FILTER",     4 },
      { "TRACE",      5 },
      { "BOARD",      6 },
      { "Q",          7},
      { "ALL",        0 },
      { 0,  0 }
}; /* end of table */

static char linkCtrlHelp[]        =   "    Syntax: LINK <link number> <action>\n\n"
"     where:\n"
"            <action>      [START | STOP]\n\n";

static char cfgReqHelp[]        =   "    Syntax: GET [GEN | LINK | FILTERS] <link number>\n\n";

static char statsReqHelp[]        =   "    Syntax: STATS <entity> [<link number>]\n\n"
"     where:\n"
"            <entity>           [LINK | GEN ]\n"
"            <link number>      not used for GEN\n\n";

static char filterCtrlHelp[]        =   "    Syntax: FILTER <link number> <entity> <filter value>\n\n"
"     where:\n"
"            <link number>      not used for GEN\n"
"            <entity>           [DPC | OPC | SIO | CLEAR]\n\n";

static char traceCtrlHelp[]     =   "    Syntax: TRACE <trace level>\n\n";

static char boardHelp[]        =   "    Syntax: BOARD <new target board number>\n\n";

static char quitHelp[]              =   "    Syntax: Q \n\n"
"    Quit mtpmgr application\n\n\n";


static char allHelp[]               =
"    LINK       Link Control\n"
"    GET        Configuration Retrieval\n"
"    STATS      Link | General statistics\n"
"    FILTER     Set link filter\n"
"    TRACE      Monitor trace Control\n"
"    BOARD      Changes target board\n"
"    Q          Quit monmgr application\n\n";

static char* helpTbl[] =
{
    allHelp,
    linkCtrlHelp,
    cfgReqHelp,
    statsReqHelp,
    filterCtrlHelp,
    traceCtrlHelp,
    boardHelp,
    quitHelp
};
/* structure for command parse tables */
#define MAX_PARMS	10

struct CmdSpec {
	char				*command;
	struct lookupEnt	*parmTbl[MAX_PARMS];
    void                (*func)(U32 *);
};

/* Help Command */
struct CmdSpec  helpCmd =
{
	"?",               /* command name */
	    { CmdLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
	    HelpCommand         /* handler function */
};

/* monitor MAPI commands */
struct CmdSpec	linkCtrlCmd =
{
	"LINK",				/* command name */
	{ 0, LinkCtrlLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0 },
	linkCtrl			/* handler function */
};

struct CmdSpec  cfgReqCmd =
{
    "GET",                              /* command name */
    { CfgReqLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },    /* parameter lookups */
    cfgReq                              /* handler function */
};

struct CmdSpec  statsReqCmd =
{
	"STATS",			   				/* command name */
    { StatsReqLookupTbl, ResetLookupTbl, ResetLookupTbl, 0, 0, 0, 0, 0, 0, 0 },  /* parameter lookups */
	statsReq							/* handler function */
};

struct CmdSpec  filterCtrlCmd =
{
    "FILTER",                             /* command name */
    { 0, FilterCtrlLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0 },   /* parameter lookups */
    filterCtrl                            /* handler function */
};

struct CmdSpec  traceCtrlCmd =
{
    "TRACE",                              /* command name */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },    /* parameter lookups */
    traceCtrl                             /* handler function */
};

struct CmdSpec	boardCmd =
{
	"BOARD",			   							/* command name */
	{  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },				/* parameter lookups */
	setCP											/* handler function */
};

struct CmdSpec	quitCmd =
{
	"Q",			   								/* command name */
		{  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },				/* parameter lookups */
		quit											/* handler function */
};

struct CmdSpec *CmdTable[] = {
		&linkCtrlCmd,
        &cfgReqCmd,
        &statsReqCmd,
        &filterCtrlCmd,
        &traceCtrlCmd,
		&boardCmd,
		&quitCmd,
        &helpCmd,
    0
};


/* Misc. globals */
char	valStr[20];
U8      cp = 1;                 /* current board number attached to */
short   quittinTime = 0;        /* set to 1 when user 'quit' command entered */

char    msg[200];               /* Used for error messages */
U8      bDebug = FALSE;         /* Turn debug (additional config display) on or not */
char    *pErrstr;               /* Used for return of error text */

U8      pcDispFmt = SS7_PCDISP_DFLT;    /* desired display format for point codes */

/* Forward prototypes */
void StatusAllLinks (void);

/* getname
 *
 * Serches lookup table for a value match and returns a pointer to
 * the name that corresponds to that value.  Returns a pointer to a
 * global string containing ascii representation of that value if
 * no matching value found in table.
 */
char *getname( struct lookupEnt *tbl, U32 val )
{
	while( tbl->name != 0 )
	{		/* while not end of table */
		if( tbl->value == val )
		{
			/* match found */
			return( tbl->name );
		}
		tbl++;
	}	/* end while */

	/* falling through loop means search failed - convert number to ascii */
    sprintf( valStr, "%ld", val );
	return( valStr );

}	/* end getname() */


/* getval
 *
 * Serches lookup table for a string match and generates the resulting
 * value if found.  Returns 1 if string found, 0 if not found.
 */
short getval( struct lookupEnt *tbl, char *s, U32 *val )
{
	while( tbl->name != 0 )
	{		/* while not end of table */
		if( strcmp( tbl->name, s ) == 0 )
		{
			/* match found */
			*val = tbl->value;
			return( 1 );
		}
		tbl++;
	}	/* end while */

	/* falling through loop means search failed */
	return( 0 );
}	/* end getval() */

/*
 * linkCtrl command handler; command syntax:
 *      LINK <link number>  START | STOP
 */
void linkCtrl( U32 *parms )
{
    MON_STATUS status;
    S16         linkno;
    U8          action;

	linkno = (S16) parms[0];
	action = (U8) parms[1];

    if (action == MON_START_LINK)
    {
        status = MonStartLink(cp, linkno);

        if ( status != MON_SUCCESS) {
            printf("MonStartLink failed: return value = %d\n", status);
            return;
        }
    }
    else if (action == MON_STOP_LINK)
    {
        status = MonStopLink(cp, linkno);

        if ( status != MON_SUCCESS) {
            printf("MonStopLink failed: return value = %d\n", status);
            return;
        }
    }
    else
            printf("Valid actions are START and STOP\n");

    return;
}

/*
 * cfgReq command handler; command syntax:
 *      GET [GEN | LINK | FILTER] <link number>
 */
void cfgReq( U32 *parms )
{
    MON_STATUS status;
    U16         linkNum;
    U8          entity;

    entity = (U8) parms[0];
    linkNum = (U16) parms[1];

    switch(entity)
    {
        case  MON_GEN_CFG:
        {
            MonGenCfg   cfg;

            status = MonGetGenCfg(cp, &cfg);
            if ( status != MON_SUCCESS) {
                printf("MonGetGenCfg failed: return value = %d\n", status);
                return;
            }

            printf("Monitor General Configuration\n");
            printf("-----------------------------\n");
            printf("recvLssu = %s\n", cfg.recvLssu ? "TRUE " : "FALSE");
            printf("recvFisu = %s\n", cfg.recvFisu ? "TRUE " : "FALSE");
            printf("mtp2Hdr = %s\n", cfg.mtp2Hdr ? "TRUE " : "FALSE");
            printf("numBuffers = %d\n", cfg.numBuffers);
            printf("sizeBuffers = %d\n", cfg.sizeBuffers);
            printf("tInd.enabled = %d\n", cfg.tInd.enable);
            printf("tInd.value = %d\n", cfg.tInd.value);
        }
        break;

        case  MON_LINK_CFG:
        {
            MonLinkCfg   cfg;
			char *linkType;

            status = MonGetLinkCfg(cp, &cfg, linkNum);
            if ( status != MON_SUCCESS) {
                printf("MonGetLinkCfg failed: return value = %d\n", status);
                return;
            }

            printf("Link %d Configuration\n", linkNum);
            printf("--------------------\n");
			if ( cfg.linkType == MON_LINK_ANSI)
				linkType = "ANSI";
			else if ( cfg.linkType == MON_LINK_ANSI_EXT)
				linkType = "ANSI_EXT";
			else if ( cfg.linkType == MON_LINK_ITU)
				linkType = "ITU";
			else if ( cfg.linkType == MON_LINK_ITU_EXT)
				linkType = "ITU_EXT";
			else
				linkType = "UNK";
            printf("linkType = %s\n", linkType);
            printf("portNum = %d\n", cfg.portNum);
            printf("portType = %s\n", (cfg.portType == MON_PTYPE_TDM) ? "TDM " : "SERIAL");
            printf("dataEnc = %s\n", (cfg.dataEnc == DAT_NRZ) ? "DAT_NRZ" : "DAT_NRZI");
            printf("maxFrameLen = %d\n", cfg.maxFrameLen);
            printf("numBuffers = %d\n", cfg.numBuffers);
            printf("sizeBuffers = %d\n", cfg.sizeBuffers);
        }
        break;

        case  MON_FILTER:
        {
            MonFilterList   list;
            int             filter;

            status = MonGetFilters(cp, &list, linkNum);
            if ( status != MON_SUCCESS) {
                printf("MonGetLinkCfg failed: return value = %d\n", status);
                return;
            }

            if (list.numActive == 0) {
                printf("There are no active filters on link %d\n", linkNum);
                return;
            }

            for (filter = 0; filter < list.numActive; filter++)
            {
                printf("Filter %d on link %d\n", filter, linkNum);
                printf("------------------\n");
                printf("type = %s\n", filterNames[(list.filter[filter].type)]);
                printf("value = %d\n", list.filter[filter].value);
            }
        }
        break;


        default:
            printf("Invalid type - use GEN | LINK | FILTER\n");
            break;
    }
}

/*
 * statsReq command handler; command syntax:
 *      STATS   LINK    <link number> [ RESET ]
 *      STATS   GEN
 */
void statsReq( U32 *parms )
{
    MON_STATUS status;
    S16         linkNum;
    Bool        reset;
    S16         entity;

    entity = (S16) parms[0];

    if (entity == MON_GEN_CFG)
    {
        MonGenStats     stats;

        reset = (U8) parms[1];

        status = MonGetGenStats(cp, &stats, reset);
        if ( status != MON_SUCCESS) {
            printf("MonGetGenStats failed: return value = %d\n", status);
            return;
        }

        printf("Monitor General Stats\n");
        printf("---------------------\n");
        printf("numAppCongDrop = %d\n", stats.numAppCongDrop);

    }
    else if (entity == MON_LINK_CFG)
    {
        MonLinkStats     stats;

        reset = (U8) parms[2];
        linkNum = (S16) parms[1];

        status = MonGetLinkStats(cp, &stats, linkNum, reset);
        if ( status != MON_SUCCESS) {
            printf("MonGetLinkStats failed for link %d: return value = %d\n", linkNum, status);
            return;
        }

        printf("Monitor Link %d Stats\n", linkNum);
        printf("--------------------\n");
        printf("numMsuRx = %d\n", stats.numMsuRx);
        printf("numLssuRx = %d\n", stats.numLssuRx);
        printf("numFisuRx = %d\n", stats.numFisuRx);
        printf("numRxFiltered = %d\n", stats.numRxFiltered);
        printf("numSUERMFail = %d\n", stats.numSUERMFail);

    }
    else
    {
        printf("Please enter GEN or LINK for the entity\n");
    }
}

/*
 * filterCtrl command handler; command syntax:
 *      FILTER <link number> [DPC|OPC|SIO|CLEAR] <value>
 */
void filterCtrl( U32 *parms )
{
    MON_STATUS  status;
    S16         linkNum;
    U32         filterVal;
    U8          entity;

    linkNum = (S16) parms[0];
    entity = (U8) parms[1];
    filterVal = (U32) parms[2];

    if (entity == MON_FILTER_CLEAR_VAL)
    {
        status = MonClearFilters(cp, linkNum);
        if ( status != MON_SUCCESS) {
            printf("MonClearFilters failed: return value = %d\n", status);
            return;
        }
    }
    else if ((entity == MON_FILTER_SIO) || (entity == MON_FILTER_OPC) || (entity == MON_FILTER_DPC))
    {
        MonFilterDef    filter;

        filter.type = entity;
        filter.value = filterVal;

        status = MonSetFilter(cp, &filter, linkNum);
        if ( status != MON_SUCCESS) {
            printf("MonSetFilter failed: return value = %d\n", status);
            return;
        }
    }
}


/*
 * traceCtrl command handler; command syntax:
 *      TRACE <trace level>
 */
void traceCtrl( U32 *parms )
{
    MON_STATUS  status;
    U16         traceLevel;

    traceLevel = (U16) parms[0];

    status = MonSetTrace(cp, traceLevel);
    if ( status != MON_SUCCESS) {
        printf("MonSetTrace failed: return value = %d\n", status);
        return;
    }
}



void HelpCommand( U32 *parms )
{
	printf( "%s", helpTbl[*parms] );
}

void setCP( U32 *parms )
{
    U8       newcp, oldcp;
    MON_STATUS status;

    oldcp = cp;
    newcp = (U8) parms[0];

	if( newcp < 1 || newcp > 8 )
	{
		printf( "CP must be in range 1..8\n" );
	}

    if (newcp == oldcp)
    {
        printf("Already on that board\n");
        return;
    }

    /* open/init the Monitor mgmt API for new board*/
    status = MonMgmtInit( newcp, MONMGR_ENT, 0  );
    if( status != MON_SUCCESS )
	{
        printf( "Board %d monMgmtInit failed [%d]\n", newcp, status );
        return;
	}
	else
	{
		/* successful, close old cp & set new cp as the current cp */
        status = MonMgmtTerm( oldcp );
		cp = newcp;
	}

}

void quit( U32 *parms )
{
	quittinTime = 1;
}


/* processCmd
 *
 * Searches command table for command match, converts parameters to
 * array of shorts, and invokes the appropriate command handler.
 */
void processCmd( char *tokens[], short cnt )
{
	short 				i, j;
	short				found;
    U32       params[MAX_PARMS];  /* numeric parameters array */
	struct lookupEnt	*lp;
    short               retval;
    unsigned int        scan_param;

	/* first search command table for command match */
	for( i = 0; CmdTable[i] != 0; i++ )
	{
		if( strcmp( tokens[0], CmdTable[i]->command ) == 0 )
		{
			break;		/* command matched */
		}
	}	/* end for */

	/* if no match found display error message and return */
	if( CmdTable[i] == 0 )
	{
		printf("Unrecognized command: %s\n", tokens[0] );
		return;
	}

    /* convert remaining tokens (i.e., parameters) to long integers,
     * using lookup tables where specified to convert text strings
	 * (names) into numbers.  Note that all parameters are positional.
	 */
	for( j = 0; j < MAX_PARMS; j++ )
		params[j] = 0;					/* first initialize to defaults */

	for( j = 1; j < cnt; j++, lp++ )
	{
		lp = CmdTable[i]->parmTbl[j - 1];
		if( lp )
		{	/* lookup table for this param ? */
			found = getval( lp, tokens[j], &params[j - 1] );
			if( found )
			{
				continue;				/* parameter succesfully converted */
			}
        }

        /* no lookup table or lookup failed - see if this is a point code in x.y.z format
         * or a number in hex or decimal
         */
        retval = CfgScanPc( tokens[j], &params[j - 1], pcDispFmt );
        if( retval == 0 )
        {
            /* successful conversion */
            continue;
        }
        else if( retval == -2 )
        {
            printf("Point code: %s out of range\n", tokens[j] );
			return;
        }
        else

        /* no lookup table - try to convert as a number */
        if( (retval = sscanf( tokens[j], "%u", &scan_param )) != 1 )
        {
            /* If it's not a number, see if it's an asterisk */
            if (*tokens[j] == '*')
                {
                params[j-1] = ALL_LINKS;            /* Unique value to denote '*' */
                continue;
                }

			/* conversion failed */
			printf("Unrecognized parameter: %s\n", tokens[j] );
			return;
		}
        params[j - 1] = (U32)scan_param;
	}	/* end for */

	/* call command handler function */

	if( CmdTable[i]->func != 0 )
	{
		CmdTable[i]->func( params );
	}
}

/* tokenize
 *
 * Takes input line and creates an array of pointers to individual
 * tokens in the input line.  First separator after each token in
 * input line is replaced with a null so that each token becomes
 * a null terminated string.
 */
short tokenize( char *buf, char *tokens[] )
{
	char	*p = buf;
	char	*endp;
	short   cnt;

	/* initialize token pointers to all null */
	for( cnt = 0; cnt <= MAX_PARMS; cnt++ )
		tokens[cnt] = 0;

	/* begin scanning for tokens */
	cnt = 0;
	while( *p != 0 && cnt <= MAX_PARMS )
	{
		/* skip leading space or tabs */
		while( *p == ' ' || *p == '\t' )
			p++;

		/* beginning of token (or end of line) */
		if( *p != 0 )
		{
			/* scan for end of token, converting to upper case as we go */
			endp = p;
			while( *endp != ' ' && *endp != '\t' && *endp != 0 )
			{
				if( islower( *endp ) )
				{
					*endp = 'A' + (*endp - 'a');	/* convert to upper */
				}
				endp++;
			}

			/* now have a begin & end of token */
			tokens[cnt++] = p;
			if( *endp == 0 )
			{
				p = endp;	/* end of input - terminate scan */
			}
			else
			{
				*endp = 0;		/* null terminate token */
				p = &endp[1];	/* start next scan at after null */
			}
		}	/* end if *p != 0 */
	}	/* end while *p != 0 .... */

	return( cnt );
}

void usage( char *pName )
{
    printf("\n usage: %s [ -b <board>] \n", pName );
    printf("\n\t<board>\tboard number to attach to [default 1]\n");
	exit(1);
}

int main( int argc, char *argv[] )
{

	char	buffer[132];
	char	*tokens[MAX_PARMS +1];
	short	cnt;
	short	i;
	char	*p;
    MON_STATUS status;
    unsigned int    scan_cp;

    printf("\n\n%s: sample monitor management application version %s %s\n\n",
            argv[0], MONMGR_VERSION, __DATE__);

	/* parse command line for interrupt & cp # */
	for(i = 1; i < argc; i++)
	{
		p = argv[i];
		if (*p == '-')
		{
			switch (*++p)
			{
			case 'b' :    /* Set Board number */
				i++;
                if((argv[i] == NULL) || (sscanf(argv[i], "%u", &scan_cp) != 1) || (scan_cp < 1) || (scan_cp > 8) )
				{
                    usage( argv[0] );
				}
                cp = (U8)scan_cp;
				break;

            case 'd' :      /* Set debug mode on */
                bDebug = TRUE;
                break;

			default:
                usage( argv[0] );
				break;
			}	/* end switch */
		}
		else
		{
            usage( argv[0] );
		}

	}	/* end for */

    /* open/init the Monitor mgmt API */
    status = MonMgmtInit( (U8) cp, MONMGR_ENT, 0  );
    if( status != MON_SUCCESS )
	{
        printf( "monMgmtInit failed [%d]\n", status );
		exit(1);
	}

    printf( "monmgr[%-1u]>", cp );

    while( (quittinTime == 0) && fgets( buffer, 132, stdin ) != 0 )
	{
        while ( (strlen(buffer) > 0) && (buffer[strlen(buffer) - 1] < ' ') )
        {
            buffer[strlen(buffer) - 1] = '\0';  /* remove any trailing non-printable characters */
        }

		cnt = tokenize( buffer, tokens );
		if( cnt > 0 )
			processCmd( tokens, cnt );

            printf( "monmgr[%-1u]>", cp );
	}	/* end while */

    status = MonMgmtTerm( (U8) cp );

}   /* end main() */

