/* rmgcomm.c
 *
 * Redundancy manager (RMG) sockets communication handler
 *
 * This module provides the communication interface to the mate node for the rest
 * of the RMG process.
 *
 * This module is composed of a receiver thread, whose job is to continuously read
 * from the socket and post any received messages to the controller FSM module, and
 * utility routines for initializing the interface, sending packets, and closing
 * down the interface.
 *
 * Connectionless datagram (UDP) sockets are employed.  All socket I/O is blocking
 * (thus the use of a separate thread for receiving).  The cleanup routine closes
 * down the scoket, which also has the effect of killing the receiver thread.
 *
 * The sending module is responsible for timing for responses and issuing any desired
 * retransmissions.
 */

#include <string.h>

#include "osdep.h"
#include "ctadef.h"
#include "ctaerr.h"
#include "nmstypes.h"
#include "rmgdefs.h"

/* Constants and defaults */
#ifdef UNIX
#define INVALID_SOCKET	   -1
#endif

/* Globals */
static TSockAddrIn   Mate_addr;  /* IP address/port for mate RMG process */
static TSockAddrIn   My_addr;    /* this RMG's IP address/port */

static TSocket  My_socket = INVALID_SOCKET; /* current open socket handle */
static short    Recv_shutdown = 0;          /* notifies receiver thread that kill is intentional */
static char     emsg[ 132 ];                /* buffer for constructing alarm messages */

/* forward function declarations */
void RmgCommRecvThread( void *pParam );

/*
 * RmgCommInit
 *
 * This routine performs comm module initialization during startup, reset, and
 * possibly during error recovery.  It creates and binds the socket and starts up
 * the receiver thread.
 *
 * Parameters:  none
 *
 * Returns:
 *
 *  0           Successful
 *  -1          Failed (alarm message has already been logged)
 */
short RmgCommInit( void )
{
  TThrId CommRcvThrId;
  THostent       *pHostent;      /* ptr to struct returned by gethostbyname() */

#ifdef WIN_JUNK /*???????????????????????????????????????????????????????? WIN32 */
  WSADATA         wsadata;        /* sockets version data */
  int             wsaret;         /* sockets return code */

  /*  intialize sockets API is first */
  wsaret = WSAStartup( WINSOCK_VERSION, &wsadata );
    if( wsaret != 0 )
    {
        printf( "ERROR Unable to initialize sockets API, error %d\n ", wsaret );
        return( -1 );
    }
    else if ( (LOBYTE( wsadata.wVersion ) != 1) || HIBYTE( wsadata.wVersion ) != 1 )
    {
        /* incompatible SOCKETS API version */
        printf( "ERROR: Unsupported sockets API version (%d.%d)\n",
                 HIBYTE( wsadata.wVersion ), LOBYTE( wsadata.wVersion ) );
        return( -1 );
    }
#endif /* ?????????????????????????????????????????????????????????????? WIN32 */

    /* next create a UDP datagram socket */
    if((SOCKET_OPEN( AF_INET, SOCK_DGRAM, 0, My_socket )) == RET_FAILED )
      {
        /* fatal error */
        printf( "ERROR: Unable to create socket, error %d", osGetLastError() );
        return( -1 );
      }

    /* convert port to network byte order and store in local address */
    My_addr.sin_port = htons( RmgLocalPort );
    My_addr.sin_family = AF_INET;
    My_addr.sin_addr.s_addr = htonl(INADDR_ANY);  /* any IP address on local host */

    /* Construct IP address of mate RMG process */
    Mate_addr.sin_port = htons( RmgRemotePort );
    Mate_addr.sin_family = AF_INET;
    if( strlen( RmgMateNodeName ) == 0 )
      {
	/* no remote node address, assume mate RMG process is local */
	Mate_addr.sin_addr.s_addr = htonl( INADDR_LOOPBACK );
      }
    else
      {
        /* remote address not empty, see if its a valid "nnn.nnn.nnn.nnn" format */
        Mate_addr.sin_addr.s_addr = inet_addr( RmgMateNodeName );
        if ( Mate_addr.sin_addr.s_addr == INADDR_NONE )
	  {
            /* not a dotted IP address, see if its a host name we can find */
	    if ((osGetHostByName( RmgMateNodeName, pHostent )) != RET_SUCCESS)
	      {
                printf( "ERROR: invalid mate IP address/host name [%s]\n",
                        RmgMateNodeName );
                return( -1 );
	      }

            /* got host info for far end, use 1st IP address */
            memcpy( (char *) &(Mate_addr.sin_addr), pHostent->h_addr,
                    pHostent->h_length);
	  }
      }

    /* Bind the socket to our configured port */
    if(SOCKET_BIND(My_socket, (TSockAddr*)&My_addr, sizeof(My_addr))
      != RET_SUCCESS)
      {
	printf( "ERROR: Unable to bind socket, error %d", osGetLastError() );
	return( -1 );
      }

    /* create the receiver thread */
    if((THREAD_CREATE(0, (TPtrThrFunc) RmgCommRecvThread, (TPtrThrArg) &My_socket, 0, &CommRcvThrId)) != RET_SUCCESS)
    {
      printf( "ERROR: Unable to create receiver thread, error %ld", osGetLastError() );
      return( -1 );
    }

    /* successful initialization */
    return( 0 );
}                               /* of RmgCommInit */


/*
 * RmgCommCleanup
 *
 * This routine performs comm module clean up for use during reset or error
 * recovery, when re-initialization and continued operation is desired.  It
 * shuts down the socket to kill the receiver thread and destroys the socket.
 *
 * Parameters:  none
 *
 * Returns:     void
 *
 */
   void RmgCommCleanup( void )
{
    /* if the socket exists, close it, that will kill receiver thread */
    if( My_socket != INVALID_SOCKET )
    {
        Recv_shutdown = 1;      /* notify receiver thread that this is intentional */
        SOCKET_CLOSE(My_socket);
        My_socket = INVALID_SOCKET;
    }
}                               /* of RmgCommCleanup */


/*
 * RmgCommRecvThread
 *
 * This is the entry point to the receiver thread; its job is to loop continuously
 * doing blocking reads on the socket and posting any received packets to the SCTL
 * module in the form of "data indication" events, until a socket error is received
 * (indicating that the socket has been shut down and its time to stop).
 *
 * Parameters:
 *
 *  pParam      pointer to socket handle to use
 *
 * Returns:     Always 0
 *
 */
void RmgCommRecvThread( void *pParam )
{
    TSocket            *pSocket;            /* pointer to socket to receive on */
    RmgCtrlMsg          msgBuf;             /* buffer for incoming messages */
    struct sockaddr_in  fromAddr;           /* packet sender's address */
    int                 fromLen;            /* length of socket address structure */
    int                 recvLen;            /* length of received message */
    U8                  done = 0;           /* flag to terminate while loop */
    static U8           dupNodeWarn = 0;    /* if non-zero, duplicate node number
                                             * warning has already been issued */
    U32                 value;              /* value providing additional information about received message */

    /* initialize local variables */
    pSocket = (TSocket *) pParam;

    while( ! done )
    {
        fromLen = sizeof( struct sockaddr_in );
        if ((SOCKET_RECVFROM( *pSocket, (char *)&msgBuf, RMG_MAX_PKTLEN, 0,
                              (TSockAddr *)&fromAddr, &fromLen, recvLen )) != RET_SUCCESS)
        {   /* Read operation failed. Check if this is severe error or not: */
            if(osGetLastError() == ERR_MSGSIZE)
            {
                fprintf( stderr, "WARNING: message from %s too long [len=%d], discarded\n",
                         inet_ntoa( fromAddr.sin_addr ), recvLen );
            }
            else
            {   /* ECONNREFUSED returned on Unix if other side isn't up - ignore for now */
#ifdef UNIX
                if (osGetLastError() != ECONNREFUSED)
#else
                if ( (osGetLastError( ) != WSAECONNREFUSED) &&
                     (osGetLastError( ) != WSAECONNRESET) )
#endif
                {   /* severe error, terminate thread */
                    if( ! Recv_shutdown )
                    {   /* not a shutdown or socket closed event - fatal error */
                        printf( "Socket receive failed %ld\n", osGetLastError( ) );
                    }
                    if (!(RmgRequireQuit))
                    {
                        done = 1;           /* done with comm thread */
                    }
                }
            }
        }
        else
        {   /* received a UDP packet; if its valid, post the event to the controller FSM's queue */
            if( msgBuf.signature != RMG_CTLR_SIGN )
            {
                sprintf( emsg, "WARNING: RMG message w/invalid signature ignored [0x%lx]", msgBuf.signature );
                ctaLogTrace( CtlrFsmCmdHd, CTA_TRACE_SEVERITY_WARNING, CTA_TRACETAG_STRING,
                             emsg, strlen( emsg ) );
                continue;
            }

            /* check the sender's node ID, warn of duplicate (1 time only) */
            RmgMateNodeNum = msgBuf.senderId;
            if( (RmgMateNodeNum == RmgNodeNum) && (dupNodeWarn == 0) )
            {
                sprintf( emsg, "WARNING: Dupliate node numbers detected [%d]", RmgNodeNum );
                ctaLogTrace ( CtlrFsmCmdHd, CTA_TRACE_SEVERITY_WARNING, CTA_TRACETAG_STRING,
                              emsg, strlen( emsg ) );
                dupNodeWarn = 1;        /* don't do this warning every recv'd msg */
            }

            /* maintain error information related to any messages with a non-zero reason code */
            if (msgBuf.reason != 0)
            {
                RmgError.rmgMessage.errorCount++;
                RmgError.rmgMessage.lastError = msgBuf.msgtype;     /* use received message type as last error */
                RmgError.rmgMessage.lastInfo = (U32)msgBuf.reason;
            }

            /* setup value providing additional information about this message */
            value = (DWORD)msgBuf.networkStatus;                    /* low-order byte is always mate's last reported network status */
            if (msgBuf.extracting)
            {   /* mate board is extracting... pass indication in value (priority override) */
                value |= RMG_VAL_MATE_EXTRACTING;
            }

            RmgSendEvent( CtlrFsmCmdHd, msgBuf.msgtype, value );
        }   /* end else ... got a packet */
    }   /* end while ! done */

    THREAD_EXIT( );
}   /* end of RmgCommRecvThread */


/*
 * RmgCommSend
 *
 * Routine to send a packet to the mate RMG process.  The message type
 * and network status are passed as parameters, the rest of the message
 * structure is filled in from constants or global data.
 *
 * Parameters:
 *
 *  msgType     message type to send
 *  msgFlags    flags providing additional control information (RMG_MSG_xxx)
 *
 * Returns:     void
 */
void RmgCommSend( DWORD msgType, U8 reason, U32 msgFlags )
{
    RmgCtrlMsg      msgBuf;         /* buffer for outgoing message */
    char            error_message[128];
    U32             status = 0;

    if (!(RmgStandAlone))
    {   /* not in standalone mode so send message to mate node */

        /* setup for message-type-specific handling */
        switch (msgType)
        {
            case RMG_EV_ACTV_RQST:      /* Activation request message */
            case RMG_EV_SWOVER_RQST:    /* Switchover request message (primary->backup) */
            case RMG_EV_TKOVER_RQST:    /* Takeover request message (backup->primary) */
            case RMG_EV_NETUP_RQST:     /* NetworkUp request message [optional Takeover] (backup->primary) */
            case RMG_EV_NETDOWN_RQST:   /* NetworkDown request message [optional Switchover] (primary->backup) */
                /* track whether this is the initial attempt to send request */
                if (msgFlags & RMG_MSG_REQ_RETRY)
                {   /* set the global used to differentiate initial request from retries */
                    RmgRequestRetry = 1;    /* there has been at least one retry to send this request */
                }
                else
                {   /* clear the global used to differentiate initial request from retries */
                    RmgRequestRetry = 0;    /* there has been no retry to send this request */
                }
                break;

            case RMG_EV_HEARTBEAT:      /* Heartbeat message (primary->backup) */
            case RMG_EV_FAILURE:        /* Primary board failed message */
            case RMG_EV_ACTV_CFM:       /* Activation confirm response */
            case RMG_EV_ACTV_REJ:       /* Activation reject response */
            case RMG_EV_SWOVER_CFM:     /* Switchover confirm response */
            case RMG_EV_SWOVER_REJ:     /* Switchover reject response */
            case RMG_EV_TKOVER_CFM:     /* Takeover confirm response */
            case RMG_EV_TKOVER_REJ:     /* Takeover reject response */
            case RMG_EV_NETUP_CFM:      /* NetworkUp confirm response [Takeover accepted] */
            case RMG_EV_NETUP_REJ:      /* NetworkUp reject response [Takeover rejected] */
            case RMG_EV_NETDOWN_CFM:    /* NetworkDown confirm response [Switchover accepted] */
            case RMG_EV_NETDOWN_REJ:    /* NetworkDown reject response [Switchover rejected] */
                /* no message-type-specific handling of these message types */
                break;

            default:
                sprintf( error_message, "Unknown message type [0x%X]\n",
                         msgType );
                LogError( "RmgCommSend", error_message );
                RmgError.commSend.errorCount++;
                RmgError.commSend.lastError = msgType;
                RmgError.commSend.lastInfo = msgFlags;
                status = 1;
        }

        if (status == 0)
        {   /* no error yet so construct and send message */
            memset( &msgBuf, 0, sizeof(RmgCtrlMsg) );
            msgBuf.signature        = RMG_CTLR_SIGN;
            msgBuf.msgtype          = msgType;
            msgBuf.protoVers        = RMG_PROTO_VERS;
            msgBuf.senderId         = (U8)RmgNodeNum;
            msgBuf.reason           = reason;               /* reason (for a rejection or other failure message) [RMG_REASON_xxx] */
            msgBuf.networkStatus    = RmgNetworkStatus;     /* always pass this node's current network status */
            if (msgFlags & RMG_MSG_REQ_EXTRACT)
            {   /* board is extracting (priority override) */
                msgBuf.extracting = 1;                      /* set indication that board is extracting (priority override) */
            }

            if (SOCKET_SENDTO( My_socket, (char *)&msgBuf, RMG_MAX_PKTLEN, 0,
                               (TSockAddr *)&Mate_addr, sizeof(TSockAddrIn) ) != RET_SUCCESS)
            {
                sprintf( error_message, "RMG Unable to send message [0x%X], error %ld\n",
                         msgType, osGetLastError( ) );
                LogError( "RmgCommSend", error_message );
                RmgError.commSend.errorCount++;
                RmgError.commSend.lastError = msgType;
                RmgError.commSend.lastInfo = msgFlags;
                status = 1;
            }
        }
    }
}
