/* rmgdefs.h
*
* Common constants, typedefs, structures, etc. available to all modules in the
* RMG process.
*/
#ifndef _RMGDEFS_H_
#define _RMGDEFS_H_

#include "ss7type.h"

/* Internal events passed between user interface/main routine and
 * redundancy controller state machine
 *
 *      Event Name          Event ID            Description
 *      -----------         --------            -----------  */
#define RMG_EV_SHUTDN       CTA_USER_EVENT(0x1) /* Request controller FSM to stop */
#define RMG_EV_HALT_CMD     CTA_USER_EVENT(0x2) /* Board halted manually by user  */
#define RMG_EV_LOAD_CMD     CTA_USER_EVENT(0x3) /* user requested board load  */
#define RMG_EV_SWOVER_CMD   CTA_USER_EVENT(0x4) /* user requested switchover */
#define RMG_EV_EXTRACT_CMD  CTA_USER_EVENT(0x5) /* ready for extraction */

/* Controller FSM timer events */
#define RMG_EV_T1           CTA_USER_EVENT(0xA) /* Timer T1 - response timer */
#define RMG_EV_T2           CTA_USER_EVENT(0xB) /* Timer T2 - deadman timer */
#define RMG_EV_T3           CTA_USER_EVENT(0xC) /* Timer T3 - heartbeat timer */
#define RMG_EV_T4           CTA_USER_EVENT(0xD) /* Timer T4 - board download timer */

/* Message types (double as event IDs) for messages passed between RMG
 * processes on the same or different nodes.
 */
#define RMG_EV_HEARTBEAT    CTA_USER_EVENT(0x11)  /* Heartbeat message (primary->backup) */
#define RMG_EV_FAILURE      CTA_USER_EVENT(0x12)  /* Primary board failed message */

#define RMG_EV_ACTV_RQST    CTA_USER_EVENT(0x21)  /* Activation request message */
#define RMG_EV_ACTV_CFM     CTA_USER_EVENT(0x22)  /* Activation confirm response */
#define RMG_EV_ACTV_REJ     CTA_USER_EVENT(0x23)  /* Activation reject response */

#define RMG_EV_SWOVER_RQST  CTA_USER_EVENT(0x31)  /* Switchover request message (primary->backup) */
#define RMG_EV_SWOVER_CFM   CTA_USER_EVENT(0x32)  /* Switchover confirm response */
#define RMG_EV_SWOVER_REJ   CTA_USER_EVENT(0x33)  /* Switchover reject response */

#define RMG_EV_TKOVER_RQST  CTA_USER_EVENT(0x41)  /* Takeover request message (backup->primary) */
#define RMG_EV_TKOVER_CFM   CTA_USER_EVENT(0x42)  /* Takeover confirm response */
#define RMG_EV_TKOVER_REJ   CTA_USER_EVENT(0x43)  /* Takeover reject response */

#define RMG_EV_NETUP_RQST   CTA_USER_EVENT(0x51)  /* NetworkUp request message [optional Takeover] (backup->primary) */
#define RMG_EV_NETUP_CFM    CTA_USER_EVENT(0x52)  /* NetworkUp confirm response [Takeover accepted] */
#define RMG_EV_NETUP_REJ    CTA_USER_EVENT(0x53)  /* NetworkUp reject response [Takeover rejected] */

#define RMG_EV_NETDOWN_RQST CTA_USER_EVENT(0x61)  /* NetworkDown request message [optional Switchover] (primary->backup) */
#define RMG_EV_NETDOWN_CFM  CTA_USER_EVENT(0x62)  /* NetworkDown confirm response [Switchover accepted] */
#define RMG_EV_NETDOWN_REJ  CTA_USER_EVENT(0x63)  /* NetworkDown reject response [Switchover rejected] */

/* internal events */
#define EVENT_STATE_CHANGE  CTA_USER_EVENT(0x80)  /* change of RMG's state */
#define EVENT_ERROR_MESSAGE CTA_USER_EVENT(0x81)  /* error message */
#define EVENT_PROGRESS      CTA_USER_EVENT(0x82)  /* tracing progress */
#define EVENT_FREEZE        CTA_USER_EVENT(0x83)  /* freeze all further RMG event processing */

#define EVENT_ALL           CTA_USER_EVENT(0x90)  /* special indicator for 'all events' */
#define EVENT_LIST_DONE     CTA_USER_EVENT(0x91)  /* special indicator for 'done with list' */

/* Messages between nodes (instances of RMG processes)
 *
 * The RMG inter-node protocol consists of messages exchanged between the 2
 * RMG processes to exchange status information regarding each other.  The
 * connection between processes is either a LAN running UDP/IP or, for a
 * single node configuration, a local UDP/IP connection between 2 processes
 * on the same node.
 *
 * Two type of communication types are employed.  "Heartbeat" messages are
 * unsolicited, unidirectional (i.e., there's no response) messages
 * sent from the active node to the standby node to announce that it is still
 * up and running. All other messages are either "request" messages (messages
 * requiring a response) or the responses to request messages.  Responses
 * may be either "confirms" (positive responses) or "rejects" (negative
 * responses).
 *
 * All messages contains the following fields.
 *
 * Signature            Four byte string, to make it less likely that a stray
 *                      message received on UDP port would be confused as a
 *                      RMG message
 *
 * MessageType          the message type, identifies the function of the message;
 *                      this field is 32 bits so that it may also double as a
 *                      CT Access event ID without further translation.
 *
 * ProtoVersion         the protocol version employed by the message sender,
 *                      to allow for version upgrades without requiring both
 *                      nodes to be brought down and restarted simultaneously,
 *                      resulting in a service outage.
 *
 * SenderId             the node number of the sender (1 or 2)
 *
 * The protocol is simple and assumes reliable, in-order delivery of packets by
 * the transport layer - this is typically an erroneous assumption for UDP, but
 * is sufficient for this sample application/prototype. A more robust production
 * quality protocol would include sequence numbers for detection of missing or
 * out-of-sequence packets, as well as a mechanism for re-synching after one
 * side or the other reboots.
 */

/* Message structure for all messages - currently 16 bytes, max. */

typedef struct
{
    U32 signature;                      /* always 'RMGR' to help detect stray msgs */
    U32 msgtype;                        /* message type value */
    U8  protoVers;                      /* sender's protocol version */
    U8  senderId;                       /* sender's node ID (usually 1 for node with precedence in race conditions; 2 for node without precendence */
    U8  reason;                         /* reason (for a rejection or other failure message) [RMG_REASON_xxx] */
    U8  networkStatus;                  /* node's network status at time message was sent */
    U8  extracting;                     /* indication that board is extracting (priority override) [0 = not extracting; 1 = extracting] */
    U8  spare[3];                       /* reserved for future use - code to zeros */
} RmgCtrlMsg;

#define RMG_MAX_PKTLEN   (sizeof( RmgCtrlMsg ))  /* length of all messages */

/* signature and current protocol version */
#define RMG_CTLR_SIGN       0x524D4752  /*RMGR*/
#define RMG_PROTO_VERS      1

/* possible values for reason field (for a rejection or other failure message) */
#define RMG_REASON_SUCCESS          0
#define RMG_REASON_NETWORK_DOWN     1   /* node's network is not up */
#define RMG_REASON_OPERATOR_CMD     2   /* node's role (primary or backup) is under the control of an operator command */
#define RMG_REASON_REMAIN_PRIMARY   3   /* node is to remain primary */
#define RMG_REASON_NODE_PRECEDENCE  4   /* node's role determined using node number precedence */
#define RMG_REASON_LOADING          5   /* node is currently loading its board */
#define RMG_REASON_OOS              6   /* node is currently out of service */
#define RMG_REASON_EXTRACTING       7   /* node's board is being extracted */
#define RMG_REASON_EXTRACTED        8   /* node's board has been extracted */

/* default UDP port for RMG process */
#define RMG_PORT_DEFAULT    1700

/* message control flags */
#define RMG_MSG_REQ_RETRY   (1<<0)      /* message is a retry to send a request */
#define RMG_MSG_REQ_EXTRACT (1<<1)      /* board is extracting (priority override) */

/* additional information provided in value parameter for received RMG messages */
#define RMG_VAL_MATE_EXTRACTING (1<<16) /* set = mate node is extracting */

/* template for state handling functions */
typedef short (* pRMGC_STATE_FUNC) (DWORD, DWORD);

/* RMG states */
#define RMGC_STATE_INITIAL      0       /* Initial state, waiting for board status */
#define RMGC_STATE_STARTING     1       /* Starting (Actv request pending) */
#define RMGC_STATE_START_UP     2       /* Starting when network came up (waiting for activation response before proceeding) */
#define RMGC_STATE_START_DOWN   3       /* Starting when network went down (waiting for activation response before proceeding) */
#define RMGC_STATE_LOADING      4       /* Board download in progress */
#define RMGC_STATE_ACTIVE       5       /* Active state */
#define RMGC_STATE_ACTIVE_SWRQ  6       /* Active (Switchover request pending) */
#define RMGC_STATE_ACTIVE_DOWN  7       /* Active (NetDown [optional switchover] request pending) */
#define RMGC_STATE_BACKUP       8       /* Backup state */
#define RMGC_STATE_TKOV_RQST    9       /* Backup (Takeover request pending) */
#define RMGC_STATE_TKOV_UP      10      /* Backup (NetUp [optional takeover] request pending) */
#define RMGC_STATE_OOS          11      /* Board dead (can't be reloaded) or manually halted */
#define RMGC_STATE_STANDALONE   12      /* Board not redundant - just monitor for failures */
#define RMGC_STATE_EXTRACTING   13      /* Board extraction in progress */
#define RMGC_STATE_EXTRACTED    14      /* Board has been extracted */
#define RMGC_STATE_FROZEN       15      /* Further RMG actions frozen (to debug current status) */

#define RMGC_LAST_STATE         RMGC_STATE_FROZEN
#define RMGC_MAX_STATES         ( RMGC_LAST_STATE + 1 )

typedef struct
{
    char                stateName[14];  /* state name for easy printing */
    pRMGC_STATE_FUNC    pHandler;       /* pointer to handler function */
} RMGC_STATE_DESC;

/* network connectivity status */
#define RMG_NETWORK_UNKNOWN     0       /* network connectivity status has not been reported */
#define RMG_NETWORK_UP          1       /* network connectivity has been reported as UP */
#define RMG_NETWORK_DOWN        2       /* network connectivity has been reported as DOWN */
#define RMG_MAX_NETWORK_STATUS  2


/* information maintained for each tracked error */
typedef struct
{
    U32     errorCount;                 /* count of errors (of given type) that have occurred */
    U32     lastError;                  /* last error code (of given type) encountered */
    U32     lastInfo;                   /* information value associated with last error */
} RMG_ERROR_INFO;

/* set of globally tracked controller errors */
typedef struct
{
    RMG_ERROR_INFO  rmgMessage;         /* error reported in RMG message received from mate */
    RMG_ERROR_INFO  startTimer;         /* error in function RmgcStartTimer */
    RMG_ERROR_INFO  cancelTimer;        /* error in function RmgcCancelTimer */
    RMG_ERROR_INFO  staleTimer;         /* timer fired should not have been running */
    RMG_ERROR_INFO  commSend;           /* error in function RmgCommSend */
    RMG_ERROR_INFO  hmiLoadBoard;       /* error in function hmiLoadBoard */
    RMG_ERROR_INFO  hmiHaltBoard;       /* error in function hmiHaltBoard */
    RMG_ERROR_INFO  hmiStop;            /* error in function hmiStop */
    RMG_ERROR_INFO  hmiStart;           /* error in function hmiStart */
    RMG_ERROR_INFO  hmiBackup;          /* error in function hmiBackup */
    RMG_ERROR_INFO  hmiPrimary;         /* error in function hmiPrimary */
    RMG_ERROR_INFO  hmiStandalone;      /* error in function hmiStandalone */
    RMG_ERROR_INFO  hmiStatus;          /* error in function hmiStatusReq */
} RMG_ERROR_SET;

/* Globals - The following global values are available for read access by any machine;
 *           Write access is always restricted to a single machine.
 */
extern RMGC_STATE_DESC RmgcStates[];  /* description of each RMG state */
extern short    RmgNodeNum;           /* Node number of this node (1..2) */
extern short    RmgMateNodeNum;       /* Node number of mate node (1..2) */
extern short    RmgBoardNum;          /* Board number to use on this node (1..8) */
extern char     RmgMateNodeName[80];  /* mate node IP address or name */
extern U16      RmgLocalPort;         /* local UDP port to use */
extern U16      RmgRemotePort;        /* mate's UDP port */

extern U8       RmgState;               /* Current board state */
extern U8       RmgStandAlone;          /* set = input parameters imply we are standalone */
extern U8       RmgNetworkStatus;       /* network connectivity status */
extern U8       RmgRequestRetry;        /* set after an initial attempt has been made to send any request to mate */

extern short    RmgTraceLvl;            /* Current configured trace level (0..1) */
extern short    RmgRequireQuit;         /* Whether to require die command to exit */

extern U32  RmgTxHeartbeats;          /* (SCTL) Number of heartbeat messages transmitted... */
                                        /* ...since last becoming active */
extern U32  RmgRxHeartbeats;          /* (SCTL) Number of heartbeat messages received... */
                                        /* ...since last becoming standby */

extern CTAQUEUEHD  CtlrFsmQueue;        /* controller FSM queue handle */
extern CTAHD       CtlrFsmAsyncHd;      /* context handle for async events */
extern CTAHD       CtlrFsmCmdHd;        /* context handle for sync commands */
extern CTAQUEUEHD  UserIfQueue;         /* user interface queue handle */
extern CTAHD       UserIfCmdHd;         /* context handle for sync commands */

extern RMG_ERROR_SET    RmgError;       /* set of error types that are tracked globally */

/* prototypes for public functions */
extern short RmgCommInit( void );
extern void RmgCommCleanup( void );
extern void RmgCommSend( DWORD msgType, U8 reason, U32 msgFlags );
extern void RmgCmdProc( void );
extern void RmgCfsmThread( void *pParam );
extern void RmgSendEvent( CTAHD ctaHd, DWORD id, DWORD value );
extern TMTimerId RmgStartTimer( DWORD msecs, DWORD event_id, BOOL cont);
extern void RmgStopTimer( TMTimerId hTimer );

/* prototypes for public logging functions */
extern char    *LogNetStatus    ( U8 status );
extern char    *LogNameState    ( U8 state );
extern char    *LogNameEvent    ( DWORD event );
extern char    *LogNameEventID  ( DWORD event );
extern DWORD    LogEvent        ( DWORD event );
extern void     LogStateChange  ( U8 state );
extern void     LogError        ( char *function, char *error_desc );
extern void     LogProgress     ( char *function, char *prog_desc );
extern DWORD    LogGetEventId   ( char *idname, DWORD prev_event );
extern void     LogSetEvent     ( DWORD event, U8 log_stdout, U8 log_memory,
                                  U8 trigger_stop, U8 freeze );
extern void     LogShowConfig   ( void );
extern void     LogShowCurrent  ( void );
extern void     LogShowCapture  ( void );
extern void     LogClearCapture ( void );

#endif                                  /* _RMGDEFS_H_ */
