/* rmglog.c
 *
 * Redundancy manager (RMG) logging functions
 *
 * This module provides an interface to a set of in-memory logs.
 * Developers can use these logs to trace the progress of the RMG process.
 *
 * The prototypes for the externally available logging functions
 * are of the form
 */
#include <stdio.h>
#include <time.h>
#include <string.h>

#include "osdep.h"
#include "ctadef.h"
#include "ctaerr.h"
#include "nmstypes.h"
#include "ss7type.h"
#include "rmgdefs.h"
#include "hmiapi.h"

/* Private Structures */

typedef struct __event_info
{
    DWORD       event;          /* event number */
    char        idname[16];     /* unique event identification name */
    U8          log_stdout;     /* whether event is logged to standard out */
    U8          log_memory;     /* whether event is placed in memory log */
    U8          trigger_stop;   /* whether event triggers logging to stop */
    U8          freeze;         /* whether event freezes all further event processing */
    char        name[32];       /* full name of event */
} EVENT_INFO;

#define LOG_FUNC_SZ     32      /* size of function name */
#define LOG_DESC_SZ     80      /* size of string descibing error */

typedef struct __log_entry
{
    DWORD       event;              /* event number */
    U8          state;              /* STATE_CHANGE: state changing to */
    U8          avail[3];
    time_t      logtime;            /* time when entry occurred */
    char        func[LOG_FUNC_SZ];  /* ERROR_MESSAGE: function detecting error */
    char        desc[LOG_DESC_SZ];  /* string describing event */
} LOG_ENTRY;

#define LOG_COUNT   256         /* number of entries in each log */

/* Private Globals */

EVENT_INFO logEventInfo[] =
{
    /*  event                   idname          stdout  memory  trigger freeze  name
     *  ======================  ==============  ======  ======  ======= ======  ====================     */
    {   HMI_EVN_CONNECTED,      "CONNECTED",    0,      1,      0,      0,      "Board Connected"        },
    {   HMI_EVN_ISOLATED,       "ISOLATED",     0,      1,      0,      0,      "Board Isolated"         },
    {   HMI_EVN_NOWPRIMARY,     "PRIMARY",      0,      1,      0,      0,      "Now Primary"            },
    {   HMI_EVN_NOWBACKUP,      "BACKUP",       0,      1,      0,      0,      "Now Backup"             },
    {   HMI_EVN_TASKDEAD,       "TASKDEAD",     0,      1,      1,      0,      "Task Failed"            },
    {   HMI_EVN_BRDDEAD,        "BRDDEAD",      0,      1,      0,      0,      "Board Failed"           },
    {   HMI_EVN_HALTED,         "HALTED",       0,      1,      0,      0,      "Board Halted"           },
    {   HMI_EVN_STARTING,       "STARTING",     0,      1,      0,      0,      "Now Starting"           },
    {   HMI_EVN_CONFLICT,       "CONFLICT",     0,      1,      0,      0,      "Board Conflict"         },
    {   HMI_EVN_NOWSTANDALONE,  "STANDALONE",   0,      1,      0,      0,      "Now Standalone"         },
    {   HMI_EVN_LOADING,        "LOADING",      0,      1,      0,      0,      "Board Loading"          },
    {   HMI_EVN_SERVICE_DOWN,   "DOWN",         0,      1,      0,      0,      "HMI Stopped"            },
    {   HMI_EVN_NO_BOARD,       "NOBOARD",      0,      1,      0,      0,      "Board Not Present"      },
    {   HMI_EVN_INSERT,         "INSERT",       0,      1,      0,      0,      "Board Inserted"         },
    {   HMI_EVN_EXTRACT,        "EXTRACT",      0,      1,      0,      0,      "Board Extract Pend"     },
    {   HMI_EVN_STOP,           "STOP",         0,      1,      0,      0,      "Stop"                   },
    {   HMI_EVN_NETWORK_UP,     "NETWORKUP",    0,      1,      0,      0,      "Network Up"             },
    {   HMI_EVN_NETWORK_DOWN,   "NETWORKDOWN",  0,      1,      0,      0,      "Network Down"           },

    {   RMG_EV_SHUTDN,          "SHUTDN",       0,      1,      1,      0,      "Shutdown"               },
    {   RMG_EV_HALT_CMD,        "HALTCMD",      0,      1,      1,      0,      "Halt Cmd"               },
    {   RMG_EV_LOAD_CMD,        "LOADCMD",      0,      1,      0,      0,      "Load Cmd"               },
    {   RMG_EV_SWOVER_CMD,      "SWOVERCMD",    0,      1,      0,      0,      "Switch Cmd"             },
    {   RMG_EV_EXTRACT_CMD,     "EXTRACTCMD",   0,      1,      0,      0,      "Extract Cmd"            },

    {   RMG_EV_T1,              "T1",           0,      1,      0,      0,      "T1:Response timer"      },
    {   RMG_EV_T2,              "T2",           0,      1,      0,      0,      "T2:Deadman timer"       },
    {   RMG_EV_T3,              "T3",           0,      0,      0,      0,      "T3:Heartbeat timer"     },
    {   RMG_EV_T4,              "T4",           0,      1,      0,      0,      "T4:Download timer"      },

    {   RMG_EV_HEARTBEAT,       "HEARTBEAT",    0,      0,      0,      0,      "Mate Heartbeat Msg"     },
    {   RMG_EV_FAILURE,         "FAILURE",      0,      1,      0,      0,      "Mate Failed"            },

    {   RMG_EV_ACTV_RQST,       "ACTVRQST",     0,      1,      0,      0,      "Activation Rqst"        },
    {   RMG_EV_ACTV_CFM,        "ACTVCFM",      0,      1,      0,      0,      "Activation Cfm"         },
    {   RMG_EV_ACTV_REJ,        "ACTVREJ",      0,      1,      0,      0,      "Activation Rej"         },

    {   RMG_EV_SWOVER_RQST,     "SWOVERRQST",   0,      1,      0,      0,      "Switchover Rqst"        },
    {   RMG_EV_SWOVER_CFM,      "SWOVERCFM",    0,      1,      0,      0,      "Switchover Cfm"         },
    {   RMG_EV_SWOVER_REJ,      "SWOVERREJ",    0,      1,      0,      0,      "Switchover Rej"         },

    {   RMG_EV_TKOVER_RQST,     "TKOVERRQST",   0,      1,      0,      0,      "Takeover Rqst"          },
    {   RMG_EV_TKOVER_CFM,      "TKOVERCFM",    0,      1,      0,      0,      "Takeover Cfm"           },
    {   RMG_EV_TKOVER_REJ,      "TKOVERREJ",    0,      1,      0,      0,      "Takeover Rej"           },

    {   RMG_EV_NETUP_RQST,      "UPTKOVRQST",   0,      1,      0,      0,      "NetUpTakeover Rqst"     },
    {   RMG_EV_NETUP_CFM,       "UPTKOVCFM",    0,      1,      0,      0,      "NetUpTakeover Cfm"      },
    {   RMG_EV_NETUP_REJ,       "UPTKOVREJ",    0,      1,      0,      0,      "NetUpTakeover Rej"      },

    {   RMG_EV_NETDOWN_RQST,    "DNSWOVRQST",   0,      1,      0,      0,      "NetDownSwover Rqst"     },
    {   RMG_EV_NETDOWN_CFM,     "DNSWOVCFM",    0,      1,      0,      0,      "NetDownSwover Cfm"      },
    {   RMG_EV_NETDOWN_REJ,     "DNSWOVREJ",    0,      1,      0,      0,      "NetDownSwover Rej"      },

    {   EVENT_STATE_CHANGE,     "STATECHANGE",  0,      1,      0,      0,      "State Change"           },
    {   EVENT_ERROR_MESSAGE,    "ERROR",        0,      1,      1,      0,      "ERROR!"                 },
    {   EVENT_PROGRESS,         "PROGRESS",     0,      1,      0,      0,      "Tracing Progress"       },
    {   EVENT_FREEZE,           "FREEZE",       0,      1,      1,      0,      "Freeze Events"          },

    {   EVENT_LIST_DONE,        "E O L",        0,      0,      0,      0,      "END OF EVENT LIST"      }
};

LOG_ENTRY   logCurrent[LOG_COUNT];  /* current log */
DWORD       logCurrent_next = 0;    /* index to next entry in current log */
LOG_ENTRY   logCapture[LOG_COUNT];  /* copy of captured log [first error (or trigger event)] */
DWORD       logCapture_event = 0;   /* event that triggered capture */

char *logFlagName[] =
{
    "   -   ",
    "  YES  "
};

char *logNetStatus[] =
{
    "UNKNOWN",  /* network connectivity status has not been reported */
    "UP",       /* network connectivity has been reported as UP */
    "DOWN"      /* network connectivity has been reported as DOWN */
};

/* Local Function Prototypes */
void LogEntry( DWORD event, U8 state, char *function, char *error_desc );
void LogShowEntry( LOG_ENTRY *entry );


/*----------------------------------------------------------------------------*
 * external logging functions
 *----------------------------------------------------------------------------*/

/*
 * LogNetStatus
 *
 * This routine returns a pointer to a static string buffer containing the
 * printable name of the specified network status.
 *
 * Parameters:
 *
 *  status      Network status whose name is to be returned
 *
 * Returns:     ptr to static buffer, suitable for use directly in sprintf
 *
 */
char *LogNetStatus( U8 status )
{
    static char invalidStatusName[32];
    char        *pName;

    /* make sure status value is in range */
    if( status > RMG_MAX_NETWORK_STATUS )
    {
        sprintf( invalidStatusName, "Unknown Status[%d]", status );
        pName = invalidStatusName;
    }
    else
    {
        pName = logNetStatus[ status ];
    }

    return( pName );
}

/*
 * LogNameState
 *
 * This routine returns a pointer to a static string buffer containing the
 * printable name of the specified state, for ease of message logging.
 *
 * Parameters:
 *
 *  state       State whose name is to be returned
 *
 * Returns:     ptr to static buffer, suitable for use directly in sprintf
 *
 */
char *LogNameState( U8 state )
{
    static char invalidStateName[32];
    char        *pName;

    /* make sure state number in range */
    if( state >= RMGC_MAX_STATES )
    {
        sprintf( invalidStateName, "Unknown State[%d]", state );
        pName = invalidStateName;
    }
    else
    {
        pName = &RmgcStates[ state ].stateName[ 0 ];
    }

    return( pName );
}

/*
 * LogNameEvent
 *
 * This routine returns a pointer to a static string buffer containing the
 * printable name of the specified event, for ease of message logging.
 *
 * Parameters:
 *
 *  event       Event whose name is to be returned
 *
 * Returns:     ptr to static buffer, suitable for use directly in sprintf
 *
 */
char *LogNameEvent( DWORD event )
{
    static char invalidEventName[32];
    char        *pEvent = NULL;
    EVENT_INFO  *eventInfo = logEventInfo;

    while( (pEvent == NULL) && (eventInfo->event != EVENT_LIST_DONE) )
    {
        if (event == eventInfo->event)
            pEvent = eventInfo->name;
        else
            eventInfo++;
    }

    if (pEvent == NULL)
    {
        sprintf( invalidEventName, "Unknown [0x%lx]", event );
        pEvent = invalidEventName;
    }

    return( pEvent );
}

/*
 * LogNameEventID
 *
 * This routine returns a pointer to a static string buffer containing the
 * printable ID name of the specified event, for ease of message logging.
 *
 * Parameters:
 *
 *  event       Event whose name ID is to be returned
 *
 * Returns:     ptr to static buffer, suitable for use directly in sprintf
 *
 */
char *LogNameEventID( DWORD event )
{
    static char invalidEventName[32];
    char        *pEvent = NULL;
    EVENT_INFO  *eventInfo = logEventInfo;

    while( (pEvent == NULL) && (eventInfo->event != EVENT_LIST_DONE) )
    {
        if (event == eventInfo->event)
            pEvent = eventInfo->idname;
        else
            eventInfo++;
    }

    if (pEvent == NULL)
    {
        sprintf( invalidEventName, "0x%08X]", event );
        pEvent = invalidEventName;
    }

    return( pEvent );
}


/*
 * LogEvent
 *
 * This routine logs RMG events.  It usually returns the same event provided
 * as input, but if the event has been configured as a FREEZE event,
 * then EVENT_FREEZE is returned instead (causing RMG to freeze all further
 * event handling).
 *
 * Parameters:
 *
 *  event       Event to be logged
 *
 * Returns:     event to issue to RMG state machine (input event or FREEZE)
 *
 */
DWORD LogEvent( DWORD event )
{
    DWORD       return_event = event;       /* default: return input event as event to issue */
    DWORD       ret;
    HmStatsData hmStatsData;
    HmTskStats  *hmTskStats;
    short       j;
    char        *error_desc = NULL;
    EVENT_INFO  *eventInfo = logEventInfo;

    /* set error description for certain error events */
    switch( event )
    {
        case HMI_EVN_TASKDEAD:
            /* task failure - do a status command and try to figure out
             * which task failed
             */
            error_desc = "Unknown";      /* default if we can't find which task failed */
            ret = hmiStatusReq( UserIfCmdHd, &hmStatsData, 0 );
            if( ret == SUCCESS )
            {
                hmTskStats = &hmStatsData.taskStats[0];
                for( j = 0; j < hmStatsData.numTsks; j++ )
                {
                    if( hmTskStats->runState == HMRS_FAILED )
                    {
                        error_desc = (char*)hmTskStats->name;  /* just show first failed task */
                        break;
                    }
                    hmTskStats++;
                }
            }
            else
            {
                RmgError.hmiStatus.errorCount++;
                RmgError.hmiStatus.lastError = ret;
                RmgError.hmiStatus.lastInfo = __LINE__;
            }
            break;
    }

    /* add this event to the event log */
    LogEntry( event, RmgState, NULL, error_desc );

    /* determine whether to convert this event to the special FREEZE event */
    while (eventInfo->event != EVENT_LIST_DONE)
    {
        if (eventInfo->event == event)
        {
            if (eventInfo->freeze)
                return_event = EVENT_FREEZE;        /* change to special FREEZE event */
            break;
        }

        eventInfo++;
    }

    return( return_event );
}


/*
 * LogStateChange
 *
 * This routine makes all changes to the RMG state (logging all state changes).
 *
 * Parameters:
 *
 *   state - new state to change to
 *
 * Returns:     void
 *
 */
void LogStateChange( U8 state )
{
    LogEntry( EVENT_STATE_CHANGE, state, NULL, NULL );

    RmgState = state;
}


/*
 * LogError
 *
 * This routine logs any errors encountered by the RMG process.
 *
 * Parameters:
 *
 *   function   - name of function that encountered error
 *   error_desc - text description of error
 *
 * Returns:     void
 *
 */
void LogError( char *function, char *error_desc )
{
    LogEntry( EVENT_ERROR_MESSAGE, RmgState, function, error_desc );
}


/*
 * LogProgress
 *
 * This routine logs the progress of the RMG process.
 *
 * Parameters:
 *
 *   function  - name of function generating log entry
 *   prog_desc - description of activity at progress log point
 *
 * Returns:     void
 *
 */
void LogProgress( char *function, char *prog_desc )
{
    LogEntry( EVENT_PROGRESS, RmgState, function, prog_desc );
}


/*
 * LogGetEventId
 *
 * This routine returns an event that matches an event ID (or next event ID name)
 *
 * Parameters:
 *
 *   idname     - event identification name (output pointer if prev_event != 0)
 *   prev_event - 0 = no previous event ID (so idname is input)
 *                else = event of previously returned event ID (idname is output)
 *
 * Returns:     event number associated with idname
 *              ALSO returns idname (if prev_event != 0)
 */
DWORD LogGetEventId( char *idname, DWORD prev_event )
{
    EVENT_INFO  *eventInfo = logEventInfo;

    if (prev_event == 0)
    {   /* requesting event that matches idname */
        while (eventInfo->event != EVENT_LIST_DONE)
        {
            if (strcmp(eventInfo->idname, idname) == 0)
                return( eventInfo->event );

            eventInfo++;
        }
    }
    else
    {   /* previous event provided, so return info about next event */
        while (eventInfo->event != EVENT_LIST_DONE)
        {
            if (eventInfo->event == prev_event)
            {
                eventInfo++;            /* advance to next event entry */
                if (eventInfo->event == EVENT_LIST_DONE)
                    return( 0 );        /* no more events */

                strcpy( idname, eventInfo->idname );
                return( eventInfo->event );
            }

            eventInfo++;
        }
    }

    return( 0 );                        /* idname does not map to event (or no match on prev_event) */
}


/*
 * LogSetEvent
 *
 * This routine sets the handling of a given event (or all events).
 *
 * Parameters:
 *
 *   event - event to store logging control for (or EVENT_ALL = all events)
 *   log_stdout   - flag indicating whether event should be logged to stdout
 *   log_memory   - flag indicating whether event should be logged to memory
 *   trigger_stop - flag indicating whether event should trigger log capture
 *   freeze       - flag indicating whether event should freeze all further event processing
 *
 * Returns:     void
 *
 */
void LogSetEvent( DWORD event, U8 log_stdout, U8 log_memory, U8 trigger_stop, U8 freeze )
{
    EVENT_INFO  *eventInfo = logEventInfo;

    if (event == EVENT_ALL)
    {   /* setting control info for all events */
        while (eventInfo->event != EVENT_LIST_DONE)
        {
            eventInfo->log_stdout = log_stdout;
            eventInfo->log_memory = log_memory;
            eventInfo->trigger_stop = trigger_stop;
            eventInfo->freeze = freeze;

            eventInfo++;
        }
    }
    else
    {   /* setting control info for a single event */
        while (eventInfo->event != EVENT_LIST_DONE)
        {
            if (eventInfo->event == event)
            {
                eventInfo->log_stdout = log_stdout;
                eventInfo->log_memory = log_memory;
                eventInfo->trigger_stop = trigger_stop;
                eventInfo->freeze = freeze;
                break;
            }

            eventInfo++;
        }

        if (eventInfo->event == EVENT_LIST_DONE)
            printf( "LogSetEvent: unknown event [0x%X]\n", event );
    }
}


/*
 * LogShowConfig
 *
 * This routine shows how logging control for each event is configured.
 *
 * Parameters:  NONE
 *
 * Returns:     void
 *
 */
void LogShowConfig( void )
{
    EVENT_INFO  *eventInfo = logEventInfo;

    printf( "Event ID         Std Out Memory  Trigger Freeze  Name\n" );
    printf( "================ ======= ======= ======= ======= ===============================\n" );
    while (eventInfo->event != EVENT_LIST_DONE)
    {
        printf( "%-16.16s %s %s %s %s %s\n",
                eventInfo->idname,
                logFlagName[eventInfo->log_stdout],
                logFlagName[eventInfo->log_memory],
                logFlagName[eventInfo->trigger_stop],
                logFlagName[eventInfo->freeze],
                eventInfo->name );

        eventInfo++;
    }
}


/*
 * LogShowCurrent
 *
 * This routine outputs the current log to stdout.
 *
 * Parameters:  NONE
 *
 * Returns:     void
 *
 */
void LogShowCurrent( void )
{
    DWORD       wrap_index,current_index;

    wrap_index = logCurrent_next;       /* oldest entry in current log */
    current_index = wrap_index;

    /* find oldest non-zero entry */
    while (logCurrent[current_index].event == 0)
    {
        current_index++;
        if (current_index >= LOG_COUNT)
            current_index = 0;

        if (current_index == wrap_index)
        {
            printf( "No current log entries\n" );
            return;
        }
    }

    printf( "Timestamp      State        Event              Description\n" );
    printf( "============== ============ ================== ================================\n" );
    do
    {
        LogShowEntry( &logCurrent[current_index] );

        current_index++;
        if (current_index >= LOG_COUNT)
            current_index = 0;

    } while ( (current_index != wrap_index) && (logCurrent[current_index].event != 0) );

    if (logCurrent[current_index].event != 0)
        LogShowEntry( &logCurrent[wrap_index] );    /* finish by showing oldest entry */
}


/*
 * LogShowCapture
 *
 * This routine outputs the captured log to stdout.
 *
 * Parameters:  NONE
 *
 * Returns:     void
 *
 */
void LogShowCapture( void )
{
    DWORD       capture_index = 0;

    if (logCapture_event == 0)
    {
        printf( "No event log has been captured\n" );
        return;
    }

    printf( "Timestamp      State        Event              Description\n" );
    printf( "============== ============ ================== ================================\n" );
    do
    {
        LogShowEntry( &logCapture[capture_index] );

        capture_index++;

    } while ( (capture_index < LOG_COUNT) && (logCapture[capture_index].event != 0) );
}


/*
 * LogClearCapture
 *
 * This routine clears any captured log (allowing a new capture event to take place).
 *
 * Parameters:  NONE
 *
 * Returns:     void
 *
 */
void LogClearCapture( void )
{
    DWORD   i;

    logCapture_event = 0;

    for (i = 0; i < LOG_COUNT; i++)
        logCapture[i].event = 0;
}


/*----------------------------------------------------------------------------*
 * local logging functions
 *----------------------------------------------------------------------------*/

/*
 * LogTimestamp
 *
 * This routine formats the time provided into a standard timestamp string.
 *
 * Parameters:
 *
 *  logtime     time to be conveted
 *
 * Returns:     timestamp string
 *
 */
char *LogTimestamp( time_t logtime )
{
    static char outstring[32];
    struct tm *timestamp = localtime( &logtime );

    sprintf( outstring, "%02d/%02d %02d:%02d:%02d",
             timestamp->tm_mon+1,
             timestamp->tm_mday,
             timestamp->tm_hour,
             timestamp->tm_min,
             timestamp->tm_sec );

    return( outstring );
}

/*
 * LogStdOut
 *
 * This routine formats log information into an output string to stdout.
 *
 * Parameters:
 *
 *  logtime     time when log entry created
 *  event       Event to be logged
 *  state       state to be logged
 *  function    name of function generating log
 *  error_desc  description of log entry
 *
 * Returns:     void
 *
 */
void LogStdOut( time_t logtime, DWORD event, U8 state, char *function, char *error_desc )
{
    char    *timestamp = LogTimestamp( logtime );

    /* handle error events specifically (and all other events generically) */
    switch( event )
    {
        case HMI_EVN_TASKDEAD:
            /* task failure - failed task name is error_desc */
            printf( "%s Node: %d, Board %d: Task [%0.8s] failed\n",
                    timestamp, RmgNodeNum, RmgBoardNum, error_desc );
            break;

        /* for all others, log based on optional parameters */
        default:
            if ( (function == NULL) && (error_desc == NULL) )
                printf( "%s Node: %d, Board %d: %s\n",
                        timestamp, RmgNodeNum, RmgBoardNum, LogNameEvent( event ) );
            else if (error_desc == NULL)
                printf( "%s Node: %d, Board %d: Function:%s %s\n",
                        timestamp, RmgNodeNum, RmgBoardNum, function, LogNameEvent( event ) );
            else if (function == NULL)
                printf( "%s Node: %d, Board %d: %s [%s]\n",
                        timestamp, RmgNodeNum, RmgBoardNum, LogNameEvent( event ),
                        error_desc );
            else
                printf( "%s Node: %d, Board %d: Function:%s %s [%s]\n",
                        timestamp, RmgNodeNum, RmgBoardNum, function,
                        LogNameEvent( event ), error_desc );
    }
}

/*
 * LogEntry
 *
 * Make entries in logs indicated and store error log (if trigger event)
 *
 * Parameters:
 *
 *  event       Event to be logged
 *
 * Returns:     void
 *
 */
void LogEntry( DWORD event, U8 state, char *function, char *error_desc )
{
    EVENT_INFO  *eventInfo = logEventInfo;
    LOG_ENTRY   *logEntry;
    DWORD       wrap_index,current_index,capture_index;
    time_t      logtime = time( NULL );
    char        *timestamp;

    /* find event descriptor */
    while (eventInfo->event != EVENT_LIST_DONE)
    {
        if (event == eventInfo->event)
            break;

        eventInfo++;
    }

    if (eventInfo->event == EVENT_LIST_DONE)
    {
        timestamp = LogTimestamp( logtime );
        printf( "%s LogEntry: unknown event [0x%X]\n",
                timestamp, event );
    }
    else
    {   /* handle the log entry based on event info */
        if (eventInfo->log_stdout)
        {
            LogStdOut( logtime, event, state, function, error_desc );
        }

        if (eventInfo->log_memory)
        {
            logEntry = &logCurrent[logCurrent_next];
            logCurrent_next++;
            if (logCurrent_next >= LOG_COUNT)
                logCurrent_next = 0;       /* wrap */

            logEntry->logtime = logtime;
            logEntry->event = event;
            logEntry->state = state;
            if (function == NULL)
                logEntry->func[0] = 0;
            else
                strcpy( logEntry->func, function );
            if (error_desc == NULL)
                logEntry->desc[0] = 0;
            else
                strcpy( logEntry->desc, error_desc );
        }

        if ( (logCapture_event == 0) && (eventInfo->trigger_stop) )
        {   /* copy current log to the capture log since trigger event */
            logCapture_event = event;           /* remember event that triggered capture */

            wrap_index = logCurrent_next;       /* oldest entry in current log */
            current_index = wrap_index;

            /* find oldest non-zero entry */
            while (logCurrent[current_index].event == 0)
            {
                current_index++;
                if (current_index >= LOG_COUNT)
                    current_index = 0;

                if (current_index == wrap_index)
                    break;
            }

            if (logCurrent[current_index].event != 0)
            {   /* there are some entries to be captured */
                capture_index = 0;              /* captured log uses [0] as oldest entry */
                do
                {
                    memcpy( &logCapture[capture_index], &logCurrent[current_index], sizeof(LOG_ENTRY) );
                    capture_index++;
                    current_index++;
                    if (current_index >= LOG_COUNT)
                        current_index = 0;
                } while (current_index != wrap_index);
            }
        }
    }
}


/*
 * LogShowEntry
 *
 * Display the provided log entry
 *
 * Parameters:
 *
 *  entry       log entry to be displayed
 *
 * Returns:     void
 *
 */
void LogShowEntry( LOG_ENTRY *entry )
{
    char   *desc = entry->desc;

    printf( "%s %-12.12s %-18.18s %s\n",
            LogTimestamp( entry->logtime ),
            LogNameState( entry->state ),
            LogNameEvent( entry->event ),
            desc );
}
