/* Redundancy Manager (RMG) Process - Main routine and utilities
 *
 * This is the main routine for the redundancy manager process;  it includes
 * process initialization, command-line parsing, and the user interface
 * (keyboard) command processing loop. It also creates the redundancy
 * controller state machine and communications receiver worker threads.
 *
 * This file also contains declarations for globals and utility functions used
 * by the worker threads.
 */
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "osdep.h"
#include "ctadef.h"
#include "ctaerr.h"
#include "nmstypes.h"
#include "rmgdefs.h"
#include "hmiapi.h"

#define RMG_VERSION  "2.0"

/* Globals exported by MAIN to other threads */
short   RmgBoardNum = 1;          /* Board number to use on this node (1..8) */
short   RmgNodeNum = 1;           /* Node number of this node (1..2) */
short   RmgMateNodeNum = 2;       /* Node number of mate node (1..2) */
U16     RmgLocalPort = RMG_PORT_DEFAULT;    /* local UDP port */
U16     RmgRemotePort = RMG_PORT_DEFAULT;   /* mate's UDP port */
char    RmgMateNodeName[80] = ""; /* default is local */
short   RmgTraceLvl = 0;          /* Current configured trace level (0..1) */
short   RmgRequireQuit = 0;       /* Whether to require quit command to exit */

/* Private Globals */
static char        emsg[132];               /* buffer for constructing alarm msgs */

/* Service name/manager pair for ctaInitialize */
CTA_SERVICE_NAME    HmiServiceNames[] = { { "HMI", "HMIMGR" } };

/* Service list for ctaOpenServices */
CTA_SERVICE_DESC    HmiOpenSvcLst[] = {
    {{"HMI", "HMIMGR"}, {0}, {0}, {0}}
};

/* Service list for ctaCloseServices */
char    *HmiCloseSvcLst[] = { "HMI" };

/* This application uses multiple queues and CT access contexts.  The
 * controller FSM uses 2 contexts attached to a single queue: one context for
 * HMI async event notifications and a separate context for HMI synchronous
 * commands.  Its queue handle is global so that other threads - the user
 * interface and the comm threads - can pass events to the controller FSM
 * using the same queue.
 *
 * The main process (user interface) uses a separate context/queue for
 * issuing its own HMI synchronous commands; it does not handle the
 * HMI asynch events.
 */
CTAQUEUEHD  CtlrFsmQueue = 0;           /* controller FSM queue handle */
CTAHD       CtlrFsmAsyncHd = 0;         /* context handle for async events */
CTAHD       CtlrFsmCmdHd = 0;           /* context handle for sync commands */

CTAQUEUEHD  UserIfQueue = 0;            /* user interface queue handle */
CTAHD       UserIfCmdHd = 0;            /* context handle for sync commands */

/* Assign HMI service "channels" for each of the contexts just created */
#define RMG_ASYNC_CHAN      0           /* HMI channel for ctlr async events */
#define RMG_CTLRCMD_CHAN    1           /* HMI channel for ctlr commands */
#define RMG_UICMD_CHAN      2           /* HMI channel for user i/f commands */

#define MAX_PARMS   10                  /* max parameters for keyboard commands */

/* Table of HMI task states, for display purposes */
#define HMI_TASKSTATE_UNKNOWN   (HMRS_FAILED + 1)

char *HmiTaskStates[] =
{
    "Not Used",         /* 0 not used                */
    "Started",          /*  HMRS_STARTED        0x01 */
    "Backup",           /*  HMRS_BACKUP         0x02 */
    "Primary",          /*  HMRS_PRIMARY        0x03 */
    "Standalone",       /*  HMRS_STANDALONE     0x04 */
    "Shutdown",         /*  HMRS_SHUTDOWN       0x05 */
    "Failed",           /*  HMRS_FAILED         0x06 */
    "0x07",             /*  reserved for future 0x07 */
    "0x08",             /*  reserved for future 0x08 */
    "0x09",             /*  reserved for future 0x09 */
    "UNKNOWN"           /*  HMRS_UNKNOWN        0x0a (must be last entry in table as catch-all for out-of-range states) */
};

/* Table of HMI task states, for display purposes */
#define HMI_LINKSTATE_UNKNOWN   (HMLS_STANDALONE + 1)

char *HmiLinkStates[] =
{
    "Not Used",         /* 0 not used                */
    "Connected",        /*  HMLS_CONNECTED      0x01 */
    "Isolated",         /*  HMLS_ISOLATED       0x02 */
    "Standalone",       /*  HMLS_STANDALONE     0x03 */
    "Unknown"           /* must be last              */
};

/* Table of HMI network states, for display purposes */
#define HMI_NETSTATE_UNKNOWN    (HMNS_UNKNOWN + 1)

char *HmiNetStates[] =
{
    "Not Used",         /* 0 not used                */
    "Connected",        /*  HMNS_CONNECTED      0x01    Task has indicated network connectivity is up */
    "Isolated",         /*  HMNS_ISOLATED       0x02    Task has indicated network connectivity is down */
    "Not Reported",     /*  HMNS_UNKNOWN        0x03    No task has reported network connectivity status */
    "Unknown"           /* must be last              */
};

char *RmgErrorReasons[] =
{ /* 1234567890 ... limit reason text to 10 characters */
    "SUCCESS",          /*  RMG_REASON_SUCCESS */
    "NET_DOWN",         /*  RMG_REASON_NETWORK_DOWN     node's network is not up */
    "OPER_CMD",         /*  RMG_REASON_OPERATOR_CMD     node's role (primary or backup) is under the control of an operator command */
    "REMAIN_PRI",       /*  RMG_REASON_REMAIN_PRIMARY   node is to remain primary */
    "PRECEDENCE",       /*  RMG_REASON_NODE_PRECEDENCE  node's role determined using node number precedence */
    "LOADING",          /*  RMG_REASON_LOADING          node is currently loading its board */
    "OOS",              /*  RMG_REASON_OOS              node is currently out of service */
    "EXTRACTING",       /*  RMG_REASON_EXTRACTING       node's board is being extracted */
    "EXTRACTED"         /*  RMG_REASON_EXTRACTED        node's board has been extracted */
};

/* forward function references */
static short scanArgs( int argc, char *argv[] );
static void usage( char *pCmdName );
static DWORD initCtAccess( void );
static DWORD openHmiService( char *pCtxName, DWORD hmiChan, DWORD servType,
                             CTAQUEUEHD ctaQ, CTAHD *pCtaHd );
static void cleanupCtAccess( void );


/* tokenize
 *
 * Takes input line and creates an array of pointers to individual
 * tokens in the input line.  First separator after each token in
 * input line is replaced with a null so that each token becomes
 * a null terminated string.
 */
static short tokenize( char *buf, char *tokens[] )
{
	char	*p = buf;
	char	*endp;
	short   cnt;

	/* initialize token pointers to all null */
	for( cnt = 0; cnt <= MAX_PARMS; cnt++ )
		tokens[cnt] = 0;

	/* begin scanning for tokens */
	cnt = 0;
	while( *p != 0 && cnt <= MAX_PARMS )
	{
		/* skip leading space or tabs */
		while( *p == ' ' || *p == '\t' )
			p++;

		/* beginning of token (or end of line) */
		if( *p != 0 )
		{
			/* scan for end of token, converting to upper case as we go */
			endp = p;
			while( *endp != ' ' && *endp != '\t' && *endp != 0 )
			{
				if( islower( *endp ) )
				{
					*endp = 'A' + (*endp - 'a');	/* convert to upper */
				}
				endp++;
			}

			/* now have a begin & end of token */
			tokens[cnt++] = p;
			if( *endp == 0 )
			{
				p = endp;	/* end of input - terminate scan */
			}
			else
			{
				*endp = 0;		/* null terminate token */
				p = &endp[1];	/* start next scan at after null */
			}
		}	/* end if *p != 0 */
	}	/* end while *p != 0 .... */

	return( cnt );
}


/* RmgStatus_ZeroErrInfo
 *
 * Zero out error information
 *
 * Parameters:
 *
 *  errInfo     pointer to error information record to be zero'd
 *
 * Return value: void
 */
static void RmgStatus_ZeroErrInfo( RMG_ERROR_INFO *errInfo )
{
    errInfo->errorCount = 0;
    errInfo->lastError = 0;
    errInfo->lastInfo = 0;
}

/* RmgStatus
 *
 * Displays status and statistics for both the RMG process and the HMI interface.
 *
 * Parameters:  None
 *
 * Return value: void
 */
static void RmgStatus( U8 zeroflag )
{
    DWORD       ret;
    HmStatsData hmStatsData;
    HmTskStats  *hmTskStats;
    short       j;

    /* first display RMG process status */
    printf( "\nRMG Board %u Status\n", RmgBoardNum );
    printf( "-------------------------------------------------------------------------------\n" );
    printf( "State          : %s\n", LogNameState( RmgState ) );
    printf( "Network Status : %s\n", LogNetStatus( RmgNetworkStatus ) );
    printf( "Heartbeats Sent: %lu    Received: %lu\n",
            RmgTxHeartbeats, RmgRxHeartbeats );
    if ( (RmgError.rmgMessage.errorCount != 0)      ||
         (RmgError.startTimer.errorCount != 0)      ||
         (RmgError.cancelTimer.errorCount != 0)     ||
         (RmgError.staleTimer.errorCount != 0)      ||
         (RmgError.commSend.errorCount != 0)        ||
         (RmgError.hmiLoadBoard.errorCount != 0)    ||
         (RmgError.hmiHaltBoard.errorCount != 0)    ||
         (RmgError.hmiStop.errorCount != 0)         ||
         (RmgError.hmiStart.errorCount != 0)        ||
         (RmgError.hmiBackup.errorCount != 0)       ||
         (RmgError.hmiPrimary.errorCount != 0)      ||
         (RmgError.hmiStandalone.errorCount != 0)   ||
         (RmgError.hmiStatus.errorCount != 0) )
    {   /* at least one of the globally tracked controller errors has occurred */
        printf( "\nController Errors\n" );
        printf( "-------------------------------------------------------------------------------\n" );
        if (RmgError.rmgMessage.errorCount != 0)
            printf( "RMG Message    : %10.10u errors (last: %10.10s, info: %10.10s)\n", RmgError.rmgMessage.errorCount,
            LogNameEventID( RmgError.rmgMessage.lastError ), RmgErrorReasons[RmgError.rmgMessage.lastInfo] );
        if (RmgError.startTimer.errorCount != 0)
            printf( "Timer Start    : %10.10u errors (last: 0x%08X, info: 0x%08X)\n", RmgError.startTimer.errorCount, RmgError.startTimer.lastError, RmgError.startTimer.lastInfo );
        if (RmgError.cancelTimer.errorCount != 0)
            printf( "Timer Cancel   : %10.10u errors (last: 0x%08X, info: 0x%08X)\n", RmgError.cancelTimer.errorCount, RmgError.cancelTimer.lastError, RmgError.cancelTimer.lastInfo );
        if (RmgError.staleTimer.errorCount != 0)
            printf( "Timer Stale    : %10.10u errors (last: 0x%08X, info: 0x%08X)\n", RmgError.staleTimer.errorCount, RmgError.staleTimer.lastError, RmgError.staleTimer.lastInfo );
        if (RmgError.commSend.errorCount != 0)
            printf( "UDP CommSend   : %10.10u errors (last: 0x%08X, info: 0x%08X)\n", RmgError.commSend.errorCount, RmgError.commSend.lastError, RmgError.commSend.lastInfo );
        if (RmgError.hmiLoadBoard.errorCount != 0)
            printf( "HMI LoadBoard  : %10.10u errors (last: 0x%08X, info: 0x%08X)\n", RmgError.hmiLoadBoard.errorCount, RmgError.hmiLoadBoard.lastError, RmgError.hmiLoadBoard.lastInfo );
        if (RmgError.hmiHaltBoard.errorCount != 0)
            printf( "HMI HaltBoard  : %10.10u errors (last: 0x%08X, info: 0x%08X)\n", RmgError.hmiHaltBoard.errorCount, RmgError.hmiHaltBoard.lastError, RmgError.hmiHaltBoard.lastInfo );
        if (RmgError.hmiStop.errorCount != 0)
            printf( "HMI Stop       : %10.10u errors (last: 0x%08X, info: 0x%08X)\n", RmgError.hmiStop.errorCount, RmgError.hmiStop.lastError, RmgError.hmiStop.lastInfo );
        if (RmgError.hmiStart.errorCount != 0)
            printf( "HMI Start      : %10.10u errors (last: 0x%08X, info: 0x%08X)\n", RmgError.hmiStart.errorCount, RmgError.hmiStart.lastError, RmgError.hmiStart.lastInfo );
        if (RmgError.hmiBackup.errorCount != 0)
            printf( "HMI Backup     : %10.10u errors (last: 0x%08X, info: 0x%08X)\n", RmgError.hmiBackup.errorCount, RmgError.hmiBackup.lastError, RmgError.hmiBackup.lastInfo );
        if (RmgError.hmiPrimary.errorCount != 0)
            printf( "HMI Primary    : %10.10u errors (last: 0x%08X, info: 0x%08X)\n", RmgError.hmiPrimary.errorCount, RmgError.hmiPrimary.lastError, RmgError.hmiPrimary.lastInfo );
        if (RmgError.hmiStandalone.errorCount != 0)
            printf( "HMI Standalone : %10.10u errors (last: 0x%08X, info: 0x%08X)\n", RmgError.hmiStandalone.errorCount, RmgError.hmiStandalone.lastError, RmgError.hmiStandalone.lastInfo );
        if (RmgError.hmiStatus.errorCount != 0)
            printf( "HMI Status     : %10.10u errors (last: 0x%08X, info: 0x%08X)\n", RmgError.hmiStatus.errorCount, RmgError.hmiStatus.lastError, RmgError.hmiStatus.lastInfo );
    }

    /* now fetch and display HMI status */
    ret = hmiStatusReq( UserIfCmdHd, &hmStatsData, zeroflag );
    if( ret != SUCCESS )
    {
        printf("Error return 0x%lx from %s\n", ret, "hmiStatusReq" );
        RmgError.hmiStatus.errorCount++;
        RmgError.hmiStatus.lastError = ret;
        RmgError.hmiStatus.lastInfo = __LINE__;
    }
    else
    {
        printf( "\nHMI Board %d Status\n", RmgBoardNum );
        printf( "-------------------------------------------------------------------------------\n" );
        printf( "Heartbeats Sent: %lu    Received: %lu\n",
                hmStatsData.linkStats.txHB, hmStatsData.linkStats.rxHB );
        if (hmStatsData.linkStats.linkState <= HMI_LINKSTATE_UNKNOWN)
        {   /* link state reported by HMI is within known range */
            printf( "Link State     : %s\n",
                    HmiLinkStates[ min(hmStatsData.linkStats.linkState, HMI_LINKSTATE_UNKNOWN) ] );
        }
        else
        {   /* link state reported by HMI is out-of-range */
            printf( "Link State     : 0x%X\n", hmStatsData.linkStats.linkState );
        }
        if (hmStatsData.netState <= HMI_NETSTATE_UNKNOWN)
        {   /* network state reported by HMI is within known range */
            printf( "Network State  : %s\n",
                    HmiNetStates[ min(hmStatsData.netState, HMI_NETSTATE_UNKNOWN) ] );
        }
        else
        {   /* network state reported by HMI is out-of-range */
            printf( "Network State  : 0x%X\n", hmStatsData.netState );
        }
        printf( "\n" );

        hmTskStats = &hmStatsData.taskStats[0];
        for( j = 0; j < hmStatsData.numTsks; j++ )
        {
            if (hmTskStats->runState <= HMRS_UNKNOWN)
            {   /* task state reported by HMI is within known range */
                printf( "%8.8s State = %s\n", hmTskStats->name,
                        HmiTaskStates[ min(hmTskStats->runState, HMI_TASKSTATE_UNKNOWN) ] );
            }
            else
            {   /* task state reported by HMI is out-of-range */
                printf( "%8.8s State = 0x%X\n", hmTskStats->name, hmTskStats->runState );
            }

            hmTskStats++;
        }
        printf( "\n" );
    }

    if (zeroflag )
    {   /* now zero all internally maintained status and statistics */
        RmgTxHeartbeats = 0;
        RmgRxHeartbeats = 0;
        RmgStatus_ZeroErrInfo( &RmgError.rmgMessage );
        RmgStatus_ZeroErrInfo( &RmgError.startTimer );
        RmgStatus_ZeroErrInfo( &RmgError.cancelTimer );
        RmgStatus_ZeroErrInfo( &RmgError.staleTimer );
        RmgStatus_ZeroErrInfo( &RmgError.commSend );
        RmgStatus_ZeroErrInfo( &RmgError.hmiLoadBoard );
        RmgStatus_ZeroErrInfo( &RmgError.hmiHaltBoard );
        RmgStatus_ZeroErrInfo( &RmgError.hmiStop );
        RmgStatus_ZeroErrInfo( &RmgError.hmiStart );
        RmgStatus_ZeroErrInfo( &RmgError.hmiBackup );
        RmgStatus_ZeroErrInfo( &RmgError.hmiPrimary );
        RmgStatus_ZeroErrInfo( &RmgError.hmiStandalone );
        RmgStatus_ZeroErrInfo( &RmgError.hmiStatus );
    }
}                                       /* RmgStatus */

/* RmgCmdProc
 *
 * This is the main command processing loop; it repeatedly prompts for
 * user input until an "Exit" command is received, at which point it
 * returns.
 *
 * Parameters:  None
 *
 * Return value: void
 */
void RmgCmdProc( void )
{
    short       done = 0;
    char        buffer[256];
    char        *tokens[MAX_PARMS +1];
    short       cnt;
    DWORD       ret;
    DWORD       event;
    U8          log_stdout,log_memory,trigger_stop,freeze;

    /* command processing loop */
    while( ! done )
    {
        /* wait for a command */
        if( (fgets( buffer, 256, stdin )) == NULL )
        {
            done = 1;               /* eof */
        }
        else
        {
            /* input line received, break it into individual tokens; note that
             * tokenize() shifts all alphas to uppercase
             */
            while ( (strlen(buffer) > 0) && (buffer[strlen(buffer) - 1] < ' ') )
                buffer[strlen(buffer) - 1] = '\0';  /* remove any trailing non-printable characters */
            cnt = tokenize( buffer, tokens );
            if( cnt == 0 )
            {
                /* just a CR with no data, give a prompt now */
                printf( "RMGR>" );
                continue;
            }

            if( (strcmp( tokens[0], "QUIT" ) == 0) || (strcmp( tokens[0], "Q" ) == 0) )
            {
                done = 1;
            }
            else if( (strcmp( tokens[0], "STATUS" ) == 0) || (strcmp( tokens[0], "S" ) == 0) )
            {
                /* display status and statistics */
                if (cnt == 1)
                {   /* show current status information */
                    RmgStatus( 0 );
                }
                else if ( (cnt == 2) &&
                          ( (strcmp( tokens[1], "ZERO" ) == 0) || (strcmp( tokens[1], "0" ) == 0) ) )
                {   /* show current status information and then zero internal counters, etc. */
                    RmgStatus( 1 );
                }
                else
                {
                    printf( "Available STATUS commands:\n\n" );
                    printf( "[S]TATUS      - Show current status information\n" );
                    printf( "[S]TATUS ZERO - Show current status, then zero internal counters\n" );
                }
            }
            else if( (strcmp( tokens[0], "CHANGE" ) == 0) || (strcmp( tokens[0], "C" ) == 0) )
            {
                /* send switchover command event to ctlr fsm */
                RmgSendEvent( CtlrFsmCmdHd, RMG_EV_SWOVER_CMD, 0 );
            }
            else if( (strcmp( tokens[0], "HALT" ) == 0) || (strcmp( tokens[0], "H" ) == 0) )
            {
                /* send halt command event to ctlr fsm */
                RmgSendEvent( CtlrFsmCmdHd, RMG_EV_HALT_CMD, 0 );
            }
            else if( (strcmp( tokens[0], "LOAD" ) == 0) || (strcmp( tokens[0], "L" ) == 0) )
            {
                /* send load board command event to ctlr fsm */
                RmgSendEvent( CtlrFsmCmdHd, RMG_EV_LOAD_CMD, 0 );
            }
            else if( (strcmp( tokens[0], "RESET" ) == 0) || (strcmp( tokens[0], "R" ) == 0) )
            {
                /* reset the HMI process */
                ret = hmiReset( UserIfCmdHd );
                if( ret != SUCCESS )
                {
                    printf("Error return 0x%lx from %s\n", ret, "hmiReset" );
                }
            }
            else if( (strcmp( tokens[0], "EXTRACT" ) == 0) || (strcmp( tokens[0], "E" ) == 0) )
            {
                /* send extract command event to ctlr fsm */
                RmgSendEvent( CtlrFsmCmdHd, RMG_EV_EXTRACT_CMD, 0 );
            }
            else if( (strcmp( tokens[0], "TRACE" ) == 0) || (strcmp( tokens[0], "T" ) == 0) )
            {
                /* display tracing info (or modify tracing config) */
                if ( (cnt == 1) || (strcmp( tokens[1], "SHOW" ) == 0) )
                {   /* show current event trace log */
                    LogShowCurrent( );
                }
                else if ( (cnt == 2) && (strcmp( tokens[1], "ERROR" ) == 0) )
                {   /* show captured [due to error or trigger event] trace log */
                    LogShowCapture( );
                }
                else if ( (cnt == 2) && (strcmp( tokens[1], "CONFIG" ) == 0) )
                {   /* show event trace configuration */
                    LogShowConfig( );
                }
                else if ( (cnt == 2) && (strcmp( tokens[1], "CLEAR" ) == 0) )
                {   /* clear captured [due to error or trigger event] trace log */
                    LogClearCapture( );
                }
                else if ( (cnt == 7) && (strcmp( tokens[1], "SET" ) == 0) )
                {   /* set event trace configuration */
                    if (strcmp( tokens[2], "ALL" ) == 0)
                        event = EVENT_ALL;          /* special event: all events */
                    else
                        event = LogGetEventId( tokens[2], 0 );

                    if (event == 0)
                        printf( "No event named %s, use TRACE CONFIG to see list of events\n", tokens[2] );
                    else
                    {
                        if (strcmp( tokens[3], "Y" ) == 0)
                            log_stdout = 1;
                        else
                            log_stdout = 0;
                        if (strcmp( tokens[4], "Y" ) == 0)
                            log_memory = 1;
                        else
                            log_memory = 0;
                        if (strcmp( tokens[5], "Y" ) == 0)
                            trigger_stop = 1;
                        else
                            trigger_stop = 0;
                        if (strcmp( tokens[6], "Y" ) == 0)
                            freeze = 1;
                        else
                            freeze = 0;
                        LogSetEvent( event, log_stdout, log_memory, trigger_stop, freeze );
                    }
                }
                else
                {   /* show trace help */
                    printf( "Available TRACE commands:\n\n" );
                    printf( "[T]RACE [SHOW] - Shows current event trace log\n" );
                    printf( "[T]RACE ERROR  - Shows captured error (or trigger) log\n" );
                    printf( "[T]RACE CONFIG - Shows event trace configuration\n" );
                    printf( "[T]RACE CLEAR  - Clears captured error (or trigger) log\n" );
                    printf( "[T]RACE SET    - Sets event trace configuration:\n" );
                    printf( "   T SET <event> <log_stdout> <log_memory> <trigger_stop> <freeze>\n" );
                    printf( "     where:\n" );
                    printf( "       <event> = name of event to set (or ALL)\n" );
                    printf( "       <log_stdout>   = Y (log event to std out)           | N (no log)\n" );
                    printf( "       <log_memory>   = Y (log event to memory)            | N (no log)\n" );
                    printf( "       <trigger_stop> = Y (event triggers log stop)        | N (no stop)\n" );
                    printf( "       <freeze>       = Y (event freezes event processing) | N (no freeze)\n" );
                }

            }
            else if( (strcmp( tokens[0], "HELP" ) == 0) || (strcmp( tokens[0], "?" ) == 0) )
            {
                /* help command - show available commands */
                printf( "Available commands:\n\n" );
                printf( "[S]TATUS\tDisplays current board status/statistics\n" );
                printf( "[C]HANGE\tSwaps current primary and backup, if possible\n" );
                printf( "[H]ALT\t\tHalts the board, taking it out of service\n" );
                printf( "[L]OAD\t\tReloads the board\n" );
                printf( "[R]ESET\t\tResets the HMI service\n" );
                printf( "[E]XTRACT\tIssues extract command\n" );
                printf( "[T]RACE\t\tDisplays tracing info (or modifies tracing config)\n" );
                printf( "[Q]UIT\t\tQuits this process, without disturbing boards\n\n" );
            }
            else
            {
                printf( "\nUnknown command >>>> %s\n\n", tokens[0] );
                printf( "RMGR>" );
            }
        }
    }                           /* end while loop */

}                               /* RmgCmdProc */

/* RmgSendEvent
 *
 * Queues an event with the specified event code to the queue associated
 * with the specified CT Access context.
 *
 * Parameters:
 *
 * Returns:     void
 */
void RmgSendEvent( CTAHD ctaHd, DWORD id, DWORD value )
{
    CTA_EVENT event;

    memset( &event, 0, sizeof(event) );
    event.id    = id;
    event.ctahd = ctaHd;
    event.value = value;
    event.size  = 0;

    ctaQueueEvent( &event );
}

/* usage()
 *
 * Prints out a description of valid command line arguments.
 *
 * Parameters:
 *
 *  pCmdName    ptr to programm name, from command line argv[0]
 *
 * Returns:     void
 */
static void usage( char *pCmdName )
{
    static char blanks8[] = "        ";

    printf( "Usage: \n\n" );
    printf( "%8.8s [ -b <board number> ]\t\t\tdefault: %d\n", pCmdName, RmgBoardNum );
    printf( "%8.8s [ -n <my node number> ]\t\tdefault: %d\n", blanks8, RmgNodeNum );
    printf( "%8.8s [ -m <mate node name/IP address> ]\tdefault: local\n", blanks8 );
    printf( "%8.8s [ -l <local UDP port> ]\t\tdefault: %d\n", blanks8, RMG_PORT_DEFAULT );
    printf( "%8.8s [ -p <remote UDP port> ]\t\tdefault: %d\n", blanks8, RMG_PORT_DEFAULT );
    printf( "%8.8s [ -t ] turns tracing on\t\tdefault: off\n", blanks8 );
    printf( "%8.8s [ -q ] require quit cmd to exit\tdefault: auto-exit\n", blanks8 );
}


/* scanArgs()
 *
 * Scans command line arguments for board number, alarm log file name, and
 * configuration file name.
 *
 * Parameters:
 *
 *  argc    number of arguments from command line
 *  argv    array of pointers to command line arguments
 *
 * Returns:
 *
 *  0       Success
 *  -1      Failed (error message has been displayed on standard output)
 */
static short scanArgs( int argc, char *argv[] )
{
    short   i;
    char    *str = NULL;
    int     temp;

    i = 1;

    while (i < argc)
    {
		str = argv[i];
        if (*str == '-')
        {
            switch (*++str)
            {
            case 'b':   /* board number to control */
                if (isdigit(*(str + 1)))
                    temp = atoi(++str);
                else if( ++i < argc )
                    temp = atoi(argv[i]);
                else
                {
                    usage( argv[0] );
                    return( -1 );
                }
                if ( (temp < 1) || (temp > 8) )
                {
                    printf( "Board number [%d] must be in range 1..8\n", temp );
                    return( -1 );
                }
                RmgBoardNum = (short) temp;
				break;

            case 'l':   /* local UDP port */
                if (isdigit(*(str + 1)))
                    temp = atoi(++str);
                else if( ++i < argc )
                    temp = atoi(argv[i]);
                else
                {
                    usage( argv[0] );
                    return( -1 );
                }
                if ( (temp <= 0) || (temp > 32766) )
                {
                    printf( "Local port [%d] must be in range 1..32766\n", temp );
                    return( -1 );
                }
                RmgLocalPort = (U16) temp;
				break;

            case 'm':
                if (isalpha(*(str + 1)))
                    strcpy( RmgMateNodeName, ++str );
                else if( ++i < argc )
                    strcpy( RmgMateNodeName, argv[i] );
                else
                {
                    usage( argv[0] );
                    return( -1 );
                }
				break;

            case 'n':   /* node number */
                if (isdigit(*(str + 1)))
                    temp = atoi(++str);
                else if( ++i < argc )
                    temp = atoi(argv[i]);
                else
                {
                    usage( argv[0] );
                    return( -1 );
                }
                if ( (temp < 1) || (temp > 2) )
                {
                    printf( "Node number [%d] must be in range 1..2\n", temp );
                    return( -1 );
                }
                RmgNodeNum = (short) temp;
				break;

            case 'p':   /* remote UDP port */
                if (isdigit(*(str + 1)))
                    temp = atoi(++str);
                else if( ++i < argc )
                    temp = atoi(argv[i]);
                else
                {
                    usage( argv[0] );
                    return( -1 );
                }
                if ( (temp <= 0) || (temp > 32766) )
                {
                    printf( "Remote port [%d] must be in range 1..32766\n", temp );
                    return( -1 );
                }
                RmgRemotePort = (U16) temp;
				break;

            case 't':   /* turn tracing to ctdaemon process on */
                RmgTraceLvl = 1;
				break;

            case 'q':   /* require quit command to exit */
                RmgRequireQuit = 1;
				break;

			default:
                usage( argv[0] );
                return( -1 );
            }                   /* end switch */
		}
		++i;
    }                           /* end while loop */

    return( 0 );                /* successful return */
}                               /* of scanArgs */

/*
 * initCtAccess
 *
 * This function initializes CT Access, creates all queues and contexts
 * required by this application, and opens all necessary instances
 * of the HMI service for both the user interface (main thread) and
 * the controller FSM thread.
 *
 * Paramters:   None
 *
 * Returns:     SUCCESS     sucessful
 *              other       CT Access error has occurred
 */
static DWORD initCtAccess( void )
{
    DWORD           ret;
    char            buf[80];
    CTA_INIT_PARMS  hmiInitparms = { 0 };
    CTA_EVENT       event = {0};

    /* Initialize CT Access health management (HMI) service */
    hmiInitparms.size = sizeof( CTA_INIT_PARMS );
    if( RmgTraceLvl > 0 )
    {
        hmiInitparms.traceflags = CTA_TRACE_ENABLE;  /* enable event tracing */
    }
    hmiInitparms.parmflags = CTA_PARM_MGMT_SHARED;
    hmiInitparms.ctacompatlevel = CTA_COMPATLEVEL;
    ret = ctaInitialize( HmiServiceNames, 1, &hmiInitparms );
    if( ret == CTAERR_SHAREMEM_ACCESS )
    {
        /* ctdaemon is not running. Initialize again without tracing */
        printf( "WARNING, ctadaemon not running, tracing not available\n" );
        hmiInitparms.traceflags = 0;
        hmiInitparms.parmflags = 0;
        ret = ctaInitialize( HmiServiceNames, 1, &hmiInitparms );
    }
    if( ret != SUCCESS )
    {
        ctaGetText( NULL_CTAHD, ret, buf, sizeof( buf ) );
        printf( "ERROR initializing CT Access: %s\n", buf);
        return( ret );
    }

    /* create the CT Access queue used to pass all events to the ctlr FSM */
    ret = ctaCreateQueue( NULL, 0, &CtlrFsmQueue );
    if( ret != SUCCESS )
    {
        ctaGetText( NULL_CTAHD, ret, buf, sizeof( buf ) );
        printf( "ERROR creating CT Access CTLR queue: %s\n", buf);
        return( ret );
    }

    /* create the controller FSM CT Access context for HMI async events
     * and open the HMI async events service on that context
     */
    ret = openHmiService( "RMGASYN", RMG_ASYNC_CHAN, HMI_RCV_EVENTS,
                          CtlrFsmQueue, &CtlrFsmAsyncHd );
    if( ret != SUCCESS )
    {
        /* openHmiService already logged an error message */
        return( ret );
    }

    /* create the controller FSM CT Access context for HMI commands
     * and open the HMI commands service on that context
     */
    ret = openHmiService( "RMGCMD", RMG_CTLRCMD_CHAN, HMI_DO_COMMANDS,
                          CtlrFsmQueue, &CtlrFsmCmdHd );
    if( ret != SUCCESS )
    {
        /* openHmiService already logged an error message */
        return( ret );
    }

    /* create the user interface CT Access queue */
    ret = ctaCreateQueue( NULL, 0, &UserIfQueue );
    if( ret != SUCCESS )
    {
        ctaGetText( NULL_CTAHD, ret, buf, sizeof( buf ) );
        printf( "ERROR creating CT Access UI queue: %s\n", buf);
        return( ret );
    }

    /* create the user interface CT Access context for HMI commands
     * and open the HMI commands service on that context
     */
    ret = openHmiService( "RMGUI", RMG_UICMD_CHAN, HMI_DO_COMMANDS,
                          UserIfQueue, &UserIfCmdHd );
    if( ret != SUCCESS )
    {
        /* openHmiService already logged an error message */
        return( ret );
    }

    return( SUCCESS );
}                                       /* initCtAccess */

/*
 * openHmiService
 *
 * This function creates a context and opens the desired CT Access HMI
 * service on the specified queue/context.
 *
 * Paramters:
 *  pCtxName        pointer to context name string (for tracing)
 *  hmiChan         HMI service channel number
 *  servType        either HMI_RCV_EVENTS or HMI_DO_COMMANDS
 *  ctaQ            CT Access queue to create context against
 *  pCtaHd          address where CT Access context handle is returned
 *
 * Returns:         SUCCESS     sucessful
 *                  other       error has occurred, cta error value returned
 */
static DWORD openHmiService( char *pCtxName, DWORD hmiChan, DWORD servType,
                             CTAQUEUEHD ctaQ, CTAHD *pCtaHd )
{
    DWORD           ret;
    char            buf[80];
    CTA_EVENT       event = {0};

    /* create the CT Access contect on the specified queue */
    ret = ctaCreateContext( ctaQ, 0, pCtxName, pCtaHd );
    if( ret != SUCCESS )
    {
        ctaGetText( NULL_CTAHD, ret, buf, sizeof( buf ) );
        printf( "ERROR creating CT Access context %s: %s\n", pCtxName, buf );
        return( ret );
    }

    /* open the specified HMI service on the context just created */
    HmiOpenSvcLst[0].svcargs.args[0] = hmiChan;
    HmiOpenSvcLst[0].svcargs.args[1] = RmgBoardNum;     /* board - use global */
    HmiOpenSvcLst[0].svcargs.args[2] = servType;
    ret = ctaOpenServices( *pCtaHd, &HmiOpenSvcLst[0], 1 );
    if( ret != SUCCESS )
    {
        ctaGetText( *pCtaHd, ret, buf, sizeof( buf ) );
        printf( "ERROR opening HMI service [%ld]: %s\n", hmiChan, buf );
        return( ret );
    }

    /* Wait for service open to complete. */
    do
    {
        ctaWaitEvent( ctaQ, &event, CTA_WAIT_FOREVER );
    } while( event.id != CTAEVN_OPEN_SERVICES_DONE );

    if( event.value != CTA_REASON_FINISHED )
    {
        ctaGetText( event.ctahd, event.value, buf, sizeof( buf ) );
        printf( "ERROR opening HMI service [%ld]: %s\n", hmiChan, buf );
        return( event.value );
    }

    return( SUCCESS );
}                                       /*  of openHmiService */

/*
 * cleanupCtAccess
 *
 * This function destroys any CT Access queues and contexts which were
 * created and closes any services opened against them.
 *
 * Paramters:   None
 *
 * Returns:     void
 */
static void cleanupCtAccess( void )
{

    /* destroy the user interface queue */
    /* NOTE: The user interface queue is destroyed before the controller FSM queue
     *       so that queues are destroyed in the opposite order that they are created in. */
    if( UserIfQueue != 0 )
    {
        ctaDestroyQueue( UserIfQueue );
        UserIfQueue = 0;
    }

    /* destroy the controller FSM queue if it was created; this will
    * also close any services and destroy any contexts associated with
    * the queue
    */
    if( CtlrFsmQueue != 0 )
    {
        ctaDestroyQueue( CtlrFsmQueue );
        CtlrFsmQueue = 0;
    }

}                                       /* cleanupCtAccess */

/* main()
 *
 * Retrieves run-time arguments, starts threads, and then monitors
 * state of individual threads, killing the process if one of the threads
 * fails.
 */
int main( int argc, char *argv[] )
{
   TThrId CfsmThrId;

    /* print version string */
    printf("%s: Redundancy manager version %s %s\n", argv[0], RMG_VERSION, __DATE__);

    /* scan command line arguments */
    if( scanArgs( argc, argv ) != 0 )
    {
        exit( 1 );
    }

    /* initialize CT Access, create all necessary queues and contexts,
     * and open the HMI service for each context.
     */
    if( initCtAccess() != SUCCESS )
    {
        /* destroy anything created before failure */
        cleanupCtAccess();
        exit( 1 );
    }

    /* decide whether input parameters imply standalone mode */
    if ( (RmgLocalPort == RmgRemotePort) && (strlen(RmgMateNodeName) == 0) )
    {
        RmgStandAlone = 1;              /* set = input parameters imply we are standalone */
    }
    else
    {
        /* initialize the communications (sockets) subsystem; this also
         * creates the receiver thread
         */
        if( RmgCommInit() != 0 )
        {
            cleanupCtAccess();
            exit( 1 );
        }
    }

    /* create the controller FSM thread */
    if((THREAD_CREATE(0, (TPtrThrFunc) RmgCfsmThread, NULL, 0, &CfsmThrId)) != RET_SUCCESS)
    {
        printf( "ERROR %d creating controller FSM thread\n", errno );
        RmgCommCleanup();
        cleanupCtAccess();
        exit( 1 );
    }

    /* start the user interface command processor; when it returns,
     * the user has entered an "exit" command.
     */
    RmgCmdProc();

    /* when we fall through to here we got an exit command; send a "quit"
     * event to the controller FSM (this will kill the thread) and kill
     * the comm receiver thread by calling the RmgCommCleanup() routine.
     */
    RmgSendEvent( CtlrFsmCmdHd, RMG_EV_SHUTDN, 0 );
    RmgCommCleanup();

    /* destroy the CT Access queues and contexts and close all services opened against them */
    cleanupCtAccess();

    exit( 0 );

    return( 0 );        /* dummy code to remove compiler warning */
}
