/* rmgtimer_ux.c
 *
 * Utility routines for RMG controller FSM starting and cancelling timers.
 * Timer mechanics are non-portable across operating systems, so this is
 * segregated into a separate file.
 *
 * Each timer expiration results in the corresponding event queued to the
 * RMG controller FSM.  Timers may be one-shots (have to be reset after
 * each expiration) or continuous (automatically restarted after each
 * expiration until cancelled).
 */
#include "osdep.h"

#include "ctadef.h"
#include "ctaerr.h"
#include "nmstypes.h"
#include "rmgdefs.h"

unsigned long  evt2fsm;	/* to make the signal being visible by the signal intercept function */

/* Forward declaration function */
void RmgTimerProc(int dwUser);

/*--------------------------------------------------------------------------
  Name:           InstallTimer()
  Description:    This routine installs an interval timer.
  Parameters:     None
  Return Values:  non-NULL  Error
                  NULL      Success
 -------------------------------------------------------------------------*/
short InstallITimer()
{
   short ret = 1;
   struct sigaction tact;

   tact.sa_handler = RmgTimerProc;
   tact.sa_flags = SA_RESTART;
   if ((ret = sigemptyset(&tact.sa_mask)) == 0)
      ret = sigaction(SIGALRM, &tact, NULL);

   return(ret);
}

/*--------------------------------------------------------------------------
  Name:           StartTimer()
  Description:    This routine starts a timer that will expire in msecs.
  Parameters:     msecs     Number of milliseconds before the timer
                            should expire.
                  event_id  Event to pass to the queue to controller
                            FSM when timer expires
                  cont      If true, restart the timer every time
                            it expires.  If false, do not.
  Return Values:  NULL      OS-specific error (message has been logged)
                  non-NULL  Success, return value is a handle to be
                            used in call to RmgStopTimer
 -------------------------------------------------------------------------*/
TMTimerId RmgStartTimer( DWORD msecs, DWORD event_id, BOOL cont)
{

   static unsigned char InstToggle;  /*keeps track of timer installation*/
   struct itimerval   itval;
   int    ret;
   unsigned int    tflags = 0;         /* timer options */

  /* check if the timer is already install */
   if(InstToggle == 0)
      if ((ret = InstallITimer()) != 0) {
         printf( "RMG timer installation failed\n" );
         InstToggle = 0;
         return (0); /* installation failed */
      }
      else
	 InstToggle++;    /* success */

  /* extend the scope of the event (needed for signal handler) */
   evt2fsm = event_id;

  /* set up time interval */
   if (msecs >=  1000) {
      itval.it_value.tv_sec = msecs/1000;
      itval.it_value.tv_usec = (msecs%1000)*1000; /*milliseconds*/
   }
   else {
      itval.it_value.tv_sec = 0;
      itval.it_value.tv_usec = msecs * 1000;
   }
   /* set continuous or one-shot */
   if( cont ) { /* TIME_PERIODIC */
      itval.it_interval = itval.it_value;
   }
   else { /* TIME_ONESHOT */
      itval.it_interval.tv_sec = 0;
      itval.it_interval.tv_usec = 0;
   }

    /* start timer */
    if ((ret = setitimer(ITIMER_REAL, &itval, NULL)) != 0) {
        printf( "RMG start timer failed\n" );
        return(0);
    }

    return( (TMTimerId)55 ); /*success*/
}   /* End RmgStartTimer */


/*--------------------------------------------------------------------------
  Name:           StopTimer()
  Description:    This routine stops the specified timer before it expires.
  Parameters:     hTimer            Timer handle returned by RmgStartTimer.
  Return Values:  void
 -------------------------------------------------------------------------*/
void RmgStopTimer( TMTimerId hTimer )
{
   struct itimerval   itval;
   int ret;

    /* set up time interval */
    itval.it_value.tv_sec = 0;
    itval.it_value.tv_usec = 0;
    itval.it_interval = itval.it_value;

    /* stop timer */
    if ((ret = setitimer(ITIMER_REAL, &itval, NULL)) != 0) {
        printf( "RMG stop timer failed\n" );
    }

}   /* End RmgStopTimer */


/*--------------------------------------------------------------------------
  Name:           TimerProc()
  Description:    This routine handles OS timer expirations.  The event to
                  queue to the controller FSM is contained in the dwUser
                  argument.
  Parameters:     dwUser            user data (event to queue)
  Return Values:  None
 -------------------------------------------------------------------------*/
void RmgTimerProc(int dwUser)
{
    RmgSendEvent( CtlrFsmCmdHd, evt2fsm, 0 );
}   /* End TimerProc */

