
/*****************************************************************************
 **
 **                         PROPRIETARY NOTICE
 **
 **  (c)Copyright 1995-2009 Dialogic Corporation.  All Rights Reserved
 **
 **  The following is licensed program material and is considered
 **  proprietary and confidential.  No part of this program may be
 **  reproduced or used in any form or by any means without the
 **  express written permission of Dialogic Corporation.
 **
 *****************************************************************************/

/* TX alarm capturing daemon */
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#ifdef WIN32
#include <process.h>
#include <conio.h>
#else
#include <poll.h>
#endif

#include <ctadef.h>
#include <ctaerr.h>
#include <nmstypes.h>
#include <txcpi.h>

#include <hmidef.h>

#include <txalarm.h>

/* ----- Modification History: --------------------------------------------- */
#define MH_VERSION      3           /* identifier of major release version */
#define MH_REVISION     0           /* identifier of release revision */

static char    *usagestr = "Usage: %s [-c chan] [-f outfilename] [-s]\n";

U32             display_options = 0;    /* alarm output display options [default: NONE] */
#define TXALM_DISPLAY_SEVERITY  (1<<0)  /* set = display alarm severity as part of alarm message output */

U32             alarms_lost[MAXBOARD + 1];  /* count of alarms lost (dropped by TX board due to queue limits) */

#ifdef WIN32
CTA_WAITOBJ     stdInput;
#endif
short           chan = DEFAULTALARMCHAN;
FILE           *fileDesc = NULL;
S32             cps[MAXBOARD + 1];
TX_HANDLE       fds[MAXBOARD + 1];
CTA_WAITOBJ     wobjs[MAXBOARD + 1];

/* Service name/manager pair for ctaInitialize */
CTA_SERVICE_NAME HmiServiceNames[] = { {"HMI", "HMIMGR"} };

/* Service list for ctaOpenServices */
CTA_SERVICE_DESC HmiOpenSvcLst[] = { {{"HMI", "HMIMGR"}, {0}, {0}, {0}} };

/* Service list for ctaCloseServices */
char           *HmiCloseSvcLst[] = { "HMI" };

/*****************************************************************************
 * ExitRoutine
 *****************************************************************************/
#ifdef WIN32
DWORD NMSSTDCALL ExitRoutine(CTAQUEUEHD hmiQueue, CTA_WAITOBJ waitObj,
                             void *arg)
#else
DWORD NMSSTDCALL ExitRoutine(CTAQUEUEHD hmiQueue, CTA_WAITOBJ * waitObj,
                             void *arg)
#endif
{
    /* make sure a key was hit */
#ifdef WIN32
    if (kbhit())
        exit(0);
    FlushConsoleInputBuffer(stdInput);
#else
    exit(0);
#endif

    return (SUCCESS);
}

/*****************************************************************************
 * ProcessAlarm - process received alarm message (format output line)
 *
 * Inputs: pkt     - CPI packet holding received alarm
 *                   or NULL = write banner header to output line
 *         outline - address where output line (or error message) stored
 *         options - set of display options
 *
 * Output: alarm message [outline holds formatted alarm string]
 *         or NULL if error detected [outline holds error message]
 *****************************************************************************/
TXLOG_ALARM_MSG *ProcessAlarm( CPIPKT *pkt, S8 *outline, U32 options )
{
    TXLOG_ALARM_MSG    *msg;
    int                 slen;
    S8                  severity;
    S8                 *legacy_taskname;
    S8                 *alarm_text;
    U16                 source_board;
    struct tm           timestamp;
    time_t              secs_epoch;

    /* ----- write banner header to output line if no packet provided */
    if (pkt == NULL)
    {
        if (options & TXALM_DISPLAY_SEVERITY)
            strcpy( outline, "<MM/DD/YY HH:MM:SS> TaskName Bd Sev Number Message\n" );
        else
            strcpy( outline, "<MM/DD/YY HH:MM:SS> TaskName Bd Number Message\n" );
        return( NULL );
    }

    /* ----- verify size of packet is big enough to hold alarm message ----- */
    if (pkt->hdr.length < (sizeof(CPIHDR) + TXLOG_ALARM_MIN_SZ))
    {
        sprintf( outline, "Message of %d bytes too small to be alarm\n", pkt->hdr.length );
        return( NULL );
    }

    /* ----- determine TX board that was source of alarm ----- */
    source_board = BOARD(pkt->hdr.src);
    if (source_board > MAXBOARD)
    {
        sprintf( outline, "Message from invalid board number: %d\n", source_board );
        return( NULL );
    }

    /* ----- perform all required field swabbing ----- */
    msg = (TXLOG_ALARM_MSG *)pkt->data;
    msg->hdr.secs_epoch = cpi_cptoh_l( msg->hdr.secs_epoch );   /* seconds since epoch - log record timestamp */
    msg->hdr.logs_lost = cpi_cptoh_l( msg->hdr.logs_lost );     /* count of log records lost [not delivered to receiver] */
    msg->alarm.number = cpi_cptoh_s( msg->alarm.number );       /* unique number identifying alarm */
    msg->alarm.severity = cpi_cptoh_s( msg->alarm.severity );   /* severity of alarm */

    /* ----- determine severity code ----- */
    if (options & TXALM_DISPLAY_SEVERITY)
    {
        switch( msg->alarm.severity )
        {
            case TXALARM_SPECIFIC:  /* severity determined by specific alarm number */
                severity = 'S';
                break;

            case TXALARM_INFO:      /* Informational */
                severity = 'I';
                break;

            case TXALARM_WARNING:   /* Warning */
                severity = 'W';
                break;

            case TXALARM_ERROR:     /* Error */
                severity = 'E';
                break;

            case TXALARM_CRITICAL:  /* Critical Error */
                severity = 'C';
                break;

            default:
                severity = '?';     /* show that severity code is unknown */
        }
    }

    /* ----- locate text of alarm message ----- */
    legacy_taskname = (S8 *)msg->alarm.message;
    slen = strlen( legacy_taskname );
    alarm_text = (S8 *)&msg->alarm.message[slen + 1];

    /* ----- compute timestamp of alarm ----- */
    /* NOTE: The TX board sends time by providing seconds since epoch.
     *       These seconds are used by the "localtime" function to calculate the local time.
     */
    secs_epoch = (time_t)msg->hdr.secs_epoch;   /* get local copy so represented as valid native time_t */
    timestamp = *localtime( &secs_epoch );      /* call the library function to calculate local time */

    /* ----- format the alarm output string ----- */
    if (options & TXALM_DISPLAY_SEVERITY)
        sprintf( outline, "<%02d/%02d/%02d %02d:%02d:%02d> %s %2d [%c] %6d %s\n",
                 timestamp.tm_mon+1,
                 timestamp.tm_mday,
                 (timestamp.tm_year+1900) % 100,
                 timestamp.tm_hour,
                 timestamp.tm_min,
                 timestamp.tm_sec,
                 msg->hdr.task_name, /* space-padded task name */
                 source_board,
                 severity,
                 msg->alarm.number,
                 alarm_text );
    else
        sprintf( outline, "<%02d/%02d/%02d %02d:%02d:%02d> %s %2d %6d %s\n",
                 timestamp.tm_mon+1,
                 timestamp.tm_mday,
                 (timestamp.tm_year+1900) % 100,
                 timestamp.tm_hour,
                 timestamp.tm_min,
                 timestamp.tm_sec,
                 msg->hdr.task_name, /* space-padded task name */
                 source_board,
                 msg->alarm.number,
                 alarm_text );

    return( msg );
}

/*****************************************************************************
 * InternalAlarm - issue internally generated alarm
 *
 * Inputs: secs_epoch - seconds since epoch to use for internal alarm
 *         board      - board number to simulate alarm from
 *         text       - text of alarm
 *
 * Output: internally generated alarm processed
 *****************************************************************************/
void InternalAlarm( U32 secs_epoch, U16 board, S8 *text )
{
    CPIPKT              pkt;
    TXLOG_ALARM_MSG    *msg = (TXLOG_ALARM_MSG *)pkt.data;
    char                outLine[256];

    memset( &pkt.hdr, 0, sizeof(CPIHDR) );
    pkt.hdr.src = PORT(board, PT_INF);          /* simulated source of alarm */
    pkt.hdr.dst = PORT(0, PT_INF);              /* simulated dest of host (board 0) */
    pkt.hdr.length = sizeof(CPIHDR) + sizeof(TXLOG_ALARM_MSG);

    memset( &msg->hdr, 0, sizeof(TXLOG_HDR) );
    msg->hdr.secs_epoch = cpi_htocp_l( secs_epoch );
    msg->hdr.logs_lost = cpi_htocp_l( 0 );
    msg->alarm.number = cpi_htocp_s( 0 );                   /* not a real alarm so use alarm number 0 */
    msg->alarm.severity = cpi_htocp_s( TXALARM_WARNING );   /* all internally generated alarms are warnings */
    strcpy( msg->hdr.task_name, "INTERNAL" );               /* use name of source task to show internally generated */
    strcpy( msg->alarm.message, "INTERNAL" );   /* message starts with source task [LEGACY reasons] */
    strcpy( &msg->alarm.message[strlen(msg->alarm.message) + 1], text );    /* place text after task_name NULL-terminator */

    ProcessAlarm( &pkt, outLine, display_options );

    printf(outLine);

    /* if logging to an open file, write alarm message there too */
    if (fileDesc)
        fprintf(fileDesc, outLine);
}

/*****************************************************************************
 * CheckLostAlarms - handle any indication of lost alarms
 *
 * Inputs: msg     - received alarm message (or NULL if receive error)
 *         board   - board number that message received from
 *
 * Output: internal 'alarms lost' alarm generated if indicated
 *****************************************************************************/
void CheckLostAlarms( TXLOG_ALARM_MSG *msg, U16 board )
{
    S8      internal_msg[80];

    if (msg == NULL)
        return;         /* receive error so no alarm message to check */

    if (alarms_lost[board] == 0)
    {   /* no alarms previously reported lost by this board */
        if (msg->hdr.logs_lost != 0)
        {   /* initial report of lost messages */
            sprintf( internal_msg, "WARNING: %d alarm messages reported lost",
                     msg->hdr.logs_lost );
            InternalAlarm( msg->hdr.secs_epoch, board, internal_msg );
        }
    }
    else if (msg->hdr.logs_lost != 0)
    {   /* alarms previously reported as lost & this alarm reporting alarms lost */
        if (msg->hdr.logs_lost < alarms_lost[board])
        {   /* board must have been rebooted and then immediately lost alarms */
            sprintf( internal_msg, "WARNING: %d alarm messages lost during reboot",
                     msg->hdr.logs_lost );
            InternalAlarm( msg->hdr.secs_epoch, board, internal_msg );
        }
        else if (msg->hdr.logs_lost > alarms_lost[board])
        {   /* new report indicating more messages were lost */
            sprintf( internal_msg, "WARNING: %d more alarm messages lost (total of %d lost)",
                     msg->hdr.logs_lost - alarms_lost[board], msg->hdr.logs_lost );
            InternalAlarm( msg->hdr.secs_epoch, board, internal_msg );
        }
        /* NOTE: if reported count == stored count then no more alarms have been lost */
    }

    alarms_lost[board] = msg->hdr.logs_lost;    /* save last reported count */
}

/*****************************************************************************
 * GetAlarm
 *****************************************************************************/
#ifdef WIN32
DWORD NMSSTDCALL GetAlarm(CTAQUEUEHD hmiQueue, CTA_WAITOBJ waitObj, void *arg)
#else
DWORD NMSSTDCALL GetAlarm(CTAQUEUEHD hmiQueue, CTA_WAITOBJ * waitObj,
                          void *arg)
#endif
{
    CPIPKT              inbuf;
    TX_HANDLE           handle = *(TX_HANDLE *) arg;
    S16                 ret, len = CPIBUFSIZE;
    char                outLine[256];
    TXLOG_ALARM_MSG    *msg;

    if ((ret = cpi_get_data(handle, &inbuf, &len)) == CPI_ERROR)
    {
        printf("Receive Error (%d)\n", cpi_get_last_error());
        return (SUCCESS);
    }

    /* process received alarm (and format alarm output string) */
    msg = ProcessAlarm( &inbuf, outLine, display_options );

    printf(outLine);

    /* if logging to an open file, write alarm message there too */
    if (fileDesc)
        fprintf(fileDesc, outLine);

    /* handle any indication of lost alarms */
    CheckLostAlarms( msg, (U16)BOARD(inbuf.hdr.src) );

    return (SUCCESS);
}

/*****************************************************************************
 * DisplayBanner - display banner header lines describing each output column
 *****************************************************************************/
void DisplayBanner( )
{
    char        outLine[256],underLine[256];
    U16         index;

    /* get the names of all output columns (based on display options) */
    ProcessAlarm( NULL, outLine, display_options );

    /* underline each column name */
    index = 0;
    while (outLine[index] != '\n')
    {
        if (outLine[index] == ' ')
            underLine[index] = ' ';
        else
            underLine[index] = '=';
        index++;
    }

    /* extend underline of last column to 80-character width */
    underLine[index] = 0;
    while (strlen(underLine) < 80)
        strcat( underLine, "=" );
    strcat( underLine, "\n" );

    printf(outLine);
    printf(underLine);

    /* if logging to an open file, write alarm message there too */
    if (fileDesc)
    {
        fprintf(fileDesc, outLine);
        fprintf(fileDesc, underLine);
    }
}

/*****************************************************************************
 * HmiHandler
 *****************************************************************************/
short HmiHandler(CTAQUEUEHD hmiQueue)
{
    CTA_EVENT       ctaEvent = { 0 };
    DWORD           ret;
    char            tmpbuf[80];
    short           result = 0;

    while (result == 0)
    {
        ctaWaitEvent(hmiQueue, &ctaEvent, CTA_WAIT_FOREVER);
        switch (ctaEvent.id)
        {
            case HMI_EVN_STOP:
                {
                    if (fds[ctaEvent.value] != CPI_INVALID_HANDLE)
                    {
                        /*
                         * DO NOT CHANGE THE ORDER OF THE CODE IN THIS BLOCK !!!!!
                         * doing so will have the effect of causing CT Access to
                         * not really allow further registrations of wait objects
                         * (the call to ctaRegisterWaitObject will succeed, but
                         * the object never gets signaled!)
                         */
                        cpi_close(fds[ctaEvent.value]);
                        if ((ret =
                             ctaUnregisterWaitObject(hmiQueue,
                                                     &wobjs[ctaEvent.
                                                            value])) !=
                            SUCCESS)
                        {
                            ctaGetText(NULL_CTAHD, ret, tmpbuf,
                                       sizeof(tmpbuf));
                            fprintf(stderr,
                                    "ctaUnregisterWaitObject() failed: %s\n",
                                    tmpbuf);
                        }
                        fds[ctaEvent.value] = CPI_INVALID_HANDLE;
                    }
                    break;
                }
            case HMI_EVN_INSERT:
            case HMI_EVN_STARTING:
            case HMI_EVN_HALTED:
            case HMI_EVN_NOWPRIMARY:
            case HMI_EVN_NOWBACKUP:
            case HMI_EVN_NOWSTANDALONE:
            case HMI_EVN_TASKDEAD:
                {
                    if (cps[ctaEvent.value])
                    {
                        if (fds[ctaEvent.value] == CPI_INVALID_HANDLE)
                        {
                            short           port =
                                PORT((short)ctaEvent.value, chan);

                            if ((fds[ctaEvent.value] =
                                 cpi_open(port, CPIM_PORT,
                                          NULL)) == CPI_INVALID_HANDLE)
                            {
                                printf("txalarm : could not open driver %d\n",
                                       cpi_get_last_error());
                            }
                            else
                            {
#ifdef WIN32
                                wobjs[ctaEvent.value] =
                                    cpi_wait_obj(fds[ctaEvent.value]);
#else
                                wobjs[ctaEvent.value].fd =
                                    cpi_wait_obj(fds[ctaEvent.value]);
                                wobjs[ctaEvent.value].events =
                                    (POLLIN | POLLPRI);
                                wobjs[ctaEvent.value].revents = 0;
#endif
                                if ((ret =
                                     ctaRegisterWaitObject(hmiQueue,
                                                           &wobjs[ctaEvent.
                                                                  value],
                                                           GetAlarm,
                                                           &fds[ctaEvent.
                                                                value],
                                                           CTA_PRIORITY_HIGH))
                                    != SUCCESS)
                                {
                                    ctaGetText(NULL_CTAHD, ret, tmpbuf,
                                               sizeof(tmpbuf));
                                    fprintf(stderr,
                                            "ctaRegisterWaitObject() failed: %s\n",
                                            tmpbuf);
                                }
                            }
                        }
                    }
                }
        }
    }
    return (result);
}

/*****************************************************************************
 * InitHMI
 *****************************************************************************/
CTAQUEUEHD InitHMI(void)
{
    DWORD           ret;
    CTAQUEUEHD      hmiQueue = 0;   /* controller FSM queue handle */
    CTAHD           hmiHndl = NULL_CTAHD;   /* context handle for async events */
    char            tmpbuf[80];
    CTA_INIT_PARMS  hmiInitparms = { 0 };
    CTA_EVENT       ctaEvent = { 0 };

    /* Initialize CT Access health management (HMI) service */
    /*hmiInitparms.traceflags = CTA_TRACE_ENABLE; */  /* enable event tracing */
    hmiInitparms.size = sizeof(CTA_INIT_PARMS);
    hmiInitparms.ctacompatlevel = CTA_COMPATLEVEL;

    if ((ret = ctaInitialize(HmiServiceNames, 1, &hmiInitparms)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, tmpbuf, sizeof(tmpbuf));
        fprintf(stderr,
                "txalarm terminating, error initializing CT Access: %s\n",
                tmpbuf);
        return NULL_CTAQUEUEHD;
    }

    /* create the CT Access queue used to receive events from HMI */
    if ((ret = ctaCreateQueue(NULL, 0, &hmiQueue)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, tmpbuf, sizeof(tmpbuf));
        fprintf(stderr,
                "txalarm terminating, error creating CT Access queue: %s\n",
                tmpbuf);
        return NULL_CTAQUEUEHD;
    }

    /* create the controller FSM CT Access context for HMI async events
     * and open the HMI async events service on that context
     */
    if ((ret = ctaCreateContext(hmiQueue, 0, "DEMO", &hmiHndl)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, tmpbuf, sizeof(tmpbuf));
        fprintf(stderr,
                "txalarm terminating, error creating CT Access context: %s\n",
                tmpbuf);
        return NULL_CTAQUEUEHD;
    }

    /* open the specified HMI service on the context just created */
    HmiOpenSvcLst[0].svcargs.args[0] = 0;
    HmiOpenSvcLst[0].svcargs.args[1] = HMI_EVENTS_ALL_BOARDS;
    HmiOpenSvcLst[0].svcargs.args[2] = HMI_RCV_EVENTS;

    if ((ret = ctaOpenServices(hmiHndl, &HmiOpenSvcLst[0], 1)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, tmpbuf, sizeof(tmpbuf));
        fprintf(stderr,
                "Error [%s] opening HMI service, hot swap functionality disabled\n",
                tmpbuf);
    }
    else if ((ret =
              ctaWaitEvent(hmiQueue, &ctaEvent, CTA_WAIT_FOREVER)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, tmpbuf, sizeof(tmpbuf));
        fprintf(stderr,
                "Error [%s] opening HMI service, hot swap functionality disabled\n",
                tmpbuf);
    }
    else if ((ctaEvent.id != CTAEVN_OPEN_SERVICES_DONE) ||
             (ctaEvent.value != CTA_REASON_FINISHED))
    {
        ctaGetText(NULL_CTAHD, ret, tmpbuf, sizeof(tmpbuf));
        fprintf(stderr,
                "Error [%s] opening HMI service, hot swap functionality disabled\n",
                tmpbuf);
    }

    return (hmiQueue);
}

/*****************************************************************************
 * main
 *****************************************************************************/
int main(argc, argv)
    int             argc;
    char           *argv[];
{
    short           i;
    char           *myname;
    char           *str = NULL;
    char            sbuf[CPSTRSIZE], *vstr = sbuf;  /* dummy for old cpi_init */
    short           port;
    char            fileName[FILENAME_MAX];
    CTAQUEUEHD      hmiQueue = 0;   /* controller FSM queue handle */
    CTA_EVENT       ctaEvent = { 0 };
    DWORD           ret;
    char            tmpbuf[80];

#ifdef UNIX
    struct pollfd   stdInPoll;
#endif

    printf("\n");
    printf("Alarm Monitor V%d.%d : Hit Return to exit\n", MH_VERSION, MH_REVISION);
    printf("(c)Copyright 1995-2009 Dialogic Corporation. All Rights Reserved\n");
    printf("\n");

    /* process command line */
    myname = argv[0];

    i = 1;
    while (i < argc)
    {
        str = argv[i];
        if (*str == '-')
        {
            str++;
            switch (*str)
            {
                case 'c':          /* which channel should we speak through */
                case 'C':
                    str++;
                    if (isdigit(*str))
                        chan = atoi(str);
                    else
                    {
                        i++;
                        if ( (i < argc) && (isdigit(*argv[i])) )
                            chan = atoi(argv[i]);
                        else
                        {
                            printf("\nError: expecting channel number after -c option.\n");
                            exit(1);
                        }
                    }
                    break;

                case 'f':
                case 'F':
                    str++;
                    if (*str != '\0')
                    {
                        strcpy(fileName, str);
                    }
                    else
                    {
                        i++;
                        if (i < argc)
                            strcpy(fileName, argv[i]);
                        else
                        {
                            printf("\nError: expecting filename after -f option.\n");
                            exit(1);
                        }
                    }
                    fileDesc = fopen(fileName, "w");
                    if (!fileDesc)
                    {
                        printf("Cannot open file %s\n", fileName);
                        printf(usagestr, myname);
                        exit(1);
                    }
                    break;

                case 's':          /* display alarm severity as part of alarm message output */
                case 'S':
                    display_options |= TXALM_DISPLAY_SEVERITY;
                    break;

                default:
                    printf(usagestr, myname);
                    exit(1);
            }
        }
        else
        {
            printf(usagestr, myname);
            exit(1);
        }
        ++i;
    }

    if (chan < 1)
    {
        printf(usagestr, argv[0]);
        exit(2);
    }

    /* initialize libraries and open channel to each TX board */
    if ((hmiQueue = InitHMI()) == NULL_CTAQUEUEHD)
        exit(3);

    cpi_init(0, &vstr);

    cpi_get_resources(MAXBOARD, cps);

    for (i = 0; i <= MAXBOARD; ++i)
    {
        if (cps[i])
        {
            port = PORT(i, chan);
            if ((fds[i] =
                 cpi_open(port, CPIM_PORT, NULL)) == CPI_INVALID_HANDLE)
            {
                printf("%s : could not open driver %d\n", argv[0],
                       cpi_get_last_error());
            }
            else
            {
#ifdef WIN32
                wobjs[i] = cpi_wait_obj(fds[i]);
#else
                wobjs[i].fd = cpi_wait_obj(fds[i]);
                wobjs[i].events = (POLLIN | POLLPRI);
                wobjs[i].revents = 0;
#endif
                if ((ret =
                     ctaRegisterWaitObject(hmiQueue, &wobjs[i], GetAlarm,
                                           &fds[i],
                                           CTA_PRIORITY_NORMAL)) != SUCCESS)
                {
                    ctaGetText(NULL_CTAHD, ret, tmpbuf, sizeof(tmpbuf));
                    fprintf(stderr, "ctaRegisterWaitObject() failed: %s\n",
                            tmpbuf);
                }

                alarms_lost[i] = 0;         /* no alarms have been reported lost */
            }
        }
        else
            fds[i] = CPI_INVALID_HANDLE;
    }

    /* register for keyboard input */
#ifdef WIN32
    stdInput = GetStdHandle(STD_INPUT_HANDLE);
    SetConsoleMode(stdInput,
                   ENABLE_LINE_INPUT | ENABLE_ECHO_INPUT |
                   ENABLE_PROCESSED_INPUT);
    ctaRegisterWaitObject(hmiQueue, &stdInput, ExitRoutine,
                          (void *)CPI_INVALID_HANDLE, CTA_PRIORITY_NORMAL);
#else
    stdInPoll.fd = 0;               /* This is stdin */
    stdInPoll.events = (POLLIN | POLLPRI);
    stdInPoll.revents = 0;
    ctaRegisterWaitObject(hmiQueue, &stdInPoll, ExitRoutine,
                          (void *)CPI_INVALID_HANDLE, CTA_PRIORITY_NORMAL);
#endif

    /* display banner */
    DisplayBanner( );

    /* enter main dispatch loop */
    HmiHandler(hmiQueue);

    /* close all the handles */
    for (i = 1; i <= MAXBOARD; ++i)
        if (fds[i] != CPI_INVALID_HANDLE)
            cpi_close(fds[i]);

    if (fileDesc)
        fclose(fileDesc);

    return (0);
}
