/* rmgctlr.c
 *
 * RMG controller state machine (RMGC)
 *
 * This module provides the overall control of the board being
 * managed by this RMG process, based on the current state of the board,
 * the current state of the mate board/node, and local user requests.
 *
 * This module is organized as a state machine, with states reflecting the state
 * of this signaling board/node.  Each state is comprised of a separate function
 * which handles all possible events for that state.  Events handled include
 * messages from the mate node, local board status events, internal timers,
 * and user requests (changeover, reset, shutdown).
 *
 */
#include <string.h>

#include "osdep.h"
#include "ctadef.h"
#include "ctaerr.h"
#include "nmstypes.h"
#include "ss7type.h"
#include "rmgdefs.h"
#include "hmiapi.h"

/* process wide globals "owned" by the RMGC module */
U8   RmgState;                  /* current state */
U8   RmgStandAlone = 0;         /* DEFAULT: input parameters DO NOT imply we are standalone */
U32  RmgTxHeartbeats;           /* (RMGC) Number of heartbeat messages transmitted... */
                                /* ...since last becoming active */
U32  RmgRxHeartbeats;           /* (RMGC) Number of heartbeat messages received... */
                                /* ...since last becoming backup */
U8   RmgNetworkStatus = RMG_NETWORK_UNKNOWN;    /* network connectivity status has not been reported */
U8   RmgRequestRetry = 0;       /* set after an initial attempt has been made to send any request to mate */

RMG_ERROR_SET   RmgError;       /* set of control errors that are tracked globally */

/* misc. globals - local to this module */
static U8   Rmgc_shutdown = 0;          /* shutdown in progress indicator */

static char emsg[132];                  /* buffer for alarms */

S8 Rmgc_defaultReason[]         = "internal decision";      /* reason for switchover always set to this default... should always be replaced with more specific reason */

/* ----- choices for reason that local vs mate node primary selection was made ----- */
S8 Rmgc_primaryPickMateExtract[]    = "mate node is extracting";
S8 Rmgc_primaryPickMateUp[]         = "mate node has better network connectivity";
S8 Rmgc_primaryPickLocalUp[]        = "local node has better network connectivity";
S8 Rmgc_primaryPickMateOper[]       = "mate node primary due to operator";
S8 Rmgc_primaryPickLocalOper[]      = "local node primary due to operator";
S8 Rmgc_primaryPickMateCurr[]       = "mate node remains primary";
S8 Rmgc_primaryPickLocalCurr[]      = "local node remains primary";
S8 Rmgc_primaryPickMatePrio[]       = "mate node has precedence";
S8 Rmgc_primaryPickLocalPrio[]      = "local node has precedence";


/* Timers */
#define RMGC_MAX_TIMERS 4
#define RMGC_TIMER_T1   0               /* response timer for request msgs */
#define RMGC_TIMER_T2   1               /* Deadman timer for heartbeats from mate */
#define RMGC_TIMER_T3   2               /* periodic heartbeat send timer */
#define RMGC_TIMER_T4   3               /* board download timer */

typedef struct
{
    char            *timerName;         /* timer name for easy printing */
    U32             currentVal;         /* current value (msecs) */
    TMTimerId       hTimer;             /* handles from RmgStartTimer */
    DWORD           event;              /* event to queue when timer fires */
} RMGC_TIMER_DESC;

RMGC_TIMER_DESC Rmgc_timers[ RMGC_MAX_TIMERS ] = {
/* Name         Value       Handle  Event */
/* ----         -----       ------  ----- */
{"TIMER_T1",    3000,       0,      RMG_EV_T1   },
{"TIMER_T2",    3000,       0,      RMG_EV_T2   },
{"TIMER_T3",    500,        0,      RMG_EV_T3   },
{"TIMER_T4",    30000,      0,      RMG_EV_T4   }
};

typedef struct
{
    DWORD           errorCode;          /* error code */
    char            *errorName;         /* name of error code */
} RMGC_ERROR_DESC;

RMGC_ERROR_DESC Rmgc_hmiError[] =
{
    { HMI_ERR_CLOSED   ,  "HMI_ERR_CLOSED"    },
    { HMI_ERR_BADCMD   ,  "HMI_ERR_BADCMD"    },
    { HMI_ERR_TIMEOUT  ,  "HMI_ERR_TIMEOUT"   },
    { HMI_ERR_INTERROR ,  "HMI_ERR_INTERROR"  },
    { HMI_ERR_INVHANDLE,  "HMI_ERR_INVHANDLE" },
    { HMI_ERR_REFUSED  ,  "HMI_ERR_REFUSED"   },
    { HMI_ERR_NOMEM    ,  "HMI_ERR_NOMEM"     },
    { 0, NULL }
};

char *RmgcMsgFailReason[] =
{
    "no reason provided",           /*  RMG_REASON_SUCCESS */
    "node's network is not up",     /*  RMG_REASON_NETWORK_DOWN */
    "operator control override",    /*  RMG_REASON_OPERATOR_CMD */
    "node is to remain primary",    /*  RMG_REASON_REMAIN_PRIMARY */
    "node number precedence",       /*  RMG_REASON_NODE_PRECEDENCE */
    "node's board is loading",      /*  RMG_REASON_LOADING */
    "node out of service",          /*  RMG_REASON_OOS */
    "node's board is extracting",   /*  RMG_REASON_EXTRACTING */
    "node's board is extracted"     /*  RMG_REASON_EXTRACTED */
};

/* state handler function prototypes */
static short RmgcInitial( DWORD event, DWORD value );
static short RmgcStarting( DWORD event, DWORD value );
static short RmgcStartDown( DWORD event, DWORD value );
static short RmgcStartUp( DWORD event, DWORD value );
static short RmgcLoading( DWORD event, DWORD value );
static short RmgcActive( DWORD event, DWORD value );
static short RmgcActiveSwreq( DWORD event, DWORD value );
static short RmgcActiveDown( DWORD event, DWORD value );
static short RmgcBackup( DWORD event, DWORD value );
static short RmgcTkreq( DWORD event, DWORD value );
static short RmgcTkreqUp( DWORD event, DWORD value );
static short RmgcOOS( DWORD event, DWORD value );
static short RmgcStandalone( DWORD event, DWORD value );
static short RmgcExtracting( DWORD event, DWORD value );
static short RmgcExtracted( DWORD event, DWORD value );
static short RmgcFrozen( DWORD event, DWORD value );

/* table of state names, handler routines, for ease of dispatching/tracing */
RMGC_STATE_DESC RmgcStates[] =
{
    { "INITIAL",        RmgcInitial     },  /* initial state entered on startup - waiting for HMI to tell if board loaded */
    { "STARTING",       RmgcStarting    },  /* board downloaded; ActvRqst sent to mate; waiting response */
    { "START_UP",       RmgcStartUp     },  /* starting when network came up (waiting for activation response before proceeding) */
    { "START_DOWN",     RmgcStartDown   },  /* starting when network went down (waiting for activation response before proceeding) */
    { "LOADING",        RmgcLoading     },  /* downloading board; waiting for HMI to indicate board STARTED */
    { "ACTIVE",         RmgcActive      },  /* this node is active (primary); monitoring backup RMG */
    { "ACTIVE_SWRQ",    RmgcActiveSwreq },  /* this node was active; switchover request sent to mate; waiting response */
    { "ACTIVE_DOWN",    RmgcActiveDown  },  /* this node was active; NetDown [optional switchover] request sent to mate; waiting response */
    { "BACKUP",         RmgcBackup      },  /* this node in standby mode (backup); monitoring primary RMG */
    { "TKOV_RQST",      RmgcTkreq       },  /* this node was backup; takeover request sent to mate; waiting response */
    { "TKOV_UP",        RmgcTkreqUp     },  /* this node was backup; NetUp [optional takeover] request sent to mate; waiting response */
    { "OOS",            RmgcOOS         },  /* board failed and could not be reloaded (or manually halted) */
    { "STANDALONE",     RmgcStandalone  },  /* this node/board is not redundant; monitoring for failures */
    { "EXTRACTING",     RmgcExtracting  },  /* board extraction in progress */
    { "EXTRACTED",      RmgcExtracted   },  /* board has been extracted */
    { "FROZEN",         RmgcFrozen      }   /* event occurred which stops further RMG processing (debug required) */
};

/* MACRO to send a Heartbeat message and increment the appropriate counter */
#define SEND_HEARTBEAT                     \
{                                          \
    RmgCommSend( RMG_EV_HEARTBEAT, 0, 0 ); \
    RmgTxHeartbeats++;                     \
}

/* MACRO to generate an internal error alarm with line number */
#define INT_ERROR( text, errval )                                                       \
    {                                                                                   \
        sprintf( emsg, "ERROR:   Internal Error %d: %s [%d]", __LINE__, text, errval ); \
        ctaLogTrace ( CtlrFsmCmdHd, CTA_TRACE_SEVERITY_ERROR, CTA_TRACETAG_STRING,      \
                      emsg, strlen( emsg ) );                                           \
        sprintf( emsg, "%s [error %d (0x%X)]", text, errval, errval );                  \
        LogError( "INT_ERROR", emsg );                                                  \
    }

/* MACRO to generate an internal warning alarm with line number */
#define INT_WARN( text, errval )                                                        \
    {                                                                                   \
        sprintf( emsg, "WARNING: Internal Error %d: %s [%d]", __LINE__, text, errval ); \
        ctaLogTrace ( CtlrFsmCmdHd, CTA_TRACE_SEVERITY_WARNING, CTA_TRACETAG_STRING,    \
                      emsg, strlen( emsg ) );                                           \
        sprintf( emsg, "%s [error %d (0x%X)]", text, errval, errval );                  \
        LogError( "INT_WARN", emsg );                                                   \
    }

/* MACRO to generate a unexpected event for current state alarm */
#define UNEX_EVENT_ALARM( eventNo )                                                     \
    {                                                                                   \
        sprintf( emsg, "ALARM:   Unexpected event %s in state %s",                      \
                 LogNameEvent( eventNo ), LogNameState( RmgState ) );                   \
        ctaLogTrace ( CtlrFsmCmdHd, CTA_TRACE_SEVERITY_WARNING, CTA_TRACETAG_STRING,    \
                      emsg, strlen( emsg ) );                                           \
        sprintf( emsg, "event %s in state %s",                                          \
                 LogNameEvent( eventNo ), LogNameState( RmgState ) );                   \
        LogError( "UNEXPECTED", emsg );                                                 \
    }

/* MACRO to generate a debugging (level 3) alarm */
#define DEBUG_ALARM( text )                                                             \
    {                                                                                   \
        sprintf( emsg, "DEBUG:   %s", text );                                           \
        ctaLogTrace ( CtlrFsmCmdHd, CTA_TRACE_SEVERITY_INFO, CTA_TRACETAG_STRING,       \
                      emsg, strlen( emsg ) );                                           \
        LogProgress( "DEBUG", text );                                                   \
    }

/* forward function declarations */
static short RmgcInit( void );
static void RmgcCleanup( void );
static void RmgcStartTimer( U16 timer );
static void RmgcCancelTimer( U16 timer );
static short RmgcDispatchFsm( U8 state, DWORD event, DWORD value );

/*
 * RmgCfsmThread
 *
 * This is the main entry point for the RMG controller FSM thread.
 * This routine handles all the thread initialization and then loops
 * continuously waiting for external events until a shutdown event
 * is received.
 *
 * Parameters:
 *
 *  pNotUsed        not used
 *
 * Returns:
 *
 *  0               Always
 */
void RmgCfsmThread( void *pNotUsed )
{
    CTA_EVENT   event;
    DWORD       ret;
    short       done = 0;
    char        emsg[80];               /* buffer for constructing alarm msgs */
    short       rc;                     /* return code from fsm */

    /* first do the one-time intialization, performed only when the
     * thread is created;
     */
    DEBUG_ALARM( "RMG Controller FSM started" );
    if (RmgcInit( ) != 0)
    {
        Rmgc_shutdown = 1;      /* alarms already generated by called routines */
    }

    /* Main loop - this loop is executed until a "shutdown" event is received
     * or a fatal error occurs.
     */
    while (!Rmgc_shutdown)
    {
        ret = ctaWaitEvent( CtlrFsmQueue, &event, CTA_WAIT_FOREVER );
        if (ret == SUCCESS)
        {
            rc = RmgcDispatchFsm( RmgState, event.id, event.value );
            if (rc < 0)
            {   /* fatal error - die */
                sprintf( emsg, "error (%d) returned from RmgcDispatchFsm", rc );
                LogError( "RmgCfsmThread", emsg );
                if (!(RmgRequireQuit))
                    Rmgc_shutdown = 1;
            }
        }
        else
        {
            ctaGetText( NULL_CTAHD, ret, emsg, sizeof( emsg ) );
            INT_ERROR( "RMGC Wait Event Error", ret );
            LogError( "RmgCfsmThread", emsg );
            if (!(RmgRequireQuit))
                Rmgc_shutdown = 1;
        }
    }

    /* final cleanup before exiting thread */
    RmgcCleanup( );

    THREAD_EXIT( );
}


/*
 * RmgGetTimerDesc
 *
 * Return timer descriptor associated with provided timer event.
 *
 * Parameters:      timer event
 *
 * Returns:         timer descriptor associated with timer event
 *                  or NULL if event is not a valid timer event
 */
RMGC_TIMER_DESC *RmgGetTimerDesc( DWORD event )
{
    DWORD   i;

    for( i = 0; i < RMGC_MAX_TIMERS; i++ )
    {
        if( Rmgc_timers[i].event == event )
            return( &Rmgc_timers[i] );
    }

    return( NULL );     /* event is not a timer event */
}


/*
 * RmgHmiResult
 *
 * if the result of an HMI function call is not SUCCESS then log the error
 *
 * Parameters:      hmi_func   - name of HMI function that returned the error
 *                  hmi_result - result returned from an HMI function
 */
void RmgHmiResult( char *hmi_func, DWORD hmi_result, RMG_ERROR_INFO *errorInfo )
{
    char            unknown_error[32];
    char            *error_name = unknown_error;     /* default: error name unknown */
    RMGC_ERROR_DESC *errorDesc = Rmgc_hmiError;
    char            error_message[80];

    if( hmi_result != SUCCESS )
    {   /* log the error returned from HMI */

        while ( (errorDesc->errorName != NULL) &&
                (hmi_result != errorDesc->errorCode) )
            errorDesc++;

        if( errorDesc->errorName == NULL )
            sprintf( error_name, "error 0x%X", hmi_result );
        else
            error_name = errorDesc->errorName;

        sprintf( error_message, "error %s returned from %s",
                 error_name, hmi_func );
        LogError( LogNameState( RmgState ), error_message );
        if (hmi_result != errorInfo->lastError)
        {   /* different error than last reported */
            INT_ERROR( error_message, hmi_result );
        }
        errorInfo->errorCount++;
        errorInfo->lastError = hmi_result;
        errorInfo->lastInfo = 0;
    }
}


/*
 * RmgHmiLoadBoard
 *
 * Call HMI function: hmiLoadBoard and log any error.
 *
 * Parameters:      NONE
 *
 * Returns: result returned by hmiLoadBoard
 */
DWORD RmgHmiLoadBoard( void )
{
    DWORD   ret;

    ret = hmiLoadBoard( CtlrFsmCmdHd );
    RmgHmiResult( "hmiLoadBoard", ret, &RmgError.hmiLoadBoard );

    return( ret );
}


/*
 * RmgHmiHaltBoard
 *
 * Call HMI function: hmiHaltBoard and log any error.
 *
 * Parameters:      NONE
 *
 * Returns: result returned by hmiHaltBoard
 */
DWORD RmgHmiHaltBoard( void )
{
    DWORD   ret;

    ret = hmiHaltBoard( CtlrFsmCmdHd );
    RmgHmiResult( "hmiHaltBoard", ret, &RmgError.hmiHaltBoard );

    return( ret );
}


/*
 * RmgHmiStop
 *
 * Call HMI function: hmiStop and log any error.
 *
 * Parameters:      NONE
 *
 * Returns: result returned by hmiStop
 */
DWORD RmgHmiStop( void )
{
    DWORD   ret;

    ret = hmiStop( CtlrFsmCmdHd );
    RmgHmiResult( "hmiStop", ret, &RmgError.hmiStop );

    return( ret );
}


/*
 * RmgHmiStart
 *
 * Call HMI function: hmiStart and log any error.
 *
 * Parameters:      NONE
 *
 * Returns: result returned by hmiStart
 */
DWORD RmgHmiStart( void )
{
    DWORD   ret;

    ret = hmiStart( CtlrFsmCmdHd );
    RmgHmiResult( "hmiStart", ret, &RmgError.hmiStart );

    return( ret );
}


/*
 * RmgHmiBackup
 *
 * Call HMI function: hmiBackup and log any error.
 *
 * Parameters:      NONE
 *
 * Returns: result returned by hmiBackup
 */
DWORD RmgHmiBackup( void )
{
    DWORD   ret;

    ret = hmiBackup( CtlrFsmCmdHd );
    RmgHmiResult( "hmiBackup", ret, &RmgError.hmiBackup );

    return( ret );
}


/*
 * RmgHmiPrimary
 *
 * Call HMI function: hmiPrimary and log any error.
 *
 * Parameters:      NONE
 *
 * Returns: result returned by hmiPrimary
 */
DWORD RmgHmiPrimary( void )
{
    DWORD   ret;

    ret = hmiPrimary( CtlrFsmCmdHd );
    RmgHmiResult( "hmiPrimary", ret, &RmgError.hmiPrimary );

    return( ret );
}


/*
 * RmgHmiStandalone
 *
 * Call HMI function: hmiStandalone and log any error.
 *
 * Parameters:      NONE
 *
 * Returns: result returned by hmiStandalone
 */
DWORD RmgHmiStandalone( void )
{
    DWORD   ret;

    ret = hmiStandalone( CtlrFsmCmdHd );
    RmgHmiResult( "hmiStandalone", ret, &RmgError.hmiStandalone );

    return( ret );
}


/*
 * RmgcInit
 *
 * This routine performs RMGC machine initialization during startup, reset, and
 * possibly during error recovery.  It initializes global variables, sends an
 * ActvRqst message to the mate node and sets the RMGC machine to the STARTING
 * state.
 *
 * Parameters:  none
 *
 * Returns:
 *
 *  0           Successful
 *  -1          Failed (alarm message has already been logged)
 */
static short RmgcInit( void )
{
    /* initialize globals back to initial values */
    Rmgc_shutdown   = 0;    /* no shutdown in progress */
    RmgRequestRetry = 0;    /* no request retry in progress */

    /* reset statistics */
    RmgTxHeartbeats = 0;
    RmgRxHeartbeats = 0;

    /* successful initialization */
    LogStateChange( RMGC_STATE_INITIAL );
    return( 0 );
}                               /* of RmgcInit */

/*
 * RmgcSetShutdown
 *
 * This routine sets a global that is used to trigger all RMG threads
 * to shutdown.
 *
 * NOTE: The shutdown trigger is not set if RMG was run with the -q option
 *       (indicating that RMG should not shutdown unless then operator
 *        enter the 'quit' command).
 *
 * Parameters:  none
 *
 * Returns:     void
 */
static void RmgcSetShutdown( void )
{
    if (!(RmgRequireQuit))
    {
        Rmgc_shutdown = 1;
    }
}


/*
 * RmgcCleanup
 *
 * This routine performs RMGC machine clean up for use during reset or error
 * recovery, when re-initialization and continued operation is desired.  It
 * currently just cancels all outstanding timers.
 *
 * Parameters:  none
 *
 * Returns:     void
 *
 */
static void RmgcCleanup( void )
{
    short   i;

    /* cancel all outstanding timers */
    for( i = 0; i < RMGC_MAX_TIMERS ; i++ )
    {
        if( Rmgc_timers[i].hTimer != 0 )
            RmgStopTimer( Rmgc_timers[i].hTimer );
    }

}                               /* of RmgcCleanup */


/*
 * RmgcStartTimer
 *
 * This routine starts the specified timer.
 *
 * Parameters:
 *
 *  timer       index into timer table for timer to start
 *
 * Returns:     void
 */
static void RmgcStartTimer( U16 timer )
{
    RMGC_TIMER_DESC *pTimer;

    if (timer >= RMGC_MAX_TIMERS)
    {
        INT_ERROR( "Starting invalid timer index", timer );
        RmgError.startTimer.errorCount++;
        RmgError.startTimer.lastError = 1;
        RmgError.startTimer.lastInfo = (U32)timer;
    }
    else
    {
        pTimer = &Rmgc_timers[timer];

        /* if requested timer is still running, cancel it before starting new timer */
        if (pTimer->hTimer != 0)
        {
            RmgStopTimer( pTimer->hTimer );
            pTimer->hTimer = 0;
        }

        /* start the timer */
        pTimer->hTimer = RmgStartTimer( pTimer->currentVal, pTimer->event, FALSE );
        if( pTimer->hTimer == 0 )
        {
            INT_ERROR( "Unable to start timer", timer );
            if (RmgError.startTimer.errorCount == 0)
            {   /* report first time unable to start timer */
                INT_ERROR( "Unable to starst timer", (U32)timer );
            }
            RmgError.startTimer.errorCount++;
            RmgError.startTimer.lastError = 2;
            RmgError.startTimer.lastInfo = (U32)timer;
        }
    }
}


/*
 * RmgcCancelTimer
 *
 * This routine cancels the specified timer. An error message is logged
 * but no other action if timer index is out of range.
 *
 * Parameters:
 *
 *  timer       index into timer table for timer to cancel
 *
 * Returns:     void
 *
 */
static void RmgcCancelTimer( U16 timer )
{
    RMGC_TIMER_DESC *pTimer;

    /* make sure timer index in range */
    if (timer >= RMGC_MAX_TIMERS)
    {
        INT_ERROR( "Cancelling invalid timer index", timer );
        RmgError.cancelTimer.errorCount++;
        RmgError.cancelTimer.lastError = 1;
        RmgError.cancelTimer.lastInfo = (U32)timer;
    }
    else
    {   /* valid timer index provided */
        /* cancel the timer if it's running */
        pTimer = &Rmgc_timers[timer];
        if (pTimer->hTimer != 0)
        {
            RmgStopTimer( pTimer->hTimer );
            pTimer->hTimer = 0;
        }
    }
}

/*
 * RmgcStaleTimer
 *
 * This routine issues a log indicating that a stale timer has fired
 * and counts the global control error .
 *
 * Parameters:
 *
 *  event       timer event that fired when not expected
 *
 * Returns:     void
 *
 */
static void RmgcStaleTimer( U32 event )
{
    RMGC_TIMER_DESC    *tDesc;
    char                message[128];

    tDesc = RmgGetTimerDesc( event );
    if (tDesc == NULL)
    {   /* can't validate timer descriptor so log the event as a generic unexpected event */
        UNEX_EVENT_ALARM( event );
    }
    else
    {   /* log the failed timer and count as global control error */
        sprintf( message, "stale %s (ignored)", LogNameEvent( event ) );
        LogProgress( LogNameState( RmgState ), message );
        INT_ERROR( message, tDesc->currentVal );
        RmgError.staleTimer.errorCount++;
        RmgError.staleTimer.lastError = event;
        RmgError.staleTimer.lastInfo = tDesc->currentVal;
    }
}


/*
 * RmgcUpdateNetworkStatus
 *
 * This routine maintains the network status that is reported through HMI.
 * Any time the network status changes a message is printed.
 *
 * Parameters:
 *
 *  rmgNetStatus    the currently reported network status
 *                  (or RMG_NETWORK_UNKNOWN whenever board is reloaded).
 *
 * Returns:     void
 *
 */
static void RmgcUpdateNetworkStatus( U8 rmgNetStatus )
{
    if (rmgNetStatus != RmgNetworkStatus)
    {   /* the network status (reported via HMI) has changed */
        switch (rmgNetStatus)
        {
            case RMG_NETWORK_UNKNOWN:   /* network connectivity status has not been reported */
                /* do not display this change since used as indicator of status not-yet reported */
                break;

            case RMG_NETWORK_UP:        /* network connectivity has been reported as UP */
                printf( "Board network connectivity UP\n" );
                break;

            case RMG_NETWORK_DOWN:      /* network connectivity has been reported as DOWN */
                printf( "Board network connectivity DOWN\n" );
                break;

            default:
                printf( "WARNING: Unknown network connectivity status [0x%X] reported\n", rmgNetStatus );
        }
        RmgNetworkStatus = rmgNetStatus;
    }
}

/*
 * RmgcWantMatePrimary
 *
 * This routine is called whenever a state/event occurrence does not identify
 * a clear choice as to which node should be primary and which should be backup.
 * The current network status of each node is used to make the decision.
 * If network status cannot be used to determine the best choice for primary
 * then whatever node is to be primary based on operator request is selected.
 * If no operator request is involved that whatever node is currently primary remains primary.
 * If neither node is currently primary then the node numbers are used
 * as the final selection criteria (lower node number is selected as primary in this case).
 *
 * Parameters:
 *
 *  value               value provided as parameter from FSM dispatch routine
 *                          low-order byte = the currently reported network status of the mate node
 *                          OPTIONAL flags (RMG_VAL_xxx) in higher-order bytes
 *  operatorPrimary     node number of node that an operator request indicates as primary
 *                          (0 = neither node is being requested as  primary by operator)
 *  currentPrimary      node number of node that is currently the primary
 *                          (0 = neither node is currently primary)
 *  reject              optional address to hold numeric reason for rejection [RMG_REASON_xxx]
 *                          (used as reason code in any reject response related to primary node selection)
 *  reason              optional address of pointer to reason that selection was made
 *
 * Returns:     0 = this node has been selected as the one to end up primary
 *              1 = mate node has been selected as the one to end up primary
 */
static U8 RmgcWantMatePrimary( U32 value, S16 operatorPrimary, S16 currentPrimary, U8 *reject, S8 **reason )
{
    U8  mateNetworkStatus = (U8)value;
    U8  mateSelected = 0;   /* default: this node selected to end up primary */

    if (value & RMG_VAL_MATE_EXTRACTING)
    {   /* indication that mate board is extracting (priority override) */
        if (reject != NULL)
        {   /* reason for rejection response due to this decision is that mate node is extracting */
            *reject = RMG_REASON_EXTRACTING;
        }
        if (reason != NULL)
        {
            *reason = Rmgc_primaryPickMateExtract;
        }
    }
    else if ( (mateNetworkStatus == RMG_NETWORK_UP) && (RmgNetworkStatus != RMG_NETWORK_UP) )
    {   /* mate node has network connectivity and we do not so mate should end up primary */
        mateSelected = 1;
        if (reject != NULL)
        {   /* reason for rejection response due to this decision is that local node's network is not up */
            *reject = RMG_REASON_NETWORK_DOWN;
        }
        if (reason != NULL)
        {
            *reason = Rmgc_primaryPickMateUp;
        }
    }
    else if ( (RmgNetworkStatus == RMG_NETWORK_UP) && (mateNetworkStatus != RMG_NETWORK_UP) )
    {   /* our board has network connectivity and mate node does not so this node should be primary */
        if (reject != NULL)
        {   /* reason for rejection response due to this decision is that mate node's network is not up */
            *reject = RMG_REASON_NETWORK_DOWN;
        }
        if (reason != NULL)
        {
            *reason = Rmgc_primaryPickLocalUp;
        }
    }
    else if (operatorPrimary != 0)
    {   /* an operator request has indicated which node should be primary */
        if (operatorPrimary == RmgMateNodeNum)
        {   /* the mate node is to end up primary (due to operator command) */
            if (reject != NULL)
            {   /* reason for rejection response due to this decision is that mate node is to be primary (by operator command) */
                *reject = RMG_REASON_OPERATOR_CMD;
            }
            mateSelected = 1;
            if (reason != NULL)
            {
                *reason = Rmgc_primaryPickMateOper;
            }
        }
        else
        {   /* the local node is to end up primary (due to operator command) */
            if (reject != NULL)
            {   /* reason for rejection response due to this decision is that local node is to be primary (by operator command) */
                *reject = RMG_REASON_OPERATOR_CMD;
            }
            if (reason != NULL)
            {
                *reason = Rmgc_primaryPickLocalOper;
            }
        }
    }
    else if (currentPrimary != 0)
    {   /* one node is viewed as the current primary so select that node to remain primary */
        if (currentPrimary == RmgMateNodeNum)
        {   /* the mate node is already primary so it remains primary */
            mateSelected = 1;
            if (reject != NULL)
            {   /* reason for rejection response due to this decision is that mate node is to remain primary */
                *reject = RMG_REASON_REMAIN_PRIMARY;
            }
            if (reason != NULL)
            {
                *reason = Rmgc_primaryPickMateCurr;
            }
        }
        else
        {   /* the local node is currently primary so it remains primary */
            if (reject != NULL)
            {   /* reason for rejection response due to this decision is that local node is to remain primary */
                *reject = RMG_REASON_REMAIN_PRIMARY;
            }
            if (reason != NULL)
            {
                *reason = Rmgc_primaryPickLocalCurr;
            }
        }
    }
    else
    {   /* cannot determine primary using network status so use node number precedence to decide */
        if (RmgMateNodeNum < RmgNodeNum)
        {   /* mate node has primary precedence */
            mateSelected = 1;
            if (reject != NULL)
            {   /* reason for rejection response due to this decision is that mate node has primary precedence */
                *reject = RMG_REASON_NODE_PRECEDENCE;
            }
            if (reason != NULL)
            {
               *reason = Rmgc_primaryPickMatePrio;
            }
        }
        else
        {   /* our node has primary precedence */
            if (reject != NULL)
            {   /* reason for rejection response due to this decision is that local node has primary precedence */
                *reject = RMG_REASON_NODE_PRECEDENCE;
            }
            if (reason != NULL)
            {
                *reason = Rmgc_primaryPickLocalPrio;
            }
        }
    }

    return( mateSelected );
}

/*
 * RmgcDispatchFsm
 *
 * Routine to dispatch the appropriate state-handler function for the current state,
 * with a new event.  FSM trace is implemented here.
 *
 * Parameters:
 *
 *  state       the current RMGC state
 *  event       the event to process
 *  value       data portion of the event (format varies by event)
 *
 * Returns:     void
 *  0           state handler returned zero (success)
 *  >0          non-fatal error - continue processing
 *  <0          fatal error - die
 *
 */
static short RmgcDispatchFsm( U8 state, DWORD event, DWORD value )
{
    short   rc;
    char    emsg[ 80 ];         /* buffer for constructing alarm messages */
    char   *event_msg = NULL;

    /* tracing variables, prevents excessive duplicates of timer/heartbeat events */
    static char     *pFlag = " ";
    static U8       lastState = 0;
    static DWORD    lastEvent = 0;

    /* validate that the state is in range */
    if( state > RMGC_LAST_STATE )
    {
        INT_ERROR( "RMGC invalid state", state );

        return( -1 );
    }

    /* count received heartbeats here, in central place */
    if( event == RMG_EV_HEARTBEAT )
        RmgRxHeartbeats++;

    /* implement tracing here - note, don't duplicate multiple timer T3 events or
     * received heartbeat messages, which happen frequently
     */
    if( ((event == RMG_EV_T3) || (event == RMG_EV_HEARTBEAT)) &&
        (lastEvent == event) && (lastState == state) )
    {
        /* don't trace duplicate T3 or heartbeat events, but flag next non-dup with "*" */
        pFlag = "*";
    }
    else
    {
        sprintf( emsg, "%sRMGC State: %-12.12s  Event: %-18.18s", pFlag,
                 LogNameState( state ), LogNameEvent( event ) );
        ctaLogTrace( CtlrFsmCmdHd, CTA_TRACE_SEVERITY_INFO, CTA_TRACETAG_STRING,
                     emsg, strlen( emsg ) );
        pFlag = " ";                  /* no longer in duplicate mode */
        lastEvent = event;
        lastState = state;
    }

    /* decide whether to display event to the screen */
    switch( event )
    {
        case HMI_EVN_CONNECTED:     /* HMI reporting that board is connected to its mate board */
           event_msg = "Connected to mate board";
           break;

        case HMI_EVN_ISOLATED:      /* HMI reporting that board is isolated from its mate board */
           event_msg = "Isolated from mate board";
           break;

        case HMI_EVN_NOWPRIMARY:    /* HMI reporting that board is now operating as primary */
           event_msg = "Now operating as primary";
           break;

        case HMI_EVN_NOWBACKUP:     /* HMI reporting that board is now operating as backup */
           event_msg = "Now operating as backup";
           break;

        case HMI_EVN_TASKDEAD:      /* HMI reporting that a task has failed */
           event_msg = "Task has failed";
           break;

        case HMI_EVN_BRDDEAD:       /* HMI reporting that board itself has failed */
           event_msg = "Board has failed";
           break;

        case HMI_EVN_HALTED:        /* HMI reporting that board is in the halted state */
           event_msg = "Board halted";
           break;

        case HMI_EVN_STARTING:      /* HMI reporting that board has loaded and is now starting */
           event_msg = "Now starting";
           break;

        case HMI_EVN_CONFLICT:      /* HMI reporting board conflict */
           event_msg = "Board conflict reported";
           break;

        case HMI_EVN_NOWSTANDALONE: /* HMI reporting that board is operating in standalone mode */
           event_msg = "Operating in standalone mode";
           break;

        case HMI_EVN_LOADING:       /* HMI reporting that board is loading */
           event_msg = "Loading board...";
           break;

        case HMI_EVN_SERVICE_DOWN:  /* HMI service has shutdown */
           event_msg = "HMI service has shutdown";
           break;

        case HMI_EVN_NO_BOARD:      /* HMI reporting board not present */
           event_msg = "Board not present";
           break;

        case HMI_EVN_INSERT:        /* HMI reporting that board has been inserted */
           event_msg = "Board has been inserted";
           break;

        case HMI_EVN_EXTRACT:       /* HMI reporting board extraction pending */
           event_msg = "Board extraction pending...";
           break;

        case HMI_EVN_STOP:          /* HMI reporting board has been stopped */
           event_msg = "HMI stop indicated";
           break;

        case RMG_EV_FAILURE:        /* mate node (primary) indicating that its board has failed */
           event_msg = "Mate failed";
           break;
    }
    if (event_msg != NULL)
        printf( "Board %d, Node %d  %s\n", RmgBoardNum, RmgNodeNum, event_msg );

    event = LogEvent( event );          /* add event to log [event may change to FREEZE if config'd by user] */

    /* call the state handler for this state */
    rc = RmgcStates[ state ].pHandler( event, value );

    return( rc );
}

/*************************************************************************
 * Start of actual controller state machine
 *************************************************************************/

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 * State: Initial
 *
 * Description:
 *
 * This is the initial state entered on startup; the HMI service has been opened
 * and we are waiting for an "asynchronous" API event to tell us if the board
 * has already been downloaded or not.  This is a transient state, the HMI board
 * state event should occur almost immediately. During this state, any messages
 * received from the mate node/RMG process are ignored (as though we are not
 * running yet).
 *
 * In this state, timer T4 is running; if no event is received from the HMI
 * service before timer T4 expires, then it is assumed that the board is not
 * yet loaded and a download is initiated.
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
static short RmgcInitial( DWORD event, DWORD value )
{
    switch( event )
    {
        case HMI_EVN_NOWPRIMARY:    /* HMI reporting that board is now operating as primary */
        case HMI_EVN_NOWBACKUP:     /* HMI reporting that board is now operating as backup */
        case HMI_EVN_STARTING:      /* HMI reporting that board has loaded and is now starting */
            /* The board is loaded, send activation request to mate
             * and proceed to STARTING state.
             */
            RmgcCancelTimer( RMGC_TIMER_T4 );
            RmgCommSend( RMG_EV_ACTV_RQST, 0, 0 );
            RmgcStartTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_STARTING );
            break;

        case HMI_EVN_NOWSTANDALONE: /* HMI reporting that board is operating in standalone mode */
            /* board not redundant - just monitor for failures */
            RmgcCancelTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_STANDALONE );
            break;

        case HMI_EVN_NO_BOARD:      /* HMI reporting board not present */
            /* The requested board doesn't exist - error msg & quit */
            printf( "ERROR: Board %d doesn't exist\n", RmgBoardNum );
            RmgcCancelTimer( RMGC_TIMER_T4 );
            LogError( LogNameState( RmgState ), "requested board does not exist" );
            RmgcSetShutdown( );     /* set trigger for all RMG threads to shutdown (unless operator 'quit' required) */
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        case HMI_EVN_TASKDEAD:      /* HMI reporting that a task has failed */
        case HMI_EVN_BRDDEAD:       /* HMI reporting that board itself has failed */
        case HMI_EVN_HALTED:        /* HMI reporting that board is in the halted state */
            RmgcCancelTimer( RMGC_TIMER_T4 );
            RmgHmiLoadBoard( );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case RMG_EV_T4:             /* timer 4 (board download timer) has fired */
            /* no event from HMI service */
            /* The board is not running - download it now */
            RmgHmiLoadBoard( );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_LOADING:       /* HMI reporting that board is loading */
            /* board loaded by another application? */
            RmgcCancelTimer( RMGC_TIMER_T4 );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case RMG_EV_SWOVER_CMD:     /* operator command to cause changeover */
        case RMG_EV_LOAD_CMD:       /* operator command to load board */
        case RMG_EV_HALT_CMD:       /* operator command to halt board */
        case RMG_EV_EXTRACT_CMD:    /* operator command indicating ready for extraction */
            /* User commands not allowed in this state. */
            printf( "%s not allowed in %s state\n",
                    LogNameEvent( event ), LogNameState( RmgState ) );
            break;

        case HMI_EVN_EXTRACT:       /* HMI reporting board extraction pending */
            RmgcCancelTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_EXTRACTING );
            break;

        case RMG_EV_SHUTDN:         /* main thread is shutting down */
            Rmgc_shutdown = 1;
            RmgcCancelTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        case HMI_EVN_SERVICE_DOWN:  /* HMI service has shutdown */
            RmgcCancelTimer( RMGC_TIMER_T4 );
            RmgcSetShutdown( );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these can happen, but nothing to do */
        case HMI_EVN_CONNECTED:     /* HMI reporting that board is connected to its mate board */
        case HMI_EVN_ISOLATED:      /* HMI reporting that board is isolated from its mate board */
        case HMI_EVN_CONFLICT:      /* HMI reporting board conflict */
        case RMG_EV_HEARTBEAT:      /* mate node (primary) providing heartbeat as health indicator */
        case RMG_EV_ACTV_RQST:      /* mate node is initializing and is asking to become primary */
        case RMG_EV_SWOVER_RQST:    /* mate node (primary) requesting that this node become primary [due to operator command or remote extraction pending] */
        case RMG_EV_TKOVER_RQST:    /* mate node (backup) requesting to become primary [due to operator command] */
        case RMG_EV_NETUP_RQST:     /* mate node (backup) network up so requesting to become primary (if primary network down) */
        case RMG_EV_NETDOWN_RQST:   /* mate node (primary) network down so requesting that this node become primary (if primary network up) */
        case RMG_EV_ACTV_CFM:       /* mate node (now backup) has accepted request for this node to be primary [due to initial startup request] */
        case RMG_EV_ACTV_REJ:       /* mate node (now primary) has rejected request for this node to be primary [due to initial startup request] */
        case RMG_EV_SWOVER_CFM:     /* mate node (now primary) has accepted this node's request to become backup */
        case RMG_EV_SWOVER_REJ:     /* mate node (now backup) has rejected this node's request to become backup */
        case RMG_EV_TKOVER_CFM:     /* mate node (now backup) has accepted this node's request to become primary */
        case RMG_EV_TKOVER_REJ:     /* mate node (now primary) has rejected this node's request to become primary */
        case RMG_EV_NETUP_CFM:      /* mate node (now backup) has accepted this node's request to become primary due to network up */
        case RMG_EV_NETUP_REJ:      /* mate node (now primary) has rejected this node's request to become primary due to network up */
        case RMG_EV_NETDOWN_CFM:    /* mate node (now primary) has accepted this node's request to become backup due to network down */
        case RMG_EV_NETDOWN_REJ:    /* mate node (now backup) has rejected this node's request to become backup due to network down */
        case RMG_EV_FAILURE:        /* mate node (primary) indicating that its board has failed */
            break;

        /* these events may happen due to a stale timer firing indication */
        case RMG_EV_T1:             /* timer 1 (retry timer) has fired */
        case RMG_EV_T2:             /* timer 2 (monitoring for heartbeat timer) has fired */
        case RMG_EV_T3:             /* timer 3 (periodic heartbeat send timer) has fired */
            RmgcStaleTimer( event );        /* issue log indicating that a stale timer has fired */
            break;

        case EVENT_FREEZE:          /* an event has been converted into indication to freeze further event processing */
            RmgcCancelTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these events shouldn't happen - log them */
        case HMI_EVN_INSERT:        /* HMI reporting that board has been inserted */
        case HMI_EVN_STOP:          /* HMI reporting board has been stopped */
        case HMI_EVN_NETWORK_UP:    /* HMI forwarding task indication that network connectivity is up */
        case HMI_EVN_NETWORK_DOWN:  /* HMI forwarding task indication that network connectivity is down */
        default:
            UNEX_EVENT_ALARM( event );

    }                                   /* end switch on event */

    return( 0 );
}

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 * State: Loading
 *
 * Description:
 *
 * Entered on startup when board is not yet loaded, or after a failure or user command
 * to reload board; HMI board load command has been issued.
 *
 * We're expecting a board state event (STARTED) from HMI to signify that board
 * has been downloaded.
 * Timer T4 is running.
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
static short RmgcLoading( DWORD event, DWORD value )
{
    switch( event )
    {
        case RMG_EV_ACTV_RQST:      /* mate node is initializing and is asking to become primary */
            /* We're still downloading, allow other node to be primary. */
            RmgCommSend( RMG_EV_ACTV_CFM, 0, 0 );
            break;

        case RMG_EV_SWOVER_RQST:    /* mate node (primary) requesting that this node become primary [due to operator command or remote extraction pending] */
            /* Other node wants us to take over - can't allow it yet. */
            RmgCommSend( RMG_EV_SWOVER_REJ, RMG_REASON_LOADING, 0 );
            break;

        case RMG_EV_TKOVER_RQST:    /* mate node (backup) requesting to become primary [due to operator command] */
            /* other node wants to go active, allow it */
            RmgCommSend( RMG_EV_TKOVER_CFM, 0, 0 );
            break;

        case RMG_EV_NETUP_RQST:     /* mate node (backup) network up so requesting to become primary (if primary network down) */
            /* We're still downloading, allow other node to be primary. */
            RmgCommSend( RMG_EV_NETUP_CFM, 0, 0 );
            break;

        case RMG_EV_NETDOWN_RQST:   /* mate node (primary) network down so requesting that this node become primary (if primary network up) */
            /* Other node wants us to take over - can't allow it yet. */
            RmgCommSend( RMG_EV_NETDOWN_REJ, RMG_REASON_LOADING, 0 );
            break;

        case HMI_EVN_STARTING:      /* HMI reporting that board has loaded and is now starting */
            /* Board load complete - send activation request to mate
             * and go to starting state to wait for response.
             */
            RmgcCancelTimer( RMGC_TIMER_T4 );
            if (RmgStandAlone)
            {
                RmgHmiStandalone( );
                LogStateChange( RMGC_STATE_STANDALONE );
            }
            else
            {
                RmgCommSend( RMG_EV_ACTV_RQST, 0, 0 );
                RmgcStartTimer( RMGC_TIMER_T1 );
                LogStateChange( RMGC_STATE_STARTING );
            }
            break;

        case HMI_EVN_NOWSTANDALONE: /* HMI reporting that board is operating in standalone mode */
            /* board not redundant - just monitor for failures */
            RmgcCancelTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_STANDALONE );
            break;

        case HMI_EVN_TASKDEAD:      /* HMI reporting that a task has failed */
        case HMI_EVN_BRDDEAD:       /* HMI reporting that board itself has failed */
        case HMI_EVN_HALTED:        /* HMI reporting that board is in the halted state */
        case RMG_EV_HALT_CMD:       /* operator command to halt board */
            RmgcCancelTimer( RMGC_TIMER_T4 );
            RmgHmiHaltBoard( );     /* halt board to be safe */
            LogStateChange( RMGC_STATE_OOS );
            break;

        case RMG_EV_T4:             /* timer 4 (board download timer) has fired */
            /* failure during reload: take board out of service */
            printf( "Timeout while loading board\n" );
            RmgHmiHaltBoard( );     /* halt board to be safe */
            LogStateChange( RMGC_STATE_OOS );
            break;

        case RMG_EV_EXTRACT_CMD:    /* operator command indicating ready for extraction */
        case RMG_EV_LOAD_CMD:       /* operator command to load board */
        case RMG_EV_SWOVER_CMD:     /* operator command to cause changeover */
            /* These operator commands not allowed while load in progress. */
            printf( "%s not allowed in %s state\n",
                    LogNameEvent( event ), LogNameState( RmgState ) );
            break;

        case HMI_EVN_EXTRACT:       /* HMI reporting board extraction pending */
            RmgcCancelTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_EXTRACTING );
            break;

        case RMG_EV_SHUTDN:         /* main thread is shutting down */
            Rmgc_shutdown = 1;
            RmgcCancelTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        case HMI_EVN_SERVICE_DOWN:  /* HMI service has shutdown */
            RmgcCancelTimer( RMGC_TIMER_T4 );
            RmgcSetShutdown( );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these can happen, but nothing to do */
        case HMI_EVN_CONNECTED:     /* HMI reporting that board is connected to its mate board */
        case HMI_EVN_ISOLATED:      /* HMI reporting that board is isolated from its mate board */
        case HMI_EVN_CONFLICT:      /* HMI reporting board conflict */
        case HMI_EVN_LOADING:       /* HMI reporting that board is loading */
        case HMI_EVN_NETWORK_UP:    /* HMI forwarding task indication that network connectivity is up */
        case HMI_EVN_NETWORK_DOWN:  /* HMI forwarding task indication that network connectivity is down */
        case RMG_EV_HEARTBEAT:      /* mate node (primary) providing heartbeat as health indicator */
        case RMG_EV_FAILURE:        /* mate node (primary) indicating that its board has failed */
        case RMG_EV_ACTV_CFM:       /* mate node (now backup) has accepted request for this node to be primary [due to initial startup request] */
        case RMG_EV_ACTV_REJ:       /* mate node (now primary) has rejected request for this node to be primary [due to initial startup request] */
        case RMG_EV_SWOVER_CFM:     /* mate node (now primary) has accepted this node's request to become backup */
        case RMG_EV_SWOVER_REJ:     /* mate node (now backup) has rejected this node's request to become backup */
        case RMG_EV_TKOVER_CFM:     /* mate node (now backup) has accepted this node's request to become primary */
        case RMG_EV_TKOVER_REJ:     /* mate node (now primary) has rejected this node's request to become primary */
        case RMG_EV_NETUP_CFM:      /* mate node (now backup) has accepted this node's request to become primary due to network up */
        case RMG_EV_NETUP_REJ:      /* mate node (now primary) has rejected this node's request to become primary due to network up */
        case RMG_EV_NETDOWN_CFM:    /* mate node (now primary) has accepted this node's request to become backup due to network down */
        case RMG_EV_NETDOWN_REJ:    /* mate node (now backup) has rejected this node's request to become backup due to network down */
            break;

        /* these events may happen due to a stale timer firing indication */
        case RMG_EV_T1:             /* timer 1 (retry timer) has fired */
        case RMG_EV_T2:             /* timer 2 (monitoring for heartbeat timer) has fired */
        case RMG_EV_T3:             /* timer 3 (periodic heartbeat send timer) has fired */
            RmgcStaleTimer( event );        /* issue log indicating that a stale timer has fired */
            break;

        case EVENT_FREEZE:          /* an event has been converted into indication to freeze further event processing */
            RmgcCancelTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these events shouldn't happen in this state - log them */
        case HMI_EVN_INSERT:        /* HMI reporting that board has been inserted */
        case HMI_EVN_STOP:          /* HMI reporting board has been stopped */
        case HMI_EVN_NOWPRIMARY:    /* HMI reporting that board is now operating as primary */
        case HMI_EVN_NOWBACKUP:     /* HMI reporting that board is now operating as backup */
        case HMI_EVN_NO_BOARD:      /* HMI reporting board not present */
        default:
            UNEX_EVENT_ALARM( event );

    }   /* end switch on event */

    return( 0 );
}

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 * State: Starting
 *
 * Description:
 *
 * Entered on startup when board is already downloaded, or after initial
 * download completes; ActvRqst has been sent to mate node.
 *
 * Waiting for response to Actv.
 * Timer T1 is running.
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
static short RmgcStarting( DWORD event, DWORD value )
{
    U8      reject = 0;             /* reason for rejection (if primary selection results in a reject message) */

    switch( event )
    {
        case RMG_EV_ACTV_RQST:      /* mate node is initializing and is asking to become primary */
            /* ~~~ simultaneous activation race condition ~~~
             * Both nodes are starting and requesting to become primary.
             * Use current network status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, 0, 0, &reject, NULL ))
            {   /* mate should end up primary -> go backup */
                RmgCommSend( RMG_EV_ACTV_CFM, 0, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must become backup -> go primary */
                RmgCommSend( RMG_EV_ACTV_REJ, reject, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_SWOVER_RQST:    /* mate node (primary) requesting that this node become primary [due to operator command or remote extraction pending] */
            /* Use current network status, operator-requested state, current primary status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, RmgNodeNum, RmgMateNodeNum, &reject, NULL ))     /* operator requested = local, current = mate */
            {   /* mate must remain primary -> go backup */
                RmgCommSend( RMG_EV_SWOVER_REJ, reject, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate can become backup -> go primary */
                RmgCommSend( RMG_EV_SWOVER_CFM, 0, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_TKOVER_RQST:    /* mate node (backup) requesting to become primary [due to operator command] */
            /* Use current network status, operator-requested state
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, RmgMateNodeNum, 0, &reject, NULL ))     /* operator requested = mate */
            {   /* mate can become primary -> go backup */
                RmgCommSend( RMG_EV_TKOVER_CFM, 0, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must remain backup -> go primary */
                RmgCommSend( RMG_EV_TKOVER_REJ, reject, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_NETUP_RQST:     /* mate node (backup) network up so requesting to become primary (if primary network down) */
            /* Use current network status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, 0, 0, &reject, NULL ))
            {   /* mate can become primary -> go backup */
                RmgCommSend( RMG_EV_NETUP_CFM, 0, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must remain backup -> go primary */
                RmgCommSend( RMG_EV_NETUP_REJ, reject, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_NETDOWN_RQST:   /* mate node (primary) network down so requesting that this node become primary (if primary network up) */
            /* Use current network status, current primary status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, 0, RmgMateNodeNum, &reject, NULL ))      /* current = mate */
            {   /* mate must remain primary -> go backup */
                RmgCommSend( RMG_EV_NETDOWN_REJ, reject, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate can become backup -> go primary */
                RmgCommSend( RMG_EV_NETDOWN_CFM, 0, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_ACTV_CFM:       /* mate node (now backup) has accepted request for this node to be primary [due to initial startup request] */
            /* Activation CFM - set board to primary */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgHmiPrimary( );
            RmgcStartTimer( RMGC_TIMER_T3 );
            SEND_HEARTBEAT;
            LogStateChange( RMGC_STATE_ACTIVE );
            break;

        case RMG_EV_ACTV_REJ:       /* mate node (now primary) has rejected request for this node to be primary [due to initial startup request] */
            /* other node already active -> go to backup */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgHmiBackup( );
            RmgcStartTimer( RMGC_TIMER_T2 );
            LogStateChange( RMGC_STATE_BACKUP );
            break;

        case RMG_EV_T1:             /* timer 1 (retry timer) has fired */
            if (!(RmgRequestRetry))
            {   /* 1st timeout - retry the request */
                RmgCommSend( RMG_EV_ACTV_RQST, 0, RMG_MSG_REQ_RETRY );
                RmgcStartTimer( RMGC_TIMER_T1 );
            }
            else
            {   /* 2nd timeout - mate not responding - go active */
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case HMI_EVN_TASKDEAD:      /* HMI reporting that a task has failed */
        case HMI_EVN_BRDDEAD:       /* HMI reporting that board itself has failed */
        case HMI_EVN_HALTED:        /* HMI reporting that board is in the halted state */
        case RMG_EV_LOAD_CMD:       /* operator command to load board */
            /* Notify mate board of failure (in case it has already sent the activation_confirm) and reload the board. */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgHmiLoadBoard( );
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_LOADING, 0 );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_LOADING:       /* HMI reporting that board is loading */
            /* board loaded by another app? */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_LOADING, 0 );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_NETWORK_UP:    /* HMI forwarding task indication that network connectivity is up */
            RmgcUpdateNetworkStatus( RMG_NETWORK_UP );
            /* We already have an activation request outstanding so we wait to receive a response from the mate node.
             * If the mate node rejects the activation request we will use that as the trigger to then send a NetUp request.
             */
            LogStateChange( RMGC_STATE_START_UP );
            break;

        case HMI_EVN_NETWORK_DOWN:  /* HMI forwarding task indication that network connectivity is down */
            RmgcUpdateNetworkStatus( RMG_NETWORK_DOWN );
            /* We have ACTV_RQST outstanding, but now we want to be backup since network is down.
             * Wait for activation response before issuing NetDown request.
             */
            LogStateChange( RMGC_STATE_START_DOWN );
            break;

        case RMG_EV_SWOVER_CMD:     /* operator command to cause changeover */
        case RMG_EV_EXTRACT_CMD:    /* operator command indicating ready for extraction */
            /* these operator commands not valid in this state */
            printf( "%s not allowed in %s state\n",
                    LogNameEvent( event ), LogNameState( RmgState ) );
            break;

        case RMG_EV_HALT_CMD:       /* operator command to halt board */
            /* Notify mate board of failure (in case it has already sent the activation_confirm) and take board out of service. */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_OOS, 0 );
            RmgHmiHaltBoard( );
            LogStateChange( RMGC_STATE_OOS );
            break;

        case HMI_EVN_STARTING:      /* HMI reporting that board has loaded and is now starting */
            /* already have ACTV_RQST outstanding so nothing else to do here */
            break;

        case HMI_EVN_EXTRACT:       /* HMI reporting board extraction pending */
            /* Since board extraction is pending we absolutely do not want mate node to consider us to be primary.
             * Issue Swover request as means of "forcing" mate to go primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgCommSend( RMG_EV_SWOVER_RQST, 0, RMG_MSG_REQ_EXTRACT );
            RmgcStartTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_EXTRACTING );
            break;

        case RMG_EV_SHUTDN:         /* main thread is shutting down */
            Rmgc_shutdown = 1;
            RmgcCancelTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        case HMI_EVN_SERVICE_DOWN:  /* HMI service has shutdown */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcSetShutdown( );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these can happen, but nothing to do */
        case RMG_EV_SWOVER_CFM:     /* mate node (now primary) has accepted this node's request to become backup */
        case RMG_EV_SWOVER_REJ:     /* mate node (now backup) has rejected this node's request to become backup */
        case RMG_EV_TKOVER_CFM:     /* mate node (now backup) has accepted this node's request to become primary */
        case RMG_EV_TKOVER_REJ:     /* mate node (now primary) has rejected this node's request to become primary */
        case RMG_EV_NETUP_CFM:      /* mate node (now backup) has accepted this node's request to become primary due to network up */
        case RMG_EV_NETUP_REJ:      /* mate node (now primary) has rejected this node's request to become primary due to network up */
        case RMG_EV_NETDOWN_CFM:    /* mate node (now primary) has accepted this node's request to become backup due to network down */
        case RMG_EV_NETDOWN_REJ:    /* mate node (now backup) has rejected this node's request to become backup due to network down */
        case HMI_EVN_CONNECTED:     /* HMI reporting that board is connected to its mate board */
        case HMI_EVN_ISOLATED:      /* HMI reporting that board is isolated from its mate board */
        case HMI_EVN_CONFLICT:      /* HMI reporting board conflict */
        case RMG_EV_FAILURE:        /* mate node (primary) indicating that its board has failed */
        case RMG_EV_HEARTBEAT:      /* mate node (primary) providing heartbeat as health indicator */
            break;

        /* these events may happen due to a stale timer firing indication */
        case RMG_EV_T2:             /* timer 2 (monitoring for heartbeat timer) has fired */
        case RMG_EV_T3:             /* timer 3 (periodic heartbeat send timer) has fired */
        case RMG_EV_T4:             /* timer 4 (board download timer) has fired */
            RmgcStaleTimer( event );        /* issue log indicating that a stale timer has fired */
            break;

        case EVENT_FREEZE:          /* an event has been converted into indication to freeze further event processing */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these events shouldn't happen in this state - log them */
        case HMI_EVN_INSERT:        /* HMI reporting that board has been inserted */
        case HMI_EVN_STOP:          /* HMI reporting board has been stopped */
        case HMI_EVN_NOWPRIMARY:    /* HMI reporting that board is now operating as primary */
        case HMI_EVN_NOWBACKUP:     /* HMI reporting that board is now operating as backup */
        case HMI_EVN_NOWSTANDALONE: /* HMI reporting that board is operating in standalone mode */
        case HMI_EVN_NO_BOARD:      /* HMI reporting board not present */
        default:
            UNEX_EVENT_ALARM( event );

    }   /* end switch on event */

    return( 0 );
}

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 * State: Start_Up
 *
 * Description:
 *
 * Entered when starting and network is reported as up;
 * Waiting to receive response from mate to pending activation request before
 * deciding whether to issue NetUp [optional takeover] request.
 *
 * Waiting for response to Actv.
 * Timer T1 is running.
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
static short RmgcStartUp( DWORD event, DWORD value )
{
    U8      reject = 0;             /* reason for rejection (if primary selection results in a reject message) */

    switch( event )
    {
        case RMG_EV_ACTV_RQST:      /* mate node is initializing and is asking to become primary */
            /* ~~~ simultaneous activation race condition ~~~
             * Both nodes are starting and requesting to become primary.
             * Use current network status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, 0, 0, &reject, NULL ))
            {   /* mate should end up primary -> go backup */
                RmgCommSend( RMG_EV_ACTV_CFM, 0, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must become backup -> go primary */
                RmgCommSend( RMG_EV_ACTV_REJ, reject, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_SWOVER_RQST:    /* mate node (primary) requesting that this node become primary [due to operator command or remote extraction pending] */
            /* Use current network status, operator-requested state, current primary status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, RmgNodeNum, RmgMateNodeNum, &reject, NULL ))     /* operator requested = local, current = mate */
            {   /* mate must remain primary -> go backup */
                RmgCommSend( RMG_EV_SWOVER_REJ, reject, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate can become backup -> go primary */
                RmgCommSend( RMG_EV_SWOVER_CFM, 0, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_TKOVER_RQST:    /* mate node (backup) requesting to become primary [due to operator command] */
            /* Use current network status, operator-requested state
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, RmgMateNodeNum, 0, &reject, NULL ))     /* operator requested = mate */
            {   /* mate can become primary -> go backup */
                RmgCommSend( RMG_EV_TKOVER_CFM, 0, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must remain backup -> go primary */
                RmgCommSend( RMG_EV_TKOVER_REJ, reject, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_NETUP_RQST:     /* mate node (backup) network up so requesting to become primary (if primary network down) */
            /* Use current network status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, 0, 0, &reject, NULL ))
            {   /* mate can become primary -> go backup */
                RmgCommSend( RMG_EV_NETUP_CFM, 0, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must remain backup -> go primary */
                RmgCommSend( RMG_EV_NETUP_REJ, reject, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_NETDOWN_RQST:   /* mate node (primary) network down so requesting that this node become primary (if primary network up) */
            /* Use current network status, current primary status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, 0, RmgMateNodeNum, &reject, NULL ))      /* current = mate */
            {   /* mate must remain primary -> go backup */
                RmgCommSend( RMG_EV_NETDOWN_REJ, reject, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate can become backup -> go primary */
                RmgCommSend( RMG_EV_NETDOWN_CFM, 0, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_ACTV_CFM:       /* mate node (now backup) has accepted request for this node to be primary [due to initial startup request] */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgHmiPrimary( );
            RmgcStartTimer( RMGC_TIMER_T3 );
            SEND_HEARTBEAT;
            LogStateChange( RMGC_STATE_ACTIVE );
            break;

        case RMG_EV_ACTV_REJ:       /* mate node (now primary) has rejected request for this node to be primary [due to initial startup request] */
            /* Other node already active -> go to backup.
             * Since we issued the activation request our board reported network up so issue immediate NetUp request */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgHmiBackup( );
            RmgcStartTimer( RMGC_TIMER_T1 );
            RmgCommSend( RMG_EV_NETUP_RQST, 0, 0 );
            LogStateChange( RMGC_STATE_TKOV_UP );
            break;

        case RMG_EV_T1:             /* timer 1 (retry timer) has fired */
            if (!(RmgRequestRetry))
            {   /* 1st timeout - retry the request */
                RmgCommSend( RMG_EV_ACTV_RQST, 0, RMG_MSG_REQ_RETRY );
                RmgcStartTimer( RMGC_TIMER_T1 );
            }
            else
            {   /* 2nd timeout - mate not responding - go active */
                /* In this case, we would prefer not be primary since our board's network is down;
                 * but since other side is not responding we must go active.
                 */
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case HMI_EVN_TASKDEAD:      /* HMI reporting that a task has failed */
        case HMI_EVN_BRDDEAD:       /* HMI reporting that board itself has failed */
        case HMI_EVN_HALTED:        /* HMI reporting that board is in the halted state */
        case RMG_EV_LOAD_CMD:       /* operator command to load board */
            /* Notify mate board of failure (in case it has already sent the activation_confirm) and reload the board. */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgHmiLoadBoard( );
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_LOADING, 0 );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_LOADING:       /* HMI reporting that board is loading */
            /* board loaded by another app? */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_LOADING, 0 );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_NETWORK_UP:    /* HMI forwarding task indication that network connectivity is up */
            RmgcUpdateNetworkStatus( RMG_NETWORK_UP );          /* update status for consistency, but we already know that network is up */
            break;

        case HMI_EVN_NETWORK_DOWN:  /* HMI forwarding task indication that network connectivity is down */
            RmgcUpdateNetworkStatus( RMG_NETWORK_DOWN );
            LogStateChange( RMGC_STATE_START_DOWN );
            break;

        case RMG_EV_SWOVER_CMD:     /* operator command to cause changeover */
        case RMG_EV_EXTRACT_CMD:    /* operator command indicating ready for extraction */
            /* these operator commands not valid in this state */
            printf( "%s not allowed in %s state\n",
                    LogNameEvent( event ), LogNameState( RmgState ) );
            break;

        case RMG_EV_HALT_CMD:       /* operator command to halt board */
            /* Notify mate board of failure (in case it has already sent the activation_confirm) and take board out of service. */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_OOS, 0 );
            RmgHmiHaltBoard( );
            LogStateChange( RMGC_STATE_OOS );
            break;

        case HMI_EVN_STARTING:      /* HMI reporting that board has loaded and is now starting */
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            /* already have ACTV_RQST outstanding so just change to Starting state */
            LogStateChange( RMGC_STATE_STARTING );
            break;

        case HMI_EVN_EXTRACT:       /* HMI reporting board extraction pending */
            /* Since board extraction is pending we absolutely do not want mate node to consider us to be primary.
             * Issue Swover request as means of "forcing" mate to go primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgCommSend( RMG_EV_SWOVER_RQST, 0, RMG_MSG_REQ_EXTRACT );
            RmgcStartTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_EXTRACTING );
            break;

        case RMG_EV_SHUTDN:         /* main thread is shutting down */
            Rmgc_shutdown = 1;
            RmgcCancelTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        case HMI_EVN_SERVICE_DOWN:  /* HMI service has shutdown */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcSetShutdown( );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these can happen, but nothing to do */
        case RMG_EV_SWOVER_CFM:     /* mate node (now primary) has accepted this node's request to become backup */
        case RMG_EV_SWOVER_REJ:     /* mate node (now backup) has rejected this node's request to become backup */
        case RMG_EV_TKOVER_CFM:     /* mate node (now backup) has accepted this node's request to become primary */
        case RMG_EV_TKOVER_REJ:     /* mate node (now primary) has rejected this node's request to become primary */
        case RMG_EV_NETUP_CFM:      /* mate node (now backup) has accepted this node's request to become primary due to network up */
        case RMG_EV_NETUP_REJ:      /* mate node (now primary) has rejected this node's request to become primary due to network up */
        case RMG_EV_NETDOWN_CFM:    /* mate node (now primary) has accepted this node's request to become backup due to network down */
        case RMG_EV_NETDOWN_REJ:    /* mate node (now backup) has rejected this node's request to become backup due to network down */
        case HMI_EVN_CONNECTED:     /* HMI reporting that board is connected to its mate board */
        case HMI_EVN_ISOLATED:      /* HMI reporting that board is isolated from its mate board */
        case HMI_EVN_CONFLICT:      /* HMI reporting board conflict */
        case RMG_EV_FAILURE:        /* mate node (primary) indicating that its board has failed */
        case RMG_EV_HEARTBEAT:      /* mate node (primary) providing heartbeat as health indicator */
            break;

        /* these events may happen due to a stale timer firing indication */
        case RMG_EV_T2:             /* timer 2 (monitoring for heartbeat timer) has fired */
        case RMG_EV_T3:             /* timer 3 (periodic heartbeat send timer) has fired */
        case RMG_EV_T4:             /* timer 4 (board download timer) has fired */
            RmgcStaleTimer( event );        /* issue log indicating that a stale timer has fired */
            break;

        case EVENT_FREEZE:          /* an event has been converted into indication to freeze further event processing */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these events shouldn't happen in this state - log them */
        case HMI_EVN_INSERT:        /* HMI reporting that board has been inserted */
        case HMI_EVN_STOP:          /* HMI reporting board has been stopped */
        case HMI_EVN_NOWPRIMARY:    /* HMI reporting that board is now operating as primary */
        case HMI_EVN_NOWBACKUP:     /* HMI reporting that board is now operating as backup */
        case HMI_EVN_NOWSTANDALONE: /* HMI reporting that board is operating in standalone mode */
        case HMI_EVN_NO_BOARD:      /* HMI reporting board not present */
        default:
            UNEX_EVENT_ALARM( event );

    }   /* end switch on event */

    return( 0 );
}

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 * State: Start_Down
 *
 * Description:
 *
 * Entered when starting and network is reported as down;
 * Waiting to receive response from mate to pending activation request before
 * deciding whether to issue NetDown [optional switchover] request.
 *
 * Waiting for response to Actv.
 * Timer T1 is running.
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
static short RmgcStartDown( DWORD event, DWORD value )
{
    U8      reject = 0;             /* reason for rejection (if primary selection results in a reject message) */

    switch( event )
    {
        case RMG_EV_ACTV_RQST:      /* mate node is initializing and is asking to become primary */
            /* ~~~ simultaneous activation race condition ~~~
             * Both nodes are starting and requesting to become primary.
             * Use current network status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, 0, 0, &reject, NULL ))
            {   /* mate should end up primary -> go backup */
                RmgCommSend( RMG_EV_ACTV_CFM, 0, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must become backup -> go primary */
                RmgCommSend( RMG_EV_ACTV_REJ, reject, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_SWOVER_RQST:    /* mate node (primary) requesting that this node become primary [due to operator command or remote extraction pending] */
            /* Use current network status, operator-requested state, current primary status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, RmgNodeNum, RmgMateNodeNum, &reject, NULL ))     /* operator requested = local, current = mate */
            {   /* mate must remain primary -> go backup */
                RmgCommSend( RMG_EV_SWOVER_REJ, reject, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate can become backup -> go primary */
                RmgCommSend( RMG_EV_SWOVER_CFM, 0, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_TKOVER_RQST:    /* mate node (backup) requesting to become primary [due to operator command] */
            /* Use current network status, operator-requested state
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, RmgMateNodeNum, 0, &reject, NULL ))     /* operator requested = mate */
            {   /* mate can become primary -> go backup */
                RmgCommSend( RMG_EV_TKOVER_CFM, 0, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must remain backup -> go primary */
                RmgCommSend( RMG_EV_TKOVER_REJ, reject, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_NETUP_RQST:     /* mate node (backup) network up so requesting to become primary (if primary network down) */
            /* Use current network status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, 0, 0, &reject, NULL ))
            {   /* mate can become primary -> go backup */
                RmgCommSend( RMG_EV_NETUP_CFM, 0, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must remain backup -> go primary */
                RmgCommSend( RMG_EV_NETUP_REJ, reject, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_NETDOWN_RQST:   /* mate node (primary) network down so requesting that this node become primary (if primary network up) */
            /* Use current network status, current primary status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, 0, RmgMateNodeNum, &reject, NULL ))      /* current = mate */
            {   /* mate should remain primary -> go backup */
                RmgCommSend( RMG_EV_NETDOWN_REJ, reject, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate can become backup -> go primary */
                RmgCommSend( RMG_EV_NETDOWN_CFM, 0, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_ACTV_CFM:       /* mate node (now backup) has accepted request for this node to be primary [due to initial startup request] */
            /* We are now primary but would prefer not to be since board has reported network down. */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgHmiPrimary( );
            RmgCommSend( RMG_EV_NETDOWN_RQST, 0, 0 );
            RmgcStartTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_ACTIVE_DOWN );
            break;

        case RMG_EV_ACTV_REJ:       /* mate node (now primary) has rejected request for this node to be primary [due to initial startup request] */
            /* Other node already active -> go to backup. */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgHmiBackup( );
            RmgcStartTimer( RMGC_TIMER_T2 );
            LogStateChange( RMGC_STATE_BACKUP );
            break;

        case RMG_EV_T1:             /* timer 1 (retry timer) has fired */
            if (!(RmgRequestRetry))
            {   /* 1st timeout - retry the request */
                /* Even though we prefer not be primary we don't know whether our Actv request was received by mate node or was lost.
                 * To avoid becoming out-of-sync with mate we retry the Actv request... decision of whether to issue NetDown will
                 * occur when Actv response is received (or another timeout occurs).
                 */
                RmgCommSend( RMG_EV_ACTV_RQST, 0, RMG_MSG_REQ_RETRY );
                RmgcStartTimer( RMGC_TIMER_T1 );
            }
            else
            {   /* 2nd timeout - mate not responding - go active */
                /* In this case, we would prefer not be primary since our board's network is down;
                 * but since other side is not responding we must go active.
                 */
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case HMI_EVN_TASKDEAD:      /* HMI reporting that a task has failed */
        case HMI_EVN_BRDDEAD:       /* HMI reporting that board itself has failed */
        case HMI_EVN_HALTED:        /* HMI reporting that board is in the halted state */
        case RMG_EV_LOAD_CMD:       /* operator command to load board */
            /* Notify mate board of failure (in case it has already sent the activation_confirm) and reload the board. */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgHmiLoadBoard( );
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_LOADING, 0 );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_LOADING:       /* HMI reporting that board is loading */
            /* board loaded by another app? */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_LOADING, 0 );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_NETWORK_UP:    /* HMI forwarding task indication that network connectivity is up */
            RmgcUpdateNetworkStatus( RMG_NETWORK_UP );
            LogStateChange( RMGC_STATE_START_UP );
            break;

        case HMI_EVN_NETWORK_DOWN:  /* HMI forwarding task indication that network connectivity is down */
            RmgcUpdateNetworkStatus( RMG_NETWORK_DOWN );        /* update status for consistency, but we already know that network is down */
            break;

        case RMG_EV_SWOVER_CMD:     /* operator command to cause changeover */
        case RMG_EV_EXTRACT_CMD:    /* operator command indicating ready for extraction */
            /* these operator commands not valid in this state */
            printf( "%s not allowed in %s state\n",
                    LogNameEvent( event ), LogNameState( RmgState ) );
            break;

        case RMG_EV_HALT_CMD:       /* operator command to halt board */
            /* Notify mate board of failure (in case it has already sent the activation_confirm) and take board out of service. */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_OOS, 0 );
            RmgHmiHaltBoard( );
            LogStateChange( RMGC_STATE_OOS );
            break;

        case HMI_EVN_STARTING:      /* HMI reporting that board has loaded and is now starting */
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            /* already have ACTV_RQST outstanding so just change to Starting state */
            LogStateChange( RMGC_STATE_STARTING );
            break;

        case HMI_EVN_EXTRACT:       /* HMI reporting board extraction pending */
            /* Since board extraction is pending we absolutely do not want mate node to consider us to be primary.
             * Issue Swover request as means of "forcing" mate to go primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgCommSend( RMG_EV_SWOVER_RQST, 0, RMG_MSG_REQ_EXTRACT );
            RmgcStartTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_EXTRACTING );
            break;

        case RMG_EV_SHUTDN:         /* main thread is shutting down */
            Rmgc_shutdown = 1;
            RmgcCancelTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        case HMI_EVN_SERVICE_DOWN:  /* HMI service has shutdown */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcSetShutdown( );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these can happen, but nothing to do */
        case RMG_EV_SWOVER_CFM:     /* mate node (now primary) has accepted this node's request to become backup */
        case RMG_EV_SWOVER_REJ:     /* mate node (now backup) has rejected this node's request to become backup */
        case RMG_EV_TKOVER_CFM:     /* mate node (now backup) has accepted this node's request to become primary */
        case RMG_EV_TKOVER_REJ:     /* mate node (now primary) has rejected this node's request to become primary */
        case RMG_EV_NETUP_CFM:      /* mate node (now backup) has accepted this node's request to become primary due to network up */
        case RMG_EV_NETUP_REJ:      /* mate node (now primary) has rejected this node's request to become primary due to network up */
        case RMG_EV_NETDOWN_CFM:    /* mate node (now primary) has accepted this node's request to become backup due to network down */
        case RMG_EV_NETDOWN_REJ:    /* mate node (now backup) has rejected this node's request to become backup due to network down */
        case HMI_EVN_CONNECTED:     /* HMI reporting that board is connected to its mate board */
        case HMI_EVN_ISOLATED:      /* HMI reporting that board is isolated from its mate board */
        case HMI_EVN_CONFLICT:      /* HMI reporting board conflict */
        case RMG_EV_FAILURE:        /* mate node (primary) indicating that its board has failed */
        case RMG_EV_HEARTBEAT:      /* mate node (primary) providing heartbeat as health indicator */
            break;

        /* these events may happen due to a stale timer firing indication */
        case RMG_EV_T2:             /* timer 2 (monitoring for heartbeat timer) has fired */
        case RMG_EV_T3:             /* timer 3 (periodic heartbeat send timer) has fired */
        case RMG_EV_T4:             /* timer 4 (board download timer) has fired */
            RmgcStaleTimer( event );        /* issue log indicating that a stale timer has fired */
            break;

        case EVENT_FREEZE:          /* an event has been converted into indication to freeze further event processing */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these events shouldn't happen in this state - log them */
        case HMI_EVN_INSERT:        /* HMI reporting that board has been inserted */
        case HMI_EVN_STOP:          /* HMI reporting board has been stopped */
        case HMI_EVN_NOWPRIMARY:    /* HMI reporting that board is now operating as primary */
        case HMI_EVN_NOWBACKUP:     /* HMI reporting that board is now operating as backup */
        case HMI_EVN_NOWSTANDALONE: /* HMI reporting that board is operating in standalone mode */
        case HMI_EVN_NO_BOARD:      /* HMI reporting board not present */
        default:
            UNEX_EVENT_ALARM( event );

    }   /* end switch on event */

    return( 0 );
}

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 * State: Active
 *
 * Description:
 *
 * This node is active!!!
 *
 * Timer T3 (periodic heartbeat) is running,
 * and we're just monitoring for failures.
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
static short RmgcActive( DWORD event, DWORD value )
{
    U8      reject = 0;             /* reason for rejection (if primary selection results in a reject message) */
    S8     *reason = Rmgc_defaultReason;

    switch( event )
    {
        case RMG_EV_ACTV_RQST:      /* mate node is initializing and is asking to become primary */
            /* Mate node is initializing, and this node is already primary.
             * Use current network status
             * (and node numbers if needed) to decide which should be primary.
             */
            if (RmgcWantMatePrimary( value, 0, RmgNodeNum, &reject, &reason ))          /* current = local */
            {   /* mate should end up primary -> go backup */
                printf( "Switchover in progress (%s)\n", reason );
                RmgcCancelTimer( RMGC_TIMER_T3 );
                RmgCommSend( RMG_EV_ACTV_CFM, 0, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must become backup -> remain primary */
                RmgCommSend( RMG_EV_ACTV_REJ, reject, 0 );
            }
            break;

        case RMG_EV_SWOVER_RQST:    /* mate node (primary) requesting that this node become primary [due to operator command or remote extraction pending] */
            /* ~~~ out-of-sync (both nodes believe they are primary) ~~~
             * Use current network status, operator-requested state, current primary status
             * (and node numbers if needed) to decide which should be primary.
             */
            if (RmgcWantMatePrimary( value, RmgNodeNum, 0, &reject, &reason ))         /* operator requested = local */
            {   /* mate must remain primary -> go backup */
                printf( "Switchover in progress (%s)\n", reason );
                RmgcCancelTimer( RMGC_TIMER_T3 );
                RmgCommSend( RMG_EV_SWOVER_REJ, reject, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate can become backup -> remain primary */
                RmgCommSend( RMG_EV_SWOVER_CFM, 0, 0 );
            }
            break;

        case RMG_EV_TKOVER_RQST:    /* mate node (backup) requesting to become primary [due to operator command] */
            /* Use current network status, operator-requested state
             * (and node numbers if needed) to decide which should be primary.
             */
            if (RmgcWantMatePrimary( value, RmgMateNodeNum, RmgNodeNum, &reject, &reason ))     /* operator requested = mate, current = local */
            {   /* mate can become primary -> go backup */
                printf( "Switchover in progress (%s)\n", reason );
                RmgcCancelTimer( RMGC_TIMER_T3 );
                RmgCommSend( RMG_EV_TKOVER_CFM, 0, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must remain backup -> remain primary */
                RmgCommSend( RMG_EV_TKOVER_REJ, reject, 0 );
            }
            break;

        case RMG_EV_NETUP_RQST:     /* mate node (backup) network up so requesting to become primary (if primary network down) */
            /* Use current network status
             * (and node numbers if needed) to decide which should be primary.
             */
            if (RmgcWantMatePrimary( value, 0, RmgNodeNum, &reject, &reason ))                  /* current = local */
            {   /* mate can become primary -> go backup */
                printf( "Switchover in progress (%s)\n", reason );
                RmgcCancelTimer( RMGC_TIMER_T3 );
                RmgCommSend( RMG_EV_NETUP_CFM, 0, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must remain backup -> remain primary */
                RmgCommSend( RMG_EV_NETUP_REJ, reject, 0 );
            }
            break;

        case RMG_EV_NETDOWN_RQST:   /* mate node (primary) network down so requesting that this node become primary (if primary network up) */
            /* ~~~ out-of-sync (both nodes believe they are primary) ~~~
             * Use current network status, current primary status
             * (and node numbers if needed) to decide which should be primary.
             */
            if (RmgcWantMatePrimary( value, 0, 0, &reject, &reason ))
            {   /* mate must remain primary -> go backup */
                printf( "Switchover in progress (%s)\n", reason );
                RmgcCancelTimer( RMGC_TIMER_T3 );
                RmgCommSend( RMG_EV_NETDOWN_REJ, reject, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate can become backup -> remain primary */
                RmgCommSend( RMG_EV_NETDOWN_CFM, 0, 0 );
            }
            break;

        case RMG_EV_HEARTBEAT:      /* mate node (primary) providing heartbeat as health indicator */
            /* ~~~ out-of-sync (both nodes believe they are primary) ~~~
             * Use current network status, current primary status
             * (and node numbers if needed) to decide which should be primary.
             */
            if (RmgcWantMatePrimary( value, 0, 0, &reject, &reason ))
            {   /* mate remains primary -> go backup */
                printf( "Switchover in progress (%s)\n", reason );
                RmgcCancelTimer( RMGC_TIMER_T3 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate should switch to backup (once it receives our heartbeat) -> remain primary */
                /* don't wait for heartbeat timer to fire - send a heartbeat now so other side see's we are primary */
                RmgcCancelTimer( RMGC_TIMER_T3 );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
            }
            break;

        case RMG_EV_T3:             /* timer 3 (periodic heartbeat send timer) has fired */
            /* time to send another heartbeat message */
            SEND_HEARTBEAT;
            RmgcStartTimer( RMGC_TIMER_T3 );
            break;

        case HMI_EVN_TASKDEAD:      /* HMI reporting that a task has failed */
        case HMI_EVN_BRDDEAD:       /* HMI reporting that board itself has failed */
        case HMI_EVN_HALTED:        /* HMI reporting that board is in the halted state */
        case RMG_EV_LOAD_CMD:       /* operator command to load board */
            /* Notify mate board of failure and reload the board. */
            RmgcCancelTimer( RMGC_TIMER_T3 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgHmiLoadBoard( );
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_LOADING, 0 );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_LOADING:       /* HMI reporting that board is loading */
            /* board reloaded by another app? */
            RmgcCancelTimer( RMGC_TIMER_T3 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_LOADING, 0 );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_NETWORK_UP:    /* HMI forwarding task indication that network connectivity is up */
            RmgcUpdateNetworkStatus( RMG_NETWORK_UP );
            /* already ACTIVE so no other action required */
            break;

        case HMI_EVN_NETWORK_DOWN:  /* HMI forwarding task indication that network connectivity is down */
            RmgcUpdateNetworkStatus( RMG_NETWORK_DOWN );
            RmgcCancelTimer( RMGC_TIMER_T3 );
            RmgCommSend( RMG_EV_NETDOWN_RQST, 0, 0 );
            RmgcStartTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_ACTIVE_DOWN );
            break;

        case RMG_EV_SWOVER_CMD:     /* operator command to cause changeover */
            /* user requested switchover to backup */
            RmgcCancelTimer( RMGC_TIMER_T3 );
            RmgCommSend( RMG_EV_SWOVER_RQST, 0, 0 );
            RmgcStartTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_ACTIVE_SWRQ );
            break;

        case RMG_EV_HALT_CMD:       /* operator command to halt board */
            /* Tell mate node that board has failed and take board out of service. */
            RmgcCancelTimer( RMGC_TIMER_T3 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_OOS, 0 );
            RmgHmiHaltBoard( );
            LogStateChange( RMGC_STATE_OOS );
            break;

        case RMG_EV_EXTRACT_CMD:    /* operator command indicating ready for extraction */
            /* extract command not valid in this state */
            printf( "%s not allowed in %s state\n",
                    LogNameEvent( event ), LogNameState( RmgState ) );
            break;

        case HMI_EVN_EXTRACT:       /* HMI reporting board extraction pending */
            RmgcCancelTimer( RMGC_TIMER_T3 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgCommSend( RMG_EV_SWOVER_RQST, 0, RMG_MSG_REQ_EXTRACT );
            RmgcStartTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_EXTRACTING );
            break;

        case RMG_EV_SHUTDN:         /* main thread is shutting down */
            Rmgc_shutdown = 1;
            RmgcCancelTimer( RMGC_TIMER_T3 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        case HMI_EVN_SERVICE_DOWN:  /* HMI service has shutdown */
            /* user requested shutdown or HMI service stopped */
            RmgcCancelTimer( RMGC_TIMER_T3 );
            RmgcSetShutdown( );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these can happen, but nothing to do */
        case HMI_EVN_NOWPRIMARY:    /* HMI reporting that board is now operating as primary */
        case HMI_EVN_CONNECTED:     /* HMI reporting that board is connected to its mate board */
        case HMI_EVN_ISOLATED:      /* HMI reporting that board is isolated from its mate board */
        case HMI_EVN_CONFLICT:      /* HMI reporting board conflict */
        case RMG_EV_FAILURE:        /* mate node (primary) indicating that its board has failed */
        case RMG_EV_ACTV_CFM:       /* mate node (now backup) has accepted request for this node to be primary [due to initial startup request] */
        case RMG_EV_ACTV_REJ:       /* mate node (now primary) has rejected request for this node to be primary [due to initial startup request] */
        case RMG_EV_SWOVER_CFM:     /* mate node (now primary) has accepted this node's request to become backup */
        case RMG_EV_SWOVER_REJ:     /* mate node (now backup) has rejected this node's request to become backup */
        case RMG_EV_TKOVER_CFM:     /* mate node (now backup) has accepted this node's request to become primary */
        case RMG_EV_TKOVER_REJ:     /* mate node (now primary) has rejected this node's request to become primary */
        case RMG_EV_NETUP_CFM:      /* mate node (now backup) has accepted this node's request to become primary due to network up */
        case RMG_EV_NETUP_REJ:      /* mate node (now primary) has rejected this node's request to become primary due to network up */
        case RMG_EV_NETDOWN_CFM:    /* mate node (now primary) has accepted this node's request to become backup due to network down */
        case RMG_EV_NETDOWN_REJ:    /* mate node (now backup) has rejected this node's request to become backup due to network down */
            break;

        /* these events may happen due to a stale timer firing indication */
        case RMG_EV_T1:             /* timer 1 (retry timer) has fired */
        case RMG_EV_T2:             /* timer 2 (monitoring for heartbeat timer) has fired */
        case RMG_EV_T4:             /* timer 4 (board download timer) has fired */
            RmgcStaleTimer( event );        /* issue log indicating that a stale timer has fired */
            break;

        case EVENT_FREEZE:          /* an event has been converted into indication to freeze further event processing */
            RmgcCancelTimer( RMGC_TIMER_T3 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these events shouldn't happen in this state - log them */
        case HMI_EVN_INSERT:        /* HMI reporting that board has been inserted */
        case HMI_EVN_STOP:          /* HMI reporting board has been stopped */
        case HMI_EVN_NOWBACKUP:     /* HMI reporting that board is now operating as backup */
        case HMI_EVN_STARTING:      /* HMI reporting that board has loaded and is now starting */
        case HMI_EVN_NOWSTANDALONE: /* HMI reporting that board is operating in standalone mode */
        case HMI_EVN_NO_BOARD:      /* HMI reporting board not present */
        default:
            UNEX_EVENT_ALARM( event );

    }   /* end switch on event */

    return( 0 );
}

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 * State: Active_Swrq
 *
 * Description:
 *
 * Node was active, switchover request has been sent to mate due to user request.
 *
 * Waiting for response to Swover.
 * Timer T1 is running.
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
static short RmgcActiveSwreq( DWORD event, DWORD value )
{
    U8      reject = 0;             /* reason for rejection (if primary selection results in a reject message) */
    S8     *reason = Rmgc_defaultReason;

    switch( event )
    {
        case RMG_EV_ACTV_RQST:      /* mate node is initializing and is asking to become primary */
            /* Mate node is initializing, and this node is already primary.
             * Use current network status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, RmgMateNodeNum, RmgNodeNum, &reject, &reason ))  /* operator requested = mate, current = local */
            {   /* mate should end up primary -> go backup */
                RmgCommSend( RMG_EV_ACTV_CFM, 0, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must become backup -> remain primary */
                printf( "Changeover failed - %s\n", reason );
                RmgCommSend( RMG_EV_ACTV_REJ, reject, 0 );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_SWOVER_RQST:    /* mate node (primary) requesting that this node become primary [due to operator command or remote extraction pending] */
            /* ~~~ out-of-sync (both nodes primary) ~~~
             * Both nodes believe they are primary and both are asking for the other node to become primary.
             * Use the current network status of each node (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, 0, 0, &reject, &reason ))
            {   /* mate should end up primary -> go backup */
                RmgCommSend( RMG_EV_SWOVER_REJ, reject, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must become backup -> go primary */
                printf( "Changeover failed - %s\n", reason );
                RmgCommSend( RMG_EV_SWOVER_CFM, 0, 0 );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_TKOVER_RQST:    /* mate node (backup) requesting to become primary [due to operator command] */
            /* We are expecting a Swover response but instead have received a takeover request.
             * Use current network status, operator-requested state
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, RmgMateNodeNum, RmgNodeNum, &reject, &reason ))  /* operator requested = mate, current = local */
            {   /* mate can become primary -> go backup */
                RmgCommSend( RMG_EV_TKOVER_CFM, 0, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must remain backup -> remain primary */
                printf( "Changeover failed - %s\n", reason );
                RmgCommSend( RMG_EV_TKOVER_REJ, reject, 0 );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_NETUP_RQST:     /* mate node (backup) network up so requesting to become primary (if primary network down) */
            /* We were expecting a Swover response, but mate node is requesting what we want.
            /* Use current network status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, RmgMateNodeNum, RmgNodeNum, &reject, &reason ))  /* operator requested = mate, current = local */
            {   /* mate can become primary -> go backup */
                RmgCommSend( RMG_EV_NETUP_CFM, 0, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must remain backup -> remain primary */
                printf( "Changeover failed - %s\n", reason );
                RmgCommSend( RMG_EV_NETUP_REJ, reject, 0 );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_NETDOWN_RQST:   /* mate node (primary) network down so requesting that this node become primary (if primary network up) */
            /* ~~~ out-of-sync (both nodes believe they are primary) ~~~
             * Use current network status, current primary status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, RmgMateNodeNum, 0, &reject, NULL ))              /* operator requested = mate */
            {   /* mate must remain primary -> go backup */
                RmgCommSend( RMG_EV_NETDOWN_REJ, reject, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate can become backup -> remain primary */
                printf( "Changeover failed - %s\n", reason );
                RmgCommSend( RMG_EV_NETDOWN_CFM, 0, 0 );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_SWOVER_CFM:     /* mate node (now primary) has accepted this node's request to become backup */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgHmiBackup( );
            RmgcStartTimer( RMGC_TIMER_T2 );
            LogStateChange( RMGC_STATE_BACKUP );
            break;

        case RMG_EV_SWOVER_REJ:     /* mate node (now backup) has rejected this node's request to become backup */
            /* Mate node can't take over as primary - remain primary. */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            printf( "Changeover failed - backup can't take over [%s]\n", RmgcMsgFailReason[RmgError.rmgMessage.lastInfo] );
            RmgcStartTimer( RMGC_TIMER_T3 );
            SEND_HEARTBEAT;
            LogStateChange( RMGC_STATE_ACTIVE );
            break;

        case RMG_EV_T1:             /* timer 1 (retry timer) has fired */
            if (!(RmgRequestRetry))
            {
                /* 1st timeout - retry the request */
                RmgCommSend( RMG_EV_SWOVER_RQST, 0, RMG_MSG_REQ_RETRY );
                RmgcStartTimer( RMGC_TIMER_T1 );
            }
            else
            {
                /* 2nd timeout - mate not responding - remain active */
                printf( "Changeover failed - no response from backup\n" );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case HMI_EVN_TASKDEAD:      /* HMI reporting that a task has failed */
        case HMI_EVN_BRDDEAD:       /* HMI reporting that board itself has failed */
        case HMI_EVN_HALTED:        /* HMI reporting that board is in the halted state */
        case RMG_EV_LOAD_CMD:       /* operator command to load board */
            /* Notify mate board of failure (in case Swover request lost
             * or Swover reject already on its way) and reload the board. */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgHmiLoadBoard( );
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_LOADING, 0 );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_LOADING:       /* HMI reporting that board is loading */
            /* board loaded by another app? */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_LOADING, 0 );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_NETWORK_UP:    /* HMI forwarding task indication that network connectivity is up */
            RmgcUpdateNetworkStatus( RMG_NETWORK_UP );
            /* already asking for switchover to become backup so wait to hear whether mate has become primary */
            break;

        case HMI_EVN_NETWORK_DOWN:  /* HMI forwarding task indication that network connectivity is down */
            RmgcUpdateNetworkStatus( RMG_NETWORK_DOWN );
            /* already asking for switchover to become backup so no other action required */
            break;

        case RMG_EV_SWOVER_CMD:     /* operator command to cause changeover */
        case RMG_EV_EXTRACT_CMD:    /* operator command indicating ready for extraction */
            /* these user commands not valid when switchover in progress */
            printf( "%s not allowed in %s state\n",
                    LogNameEvent( event ), LogNameState( RmgState ) );
            break;

        case RMG_EV_HALT_CMD:       /* operator command to halt board */
            /* Tell mate node that board has failed and take board out of service. */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_OOS, 0 );
            RmgHmiHaltBoard( );
            LogStateChange( RMGC_STATE_OOS );
            break;

        case HMI_EVN_EXTRACT:       /* HMI reporting board extraction pending */
            /* send another switchover request after updating network status to "force" mate node to become primary */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgCommSend( RMG_EV_SWOVER_RQST, 0, RMG_MSG_REQ_EXTRACT );
            RmgcStartTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_EXTRACTING );
            break;

        case RMG_EV_SHUTDN:         /* main thread is shutting down */
            Rmgc_shutdown = 1;
            RmgcCancelTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        case HMI_EVN_SERVICE_DOWN:  /* HMI service has shutdown */
            /* user requested shutdown or HMI service stopped */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcSetShutdown( );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these can happen, but nothing to do */
        case HMI_EVN_NOWPRIMARY:    /* HMI reporting that board is now operating as primary */
        case HMI_EVN_CONNECTED:     /* HMI reporting that board is connected to its mate board */
        case HMI_EVN_ISOLATED:      /* HMI reporting that board is isolated from its mate board */
        case HMI_EVN_CONFLICT:      /* HMI reporting board conflict */
        case RMG_EV_FAILURE:        /* mate node (primary) indicating that its board has failed */
        case RMG_EV_HEARTBEAT:      /* mate node (primary) providing heartbeat as health indicator */
        case RMG_EV_ACTV_CFM:       /* mate node (now backup) has accepted request for this node to be primary [due to initial startup request] */
        case RMG_EV_ACTV_REJ:       /* mate node (now primary) has rejected request for this node to be primary [due to initial startup request] */
        case RMG_EV_TKOVER_CFM:     /* mate node (now backup) has accepted this node's request to become primary */
        case RMG_EV_TKOVER_REJ:     /* mate node (now primary) has rejected this node's request to become primary */
        case RMG_EV_NETUP_CFM:      /* mate node (now backup) has accepted this node's request to become primary due to network up */
        case RMG_EV_NETUP_REJ:      /* mate node (now primary) has rejected this node's request to become primary due to network up */
        case RMG_EV_NETDOWN_CFM:    /* mate node (now primary) has accepted this node's request to become backup due to network down */
        case RMG_EV_NETDOWN_REJ:    /* mate node (now backup) has rejected this node's request to become backup due to network down */
            break;

        /* these events may happen due to a stale timer firing indication */
        case RMG_EV_T2:             /* timer 2 (monitoring for heartbeat timer) has fired */
        case RMG_EV_T3:             /* timer 3 (periodic heartbeat send timer) has fired */
        case RMG_EV_T4:             /* timer 4 (board download timer) has fired */
            RmgcStaleTimer( event );        /* issue log indicating that a stale timer has fired */
            break;

        case EVENT_FREEZE:          /* an event has been converted into indication to freeze further event processing */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these events shouldn't happen in this state - log them */
        case HMI_EVN_INSERT:        /* HMI reporting that board has been inserted */
        case HMI_EVN_STOP:          /* HMI reporting board has been stopped */
        case HMI_EVN_NOWBACKUP:     /* HMI reporting that board is now operating as backup */
        case HMI_EVN_STARTING:      /* HMI reporting that board has loaded and is now starting */
        case HMI_EVN_NOWSTANDALONE: /* HMI reporting that board is operating in standalone mode */
        case HMI_EVN_NO_BOARD:      /* HMI reporting board not present */
        default:
            UNEX_EVENT_ALARM( event );

    }   /* end switch on event */

    return( 0 );
}

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 * State: Active_Down
 *
 * Description:
 *
 * Node was active, NetDown [optional switchover] request has been sent to mate
 * due to network connectivity going down.
 *
 * Waiting for response to NetDown.
 * Timer T1 is running.
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
static short RmgcActiveDown( DWORD event, DWORD value )
{
    U8      reject = 0;             /* reason for rejection (if primary selection results in a reject message) */
    S8     *reason = Rmgc_defaultReason;

    switch( event )
    {
        case RMG_EV_ACTV_RQST:      /* mate node is initializing and is asking to become primary */
            /* We are expecting a NetUp response but instead have received an activation request.
             * The mate node must have restarted and is now asking to be primary.
             * Use current network status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, 0, RmgNodeNum, &reject, &reason ))          /* current = local */
            {   /* mate should end up primary -> go backup */
                printf( "Switchover in progress (%s)\n", reason );
                RmgCommSend( RMG_EV_ACTV_CFM, 0, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must become backup -> remain primary */
                RmgCommSend( RMG_EV_ACTV_REJ, reject, 0 );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_SWOVER_RQST:    /* mate node (primary) requesting that this node become primary [due to operator command or remote extraction pending] */
            /* ~~~ out-of-sync (both nodes believe they are primary) ~~~
             * Use current network status, operator-requested state, current primary status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, RmgNodeNum, 0, &reject, &reason ))         /* operator requested = local */
            {   /* mate must remain primary -> go backup */
                printf( "Switchover in progress (%s)\n", reason );
                RmgCommSend( RMG_EV_SWOVER_REJ, reject, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate can become backup -> remain primary */
                RmgCommSend( RMG_EV_SWOVER_CFM, 0, 0 );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_TKOVER_RQST:    /* mate node (backup) requesting to become primary [due to operator command] */
            /* We are expecting a NetUp response but instead have received a takeover request.
             * Use current network status, operator-requested state
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, RmgMateNodeNum, RmgNodeNum, &reject, &reason ))     /* operator requested = mate, current = local */
            {   /* mate can become primary -> go backup */
                printf( "Switchover in progress (%s)\n", reason );
                RmgCommSend( RMG_EV_TKOVER_CFM, 0, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must remain backup -> remain primary */
                RmgCommSend( RMG_EV_TKOVER_REJ, reject, 0 );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_NETUP_RQST:     /* mate node (backup) network up so requesting to become primary (if primary network down) */
            /* We were expecting a NetDown response, but mate node is instead requesting to be primary.
             * Use current network status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, 0, RmgNodeNum, &reject, &reason ))                  /* current = local */
            {   /* mate can become primary -> go backup */
                printf( "Switchover in progress (%s)\n", reason );
                RmgCommSend( RMG_EV_NETUP_CFM, 0, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must remain backup -> remain primary */
                RmgCommSend( RMG_EV_NETUP_REJ, reject, 0 );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_NETDOWN_RQST:   /* mate node (primary) network down so requesting that this node become primary (if primary network up) */
            /* ~~~ out-of-sync (both nodes believe they are primary) ~~~
             * Use current network status, current primary status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, 0, 0, &reject, &reason ))
            {   /* mate must remain primary -> go backup */
                printf( "Switchover in progress (%s)\n", reason );
                RmgCommSend( RMG_EV_NETDOWN_REJ, reject, 0 );
                RmgHmiBackup( );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate can become backup -> remain primary */
                RmgCommSend( RMG_EV_NETDOWN_CFM, 0, 0 );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_NETDOWN_CFM:    /* mate node (now primary) has accepted this node's request to become backup due to network down */
            /* Mate node has become primary as we requested -> go backup. */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            printf( "Switchover in progress (board network connection down)\n" );
            RmgHmiBackup( );
            RmgcStartTimer( RMGC_TIMER_T2 );
            LogStateChange( RMGC_STATE_BACKUP );
            break;

        case RMG_EV_NETDOWN_REJ:    /* mate node (now backup) has rejected this node's request to become backup due to network down */
            /* Backup can't take over - remain active. */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcStartTimer( RMGC_TIMER_T3 );
            SEND_HEARTBEAT;
            LogStateChange( RMGC_STATE_ACTIVE );
            break;

        case RMG_EV_T1:             /* timer 1 (retry timer) has fired */
            if (!(RmgRequestRetry))
            {
                /* 1st timeout - retry the request */
                RmgCommSend( RMG_EV_SWOVER_RQST, 0, RMG_MSG_REQ_RETRY );
                RmgcStartTimer( RMGC_TIMER_T1 );
            }
            else
            {
                /* 2nd timeout - mate not responding - remain active */
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case HMI_EVN_TASKDEAD:      /* HMI reporting that a task has failed */
        case HMI_EVN_BRDDEAD:       /* HMI reporting that board itself has failed */
        case HMI_EVN_HALTED:        /* HMI reporting that board is in the halted state */
        case RMG_EV_LOAD_CMD:       /* operator command to load board */
            /* Notify mate board of failure (in case NetDown request lost
             * or NetDown reject is already on its way) and reload the board. */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgHmiLoadBoard( );
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_LOADING, 0 );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_LOADING:       /* HMI reporting that board is loading */
            /* board loaded by another app? */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_LOADING, 0 );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_NETWORK_UP:    /* HMI forwarding task indication that network connectivity is up */
            RmgcUpdateNetworkStatus( RMG_NETWORK_UP );
            /* already asking for switchover to become backup so wait to hear whether mate has become primary */
            break;

        case HMI_EVN_NETWORK_DOWN:  /* HMI forwarding task indication that network connectivity is down */
            RmgcUpdateNetworkStatus( RMG_NETWORK_DOWN );
            /* already asking for switchover to become backup so no other action required */
            break;

        case RMG_EV_SWOVER_CMD:     /* operator command to cause changeover */
            /* user requested switchover to backup */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgCommSend( RMG_EV_SWOVER_RQST, 0, 0 );
            RmgcStartTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_ACTIVE_SWRQ );
            break;

        case RMG_EV_EXTRACT_CMD:    /* operator command indicating ready for extraction */
            /* extract command not valid in this state */
            printf( "%s not allowed in %s state\n",
                    LogNameEvent( event ), LogNameState( RmgState ) );
            break;

        case RMG_EV_HALT_CMD:       /* operator command to halt board */
            /* Tell mate node that board has failed and take board out of service. */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_OOS, 0 );
            RmgHmiHaltBoard( );
            LogStateChange( RMGC_STATE_OOS );
            break;

        case HMI_EVN_EXTRACT:       /* HMI reporting board extraction pending */
            /* send switchover request after updating network status to "force" mate node to become primary */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgCommSend( RMG_EV_SWOVER_RQST, 0, RMG_MSG_REQ_EXTRACT );
            RmgcStartTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_EXTRACTING );
            break;

        case RMG_EV_SHUTDN:         /* main thread is shutting down */
            Rmgc_shutdown = 1;
            RmgcCancelTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        case HMI_EVN_SERVICE_DOWN:  /* HMI service has shutdown */
            /* user requested shutdown or HMI service stopped */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcSetShutdown( );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these can happen, but nothing to do */
        case HMI_EVN_NOWPRIMARY:    /* HMI reporting that board is now operating as primary */
        case HMI_EVN_CONNECTED:     /* HMI reporting that board is connected to its mate board */
        case HMI_EVN_ISOLATED:      /* HMI reporting that board is isolated from its mate board */
        case HMI_EVN_CONFLICT:      /* HMI reporting board conflict */
        case RMG_EV_FAILURE:        /* mate node (primary) indicating that its board has failed */
        case RMG_EV_HEARTBEAT:      /* mate node (primary) providing heartbeat as health indicator */
        case RMG_EV_ACTV_CFM:       /* mate node (now backup) has accepted request for this node to be primary [due to initial startup request] */
        case RMG_EV_ACTV_REJ:       /* mate node (now primary) has rejected request for this node to be primary [due to initial startup request] */
        case RMG_EV_SWOVER_CFM:     /* mate node (now primary) has accepted this node's request to become backup */
        case RMG_EV_SWOVER_REJ:     /* mate node (now backup) has rejected this node's request to become backup */
        case RMG_EV_TKOVER_CFM:     /* mate node (now backup) has accepted this node's request to become primary */
        case RMG_EV_TKOVER_REJ:     /* mate node (now primary) has rejected this node's request to become primary */
        case RMG_EV_NETUP_CFM:      /* mate node (now backup) has accepted this node's request to become primary due to network up */
        case RMG_EV_NETUP_REJ:      /* mate node (now primary) has rejected this node's request to become primary due to network up */
            break;

        /* these events may happen due to a stale timer firing indication */
        case RMG_EV_T2:             /* timer 2 (monitoring for heartbeat timer) has fired */
        case RMG_EV_T3:             /* timer 3 (periodic heartbeat send timer) has fired */
        case RMG_EV_T4:             /* timer 4 (board download timer) has fired */
            RmgcStaleTimer( event );        /* issue log indicating that a stale timer has fired */
            break;

        case EVENT_FREEZE:          /* an event has been converted into indication to freeze further event processing */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these events shouldn't happen in this state - log them */
        case HMI_EVN_INSERT:        /* HMI reporting that board has been inserted */
        case HMI_EVN_STOP:          /* HMI reporting board has been stopped */
        case HMI_EVN_NOWBACKUP:     /* HMI reporting that board is now operating as backup */
        case HMI_EVN_STARTING:      /* HMI reporting that board has loaded and is now starting */
        case HMI_EVN_NOWSTANDALONE: /* HMI reporting that board is operating in standalone mode */
        case HMI_EVN_NO_BOARD:      /* HMI reporting board not present */
        default:
            UNEX_EVENT_ALARM( event );

    }   /* end switch on event */

    return( 0 );
}

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 * State: Backup
 *
 * Description:
 *
 * This node is standby.
 *
 * Timer T2 (monitoring for heartbeat) is running,
 * and we're just monitoring for failures.
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
static short RmgcBackup( DWORD event, DWORD value )
{
    U8      reject = 0;             /* reason for rejection (if primary selection results in a reject message) */
    S8     *reason = Rmgc_defaultReason;

    switch( event )
    {
        case RMG_EV_ACTV_RQST:      /* mate node is initializing and is asking to become primary */
            /* Use current network status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T2 );
            if (RmgcWantMatePrimary( value, 0, RmgMateNodeNum, &reject, &reason ))      /* 'current' = mate */
            {   /* mate should end up primary -> remain backup */
                RmgCommSend( RMG_EV_ACTV_CFM, 0, 0 );
                RmgcStartTimer( RMGC_TIMER_T2 );
            }
            else
            {   /* mate must become backup -> go primary */
                printf( "Switchover in progress (%s)\n", reason );
                RmgCommSend( RMG_EV_ACTV_REJ, reject, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_SWOVER_RQST:    /* mate node (primary) requesting that this node become primary [due to operator command or remote extraction pending] */
            /* Use current network status, operator-requested state, current primary status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T2 );
            if (RmgcWantMatePrimary( value, RmgNodeNum, RmgMateNodeNum, &reject, &reason ))     /* operator requested = local, current = mate */
            {   /* mate must remain primary -> remain backup */
                RmgCommSend( RMG_EV_SWOVER_REJ, reject, 0 );
                RmgcStartTimer( RMGC_TIMER_T2 );
            }
            else
            {   /* mate can become backup -> go primary */
                printf( "Switchover in progress (%s)\n", reason );
                RmgCommSend( RMG_EV_SWOVER_CFM, 0, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_TKOVER_RQST:    /* mate node (backup) requesting to become primary [due to operator command] */
            /* ~~~ out-of-sync (both nodes believe they are backup) ~~~
             * Use current network status, operator-requested state
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T2 );
            if (RmgcWantMatePrimary( value, RmgMateNodeNum, 0, &reject, &reason ))      /* operator requested = mate */
            {   /* mate can become primary -> remain backup */
                RmgCommSend( RMG_EV_TKOVER_CFM, 0, 0 );
                RmgcStartTimer( RMGC_TIMER_T2 );
            }
            else
            {   /* mate must remain backup -> go primary */
                printf( "Switchover in progress (%s)\n", reason );
                RmgCommSend( RMG_EV_TKOVER_REJ, reject, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_NETUP_RQST:     /* mate node (backup) network up so requesting to become primary (if primary network down) */
            /* ~~~ out-of-sync (both nodes believe they are backup) ~~~
             * Use current network status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T2 );
            if (RmgcWantMatePrimary( value, 0, 0, &reject, &reason ))
            {   /* mate can become primary -> remain backup */
                RmgCommSend( RMG_EV_NETUP_CFM, 0, 0 );
                RmgcStartTimer( RMGC_TIMER_T2 );
            }
            else
            {   /* mate must remain backup -> go primary */
                printf( "Switchover in progress (%s)\n", reason );
                RmgCommSend( RMG_EV_NETUP_REJ, reject, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_NETDOWN_RQST:   /* mate node (primary) network down so requesting that this node become primary (if primary network up) */
            /* Use current network status, current primary status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T2 );
            if (RmgcWantMatePrimary( value, 0, RmgMateNodeNum, &reject, &reason ))           /* current = mate */
            {   /* mate must remain primary -> remain backup */
                RmgCommSend( RMG_EV_NETDOWN_REJ, reject, 0 );
                RmgcStartTimer( RMGC_TIMER_T2 );
            }
            else
            {   /* mate can become backup -> go primary */
                printf( "Switchover in progress (%s)\n", reason );
                RmgCommSend( RMG_EV_NETDOWN_CFM, 0, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_HEARTBEAT:      /* mate node (primary) providing heartbeat as health indicator */
            /* Mate is still alive.
             * Use current network status, current primary status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T2 );
            if (RmgcWantMatePrimary( value, 0, RmgMateNodeNum, &reject, &reason ))       /* current = mate */
            {   /* mate remains primary -> remain backup */
                RmgcStartTimer( RMGC_TIMER_T2 );
            }
            else
            {   /* Our network status must be up while mate board's is down.
                 * Issue a NetUp request to take over being primary.
                 */
                printf( "Switchover in progress (%s)\n", reason );
                RmgcStartTimer( RMGC_TIMER_T1 );
                RmgCommSend( RMG_EV_NETUP_RQST, 0, 0 );
                LogStateChange( RMGC_STATE_TKOV_UP );
            }
            break;

        case RMG_EV_FAILURE:        /* mate node (primary) indicating that its board has failed */
            RmgcCancelTimer( RMGC_TIMER_T2 );
            printf( "Switchover in progress (mate node failed)\n" );
            RmgHmiPrimary( );
            RmgcStartTimer( RMGC_TIMER_T3 );
            SEND_HEARTBEAT;
            LogStateChange( RMGC_STATE_ACTIVE );
            break;

        case RMG_EV_T2:             /* timer 2 (monitoring for heartbeat timer) has fired */
            /* Active node not sending heartbeats, spring into action */
            printf( "Switchover in progress (mate node not heartbeating)\n" );
            RmgHmiPrimary( );
            RmgcStartTimer( RMGC_TIMER_T3 );
            SEND_HEARTBEAT;
            LogStateChange( RMGC_STATE_ACTIVE );
            break;

        case HMI_EVN_TASKDEAD:      /* HMI reporting that a task has failed */
        case HMI_EVN_BRDDEAD:       /* HMI reporting that board itself has failed */
        case HMI_EVN_HALTED:        /* HMI reporting that board is in the halted state */
        case RMG_EV_LOAD_CMD:       /* operator command to load board */
            /* reload the board */
            RmgcCancelTimer( RMGC_TIMER_T2 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgcStartTimer( RMGC_TIMER_T4 );
            RmgHmiLoadBoard( );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_LOADING:       /* HMI reporting that board is loading */
            /* board loaded by another application? */
            RmgcCancelTimer( RMGC_TIMER_T2 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_NETWORK_UP:    /* HMI forwarding task indication that network connectivity is up */
            RmgcUpdateNetworkStatus( RMG_NETWORK_UP );
            /* now that network is up this node may be better choice to be primary so ask for takeover */
            RmgcCancelTimer( RMGC_TIMER_T2 );
            RmgcStartTimer( RMGC_TIMER_T1 );
            RmgCommSend( RMG_EV_NETUP_RQST, 0, 0 );
            LogStateChange( RMGC_STATE_TKOV_UP );
            break;

        case HMI_EVN_NETWORK_DOWN:  /* HMI forwarding task indication that network connectivity is down */
            RmgcUpdateNetworkStatus( RMG_NETWORK_DOWN );
            /* already backup so no other action required */
            break;

        case RMG_EV_SWOVER_CMD:     /* operator command to cause changeover */
            RmgcCancelTimer( RMGC_TIMER_T2 );
            RmgcStartTimer( RMGC_TIMER_T1 );
            RmgCommSend( RMG_EV_TKOVER_RQST, 0, 0 );
            LogStateChange( RMGC_STATE_TKOV_RQST );
            break;

        case RMG_EV_HALT_CMD:       /* operator command to halt board */
            /* take board out of service */
            RmgcCancelTimer( RMGC_TIMER_T2 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgHmiHaltBoard( );
            LogStateChange( RMGC_STATE_OOS );
            break;

        case RMG_EV_EXTRACT_CMD:    /* operator command indicating ready for extraction */
            /* user extract command not valid in this state */
            printf( "%s not allowed in %s state\n",
                    LogNameEvent( event ), LogNameState( RmgState ) );
            break;

        case HMI_EVN_EXTRACT:       /* HMI reporting board extraction pending */
            RmgcCancelTimer( RMGC_TIMER_T2 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            LogStateChange( RMGC_STATE_EXTRACTING );
            break;

        case RMG_EV_SHUTDN:         /* main thread is shutting down */
            Rmgc_shutdown = 1;
            RmgcCancelTimer( RMGC_TIMER_T2 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        case HMI_EVN_SERVICE_DOWN:  /* HMI service has shutdown */
            RmgcCancelTimer( RMGC_TIMER_T2 );
            RmgcSetShutdown( );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these can happen, but nothing to do */
        case HMI_EVN_NOWBACKUP:     /* HMI reporting that board is now operating as backup */
        case HMI_EVN_CONNECTED:     /* HMI reporting that board is connected to its mate board */
        case HMI_EVN_ISOLATED:      /* HMI reporting that board is isolated from its mate board */
        case HMI_EVN_CONFLICT:      /* HMI reporting board conflict */
        case RMG_EV_ACTV_CFM:       /* mate node (now backup) has accepted request for this node to be primary [due to initial startup request] */
        case RMG_EV_ACTV_REJ:       /* mate node (now primary) has rejected request for this node to be primary [due to initial startup request] */
        case RMG_EV_SWOVER_CFM:     /* mate node (now primary) has accepted this node's request to become backup */
        case RMG_EV_SWOVER_REJ:     /* mate node (now backup) has rejected this node's request to become backup */
        case RMG_EV_TKOVER_CFM:     /* mate node (now backup) has accepted this node's request to become primary */
        case RMG_EV_TKOVER_REJ:     /* mate node (now primary) has rejected this node's request to become primary */
        case RMG_EV_NETUP_CFM:      /* mate node (now backup) has accepted this node's request to become primary due to network up */
        case RMG_EV_NETUP_REJ:      /* mate node (now primary) has rejected this node's request to become primary due to network up */
        case RMG_EV_NETDOWN_CFM:    /* mate node (now primary) has accepted this node's request to become backup due to network down */
        case RMG_EV_NETDOWN_REJ:    /* mate node (now backup) has rejected this node's request to become backup due to network down */
            break;

        /* these events may happen due to a stale timer firing indication */
        case RMG_EV_T1:             /* timer 1 (retry timer) has fired */
        case RMG_EV_T3:             /* timer 3 (periodic heartbeat send timer) has fired */
        case RMG_EV_T4:             /* timer 4 (board download timer) has fired */
            RmgcStaleTimer( event );        /* issue log indicating that a stale timer has fired */
            break;

        case EVENT_FREEZE:          /* an event has been converted into indication to freeze further event processing */
            RmgcCancelTimer( RMGC_TIMER_T2 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these events shouldn't happen in this state - log them */
        case HMI_EVN_STOP:          /* HMI reporting board has been stopped */
        case HMI_EVN_NOWPRIMARY:    /* HMI reporting that board is now operating as primary */
        case HMI_EVN_STARTING:      /* HMI reporting that board has loaded and is now starting */
        case HMI_EVN_NOWSTANDALONE: /* HMI reporting that board is operating in standalone mode */
        case HMI_EVN_NO_BOARD:      /* HMI reporting board not present */
        default:
            UNEX_EVENT_ALARM( event );

    }   /* end switch on event */

    return( 0 );
}

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 * State: Tkov_Rqst
 *
 * Description:
 *
 * Node was backup, takeover request has been sent to mate due to user request.
 *
 * Waiting for response to Tkover.
 * Timer T1 is running.
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
static short RmgcTkreq( DWORD event, DWORD value )
{
    U8      reject = 0;             /* reason for rejection (if primary selection results in a reject message) */
    S8     *reason = Rmgc_defaultReason;

    switch( event )
    {
        case RMG_EV_ACTV_RQST:      /* mate node is initializing and is asking to become primary */
            /* We are expecting a response to our request to become primary
             * but instead we have received an activation request.
             * Use current network status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, RmgNodeNum, 0, &reject, &reason ))      /* operator requested = local */
            {   /* mate should end up primary -> remain backup */
                printf( "Changeover failed - %s\n", reason );
                RmgCommSend( RMG_EV_ACTV_CFM, 0, 0 );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must become backup -> go primary */
                RmgCommSend( RMG_EV_ACTV_REJ, reject, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_SWOVER_RQST:    /* mate node (primary) requesting that this node become primary [due to operator command or remote extraction pending] */
            /* The local operator has requested a takeover for this node to become primary
             * at the same time that the mate node's operator caused the mate (primary) to request a switchover to become backup.
             * Use current network status, operator-requested state, current primary status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, 0, RmgMateNodeNum, &reject, &reason ))       /* current = mate */
            {   /* mate must remain primary -> remain backup */
                printf( "Changeover failed - %s\n", reason );
                RmgCommSend( RMG_EV_SWOVER_REJ, reject, 0 );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate can become backup -> go primary */
                RmgCommSend( RMG_EV_SWOVER_CFM, 0, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_TKOVER_RQST:    /* mate node (backup) requesting to become primary [due to operator command] */
            /* ~~~ out-of-sync (both nodes believe they are backup) ~~~
             * Use current network status, operator-requested state
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, 0, 0, &reject, &reason ))
            {   /* mate can become primary -> remain backup */
                printf( "Changeover failed - %s\n", reason );
                RmgCommSend( RMG_EV_TKOVER_CFM, 0, 0 );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must remain backup -> go primary */
                RmgCommSend( RMG_EV_TKOVER_REJ, reject, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_NETUP_RQST:     /* mate node (backup) network up so requesting to become primary (if primary network down) */
            /* ~~~ out-of-sync (both nodes believe they are backup) ~~~
             * Use current network status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, RmgNodeNum, 0, &reject, &reason ))               /* operator requested = local */
            {   /* mate can become primary -> remain backup */
                printf( "Changeover failed - %s\n", reason );
                RmgCommSend( RMG_EV_NETUP_CFM, 0, 0 );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must remain backup -> go primary */
                RmgCommSend( RMG_EV_NETUP_REJ, reject, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_NETDOWN_RQST:   /* mate node (primary) network down so requesting that this node become primary (if primary network up) */
            /* We are requesting to takeover being primary due to operator command
             * and mate node wants to be backup since mate board's network is down.
             * Use current network status, current primary status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, RmgNodeNum, RmgMateNodeNum, &reject, &reason ))              /* operator requested = local, current = mate */
            {   /* mate must remain primary -> remain backup */
                printf( "Changeover failed - %s\n", reason );
                RmgCommSend( RMG_EV_NETDOWN_REJ, reject, 0 );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate can become backup -> go primary */
                RmgCommSend( RMG_EV_NETDOWN_CFM, 0, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_TKOVER_CFM:     /* mate node (now backup) has accepted this node's request to become primary */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgHmiPrimary( );
            RmgcStartTimer( RMGC_TIMER_T3 );
            SEND_HEARTBEAT;
            LogStateChange( RMGC_STATE_ACTIVE );
            break;

        case RMG_EV_TKOVER_REJ:     /* mate node (now primary) has rejected this node's request to become primary */
            /* takeover rejected by mate -> remain backup */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcStartTimer( RMGC_TIMER_T2 );
            printf( "Changeover failed - refused by primary [%s]\n", RmgcMsgFailReason[RmgError.rmgMessage.lastInfo] );
            LogStateChange( RMGC_STATE_BACKUP );
            break;

        case RMG_EV_FAILURE:        /* mate node (primary) indicating that its board has failed */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgHmiPrimary( );
            RmgcStartTimer( RMGC_TIMER_T3 );
            SEND_HEARTBEAT;
            LogStateChange( RMGC_STATE_ACTIVE );
            break;

        case RMG_EV_T1:             /* timer 1 (retry timer) has fired */
            if (!(RmgRequestRetry))
            {   /* 1st timeout - retry the request */
                RmgCommSend( RMG_EV_TKOVER_RQST, 0, RMG_MSG_REQ_RETRY );
                RmgcStartTimer( RMGC_TIMER_T1 );
            }
            else
            {   /* 2nd timeout - mate not responding - remain backup */
                RmgcStartTimer( RMGC_TIMER_T2 );
                printf( "Changeover failed - no response from primary\n" );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            break;

        case HMI_EVN_TASKDEAD:      /* HMI reporting that a task has failed */
        case HMI_EVN_BRDDEAD:       /* HMI reporting that board itself has failed */
        case HMI_EVN_HALTED:        /* HMI reporting that board is in the halted state */
        case RMG_EV_LOAD_CMD:       /* operator command to load board */
            /* Notify mate board of failure (in case Tkover confirm is already on its way)
             * and reload the board. */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgHmiLoadBoard( );
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_LOADING, 0 );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_LOADING:       /* HMI reporting that board is loading */
            /* board loaded by another app? */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_LOADING, 0 );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_NETWORK_UP:    /* HMI forwarding task indication that network connectivity is up */
            RmgcUpdateNetworkStatus( RMG_NETWORK_UP );
            /* already have TKOVER_RQST outstanding so no other action required */
            break;

        case HMI_EVN_NETWORK_DOWN:  /* HMI forwarding task indication that network connectivity is down */
            RmgcUpdateNetworkStatus( RMG_NETWORK_DOWN );
            /* stil want the switchover to occur regardless of network state so nothing to do here */
            break;

        case RMG_EV_HALT_CMD:       /* operator command to halt board */
            /* Notify mate board of failure (in case it has already sent the Tkover confirm) and take board out of service. */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_OOS, 0 );
            RmgHmiHaltBoard( );
            LogStateChange( RMGC_STATE_OOS );
            break;

        case RMG_EV_SWOVER_CMD:     /* operator command to cause changeover */
        case RMG_EV_EXTRACT_CMD:    /* operator command indicating ready for extraction */
            /* these operator commands not valid when takeover in progress */
            printf( "%s not allowed in %s state\n",
                    LogNameEvent( event ), LogNameState( RmgState ) );
            break;

        case HMI_EVN_EXTRACT:       /* HMI reporting board extraction pending */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            LogStateChange( RMGC_STATE_EXTRACTING );
            break;

        case RMG_EV_SHUTDN:         /* main thread is shutting down */
            Rmgc_shutdown = 1;
            RmgcCancelTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        case HMI_EVN_SERVICE_DOWN:  /* HMI service has shutdown */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcSetShutdown( );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these can happen, but nothing to do */
        case HMI_EVN_NOWBACKUP:     /* HMI reporting that board is now operating as backup */
        case HMI_EVN_CONNECTED:     /* HMI reporting that board is connected to its mate board */
        case HMI_EVN_ISOLATED:      /* HMI reporting that board is isolated from its mate board */
        case HMI_EVN_CONFLICT:      /* HMI reporting board conflict */
        case RMG_EV_ACTV_CFM:       /* mate node (now backup) has accepted request for this node to be primary [due to initial startup request] */
        case RMG_EV_ACTV_REJ:       /* mate node (now primary) has rejected request for this node to be primary [due to initial startup request] */
        case RMG_EV_SWOVER_CFM:     /* mate node (now primary) has accepted this node's request to become backup */
        case RMG_EV_SWOVER_REJ:     /* mate node (now backup) has rejected this node's request to become backup */
        case RMG_EV_NETUP_CFM:      /* mate node (now backup) has accepted this node's request to become primary due to network up */
        case RMG_EV_NETUP_REJ:      /* mate node (now primary) has rejected this node's request to become primary due to network up */
        case RMG_EV_NETDOWN_CFM:    /* mate node (now primary) has accepted this node's request to become backup due to network down */
        case RMG_EV_NETDOWN_REJ:    /* mate node (now backup) has rejected this node's request to become backup due to network down */
        case RMG_EV_HEARTBEAT:      /* mate node (primary) providing heartbeat as health indicator */
            break;

        /* these events may happen due to a stale timer firing indication */
        case RMG_EV_T2:             /* timer 2 (monitoring for heartbeat timer) has fired */
        case RMG_EV_T3:             /* timer 3 (periodic heartbeat send timer) has fired */
        case RMG_EV_T4:             /* timer 4 (board download timer) has fired */
            RmgcStaleTimer( event );        /* issue log indicating that a stale timer has fired */
            break;

        case EVENT_FREEZE:          /* an event has been converted into indication to freeze further event processing */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these events shouldn't happen in this state - log them */
        case HMI_EVN_INSERT:        /* HMI reporting that board has been inserted */
        case HMI_EVN_STOP:          /* HMI reporting board has been stopped */
        case HMI_EVN_NOWPRIMARY:    /* HMI reporting that board is now operating as primary */
        case HMI_EVN_STARTING:      /* HMI reporting that board has loaded and is now starting */
        case HMI_EVN_NOWSTANDALONE: /* HMI reporting that board is operating in standalone mode */
        case HMI_EVN_NO_BOARD:      /* HMI reporting board not present */
        default:
            UNEX_EVENT_ALARM( event );

    }   /* end switch on event */

    return( 0 );
}

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 * State: Tkov_Up
 *
 * Description:
 *
 * Node was backup, NetUp [optional takeover] request has been sent to mate
 * due to network connectivity coming up.
 *
 * Waiting for response to NetUp.
 * Timer T1 is running.
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
static short RmgcTkreqUp( DWORD event, DWORD value )
{
    U8      reject = 0;             /* reason for rejection (if primary selection results in a reject message) */
    S8     *reason = Rmgc_defaultReason;

    switch( event )
    {
        case RMG_EV_ACTV_RQST:      /* mate node is initializing and is asking to become primary */
            /* We are expecting a response to our request to become primary,
             * but instead we have received an activation request.
             * Use current network status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, 0, RmgMateNodeNum, &reject, &reason ))      /* 'current' = mate */
            {   /* mate should end up primary -> remain backup */
                RmgCommSend( RMG_EV_ACTV_CFM, 0, 0 );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must become backup -> go primary */
                printf( "Switchover in progress (%s)\n", reason );
                RmgCommSend( RMG_EV_ACTV_REJ, reject, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_SWOVER_RQST:    /* mate node (primary) requesting that this node become primary [due to operator command or remote extraction pending] */
            /* We are expecting a response to our NetUp request to become primary,
             * but instead we have received a switchover request.
             * Use current network status, operator-requested state, current primary status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, RmgNodeNum, RmgMateNodeNum, &reject, &reason ))     /* operator requested = local, current = mate */
            {   /* mate must remain primary -> remain backup */
                RmgCommSend( RMG_EV_SWOVER_REJ, reject, 0 );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate can become backup -> go primary */
                printf( "Switchover in progress (%s)\n", reason );
                RmgCommSend( RMG_EV_SWOVER_CFM, 0, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_TKOVER_RQST:    /* mate node (backup) requesting to become primary [due to operator command] */
            /* ~~~ out-of-sync (both nodes believe they are backup) ~~~
             * Use current network status, operator-requested state
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, RmgMateNodeNum, 0, &reject, &reason ))               /* operator requested = mate */
            {   /* mate can become primary -> remain backup */
                RmgCommSend( RMG_EV_TKOVER_CFM, 0, 0 );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must remain backup -> go primary */
                printf( "Switchover in progress (%s)\n", reason );
                RmgCommSend( RMG_EV_TKOVER_REJ, reject, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_NETUP_RQST:     /* mate node (backup) network up so requesting to become primary (if primary network down) */
            /* ~~~ out-of-sync (both nodes believe they are backup) ~~~
             * Use current network status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, 0, 0, &reject, &reason ))
            {   /* mate can become primary -> remain backup */
                RmgCommSend( RMG_EV_NETUP_CFM, 0, 0 );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate must remain backup -> go primary */
                printf( "Switchover in progress (%s)\n", reason );
                RmgCommSend( RMG_EV_NETUP_REJ, reject, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_NETDOWN_RQST:   /* mate node (primary) network down so requesting that this node become primary (if primary network up) */
            /* We are requesting a NetUp takeover snce our board's network is up,
             * and mate node wants to be backup since mate board's network is down.
             * Use current network status, current primary status
             * (and node numbers if needed) to decide which should be primary.
             */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            if (RmgcWantMatePrimary( value, 0, RmgMateNodeNum, &reject, &reason ))               /* current = mate */
            {   /* mate must remain primary -> remain backup */
                RmgCommSend( RMG_EV_NETDOWN_REJ, reject, 0 );
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            else
            {   /* mate can become backup -> go primary */
                printf( "Switchover in progress (%s)\n", reason );
                RmgCommSend( RMG_EV_NETDOWN_CFM, 0, 0 );
                RmgHmiPrimary( );
                RmgcStartTimer( RMGC_TIMER_T3 );
                SEND_HEARTBEAT;
                LogStateChange( RMGC_STATE_ACTIVE );
            }
            break;

        case RMG_EV_NETUP_CFM:      /* mate node (now backup) has accepted this node's request to become primary due to network up */
            printf( "Switchover in progress (local network up)\n" );
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgHmiPrimary( );
            RmgcStartTimer( RMGC_TIMER_T3 );
            SEND_HEARTBEAT;
            LogStateChange( RMGC_STATE_ACTIVE );
            break;

        case RMG_EV_NETUP_REJ:      /* mate node (now primary) has rejected this node's request to become primary due to network up */
            /* Takeover rejected by mate node - resume being backup */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcStartTimer( RMGC_TIMER_T2 );
            LogStateChange( RMGC_STATE_BACKUP );
            break;

        case RMG_EV_FAILURE:        /* mate node (primary) indicating that its board has failed */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            printf( "Switchover in progress (mate node failed)\n" );
            RmgHmiPrimary( );
            RmgcStartTimer( RMGC_TIMER_T3 );
            SEND_HEARTBEAT;
            LogStateChange( RMGC_STATE_ACTIVE );
            break;

        case RMG_EV_T1:             /* timer 1 (retry timer) has fired */
            if (!(RmgRequestRetry))
            {   /* 1st timeout - retry the request */
                RmgCommSend( RMG_EV_NETUP_RQST, 0, RMG_MSG_REQ_RETRY );
                RmgcStartTimer( RMGC_TIMER_T1 );
            }
            else
            {   /* 2nd timeout - mate not responding - remain backup */
                RmgcStartTimer( RMGC_TIMER_T2 );
                LogStateChange( RMGC_STATE_BACKUP );
            }
            break;

        case HMI_EVN_TASKDEAD:      /* HMI reporting that a task has failed */
        case HMI_EVN_BRDDEAD:       /* HMI reporting that board itself has failed */
        case HMI_EVN_HALTED:        /* HMI reporting that board is in the halted state */
        case RMG_EV_LOAD_CMD:       /* operator command to load board */
            /* Notify mate board of failure (in case NetUp confirm is already on its way)
             * and reload the board. */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgHmiLoadBoard( );
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_LOADING, 0 );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_LOADING:       /* HMI reporting that board is loading */
            /* board loaded by another app? */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_LOADING, 0 );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_NETWORK_UP:    /* HMI forwarding task indication that network connectivity is up */
            RmgcUpdateNetworkStatus( RMG_NETWORK_UP );
            /* already have NetUp request outstanding so no other action required */
            break;

        case HMI_EVN_NETWORK_DOWN:  /* HMI forwarding task indication that network connectivity is down */
            RmgcUpdateNetworkStatus( RMG_NETWORK_DOWN );
            /* We have a network-up [optional switchover] request outstanding and now the network is down;
             * wait for the remote node to respond to the NetUp request before requesting another state change.
             */
            break;

        case RMG_EV_HALT_CMD:       /* operator command to halt board */
            /* Notify mate board of failure (in case it has already sent the NetUp confirm) and take board out of service. */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            RmgCommSend( RMG_EV_FAILURE, RMG_REASON_OOS, 0 );
            RmgHmiHaltBoard( );
            LogStateChange( RMGC_STATE_OOS );
            break;

        case RMG_EV_SWOVER_CMD:     /* operator command to cause changeover */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcStartTimer( RMGC_TIMER_T1 );
            RmgCommSend( RMG_EV_TKOVER_RQST, 0, 0 );
            LogStateChange( RMGC_STATE_TKOV_RQST );
            break;

        case RMG_EV_EXTRACT_CMD:    /* operator command indicating ready for extraction */
            /* extract command not valid in this state */
            printf( "%s not allowed in %s state\n",
                    LogNameEvent( event ), LogNameState( RmgState ) );
            break;

        case HMI_EVN_EXTRACT:       /* HMI reporting board extraction pending */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcUpdateNetworkStatus( RMG_NETWORK_UNKNOWN );     /* network connectivity status no longer known */
            LogStateChange( RMGC_STATE_EXTRACTING );
            break;

        case RMG_EV_SHUTDN:         /* main thread is shutting down */
            Rmgc_shutdown = 1;
            RmgcCancelTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        case HMI_EVN_SERVICE_DOWN:  /* HMI service has shutdown */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            RmgcSetShutdown( );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these can happen, but nothing to do */
        case HMI_EVN_NOWBACKUP:     /* HMI reporting that board is now operating as backup */
        case HMI_EVN_CONNECTED:     /* HMI reporting that board is connected to its mate board */
        case HMI_EVN_ISOLATED:      /* HMI reporting that board is isolated from its mate board */
        case HMI_EVN_CONFLICT:      /* HMI reporting board conflict */
        case RMG_EV_ACTV_CFM:       /* mate node (now backup) has accepted request for this node to be primary [due to initial startup request] */
        case RMG_EV_ACTV_REJ:       /* mate node (now primary) has rejected request for this node to be primary [due to initial startup request] */
        case RMG_EV_SWOVER_CFM:     /* mate node (now primary) has accepted this node's request to become backup */
        case RMG_EV_SWOVER_REJ:     /* mate node (now backup) has rejected this node's request to become backup */
        case RMG_EV_TKOVER_CFM:     /* mate node (now backup) has accepted this node's request to become primary */
        case RMG_EV_TKOVER_REJ:     /* mate node (now primary) has rejected this node's request to become primary */
        case RMG_EV_NETDOWN_CFM:    /* mate node (now primary) has accepted this node's request to become backup due to network down */
        case RMG_EV_NETDOWN_REJ:    /* mate node (now backup) has rejected this node's request to become backup due to network down */
        case RMG_EV_HEARTBEAT:      /* mate node (primary) providing heartbeat as health indicator */
            break;

        /* these events may happen due to a stale timer firing indication */
        case RMG_EV_T2:             /* timer 2 (monitoring for heartbeat timer) has fired */
        case RMG_EV_T3:             /* timer 3 (periodic heartbeat send timer) has fired */
        case RMG_EV_T4:             /* timer 4 (board download timer) has fired */
            RmgcStaleTimer( event );        /* issue log indicating that a stale timer has fired */
            break;

        case EVENT_FREEZE:          /* an event has been converted into indication to freeze further event processing */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these events shouldn't happen in this state - log them */
        case HMI_EVN_INSERT:        /* HMI reporting that board has been inserted */
        case HMI_EVN_STOP:          /* HMI reporting board has been stopped */
        case HMI_EVN_NOWPRIMARY:    /* HMI reporting that board is now operating as primary */
        case HMI_EVN_STARTING:      /* HMI reporting that board has loaded and is now starting */
        case HMI_EVN_NOWSTANDALONE: /* HMI reporting that board is operating in standalone mode */
        case HMI_EVN_NO_BOARD:      /* HMI reporting board not present */
        default:
            UNEX_EVENT_ALARM( event );

    }   /* end switch on event */

    return( 0 );
}

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 * State: Out_of_Service
 *
 * Description:
 *
 * The board failed and could not be reloaded or was manually halted.
 * Manual intervention is required to get things going again.
 *
 * No timers are running in this state.
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
static short RmgcOOS( DWORD event, DWORD value )
{
    switch( event )
    {
        case RMG_EV_ACTV_RQST:      /* mate node is initializing and is asking to become primary */
            /* Mate node is initializing, let it be active */
            RmgCommSend( RMG_EV_ACTV_CFM, 0, 0 );
            break;

        case RMG_EV_SWOVER_RQST:    /* mate node (primary) requesting that this node become primary [due to operator command or remote extraction pending] */
            /* Other node wants us to take over - refuse. */
            RmgCommSend( RMG_EV_SWOVER_REJ, RMG_REASON_OOS, 0 );
            break;

        case RMG_EV_TKOVER_RQST:    /* mate node (backup) requesting to become primary [due to operator command] */
            /* Other node wants to go active - confirm it. */
            RmgCommSend( RMG_EV_TKOVER_CFM, 0, 0 );
            break;

        case RMG_EV_NETUP_RQST:     /* mate node (backup) network up so requesting to become primary (if primary network down) */
            /* Other node wants to go active - confirm it. */
            RmgCommSend( RMG_EV_NETUP_CFM, 0, 0 );
            break;

        case RMG_EV_NETDOWN_RQST:   /* mate node (primary) network down so requesting that this node become primary (if primary network up) */
            /* Other node wants us to take over - refuse. */
            RmgCommSend( RMG_EV_NETDOWN_REJ, RMG_REASON_OOS, 0 );
            break;

        case RMG_EV_LOAD_CMD:       /* operator command to load board */
            /* Reload the board. */
            RmgHmiLoadBoard( );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case RMG_EV_HALT_CMD:       /* operator command to halt board */
        case RMG_EV_SWOVER_CMD:     /* operator command to cause changeover */
        case RMG_EV_EXTRACT_CMD:    /* operator command indicating ready for extraction */
            /* These operator commands not allowed in this state */
            printf( "%s not allowed in %s state\n",
                    LogNameEvent( event ), LogNameState( RmgState ) );
            break;

        case HMI_EVN_EXTRACT:       /* HMI reporting board extraction pending */
            LogStateChange( RMGC_STATE_EXTRACTING );
            break;

        case RMG_EV_SHUTDN:         /* main thread is shutting down */
            Rmgc_shutdown = 1;
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        case HMI_EVN_SERVICE_DOWN:  /* HMI service has shutdown */
            RmgcSetShutdown( );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these can happen, but nothing to do */
        case HMI_EVN_TASKDEAD:      /* HMI reporting that a task has failed */
        case HMI_EVN_BRDDEAD:       /* HMI reporting that board itself has failed */
        case HMI_EVN_HALTED:        /* HMI reporting that board is in the halted state */
        case HMI_EVN_CONNECTED:     /* HMI reporting that board is connected to its mate board */
        case HMI_EVN_ISOLATED:      /* HMI reporting that board is isolated from its mate board */
        case HMI_EVN_CONFLICT:      /* HMI reporting board conflict */
        case HMI_EVN_NETWORK_UP:    /* HMI forwarding task indication that network connectivity is up */
        case HMI_EVN_NETWORK_DOWN:  /* HMI forwarding task indication that network connectivity is down */
        case RMG_EV_FAILURE:        /* mate node (primary) indicating that its board has failed */
        case RMG_EV_ACTV_CFM:       /* mate node (now backup) has accepted request for this node to be primary [due to initial startup request] */
        case RMG_EV_ACTV_REJ:       /* mate node (now primary) has rejected request for this node to be primary [due to initial startup request] */
        case RMG_EV_SWOVER_CFM:     /* mate node (now primary) has accepted this node's request to become backup */
        case RMG_EV_SWOVER_REJ:     /* mate node (now backup) has rejected this node's request to become backup */
        case RMG_EV_TKOVER_CFM:     /* mate node (now backup) has accepted this node's request to become primary */
        case RMG_EV_TKOVER_REJ:     /* mate node (now primary) has rejected this node's request to become primary */
        case RMG_EV_NETUP_CFM:      /* mate node (now backup) has accepted this node's request to become primary due to network up */
        case RMG_EV_NETUP_REJ:      /* mate node (now primary) has rejected this node's request to become primary due to network up */
        case RMG_EV_NETDOWN_CFM:    /* mate node (now primary) has accepted this node's request to become backup due to network down */
        case RMG_EV_NETDOWN_REJ:    /* mate node (now backup) has rejected this node's request to become backup due to network down */
        case RMG_EV_HEARTBEAT:      /* mate node (primary) providing heartbeat as health indicator */
            break;

        /* these events may happen due to a stale timer firing indication */
        case RMG_EV_T1:             /* timer 1 (retry timer) has fired */
        case RMG_EV_T2:             /* timer 2 (monitoring for heartbeat timer) has fired */
        case RMG_EV_T3:             /* timer 3 (periodic heartbeat send timer) has fired */
        case RMG_EV_T4:             /* timer 4 (board download timer) has fired */
            RmgcStaleTimer( event );        /* issue log indicating that a stale timer has fired */
            break;

        case EVENT_FREEZE:          /* an event has been converted into indication to freeze further event processing */
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these events shouldn't happen in this state - log them */
        case HMI_EVN_INSERT:        /* HMI reporting that board has been inserted */
        case HMI_EVN_STOP:          /* HMI reporting board has been stopped */
        case HMI_EVN_LOADING:       /* HMI reporting that board is loading */
        case HMI_EVN_NOWPRIMARY:    /* HMI reporting that board is now operating as primary */
        case HMI_EVN_NOWBACKUP:     /* HMI reporting that board is now operating as backup */
        case HMI_EVN_STARTING:      /* HMI reporting that board has loaded and is now starting */
        case HMI_EVN_NOWSTANDALONE: /* HMI reporting that board is operating in standalone mode */
        case HMI_EVN_NO_BOARD:      /* HMI reporting board not present */
        default:
            UNEX_EVENT_ALARM( event );

    }   /* end switch on event */

    return( 0 );
}

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 * State: Standalone
 *
 * Description:
 *
 * This node/board is not redundant, we're just monitoring for failures
 * to auto-reload a failed board.
 *
 * No timers are running in this state.
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
static short RmgcStandalone( DWORD event, DWORD value )
{
    switch( event )
    {
        case HMI_EVN_TASKDEAD:      /* HMI reporting that a task has failed */
        case HMI_EVN_BRDDEAD:       /* HMI reporting that board itself has failed */
        case HMI_EVN_HALTED:        /* HMI reporting that board is in the halted state */
        case RMG_EV_LOAD_CMD:       /* operator command to load board */
            /* initiate a reload of the board */
            RmgHmiLoadBoard( );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_LOADING:       /* HMI reporting that board is loading */
            /* board reloaded by another app? */
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case HMI_EVN_NETWORK_UP:    /* HMI forwarding task indication that network connectivity is up */
            RmgcUpdateNetworkStatus( RMG_NETWORK_UP );
            break;

        case HMI_EVN_NETWORK_DOWN:  /* HMI forwarding task indication that network connectivity is down */
            RmgcUpdateNetworkStatus( RMG_NETWORK_DOWN );
            break;

        case RMG_EV_SWOVER_CMD:     /* operator command to cause changeover */
        case RMG_EV_EXTRACT_CMD:    /* operator command indicating ready for extraction */
            printf( "%s not allowed in %s state\n",
                    LogNameEvent( event ), LogNameState( RmgState ) );
            break;

        case RMG_EV_HALT_CMD:       /* operator command to halt board */
            /* Take board out of service. */
            RmgHmiHaltBoard( );
            LogStateChange( RMGC_STATE_OOS );
            break;

        case HMI_EVN_EXTRACT:       /* HMI reporting board extraction pending */
            LogStateChange( RMGC_STATE_EXTRACTING );
            break;

        case RMG_EV_SHUTDN:         /* main thread is shutting down */
            Rmgc_shutdown = 1;
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        case HMI_EVN_SERVICE_DOWN:  /* HMI service has shutdown */
            /* user requested shutdown or HMI service stopped */
            RmgcSetShutdown( );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these can happen, but nothing to do */
        case HMI_EVN_NOWSTANDALONE: /* HMI reporting that board is operating in standalone mode */
        case HMI_EVN_ISOLATED:      /* HMI reporting that board is isolated from its mate board */
            break;

        /* these events may happen due to a stale timer firing indication */
        case RMG_EV_T1:             /* timer 1 (retry timer) has fired */
        case RMG_EV_T2:             /* timer 2 (monitoring for heartbeat timer) has fired */
        case RMG_EV_T3:             /* timer 3 (periodic heartbeat send timer) has fired */
        case RMG_EV_T4:             /* timer 4 (board download timer) has fired */
            RmgcStaleTimer( event );        /* issue log indicating that a stale timer has fired */
            break;

        case EVENT_FREEZE:          /* an event has been converted into indication to freeze further event processing */
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these events shouldn't happen in this state - log them */
        case HMI_EVN_INSERT:        /* HMI reporting that board has been inserted */
        case RMG_EV_ACTV_RQST:      /* mate node is initializing and is asking to become primary */
        case RMG_EV_ACTV_CFM:       /* mate node (now backup) has accepted request for this node to be primary [due to initial startup request] */
        case RMG_EV_ACTV_REJ:       /* mate node (now primary) has rejected request for this node to be primary [due to initial startup request] */
        case RMG_EV_SWOVER_RQST:    /* mate node (primary) requesting that this node become primary [due to operator command or remote extraction pending] */
        case RMG_EV_SWOVER_CFM:     /* mate node (now primary) has accepted this node's request to become backup */
        case RMG_EV_SWOVER_REJ:     /* mate node (now backup) has rejected this node's request to become backup */
        case RMG_EV_TKOVER_RQST:    /* mate node (backup) requesting to become primary [due to operator command] */
        case RMG_EV_TKOVER_CFM:     /* mate node (now backup) has accepted this node's request to become primary */
        case RMG_EV_TKOVER_REJ:     /* mate node (now primary) has rejected this node's request to become primary */
        case RMG_EV_NETUP_RQST:     /* mate node (backup) network up so requesting to become primary (if primary network down) */
        case RMG_EV_NETUP_CFM:      /* mate node (now backup) has accepted this node's request to become primary due to network up */
        case RMG_EV_NETUP_REJ:      /* mate node (now primary) has rejected this node's request to become primary due to network up */
        case RMG_EV_NETDOWN_RQST:   /* mate node (primary) network down so requesting that this node become primary (if primary network up) */
        case RMG_EV_NETDOWN_CFM:    /* mate node (now primary) has accepted this node's request to become backup due to network down */
        case RMG_EV_NETDOWN_REJ:    /* mate node (now backup) has rejected this node's request to become backup due to network down */
        case RMG_EV_FAILURE:        /* mate node (primary) indicating that its board has failed */
        case RMG_EV_HEARTBEAT:      /* mate node (primary) providing heartbeat as health indicator */
        case HMI_EVN_STOP:          /* HMI reporting board has been stopped */
        case HMI_EVN_NOWPRIMARY:    /* HMI reporting that board is now operating as primary */
        case HMI_EVN_CONNECTED:     /* HMI reporting that board is connected to its mate board */
        case HMI_EVN_CONFLICT:      /* HMI reporting board conflict */
        case HMI_EVN_NOWBACKUP:     /* HMI reporting that board is now operating as backup */
        case HMI_EVN_STARTING:      /* HMI reporting that board has loaded and is now starting */
        case HMI_EVN_NO_BOARD:      /* HMI reporting board not present */
        default:
            UNEX_EVENT_ALARM( event );

    }   /* end switch on event */

    return( 0 );
}

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 * State: Extracting
 *
 * Description:
 *
 * Board extraction in progress (as reported by HMI). If the extraction pending
 * event was received when this node was acting as the primary then a
 * switchover request has been sent to the mate node.
 *
 * Waiting for HMI stop indication.
 * [May receive response to Swover (if extraction event when primary)]
 * [Timer T1 is running (if extraction event when primary)]
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
static short RmgcExtracting( DWORD event, DWORD value )
{
    switch( event )
    {
        case RMG_EV_ACTV_RQST:      /* mate node is initializing and is asking to become primary */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            /* allow other node to be primary */
            RmgCommSend( RMG_EV_ACTV_CFM, 0, 0 );
            break;

        case RMG_EV_SWOVER_RQST:    /* mate node (primary) requesting that this node become primary [due to operator command or remote extraction pending] */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            /* Other node wants us to take over - can't allow it when extracting. */
            RmgCommSend( RMG_EV_SWOVER_REJ, RMG_REASON_EXTRACTING, 0 );
            break;

        case RMG_EV_TKOVER_RQST:    /* mate node (backup) requesting to become primary [due to operator command] */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            /* other node wants to go active, allow it */
            RmgCommSend( RMG_EV_TKOVER_CFM, 0, 0 );
            break;

        case RMG_EV_NETUP_RQST:     /* mate node (backup) network up so requesting to become primary (if primary network down) */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            /* Other node wants to go active, allow it. */
            RmgCommSend( RMG_EV_NETUP_CFM, 0, 0 );
            break;

        case RMG_EV_NETDOWN_RQST:   /* mate node (primary) network down so requesting that this node become primary (if primary network up) */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            /* Other node wants us to take over - can't allow it when extracting. */
            RmgCommSend( RMG_EV_NETDOWN_REJ, RMG_REASON_EXTRACTING, 0 );
            break;

        case HMI_EVN_STOP:          /* HMI reporting board has been stopped */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_EXTRACTED );
            break;

        case RMG_EV_SWOVER_CMD:     /* operator command to cause changeover */
        case RMG_EV_LOAD_CMD:       /* operator command to load board */
        case RMG_EV_HALT_CMD:       /* operator command to halt board */
            /* These operator commands not allowed in this state */
            printf( "%s not allowed in %s state\n",
                    LogNameEvent( event ), LogNameState( RmgState ) );
            break;

        case RMG_EV_T1:             /* timer 1 (retry timer) has fired */
            RmgCommSend( RMG_EV_SWOVER_RQST, 0, 0 );            /* always send this switchover request without the retry flag set (so we retry forever) */
            RmgcStartTimer( RMGC_TIMER_T1 );
            break;

        case RMG_EV_SWOVER_CFM:     /* mate node (now primary) has accepted this node's request to become backup */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            break;

        case RMG_EV_EXTRACT_CMD:    /* operator command indicating ready for extraction */
            RmgHmiStop();
            break;

        case RMG_EV_SHUTDN:         /* main thread is shutting down */
            Rmgc_shutdown = 1;
            RmgcCancelTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        case HMI_EVN_SERVICE_DOWN:  /* HMI service has shutdown */
            RmgcSetShutdown( );
            RmgcCancelTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these can happen, but nothing to do */
        case RMG_EV_HEARTBEAT:      /* mate node (primary) providing heartbeat as health indicator */
        case HMI_EVN_HALTED:        /* HMI reporting that board is in the halted state */
        case HMI_EVN_NOWPRIMARY:    /* HMI reporting that board is now operating as primary */
        case HMI_EVN_NOWBACKUP:     /* HMI reporting that board is now operating as backup */
        case HMI_EVN_STARTING:      /* HMI reporting that board has loaded and is now starting */
        case HMI_EVN_NOWSTANDALONE: /* HMI reporting that board is operating in standalone mode */
        case HMI_EVN_TASKDEAD:      /* HMI reporting that a task has failed */
        case HMI_EVN_BRDDEAD:       /* HMI reporting that board itself has failed */
        case HMI_EVN_LOADING:       /* HMI reporting that board is loading */
        case HMI_EVN_CONNECTED:     /* HMI reporting that board is connected to its mate board */
        case HMI_EVN_ISOLATED:      /* HMI reporting that board is isolated from its mate board */
        case HMI_EVN_NETWORK_UP:    /* HMI forwarding task indication that network connectivity is up */
        case HMI_EVN_NETWORK_DOWN:  /* HMI forwarding task indication that network connectivity is down */
        case RMG_EV_ACTV_CFM:       /* mate node (now backup) has accepted request for this node to be primary [due to initial startup request] */
        case RMG_EV_ACTV_REJ:       /* mate node (now primary) has rejected request for this node to be primary [due to initial startup request] */
        case RMG_EV_SWOVER_REJ:     /* mate node (now backup) has rejected this node's request to become backup */
        case RMG_EV_TKOVER_CFM:     /* mate node (now backup) has accepted this node's request to become primary */
        case RMG_EV_TKOVER_REJ:     /* mate node (now primary) has rejected this node's request to become primary */
        case RMG_EV_NETUP_CFM:      /* mate node (now backup) has accepted this node's request to become primary due to network up */
        case RMG_EV_NETUP_REJ:      /* mate node (now primary) has rejected this node's request to become primary due to network up */
        case RMG_EV_NETDOWN_CFM:    /* mate node (now primary) has accepted this node's request to become backup due to network down */
        case RMG_EV_NETDOWN_REJ:    /* mate node (now backup) has rejected this node's request to become backup due to network down */
        case RMG_EV_FAILURE:        /* mate node (primary) indicating that its board has failed */
            break;

        /* these events may happen due to a stale timer firing indication */
        case RMG_EV_T2:             /* timer 2 (monitoring for heartbeat timer) has fired */
        case RMG_EV_T3:             /* timer 3 (periodic heartbeat send timer) has fired */
        case RMG_EV_T4:             /* timer 4 (board download timer) has fired */
            RmgcStaleTimer( event );        /* issue log indicating that a stale timer has fired */
            break;

        case EVENT_FREEZE:          /* an event has been converted into indication to freeze further event processing */
            RmgcCancelTimer( RMGC_TIMER_T1 );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these events shouldn't happen - log them */
        case HMI_EVN_NO_BOARD:      /* HMI reporting board not present */
        case HMI_EVN_CONFLICT:      /* HMI reporting board conflict */
        case HMI_EVN_EXTRACT:       /* HMI reporting board extraction pending */
        case HMI_EVN_INSERT:        /* HMI reporting that board has been inserted */
        default:
            UNEX_EVENT_ALARM( event );

    }   /* end switch on event */

    return( 0 );
}

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 * State: Extracted
 *
 * Description: board has been extracted
 *
 * No timers are running in this state.
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
static short RmgcExtracted( DWORD event, DWORD value )
{
    switch( event )
    {
        case RMG_EV_ACTV_RQST:      /* mate node is initializing and is asking to become primary */
            /* allow other node to be primary */
            RmgCommSend( RMG_EV_ACTV_CFM, 0, 0 );
            break;

        case RMG_EV_SWOVER_RQST:    /* mate node (primary) requesting that this node become primary [due to operator command or remote extraction pending] */
            /* Other node wants us to take over - can't allow it when extracting. */
            RmgCommSend( RMG_EV_SWOVER_REJ, RMG_REASON_EXTRACTED, 0 );
            break;

        case RMG_EV_TKOVER_RQST:    /* mate node (backup) requesting to become primary [due to operator command] */
            /* Other node wants to go active, allow it. */
            RmgCommSend( RMG_EV_TKOVER_CFM, 0, 0 );
            break;

        case RMG_EV_NETUP_RQST:     /* mate node (backup) network up so requesting to become primary (if primary network down) */
            /* Other node wants to go active, allow it. */
            RmgCommSend( RMG_EV_NETUP_CFM, 0, 0 );
            break;

        case RMG_EV_NETDOWN_RQST:   /* mate node (primary) network down so requesting that this node become primary (if primary network up) */
            /* Other node wants us to take over - can't allow it when extracting. */
            RmgCommSend( RMG_EV_NETDOWN_REJ, RMG_REASON_EXTRACTED, 0 );
            break;

        case HMI_EVN_HALTED:        /* HMI reporting that board is in the halted state */
            RmgHmiLoadBoard( );
            RmgcStartTimer( RMGC_TIMER_T4 );
            LogStateChange( RMGC_STATE_LOADING );
            break;

        case RMG_EV_HALT_CMD:       /* operator command to halt board */
        case RMG_EV_LOAD_CMD:       /* operator command to load board */
        case RMG_EV_SWOVER_CMD:     /* operator command to cause changeover */
        case RMG_EV_EXTRACT_CMD:    /* operator command indicating ready for extraction */
            /* These operator commands not allowed in this state */
            printf( "%s not allowed in %s state\n",
                    LogNameEvent( event ), LogNameState( RmgState ) );
            break;

        case RMG_EV_SHUTDN:         /* main thread is shutting down */
            Rmgc_shutdown = 1;
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        case HMI_EVN_SERVICE_DOWN:  /* HMI service has shutdown */
            RmgcSetShutdown( );
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these can happen, but nothing to do */
        case HMI_EVN_INSERT:        /* HMI reporting that board has been inserted */
        case RMG_EV_HEARTBEAT:      /* mate node (primary) providing heartbeat as health indicator */
        case HMI_EVN_NOWPRIMARY:    /* HMI reporting that board is now operating as primary */
        case HMI_EVN_NOWBACKUP:     /* HMI reporting that board is now operating as backup */
        case HMI_EVN_STARTING:      /* HMI reporting that board has loaded and is now starting */
        case HMI_EVN_NOWSTANDALONE: /* HMI reporting that board is operating in standalone mode */
        case HMI_EVN_TASKDEAD:      /* HMI reporting that a task has failed */
        case HMI_EVN_BRDDEAD:       /* HMI reporting that board itself has failed */
        case HMI_EVN_LOADING:       /* HMI reporting that board is loading */
        case HMI_EVN_CONNECTED:     /* HMI reporting that board is connected to its mate board */
        case HMI_EVN_ISOLATED:      /* HMI reporting that board is isolated from its mate board */
        case HMI_EVN_NETWORK_UP:    /* HMI forwarding task indication that network connectivity is up */
        case HMI_EVN_NETWORK_DOWN:  /* HMI forwarding task indication that network connectivity is down */
        case RMG_EV_ACTV_CFM:       /* mate node (now backup) has accepted request for this node to be primary [due to initial startup request] */
        case RMG_EV_ACTV_REJ:       /* mate node (now primary) has rejected request for this node to be primary [due to initial startup request] */
        case RMG_EV_SWOVER_CFM:     /* mate node (now primary) has accepted this node's request to become backup */
        case RMG_EV_SWOVER_REJ:     /* mate node (now backup) has rejected this node's request to become backup */
        case RMG_EV_TKOVER_CFM:     /* mate node (now backup) has accepted this node's request to become primary */
        case RMG_EV_TKOVER_REJ:     /* mate node (now primary) has rejected this node's request to become primary */
        case RMG_EV_NETUP_CFM:      /* mate node (now backup) has accepted this node's request to become primary due to network up */
        case RMG_EV_NETUP_REJ:      /* mate node (now primary) has rejected this node's request to become primary due to network up */
        case RMG_EV_NETDOWN_CFM:    /* mate node (now primary) has accepted this node's request to become backup due to network down */
        case RMG_EV_NETDOWN_REJ:    /* mate node (now backup) has rejected this node's request to become backup due to network down */
        case RMG_EV_FAILURE:        /* mate node (primary) indicating that its board has failed */
            break;

        /* these events may happen due to a stale timer firing indication */
        case RMG_EV_T1:             /* timer 1 (retry timer) has fired */
        case RMG_EV_T2:             /* timer 2 (monitoring for heartbeat timer) has fired */
        case RMG_EV_T3:             /* timer 3 (periodic heartbeat send timer) has fired */
        case RMG_EV_T4:             /* timer 4 (board download timer) has fired */
            RmgcStaleTimer( event );        /* issue log indicating that a stale timer has fired */
            break;

        case EVENT_FREEZE:          /* an event has been converted into indication to freeze further event processing */
            LogStateChange( RMGC_STATE_FROZEN );
            break;

        /* these events shouldn't happen - log them */
        case HMI_EVN_NO_BOARD:      /* HMI reporting board not present */
        case HMI_EVN_CONFLICT:      /* HMI reporting board conflict */
        case HMI_EVN_EXTRACT:       /* HMI reporting board extraction pending */
        case HMI_EVN_STOP:          /* HMI reporting board has been stopped */
        default:
            UNEX_EVENT_ALARM( event );

    }   /* end switch on event */

    return( 0 );
}

/*@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*
 * State: Frozen
 *
 * Description:
 *
 * An event has occurred that has been configured to stop further RMG processing
 * (to leave the system in a state that can be manually diagnosed).
 * The RMG process must be re-started to continue from this state.
 *
 * No timers are running in this state.
 *@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@*/
static short RmgcFrozen( DWORD event, DWORD value )
{
    switch( event )
    {
        case RMG_EV_SHUTDN:         /* main thread is shutting down */
            Rmgc_shutdown = 1;
            break;

        /* these can happen, but nothing to do */
        case RMG_EV_ACTV_RQST:      /* mate node is initializing and is asking to become primary */
        case RMG_EV_SWOVER_RQST:    /* mate node (primary) requesting that this node become primary [due to operator command or remote extraction pending] */
        case RMG_EV_TKOVER_RQST:    /* mate node (backup) requesting to become primary [due to operator command] */
        case RMG_EV_NETUP_RQST:     /* mate node (backup) network up so requesting to become primary (if primary network down) */
        case RMG_EV_NETDOWN_RQST:   /* mate node (primary) network down so requesting that this node become primary (if primary network up) */
        case RMG_EV_ACTV_CFM:       /* mate node (now backup) has accepted request for this node to be primary [due to initial startup request] */
        case RMG_EV_ACTV_REJ:       /* mate node (now primary) has rejected request for this node to be primary [due to initial startup request] */
        case RMG_EV_SWOVER_CFM:     /* mate node (now primary) has accepted this node's request to become backup */
        case RMG_EV_SWOVER_REJ:     /* mate node (now backup) has rejected this node's request to become backup */
        case RMG_EV_TKOVER_CFM:     /* mate node (now backup) has accepted this node's request to become primary */
        case RMG_EV_TKOVER_REJ:     /* mate node (now primary) has rejected this node's request to become primary */
        case RMG_EV_NETUP_CFM:      /* mate node (now backup) has accepted this node's request to become primary due to network up */
        case RMG_EV_NETUP_REJ:      /* mate node (now primary) has rejected this node's request to become primary due to network up */
        case RMG_EV_NETDOWN_CFM:    /* mate node (now primary) has accepted this node's request to become backup due to network down */
        case RMG_EV_NETDOWN_REJ:    /* mate node (now backup) has rejected this node's request to become backup due to network down */
        case RMG_EV_HEARTBEAT:      /* mate node (primary) providing heartbeat as health indicator */
        case RMG_EV_FAILURE:        /* mate node (primary) indicating that its board has failed */
        case RMG_EV_T1:             /* timer 1 (retry timer) has fired */
        case RMG_EV_T2:             /* timer 2 (monitoring for heartbeat timer) has fired */
        case RMG_EV_T3:             /* timer 3 (periodic heartbeat send timer) has fired */
        case RMG_EV_T4:             /* timer 4 (board download timer) has fired */
        case RMG_EV_LOAD_CMD:       /* operator command to load board */
        case RMG_EV_SWOVER_CMD:     /* operator command to cause changeover */
        case RMG_EV_HALT_CMD:       /* operator command to halt board */
        case RMG_EV_EXTRACT_CMD:    /* operator command indicating ready for extraction */
        case HMI_EVN_NOWPRIMARY:    /* HMI reporting that board is now operating as primary */
        case HMI_EVN_NOWBACKUP:     /* HMI reporting that board is now operating as backup */
        case HMI_EVN_STARTING:      /* HMI reporting that board has loaded and is now starting */
        case HMI_EVN_NOWSTANDALONE: /* HMI reporting that board is operating in standalone mode */
        case HMI_EVN_NO_BOARD:      /* HMI reporting board not present */
        case HMI_EVN_LOADING:       /* HMI reporting that board is loading */
        case HMI_EVN_CONNECTED:     /* HMI reporting that board is connected to its mate board */
        case HMI_EVN_ISOLATED:      /* HMI reporting that board is isolated from its mate board */
        case HMI_EVN_CONFLICT:      /* HMI reporting board conflict */
        case HMI_EVN_NETWORK_UP:    /* HMI forwarding task indication that network connectivity is up */
        case HMI_EVN_NETWORK_DOWN:  /* HMI forwarding task indication that network connectivity is down */
        case HMI_EVN_TASKDEAD:      /* HMI reporting that a task has failed */
        case HMI_EVN_BRDDEAD:       /* HMI reporting that board itself has failed */
        case HMI_EVN_HALTED:        /* HMI reporting that board is in the halted state */
        case HMI_EVN_EXTRACT:       /* HMI reporting board extraction pending */
        case HMI_EVN_INSERT:        /* HMI reporting that board has been inserted */
        case HMI_EVN_STOP:          /* HMI reporting board has been stopped */
        case HMI_EVN_SERVICE_DOWN:  /* HMI service has shutdown */
        case EVENT_FREEZE:          /* an event has been converted into indication to freeze further event processing */
            break;

        /* these events shouldn't happen - log them */
        default:
            UNEX_EVENT_ALARM( event );

    }   /* end switch on event */

    return( 0 );
}
